@@ -16,6 +16,8 @@ constexpr char kParent[] = "Parent";
 constexpr char kResources[] = "Resources";
 constexpr char kMediaBox[] = "MediaBox";
 constexpr char kCropBox[] = "CropBox";
+constexpr char kBleedBox[] = "BleedBox";
+constexpr char kTrimBox[] = "TrimBox";
 constexpr char kArtBox[] = "ArtBox";
 constexpr char kContents[] = "Contents";
 constexpr char kRotate[] = "Rotate";
