@@ -12,10 +12,12 @@
 #include <tuple>
 #include <utility>
 
+#include "core/fpdfapi/edit/cpdf_contentstream_write_utils.h"
 #include "core/fpdfapi/edit/cpdf_pagecontentmanager.h"
 #include "core/fpdfapi/edit/cpdf_stringarchivestream.h"
-#include "core/fpdfapi/font/cpdf_font.h"
-#include "core/fpdfapi/page/cpdf_contentmark.h"
+#include "core/fpdfapi/font/cpdf_truetypefont.h"
+#include "core/fpdfapi/font/cpdf_type1font.h"
+#include "core/fpdfapi/page/cpdf_contentmarks.h"
 #include "core/fpdfapi/page/cpdf_docpagedata.h"
 #include "core/fpdfapi/page/cpdf_image.h"
 #include "core/fpdfapi/page/cpdf_imageobject.h"
@@ -32,17 +34,12 @@
 #include "core/fpdfapi/parser/cpdf_stream.h"
 #include "core/fpdfapi/parser/fpdf_parser_decode.h"
 #include "core/fpdfapi/parser/fpdf_parser_utility.h"
+#include "core/fxge/render_defines.h"
+#include "third_party/base/ptr_util.h"
 #include "third_party/base/stl_util.h"
-#include "third_party/skia_shared/SkFloatToDecimal.h"
 
 namespace {
 
-std::ostream& operator<<(std::ostream& ar, const CFX_Matrix& matrix) {
-  ar << matrix.a << " " << matrix.b << " " << matrix.c << " " << matrix.d << " "
-     << matrix.e << " " << matrix.f;
-  return ar;
-}
-
 bool GetColor(const CPDF_Color* pColor, float* rgb) {
   int intRGB[3];
   if (!pColor || !pColor->IsColorSpaceRGB() ||
@@ -60,7 +57,7 @@ bool GetColor(const CPDF_Color* pColor, float* rgb) {
 CPDF_PageContentGenerator::CPDF_PageContentGenerator(
     CPDF_PageObjectHolder* pObjHolder)
     : m_pObjHolder(pObjHolder), m_pDocument(pObjHolder->GetDocument()) {
-  for (const auto& pObj : *pObjHolder->GetPageObjectList()) {
+  for (const auto& pObj : *pObjHolder) {
     if (pObj)
       m_pageObjects.emplace_back(pObj.get());
   }
@@ -88,17 +85,16 @@ CPDF_PageContentGenerator::GenerateModifiedStreams() {
     if (pPageObj->IsDirty())
       all_dirty_streams.insert(pPageObj->GetContentStream());
   }
-  const std::set<int32_t>& marked_dirty_streams =
-      m_pObjHolder->GetDirtyStreams();
+  std::set<int32_t> marked_dirty_streams = m_pObjHolder->TakeDirtyStreams();
   all_dirty_streams.insert(marked_dirty_streams.begin(),
                            marked_dirty_streams.end());
 
   // Start regenerating dirty streams.
   std::map<int32_t, std::unique_ptr<std::ostringstream>> streams;
   std::set<int32_t> empty_streams;
-  std::unique_ptr<const CPDF_ContentMark> empty_content_mark =
-      pdfium::MakeUnique<CPDF_ContentMark>();
-  std::map<int32_t, const CPDF_ContentMark*> current_content_mark;
+  std::unique_ptr<const CPDF_ContentMarks> empty_content_marks =
+      pdfium::MakeUnique<CPDF_ContentMarks>();
+  std::map<int32_t, const CPDF_ContentMarks*> current_content_marks;
 
   for (int32_t dirty_stream : all_dirty_streams) {
     std::unique_ptr<std::ostringstream> buf =
@@ -113,7 +109,7 @@ CPDF_PageContentGenerator::GenerateModifiedStreams() {
 
     streams[dirty_stream] = std::move(buf);
     empty_streams.insert(dirty_stream);
-    current_content_mark[dirty_stream] = empty_content_mark.get();
+    current_content_marks[dirty_stream] = empty_content_marks.get();
   }
 
   // Process the page objects, write into each dirty stream.
@@ -125,8 +121,8 @@ CPDF_PageContentGenerator::GenerateModifiedStreams() {
 
     std::ostringstream* buf = it->second.get();
     empty_streams.erase(stream_index);
-    current_content_mark[stream_index] = ProcessContentMarks(
-        buf, pPageObj.Get(), current_content_mark[stream_index]);
+    current_content_marks[stream_index] = ProcessContentMarks(
+        buf, pPageObj.Get(), current_content_marks[stream_index]);
     ProcessPageObject(buf, pPageObj.Get());
   }
 
@@ -137,16 +133,13 @@ CPDF_PageContentGenerator::GenerateModifiedStreams() {
       // Clear to show that this stream needs to be deleted.
       buf->str("");
     } else {
-      FinishMarks(buf, current_content_mark[dirty_stream]);
+      FinishMarks(buf, current_content_marks[dirty_stream]);
 
       // Return graphics to original state
       *buf << "Q\n";
     }
   }
 
-  // Clear dirty streams in m_pObjHolder
-  m_pObjHolder->ClearDirtyStreams();
-
   return streams;
 }
 
@@ -176,7 +169,7 @@ void CPDF_PageContentGenerator::UpdateContentStreams(
     if (buf->tellp() <= 0)
       page_content_manager.ScheduleRemoveStreamByIndex(stream_index);
     else
-      old_stream->SetDataFromStringstream(buf);
+      old_stream->SetDataFromStringstreamAndRemoveFilter(buf);
   }
 
   page_content_manager.ExecuteScheduledRemovals();
@@ -187,10 +180,11 @@ ByteString CPDF_PageContentGenerator::RealizeResource(
     const ByteString& bsType) const {
   ASSERT(pResource);
   if (!m_pObjHolder->m_pResources) {
-    m_pObjHolder->m_pResources = m_pDocument->NewIndirect<CPDF_Dictionary>();
-    m_pObjHolder->GetDict()->SetFor(
-        "Resources",
-        m_pObjHolder->m_pResources->MakeReference(m_pDocument.Get()));
+    m_pObjHolder->m_pResources.Reset(
+        m_pDocument->NewIndirect<CPDF_Dictionary>());
+    m_pObjHolder->GetDict()->SetNewFor<CPDF_Reference>(
+        "Resources", m_pDocument.Get(),
+        m_pObjHolder->m_pResources->GetObjNum());
   }
   CPDF_Dictionary* pResList = m_pObjHolder->m_pResources->GetDictFor(bsType);
   if (!pResList)
@@ -205,25 +199,26 @@ ByteString CPDF_PageContentGenerator::RealizeResource(
 
     idnum++;
   }
-  pResList->SetFor(name, pResource->MakeReference(m_pDocument.Get()));
+  pResList->SetNewFor<CPDF_Reference>(name, m_pDocument.Get(),
+                                      pResource->GetObjNum());
   return name;
 }
 
 bool CPDF_PageContentGenerator::ProcessPageObjects(std::ostringstream* buf) {
   bool bDirty = false;
-  std::unique_ptr<const CPDF_ContentMark> empty_content_mark =
-      pdfium::MakeUnique<CPDF_ContentMark>();
-  const CPDF_ContentMark* content_mark = empty_content_mark.get();
+  std::unique_ptr<const CPDF_ContentMarks> empty_content_marks =
+      pdfium::MakeUnique<CPDF_ContentMarks>();
+  const CPDF_ContentMarks* content_marks = empty_content_marks.get();
 
   for (auto& pPageObj : m_pageObjects) {
     if (m_pObjHolder->IsPage() && !pPageObj->IsDirty())
       continue;
 
     bDirty = true;
-    content_mark = ProcessContentMarks(buf, pPageObj.Get(), content_mark);
+    content_marks = ProcessContentMarks(buf, pPageObj.Get(), content_marks);
     ProcessPageObject(buf, pPageObj.Get());
   }
-  FinishMarks(buf, content_mark);
+  FinishMarks(buf, content_marks);
   return bDirty;
 }
 
@@ -235,11 +230,11 @@ void CPDF_PageContentGenerator::UpdateStreamlessPageObjects(
   }
 }
 
-const CPDF_ContentMark* CPDF_PageContentGenerator::ProcessContentMarks(
+const CPDF_ContentMarks* CPDF_PageContentGenerator::ProcessContentMarks(
     std::ostringstream* buf,
     const CPDF_PageObject* pPageObj,
-    const CPDF_ContentMark* pPrev) {
-  const CPDF_ContentMark* pNext = &pPageObj->m_ContentMark;
+    const CPDF_ContentMarks* pPrev) {
+  const CPDF_ContentMarks* pNext = &pPageObj->m_ContentMarks;
 
   size_t first_different = pPrev->FindFirstDifference(pNext);
 
@@ -255,23 +250,23 @@ const CPDF_ContentMark* CPDF_PageContentGenerator::ProcessContentMarks(
     const CPDF_ContentMarkItem* item = pNext->GetItem(i);
 
     // Write mark tag.
-    *buf << "/" << item->GetName() << " ";
+    *buf << "/" << PDF_NameEncode(item->GetName()) << " ";
 
     // If there are no parameters, write a BMC (begin marked content) operator.
-    if (item->GetParamType() == CPDF_ContentMarkItem::None) {
+    if (item->GetParamType() == CPDF_ContentMarkItem::kNone) {
       *buf << "BMC\n";
       continue;
     }
 
     // If there are parameters, write properties, direct or indirect.
     switch (item->GetParamType()) {
-      case CPDF_ContentMarkItem::DirectDict: {
+      case CPDF_ContentMarkItem::kDirectDict: {
         CPDF_StringArchiveStream archive_stream(buf);
         item->GetParam()->WriteTo(&archive_stream, nullptr);
         *buf << " ";
         break;
       }
-      case CPDF_ContentMarkItem::PropertiesDict: {
+      case CPDF_ContentMarkItem::kPropertiesDict: {
         *buf << "/" << item->GetPropertyName() << " ";
         break;
       }
@@ -289,11 +284,11 @@ const CPDF_ContentMark* CPDF_PageContentGenerator::ProcessContentMarks(
 
 void CPDF_PageContentGenerator::FinishMarks(
     std::ostringstream* buf,
-    const CPDF_ContentMark* pContentMark) {
+    const CPDF_ContentMarks* pContentMarks) {
   // Technically we should iterate backwards to close from the top to the
   // bottom, but since the EMC operators do not identify which mark they are
   // closing, it does not matter.
-  for (size_t i = 0; i < pContentMark->CountItems(); ++i)
+  for (size_t i = 0; i < pContentMarks->CountItems(); ++i)
     *buf << "EMC\n";
 }
 
@@ -329,9 +324,10 @@ void CPDF_PageContentGenerator::ProcessImage(std::ostringstream* buf,
     pImage->ConvertStreamToIndirectObject();
 
   ByteString name = RealizeResource(pStream, "XObject");
-  if (bWasInline)
-    pImageObj->SetImage(
-        m_pDocument->GetPageData()->GetImage(pStream->GetObjNum()));
+  if (bWasInline) {
+    auto* pPageData = CPDF_DocPageData::FromDocument(m_pDocument.Get());
+    pImageObj->SetImage(pPageData->GetImage(pStream->GetObjNum()));
+  }
 
   *buf << "/" << PDF_NameEncode(name) << " Do Q\n";
 }
@@ -351,24 +347,18 @@ void CPDF_PageContentGenerator::ProcessPath(std::ostringstream* buf,
                                             CPDF_PathObject* pPathObj) {
   ProcessGraphics(buf, pPathObj);
 
-  *buf << pPathObj->m_Matrix << " cm ";
+  *buf << pPathObj->matrix() << " cm ";
 
-  auto& pPoints = pPathObj->m_Path.GetPoints();
-  if (pPathObj->m_Path.IsRect()) {
+  const auto& pPoints = pPathObj->path().GetPoints();
+  if (pPathObj->path().IsRect()) {
     CFX_PointF diff = pPoints[2].m_Point - pPoints[0].m_Point;
-    *buf << pPoints[0].m_Point.x << " " << pPoints[0].m_Point.y << " " << diff.x
-         << " " << diff.y << " re";
+    *buf << pPoints[0].m_Point << " " << diff << " re";
   } else {
     for (size_t i = 0; i < pPoints.size(); i++) {
       if (i > 0)
         *buf << " ";
 
-      char buffer[pdfium::skia::kMaximumSkFloatToDecimalLength];
-      unsigned size =
-          pdfium::skia::SkFloatToDecimal(pPoints[i].m_Point.x, buffer);
-      buf->write(buffer, size) << " ";
-      size = pdfium::skia::SkFloatToDecimal(pPoints[i].m_Point.y, buffer);
-      buf->write(buffer, size);
+      *buf << pPoints[i].m_Point;
 
       FXPT_TYPE pointType = pPoints[i].m_Type;
       if (pointType == FXPT_TYPE::MoveTo) {
@@ -384,21 +374,21 @@ void CPDF_PageContentGenerator::ProcessPath(std::ostringstream* buf,
           *buf << " h";
           break;
         }
-        *buf << " " << pPoints[i + 1].m_Point.x << " "
-             << pPoints[i + 1].m_Point.y << " " << pPoints[i + 2].m_Point.x
-             << " " << pPoints[i + 2].m_Point.y << " c";
+        *buf << " ";
+        *buf << pPoints[i + 1].m_Point << " ";
+        *buf << pPoints[i + 2].m_Point << " c";
         i += 2;
       }
       if (pPoints[i].m_CloseFigure)
         *buf << " h";
     }
   }
-  if (pPathObj->m_FillType == 0)
-    *buf << (pPathObj->m_bStroke ? " S" : " n");
-  else if (pPathObj->m_FillType == FXFILL_WINDING)
-    *buf << (pPathObj->m_bStroke ? " B" : " f");
-  else if (pPathObj->m_FillType == FXFILL_ALTERNATE)
-    *buf << (pPathObj->m_bStroke ? " B*" : " f*");
+  if (pPathObj->filltype() == 0)
+    *buf << (pPathObj->stroke() ? " S" : " n");
+  else if (pPathObj->filltype() == FXFILL_WINDING)
+    *buf << (pPathObj->stroke() ? " B" : " f");
+  else if (pPathObj->filltype() == FXFILL_ALTERNATE)
+    *buf << (pPathObj->stroke() ? " B*" : " f*");
   *buf << " Q\n";
 }
 
@@ -425,7 +415,7 @@ void CPDF_PageContentGenerator::ProcessGraphics(std::ostringstream* buf,
   }
   float lineWidth = pPageObj->m_GraphState.GetLineWidth();
   if (lineWidth != 1.0f)
-    *buf << lineWidth << " w ";
+    WriteFloat(*buf, lineWidth) << " w ";
   CFX_GraphStateData::LineCap lineCap = pPageObj->m_GraphState.GetLineCap();
   if (lineCap != CFX_GraphStateData::LineCapButt)
     *buf << static_cast<int>(lineCap) << " J ";
@@ -438,8 +428,7 @@ void CPDF_PageContentGenerator::ProcessGraphics(std::ostringstream* buf,
   graphD.strokeAlpha = pPageObj->m_GeneralState.GetStrokeAlpha();
   graphD.blendType = pPageObj->m_GeneralState.GetBlendType();
   if (graphD.fillAlpha == 1.0f && graphD.strokeAlpha == 1.0f &&
-      (graphD.blendType == FXDIB_BLEND_UNSUPPORTED ||
-       graphD.blendType == FXDIB_BLEND_NORMAL)) {
+      graphD.blendType == BlendMode::kNormal) {
     return;
   }
 
@@ -448,19 +437,18 @@ void CPDF_PageContentGenerator::ProcessGraphics(std::ostringstream* buf,
   if (it != m_pObjHolder->m_GraphicsMap.end()) {
     name = it->second;
   } else {
-    auto gsDict = pdfium::MakeUnique<CPDF_Dictionary>();
+    auto gsDict = pdfium::MakeRetain<CPDF_Dictionary>();
     if (graphD.fillAlpha != 1.0f)
       gsDict->SetNewFor<CPDF_Number>("ca", graphD.fillAlpha);
 
     if (graphD.strokeAlpha != 1.0f)
       gsDict->SetNewFor<CPDF_Number>("CA", graphD.strokeAlpha);
 
-    if (graphD.blendType != FXDIB_BLEND_UNSUPPORTED &&
-        graphD.blendType != FXDIB_BLEND_NORMAL) {
+    if (graphD.blendType != BlendMode::kNormal) {
       gsDict->SetNewFor<CPDF_Name>("BM",
                                    pPageObj->m_GeneralState.GetBlendMode());
     }
-    CPDF_Object* pDict = m_pDocument->AddIndirectObject(std::move(gsDict));
+    CPDF_Object* pDict = m_pDocument->AddIndirectObject(gsDict);
     name = RealizeResource(pDict, "ExtGState");
     m_pObjHolder->m_GraphicsMap[graphD] = name;
   }
@@ -480,7 +468,7 @@ ByteString CPDF_PageContentGenerator::GetOrCreateDefaultGraphics() const {
   GraphicsData defaultGraphics;
   defaultGraphics.fillAlpha = 1.0f;
   defaultGraphics.strokeAlpha = 1.0f;
-  defaultGraphics.blendType = FXDIB_BLEND_NORMAL;
+  defaultGraphics.blendType = BlendMode::kNormal;
   auto it = m_pObjHolder->m_GraphicsMap.find(defaultGraphics);
 
   // If default graphics already exists, return it.
@@ -488,11 +476,11 @@ ByteString CPDF_PageContentGenerator::GetOrCreateDefaultGraphics() const {
     return it->second;
 
   // Otherwise, create them.
-  auto gsDict = pdfium::MakeUnique<CPDF_Dictionary>();
+  auto gsDict = pdfium::MakeRetain<CPDF_Dictionary>();
   gsDict->SetNewFor<CPDF_Number>("ca", defaultGraphics.fillAlpha);
   gsDict->SetNewFor<CPDF_Number>("CA", defaultGraphics.strokeAlpha);
   gsDict->SetNewFor<CPDF_Name>("BM", "Normal");
-  CPDF_Object* pDict = m_pDocument->AddIndirectObject(std::move(gsDict));
+  CPDF_Object* pDict = m_pDocument->AddIndirectObject(gsDict);
   ByteString name = RealizeResource(pDict, "ExtGState");
   m_pObjHolder->m_GraphicsMap[defaultGraphics] = name;
   return name;
@@ -506,20 +494,25 @@ void CPDF_PageContentGenerator::ProcessText(std::ostringstream* buf,
                                             CPDF_TextObject* pTextObj) {
   ProcessGraphics(buf, pTextObj);
   *buf << "BT " << pTextObj->GetTextMatrix() << " Tm ";
-  CPDF_Font* pFont = pTextObj->GetFont();
+  RetainPtr<CPDF_Font> pFont(pTextObj->GetFont());
   if (!pFont)
     pFont = CPDF_Font::GetStockFont(m_pDocument.Get(), "Helvetica");
-  FontData fontD;
-  if (pFont->IsType1Font())
-    fontD.type = "Type1";
-  else if (pFont->IsTrueTypeFont())
-    fontD.type = "TrueType";
-  else if (pFont->IsCIDFont())
-    fontD.type = "Type0";
-  else
+
+  FontData data;
+  const CPDF_FontEncoding* pEncoding = nullptr;
+  if (pFont->IsType1Font()) {
+    data.type = "Type1";
+    pEncoding = pFont->AsType1Font()->GetEncoding();
+  } else if (pFont->IsTrueTypeFont()) {
+    data.type = "TrueType";
+    pEncoding = pFont->AsTrueTypeFont()->GetEncoding();
+  } else if (pFont->IsCIDFont()) {
+    data.type = "Type0";
+  } else {
     return;
-  fontD.baseFont = pFont->GetBaseFont();
-  auto it = m_pObjHolder->m_FontsMap.find(fontD);
+  }
+  data.baseFont = pFont->GetBaseFontName();
+  auto it = m_pObjHolder->m_FontsMap.find(data);
   ByteString dictName;
   if (it != m_pObjHolder->m_FontsMap.end()) {
     dictName = it->second;
@@ -527,17 +520,21 @@ void CPDF_PageContentGenerator::ProcessText(std::ostringstream* buf,
     CPDF_Object* pIndirectFont = pFont->GetFontDict();
     if (pIndirectFont->IsInline()) {
       // In this case we assume it must be a standard font
-      auto fontDict = pdfium::MakeUnique<CPDF_Dictionary>();
-      fontDict->SetNewFor<CPDF_Name>("Type", "Font");
-      fontDict->SetNewFor<CPDF_Name>("Subtype", fontD.type);
-      fontDict->SetNewFor<CPDF_Name>("BaseFont", fontD.baseFont);
-      pIndirectFont = m_pDocument->AddIndirectObject(std::move(fontDict));
+      auto pFontDict = pdfium::MakeRetain<CPDF_Dictionary>();
+      pFontDict->SetNewFor<CPDF_Name>("Type", "Font");
+      pFontDict->SetNewFor<CPDF_Name>("Subtype", data.type);
+      pFontDict->SetNewFor<CPDF_Name>("BaseFont", data.baseFont);
+      if (pEncoding) {
+        pFontDict->SetFor("Encoding",
+                          pEncoding->Realize(m_pDocument->GetByteStringPool()));
+      }
+      pIndirectFont = m_pDocument->AddIndirectObject(pFontDict);
     }
     dictName = RealizeResource(pIndirectFont, "Font");
-    m_pObjHolder->m_FontsMap[fontD] = dictName;
+    m_pObjHolder->m_FontsMap[data] = dictName;
   }
-  *buf << "/" << PDF_NameEncode(dictName) << " " << pTextObj->GetFontSize()
-       << " Tf ";
+  *buf << "/" << PDF_NameEncode(dictName) << " ";
+  WriteFloat(*buf, pTextObj->GetFontSize()) << " Tf ";
   ByteString text;
   for (uint32_t charcode : pTextObj->GetCharCodes()) {
     if (charcode != CPDF_Font::kInvalidCharCode)
