@@ -9,17 +9,15 @@
 
 #include <memory>
 
-#include "core/fxcrt/fx_memory.h"
 #include "core/fxcrt/fx_string.h"
 #include "core/fxcrt/fx_system.h"
 #include "core/fxcrt/retain_ptr.h"
-#include "core/fxcrt/unowned_ptr.h"
 
 class CPDF_Dictionary;
 
 class CPDF_ContentMarkItem final : public Retainable {
  public:
-  enum ParamType { None, PropertiesDict, DirectDict };
+  enum ParamType { kNone, kPropertiesDict, kDirectDict };
 
   explicit CPDF_ContentMarkItem(ByteString name);
   ~CPDF_ContentMarkItem() override;
@@ -31,16 +29,16 @@ class CPDF_ContentMarkItem final : public Retainable {
   const ByteString& GetPropertyName() const { return m_PropertyName; }
   bool HasMCID() const;
 
-  void SetDirectDict(std::unique_ptr<CPDF_Dictionary> pDict);
-  void SetPropertiesDict(CPDF_Dictionary* pDict,
-                         const ByteString& property_name);
+  void SetDirectDict(RetainPtr<CPDF_Dictionary> pDict);
+  void SetPropertiesHolder(CPDF_Dictionary* pHolder,
+                           const ByteString& property_name);
 
  private:
+  ParamType m_ParamType = kNone;
   ByteString m_MarkName;
-  ParamType m_ParamType = None;
-  UnownedPtr<CPDF_Dictionary> m_pPropertiesDict;
   ByteString m_PropertyName;
-  std::unique_ptr<CPDF_Dictionary> m_pDirectDict;
+  RetainPtr<CPDF_Dictionary> m_pPropertiesHolder;
+  RetainPtr<CPDF_Dictionary> m_pDirectDict;
 };
 
 #endif  // CORE_FPDFAPI_PAGE_CPDF_CONTENTMARKITEM_H_
