@@ -13,7 +13,7 @@
 #include <set>
 #include <vector>
 
-#include "core/fpdfapi/render/cpdf_transparency.h"
+#include "core/fpdfapi/page/cpdf_transparency.h"
 #include "core/fxcrt/fx_coordinates.h"
 #include "core/fxcrt/fx_string.h"
 #include "core/fxcrt/fx_system.h"
@@ -66,11 +66,8 @@ class CPDF_PageObjectHolder {
 
   CPDF_Document* GetDocument() const { return m_pDocument.Get(); }
 
-  // TODO(thestig): Can this return nullptr? If not, audit callers and simplify
-  // the ones that assume it can.
   CPDF_Dictionary* GetDict() const { return m_pDict.Get(); }
-
-  size_t GetPageObjectCount() const;
+  size_t GetPageObjectCount() const { return m_PageObjectList.size(); }
   CPDF_PageObject* GetPageObjectByIndex(size_t index) const;
   void AppendPageObject(std::unique_ptr<CPDF_PageObject> pPageObj);
   bool RemovePageObject(CPDF_PageObject* pPageObj);
@@ -97,9 +94,8 @@ class CPDF_PageObjectHolder {
   }
   void AddImageMaskBoundingBox(const CFX_FloatRect& box);
   void Transform(const CFX_Matrix& matrix);
-  CFX_FloatRect CalcBoundingBox() const;
-  const std::set<int32_t>& GetDirtyStreams() const { return m_DirtyStreams; }
-  void ClearDirtyStreams() { m_DirtyStreams.clear(); }
+  bool HasDirtyStreams() const { return !m_DirtyStreams.empty(); }
+  std::set<int32_t> TakeDirtyStreams();
 
   RetainPtr<CPDF_Dictionary> m_pPageResources;
   RetainPtr<CPDF_Dictionary> m_pResources;
