@@ -27,10 +27,10 @@ void CPDF_ShadingObject::Transform(const CFX_Matrix& matrix) {
   m_Matrix.Concat(matrix);
   if (m_ClipPath.HasRef()) {
     CalcBoundingBox();
-  } else {
-    std::tie(m_Left, m_Right, m_Top, m_Bottom) =
-        matrix.TransformRect(m_Left, m_Right, m_Top, m_Bottom);
+    return;
   }
+
+  SetRect(matrix.TransformRect(GetRect()));
 }
 
 bool CPDF_ShadingObject::IsShading() const {
@@ -48,9 +48,5 @@ const CPDF_ShadingObject* CPDF_ShadingObject::AsShading() const {
 void CPDF_ShadingObject::CalcBoundingBox() {
   if (!m_ClipPath.HasRef())
     return;
-  CFX_FloatRect rect = m_ClipPath.GetClipBox();
-  m_Left = rect.left;
-  m_Bottom = rect.bottom;
-  m_Right = rect.right;
-  m_Top = rect.top;
+  SetRect(m_ClipPath.GetClipBox());
 }
