@@ -10,25 +10,29 @@
 #include <map>
 #include <memory>
 #include <set>
+#include <stack>
 #include <vector>
 
-#include "core/fpdfapi/page/cpdf_contentmark.h"
-#include "core/fpdfapi/parser/cpdf_stream.h"
+#include "core/fpdfapi/page/cpdf_contentmarks.h"
+#include "core/fxcrt/fx_number.h"
 #include "core/fxcrt/fx_string.h"
+#include "core/fxcrt/retain_ptr.h"
 #include "core/fxge/cfx_pathdata.h"
 
 class CPDF_AllStates;
+class CPDF_ColorSpace;
 class CPDF_Dictionary;
 class CPDF_Document;
 class CPDF_Font;
 class CPDF_Image;
 class CPDF_ImageObject;
+class CPDF_Object;
 class CPDF_PageObject;
 class CPDF_PageObjectHolder;
+class CPDF_Pattern;
+class CPDF_Stream;
 class CPDF_StreamParser;
 class CPDF_TextObject;
-class CPDF_ColorSpace;
-class CPDF_Pattern;
 
 class CPDF_StreamContentParser {
  public:
@@ -36,26 +40,28 @@ class CPDF_StreamContentParser {
                            CPDF_Dictionary* pPageResources,
                            CPDF_Dictionary* pParentResources,
                            const CFX_Matrix* pmtContentToUser,
-                           CPDF_PageObjectHolder* pObjectHolder,
+                           CPDF_PageObjectHolder* pObjHolder,
                            CPDF_Dictionary* pResources,
                            const CFX_FloatRect& rcBBox,
-                           CPDF_AllStates* pAllStates,
-                           std::set<const uint8_t*>* parsedSet);
+                           const CPDF_AllStates* pStates,
+                           std::set<const uint8_t*>* pParsedSet);
   ~CPDF_StreamContentParser();
 
-  uint32_t Parse(const uint8_t* pData, uint32_t dwSize, uint32_t max_cost);
+  uint32_t Parse(const uint8_t* pData,
+                 uint32_t dwSize,
+                 uint32_t start_offset,
+                 uint32_t max_cost,
+                 const std::vector<uint32_t>& stream_start_offsets);
   CPDF_PageObjectHolder* GetPageObjectHolder() const {
     return m_pObjectHolder.Get();
   }
   CPDF_AllStates* GetCurStates() const { return m_pCurStates.get(); }
   bool IsColored() const { return m_bColored; }
   const float* GetType3Data() const { return m_Type3Data; }
-  CPDF_Font* FindFont(const ByteString& name);
+  RetainPtr<CPDF_Font> FindFont(const ByteString& name);
 
-  static ByteStringView FindKeyAbbreviationForTesting(
-      const ByteStringView& abbr);
-  static ByteStringView FindValueAbbreviationForTesting(
-      const ByteStringView& abbr);
+  static ByteStringView FindKeyAbbreviationForTesting(ByteStringView abbr);
+  static ByteStringView FindValueAbbreviationForTesting(ByteStringView abbr);
 
  private:
   struct ContentParam {
@@ -65,18 +71,9 @@ class CPDF_StreamContentParser {
     ~ContentParam();
 
     Type m_Type;
-    std::unique_ptr<CPDF_Object> m_pObject;
-    struct {
-      bool m_bInteger;
-      union {
-        int m_Integer;
-        float m_Float;
-      };
-    } m_Number;
-    struct {
-      int m_Len;
-      char m_Buffer[32];
-    } m_Name;
+    FX_Number m_Number;
+    ByteString m_Name;
+    RetainPtr<CPDF_Object> m_pObject;
   };
 
   static const int kParamBufSize = 16;
@@ -84,27 +81,34 @@ class CPDF_StreamContentParser {
   using OpCodes = std::map<uint32_t, void (CPDF_StreamContentParser::*)()>;
   static OpCodes InitializeOpCodes();
 
-  void AddNameParam(const ByteStringView& str);
-  void AddNumberParam(const ByteStringView& str);
-  void AddObjectParam(std::unique_ptr<CPDF_Object> pObj);
+  void AddNameParam(ByteStringView bsName);
+  void AddNumberParam(ByteStringView str);
+  void AddObjectParam(RetainPtr<CPDF_Object> pObj);
   int GetNextParamPos();
   void ClearAllParams();
   CPDF_Object* GetObject(uint32_t index);
-  ByteString GetString(uint32_t index);
-  float GetNumber(uint32_t index);
-  int GetInteger(uint32_t index) { return (int32_t)(GetNumber(index)); }
-  void OnOperator(const ByteStringView& op);
-  void AddTextObject(ByteString* pText,
+  ByteString GetString(uint32_t index) const;
+  float GetNumber(uint32_t index) const;
+  // Calls GetNumber() |count| times and returns the values in reverse order.
+  // e.g. for |count| = 3, returns [GetNumber(2), GetNumber(1), GetNumber(0)].
+  std::vector<float> GetNumbers(size_t count) const;
+  int GetInteger(uint32_t index) const {
+    return static_cast<int>(GetNumber(index));
+  }
+  void OnOperator(ByteStringView op);
+  void AddTextObject(const ByteString* pStrs,
                      float fInitKerning,
-                     float* pKerning,
-                     int count);
+                     const std::vector<float>& kernings,
+                     size_t nSegs);
+  float GetHorizontalTextSize(float fKerning) const;
+  float GetVerticalTextSize(float fKerning) const;
 
   void OnChangeTextMatrix();
   void ParsePathObject();
   void AddPathPoint(float x, float y, FXPT_TYPE type, bool close);
   void AddPathRect(float x, float y, float w, float h);
   void AddPathObject(int FillType, bool bStroke);
-  CPDF_ImageObject* AddImage(std::unique_ptr<CPDF_Stream> pStream);
+  CPDF_ImageObject* AddImage(RetainPtr<CPDF_Stream> pStream);
   CPDF_ImageObject* AddImage(uint32_t streamObjNum);
   CPDF_ImageObject* AddImage(const RetainPtr<CPDF_Image>& pImage);
 
@@ -113,13 +117,18 @@ class CPDF_StreamContentParser {
                         bool bColor,
                         bool bText,
                         bool bGraph);
-  CPDF_ColorSpace* FindColorSpace(const ByteString& name);
-  CPDF_Pattern* FindPattern(const ByteString& name, bool bShading);
+  RetainPtr<CPDF_ColorSpace> FindColorSpace(const ByteString& name);
+  RetainPtr<CPDF_Pattern> FindPattern(const ByteString& name, bool bShading);
+  CPDF_Dictionary* FindResourceHolder(const ByteString& type);
   CPDF_Object* FindResourceObj(const ByteString& type, const ByteString& name);
 
   // Takes ownership of |pImageObj|, returns unowned pointer to it.
   CPDF_ImageObject* AddImageObject(std::unique_ptr<CPDF_ImageObject> pImageObj);
 
+  std::vector<float> GetColors() const;
+  std::vector<float> GetNamedColors() const;
+  int32_t GetCurrentStreamIndex();
+
   void Handle_CloseFillStrokePath();
   void Handle_FillStrokePath();
   void Handle_CloseEOFillStrokePath();
@@ -194,34 +203,40 @@ class CPDF_StreamContentParser {
   void Handle_Invalid();
 
   UnownedPtr<CPDF_Document> const m_pDocument;
-  UnownedPtr<CPDF_Dictionary> m_pPageResources;
-  UnownedPtr<CPDF_Dictionary> m_pParentResources;
-  UnownedPtr<CPDF_Dictionary> m_pResources;
-  UnownedPtr<CPDF_PageObjectHolder> m_pObjectHolder;
-  UnownedPtr<std::set<const uint8_t*>> m_ParsedSet;
+  RetainPtr<CPDF_Dictionary> const m_pPageResources;
+  RetainPtr<CPDF_Dictionary> const m_pParentResources;
+  RetainPtr<CPDF_Dictionary> const m_pResources;
+  UnownedPtr<CPDF_PageObjectHolder> const m_pObjectHolder;
+  UnownedPtr<std::set<const uint8_t*>> const m_ParsedSet;
   CFX_Matrix m_mtContentToUser;
   const CFX_FloatRect m_BBox;
-  ContentParam m_ParamBuf[kParamBufSize];
-  uint32_t m_ParamStartPos;
-  uint32_t m_ParamCount;
-  CPDF_StreamParser* m_pSyntax;
+  uint32_t m_ParamStartPos = 0;
+  uint32_t m_ParamCount = 0;
+  UnownedPtr<CPDF_StreamParser> m_pSyntax;
   std::unique_ptr<CPDF_AllStates> m_pCurStates;
-  CPDF_ContentMark m_CurContentMark;
+  std::stack<std::unique_ptr<CPDF_ContentMarks>> m_ContentMarksStack;
   std::vector<std::unique_ptr<CPDF_TextObject>> m_ClipTextList;
   UnownedPtr<CPDF_TextObject> m_pLastTextObject;
-  float m_DefFontSize;
   std::vector<FX_PATHPOINT> m_PathPoints;
-  float m_PathStartX;
-  float m_PathStartY;
-  float m_PathCurrentX;
-  float m_PathCurrentY;
-  uint8_t m_PathClipType;
+  float m_PathStartX = 0.0f;
+  float m_PathStartY = 0.0f;
+  float m_PathCurrentX = 0.0f;
+  float m_PathCurrentY = 0.0f;
+  uint8_t m_PathClipType = 0;
   ByteString m_LastImageName;
   RetainPtr<CPDF_Image> m_pLastImage;
-  bool m_bColored;
-  float m_Type3Data[6];
-  bool m_bResourceMissing;
+  bool m_bColored = false;
+  bool m_bResourceMissing = false;
   std::vector<std::unique_ptr<CPDF_AllStates>> m_StateStack;
+  float m_Type3Data[6] = {0.0f};
+  ContentParam m_ParamBuf[kParamBufSize];
+
+  // The merged stream offsets at which a content stream ends and another
+  // begins.
+  std::vector<uint32_t> m_StreamStartOffsets;
+
+  // The merged stream offset at which the last |m_pSyntax| started parsing.
+  uint32_t m_StartParseOffset = 0;
 };
 
 #endif  // CORE_FPDFAPI_PAGE_CPDF_STREAMCONTENTPARSER_H_
