@@ -16,31 +16,36 @@
 class CPDF_Document;
 class CPDF_Form;
 class CPDF_Object;
+class CPDF_PageObject;
 
 class CPDF_TilingPattern final : public CPDF_Pattern {
  public:
-  CPDF_TilingPattern(CPDF_Document* pDoc,
-                     CPDF_Object* pPatternObj,
-                     const CFX_Matrix& parentMatrix);
+  template <typename T, typename... Args>
+  friend RetainPtr<T> pdfium::MakeRetain(Args&&... args);
+
   ~CPDF_TilingPattern() override;
 
+  // CPDF_Pattern:
   CPDF_TilingPattern* AsTilingPattern() override;
-  CPDF_ShadingPattern* AsShadingPattern() override;
 
-  bool Load();
+  std::unique_ptr<CPDF_Form> Load(CPDF_PageObject* pPageObj);
 
   bool colored() const { return m_bColored; }
   const CFX_FloatRect& bbox() const { return m_BBox; }
   float x_step() const { return m_XStep; }
   float y_step() const { return m_YStep; }
-  CPDF_Form* form() const { return m_pForm.get(); }
 
  private:
+  CPDF_TilingPattern(CPDF_Document* pDoc,
+                     CPDF_Object* pPatternObj,
+                     const CFX_Matrix& parentMatrix);
+  CPDF_TilingPattern(const CPDF_TilingPattern&) = delete;
+  CPDF_TilingPattern& operator=(const CPDF_TilingPattern&) = delete;
+
   bool m_bColored;
   CFX_FloatRect m_BBox;
   float m_XStep;
   float m_YStep;
-  std::unique_ptr<CPDF_Form> m_pForm;
 };
 
 #endif  // CORE_FPDFAPI_PAGE_CPDF_TILINGPATTERN_H_
