@@ -9,6 +9,7 @@
 #include <set>
 #include <utility>
 
+#include "core/fpdfapi/parser/cpdf_boolean.h"
 #include "core/fpdfapi/parser/cpdf_name.h"
 #include "core/fpdfapi/parser/cpdf_number.h"
 #include "core/fpdfapi/parser/cpdf_reference.h"
@@ -16,9 +17,10 @@
 #include "core/fpdfapi/parser/cpdf_string.h"
 #include "core/fxcrt/fx_stream.h"
 #include "third_party/base/logging.h"
+#include "third_party/base/ptr_util.h"
 #include "third_party/base/stl_util.h"
 
-CPDF_Array::CPDF_Array() {}
+CPDF_Array::CPDF_Array() = default;
 
 CPDF_Array::CPDF_Array(const WeakPtr<ByteStringPool>& pPool) : m_pPool(pPool) {}
 
@@ -27,12 +29,12 @@ CPDF_Array::~CPDF_Array() {
   m_ObjNum = kInvalidObjNum;
   for (auto& it : m_Objects) {
     if (it && it->GetObjNum() == kInvalidObjNum)
-      it.release();
+      it.Leak();
   }
 }
 
 CPDF_Object::Type CPDF_Array::GetType() const {
-  return ARRAY;
+  return kArray;
 }
 
 bool CPDF_Array::IsArray() const {
@@ -47,28 +49,28 @@ const CPDF_Array* CPDF_Array::AsArray() const {
   return this;
 }
 
-std::unique_ptr<CPDF_Object> CPDF_Array::Clone() const {
+RetainPtr<CPDF_Object> CPDF_Array::Clone() const {
   return CloneObjectNonCyclic(false);
 }
 
-std::unique_ptr<CPDF_Object> CPDF_Array::CloneNonCyclic(
+RetainPtr<CPDF_Object> CPDF_Array::CloneNonCyclic(
     bool bDirect,
     std::set<const CPDF_Object*>* pVisited) const {
   pVisited->insert(this);
-  auto pCopy = pdfium::MakeUnique<CPDF_Array>();
+  auto pCopy = pdfium::MakeRetain<CPDF_Array>();
   for (const auto& pValue : m_Objects) {
-    if (!pdfium::ContainsKey(*pVisited, pValue.get())) {
+    if (!pdfium::ContainsKey(*pVisited, pValue.Get())) {
       std::set<const CPDF_Object*> visited(*pVisited);
       if (auto obj = pValue->CloneNonCyclic(bDirect, &visited))
         pCopy->m_Objects.push_back(std::move(obj));
     }
   }
-  return std::move(pCopy);
+  return pCopy;
 }
 
 CFX_FloatRect CPDF_Array::GetRect() const {
   CFX_FloatRect rect;
-  if (!IsArray() || m_Objects.size() != 4)
+  if (m_Objects.size() != 4)
     return rect;
 
   rect.left = GetNumberAt(0);
@@ -79,64 +81,70 @@ CFX_FloatRect CPDF_Array::GetRect() const {
 }
 
 CFX_Matrix CPDF_Array::GetMatrix() const {
-  CFX_Matrix matrix;
-  if (!IsArray() || m_Objects.size() != 6)
+  if (m_Objects.size() != 6)
     return CFX_Matrix();
 
   return CFX_Matrix(GetNumberAt(0), GetNumberAt(1), GetNumberAt(2),
                     GetNumberAt(3), GetNumberAt(4), GetNumberAt(5));
 }
 
-CPDF_Object* CPDF_Array::GetObjectAt(size_t i) {
-  if (i >= m_Objects.size())
+CPDF_Object* CPDF_Array::GetObjectAt(size_t index) {
+  if (index >= m_Objects.size())
     return nullptr;
-  return m_Objects[i].get();
+  return m_Objects[index].Get();
 }
 
-const CPDF_Object* CPDF_Array::GetObjectAt(size_t i) const {
-  if (i >= m_Objects.size())
+const CPDF_Object* CPDF_Array::GetObjectAt(size_t index) const {
+  if (index >= m_Objects.size())
     return nullptr;
-  return m_Objects[i].get();
+  return m_Objects[index].Get();
 }
 
-CPDF_Object* CPDF_Array::GetDirectObjectAt(size_t i) {
-  if (i >= m_Objects.size())
+CPDF_Object* CPDF_Array::GetDirectObjectAt(size_t index) {
+  if (index >= m_Objects.size())
     return nullptr;
-  return m_Objects[i]->GetDirect();
+  return m_Objects[index]->GetDirect();
 }
 
-const CPDF_Object* CPDF_Array::GetDirectObjectAt(size_t i) const {
-  if (i >= m_Objects.size())
+const CPDF_Object* CPDF_Array::GetDirectObjectAt(size_t index) const {
+  if (index >= m_Objects.size())
     return nullptr;
-  return m_Objects[i]->GetDirect();
+  return m_Objects[index]->GetDirect();
 }
 
-ByteString CPDF_Array::GetStringAt(size_t i) const {
-  if (i >= m_Objects.size())
+ByteString CPDF_Array::GetStringAt(size_t index) const {
+  if (index >= m_Objects.size())
     return ByteString();
-  return m_Objects[i]->GetString();
+  return m_Objects[index]->GetString();
 }
 
-WideString CPDF_Array::GetUnicodeTextAt(size_t i) const {
-  if (i >= m_Objects.size())
+WideString CPDF_Array::GetUnicodeTextAt(size_t index) const {
+  if (index >= m_Objects.size())
     return WideString();
-  return m_Objects[i]->GetUnicodeText();
+  return m_Objects[index]->GetUnicodeText();
 }
 
-int CPDF_Array::GetIntegerAt(size_t i) const {
-  if (i >= m_Objects.size())
+bool CPDF_Array::GetBooleanAt(size_t index, bool bDefault) const {
+  if (index >= m_Objects.size())
+    return bDefault;
+  const CPDF_Object* p = m_Objects[index].Get();
+  return ToBoolean(p) ? p->GetInteger() != 0 : bDefault;
+}
+
+int CPDF_Array::GetIntegerAt(size_t index) const {
+  if (index >= m_Objects.size())
     return 0;
-  return m_Objects[i]->GetInteger();
+  return m_Objects[index]->GetInteger();
 }
 
-float CPDF_Array::GetNumberAt(size_t i) const {
-  if (i >= m_Objects.size())
+float CPDF_Array::GetNumberAt(size_t index) const {
+  if (index >= m_Objects.size())
     return 0;
-  return m_Objects[i]->GetNumber();
+  return m_Objects[index]->GetNumber();
 }
 
-CPDF_Dictionary* CPDF_Array::GetDictAt(size_t i) {
-  CPDF_Object* p = GetDirectObjectAt(i);
+CPDF_Dictionary* CPDF_Array::GetDictAt(size_t index) {
+  CPDF_Object* p = GetDirectObjectAt(index);
   if (!p)
     return nullptr;
   if (CPDF_Dictionary* pDict = p->AsDictionary())
@@ -146,8 +154,8 @@ CPDF_Dictionary* CPDF_Array::GetDictAt(size_t i) {
   return nullptr;
 }
 
-const CPDF_Dictionary* CPDF_Array::GetDictAt(size_t i) const {
-  const CPDF_Object* p = GetDirectObjectAt(i);
+const CPDF_Dictionary* CPDF_Array::GetDictAt(size_t index) const {
+  const CPDF_Object* p = GetDirectObjectAt(index);
   if (!p)
     return nullptr;
   if (const CPDF_Dictionary* pDict = p->AsDictionary())
@@ -157,60 +165,62 @@ const CPDF_Dictionary* CPDF_Array::GetDictAt(size_t i) const {
   return nullptr;
 }
 
-CPDF_Stream* CPDF_Array::GetStreamAt(size_t i) {
-  return ToStream(GetDirectObjectAt(i));
+CPDF_Stream* CPDF_Array::GetStreamAt(size_t index) {
+  return ToStream(GetDirectObjectAt(index));
 }
 
-const CPDF_Stream* CPDF_Array::GetStreamAt(size_t i) const {
-  return ToStream(GetDirectObjectAt(i));
+const CPDF_Stream* CPDF_Array::GetStreamAt(size_t index) const {
+  return ToStream(GetDirectObjectAt(index));
 }
 
-CPDF_Array* CPDF_Array::GetArrayAt(size_t i) {
-  return ToArray(GetDirectObjectAt(i));
+CPDF_Array* CPDF_Array::GetArrayAt(size_t index) {
+  return ToArray(GetDirectObjectAt(index));
 }
 
-const CPDF_Array* CPDF_Array::GetArrayAt(size_t i) const {
-  return ToArray(GetDirectObjectAt(i));
+const CPDF_Array* CPDF_Array::GetArrayAt(size_t index) const {
+  return ToArray(GetDirectObjectAt(index));
 }
 
 void CPDF_Array::Clear() {
+  CHECK(!IsLocked());
   m_Objects.clear();
 }
 
-void CPDF_Array::RemoveAt(size_t i) {
-  if (i < m_Objects.size())
-    m_Objects.erase(m_Objects.begin() + i);
+void CPDF_Array::RemoveAt(size_t index) {
+  CHECK(!IsLocked());
+  if (index < m_Objects.size())
+    m_Objects.erase(m_Objects.begin() + index);
 }
 
-void CPDF_Array::ConvertToIndirectObjectAt(size_t i,
+void CPDF_Array::ConvertToIndirectObjectAt(size_t index,
                                            CPDF_IndirectObjectHolder* pHolder) {
-  if (i >= m_Objects.size())
+  CHECK(!IsLocked());
+  if (index >= m_Objects.size())
     return;
 
-  if (!m_Objects[i] || m_Objects[i]->IsReference())
+  if (!m_Objects[index] || m_Objects[index]->IsReference())
     return;
 
-  CPDF_Object* pNew = pHolder->AddIndirectObject(std::move(m_Objects[i]));
-  m_Objects[i] = pNew->MakeReference(pHolder);
+  CPDF_Object* pNew = pHolder->AddIndirectObject(std::move(m_Objects[index]));
+  m_Objects[index] = pNew->MakeReference(pHolder);
 }
 
-CPDF_Object* CPDF_Array::SetAt(size_t i, std::unique_ptr<CPDF_Object> pObj) {
-  ASSERT(IsArray());
+CPDF_Object* CPDF_Array::SetAt(size_t index, RetainPtr<CPDF_Object> pObj) {
+  CHECK(!IsLocked());
   ASSERT(!pObj || pObj->IsInline());
-  if (i >= m_Objects.size()) {
+  if (index >= m_Objects.size()) {
     NOTREACHED();
     return nullptr;
   }
-  CPDF_Object* pRet = pObj.get();
-  m_Objects[i] = std::move(pObj);
+  CPDF_Object* pRet = pObj.Get();
+  m_Objects[index] = std::move(pObj);
   return pRet;
 }
 
-CPDF_Object* CPDF_Array::InsertAt(size_t index,
-                                  std::unique_ptr<CPDF_Object> pObj) {
-  ASSERT(IsArray());
+CPDF_Object* CPDF_Array::InsertAt(size_t index, RetainPtr<CPDF_Object> pObj) {
+  CHECK(!IsLocked());
   CHECK(!pObj || pObj->IsInline());
-  CPDF_Object* pRet = pObj.get();
+  CPDF_Object* pRet = pObj.Get();
   if (index >= m_Objects.size()) {
     // Allocate space first.
     m_Objects.resize(index + 1);
@@ -222,10 +232,10 @@ CPDF_Object* CPDF_Array::InsertAt(size_t index,
   return pRet;
 }
 
-CPDF_Object* CPDF_Array::Add(std::unique_ptr<CPDF_Object> pObj) {
-  ASSERT(IsArray());
+CPDF_Object* CPDF_Array::Add(RetainPtr<CPDF_Object> pObj) {
+  CHECK(!IsLocked());
   CHECK(!pObj || pObj->IsInline());
-  CPDF_Object* pRet = pObj.get();
+  CPDF_Object* pRet = pObj.Get();
   m_Objects.push_back(std::move(pObj));
   return pRet;
 }
@@ -235,9 +245,18 @@ bool CPDF_Array::WriteTo(IFX_ArchiveStream* archive,
   if (!archive->WriteString("["))
     return false;
 
-  for (size_t i = 0; i < GetCount(); ++i) {
+  for (size_t i = 0; i < size(); ++i) {
     if (!GetObjectAt(i)->WriteTo(archive, encryptor))
       return false;
   }
   return archive->WriteString("]");
 }
+
+CPDF_ArrayLocker::CPDF_ArrayLocker(const CPDF_Array* pArray)
+    : m_pArray(pArray) {
+  m_pArray->m_LockCount++;
+}
+
+CPDF_ArrayLocker::~CPDF_ArrayLocker() {
+  m_pArray->m_LockCount--;
+}
