@@ -5,6 +5,7 @@
 // Original code copyright 2014 Foxit Software Inc. http://www.foxitsoftware.com
 
 #include "core/fpdfapi/parser/cpdf_boolean.h"
+
 #include "core/fxcrt/fx_stream.h"
 #include "third_party/base/ptr_util.h"
 
@@ -15,11 +16,11 @@ CPDF_Boolean::CPDF_Boolean(bool value) : m_bValue(value) {}
 CPDF_Boolean::~CPDF_Boolean() {}
 
 CPDF_Object::Type CPDF_Boolean::GetType() const {
-  return BOOLEAN;
+  return kBoolean;
 }
 
-std::unique_ptr<CPDF_Object> CPDF_Boolean::Clone() const {
-  return pdfium::MakeUnique<CPDF_Boolean>(m_bValue);
+RetainPtr<CPDF_Object> CPDF_Boolean::Clone() const {
+  return pdfium::MakeRetain<CPDF_Boolean>(m_bValue);
 }
 
 ByteString CPDF_Boolean::GetString() const {
