@@ -20,6 +20,7 @@
 #include "core/fxcrt/cfx_bitstream.h"
 #include "core/fxcrt/fx_safe_types.h"
 #include "third_party/base/numerics/safe_conversions.h"
+#include "third_party/base/ptr_util.h"
 #include "third_party/base/span.h"
 
 namespace {
@@ -60,7 +61,7 @@ std::unique_ptr<CPDF_HintTables> CPDF_HintTables::Parse(
   }
 
   parser->SetPos(szHintStart);
-  std::unique_ptr<CPDF_Stream> hints_stream = ToStream(
+  RetainPtr<CPDF_Stream> hints_stream = ToStream(
       parser->GetIndirectObject(nullptr, CPDF_SyntaxParser::ParseType::kLoose));
 
   if (!hints_stream)
@@ -68,7 +69,7 @@ std::unique_ptr<CPDF_HintTables> CPDF_HintTables::Parse(
 
   auto pHintTables = pdfium::MakeUnique<CPDF_HintTables>(
       parser->GetValidator().Get(), pLinearized);
-  if (!pHintTables->LoadHintStream(hints_stream.get()))
+  if (!pHintTables->LoadHintStream(hints_stream.Get()))
     return nullptr;
 
   return pHintTables;
