@@ -10,9 +10,9 @@
 #include <vector>
 
 #include "constants/stream_dict_common.h"
-#include "core/fpdfapi/edit/cpdf_encryptor.h"
-#include "core/fpdfapi/edit/cpdf_flateencoder.h"
 #include "core/fpdfapi/parser/cpdf_dictionary.h"
+#include "core/fpdfapi/parser/cpdf_encryptor.h"
+#include "core/fpdfapi/parser/cpdf_flateencoder.h"
 #include "core/fpdfapi/parser/cpdf_number.h"
 #include "core/fpdfapi/parser/cpdf_stream_acc.h"
 #include "core/fpdfapi/parser/fpdf_parser_decode.h"
@@ -34,27 +34,27 @@ CPDF_Stream::CPDF_Stream() {}
 
 CPDF_Stream::CPDF_Stream(std::unique_ptr<uint8_t, FxFreeDeleter> pData,
                          uint32_t size,
-                         std::unique_ptr<CPDF_Dictionary> pDict)
+                         RetainPtr<CPDF_Dictionary> pDict)
     : m_pDict(std::move(pDict)) {
-  SetData(std::move(pData), size);
+  TakeData(std::move(pData), size);
 }
 
 CPDF_Stream::~CPDF_Stream() {
   m_ObjNum = kInvalidObjNum;
   if (m_pDict && m_pDict->GetObjNum() == kInvalidObjNum)
-    m_pDict.release();  // lowercase release, release ownership.
+    m_pDict.Leak();  // lowercase release, release ownership.
 }
 
 CPDF_Object::Type CPDF_Stream::GetType() const {
-  return STREAM;
+  return kStream;
 }
 
 CPDF_Dictionary* CPDF_Stream::GetDict() {
-  return m_pDict.get();
+  return m_pDict.Get();
 }
 
 const CPDF_Dictionary* CPDF_Stream::GetDict() const {
-  return m_pDict.get();
+  return m_pDict.Get();
 }
 
 bool CPDF_Stream::IsStream() const {
@@ -70,28 +70,27 @@ const CPDF_Stream* CPDF_Stream::AsStream() const {
 }
 
 void CPDF_Stream::InitStream(pdfium::span<const uint8_t> pData,
-                             std::unique_ptr<CPDF_Dictionary> pDict) {
+                             RetainPtr<CPDF_Dictionary> pDict) {
   m_pDict = std::move(pDict);
   SetData(pData);
 }
 
 void CPDF_Stream::InitStreamFromFile(
     const RetainPtr<IFX_SeekableReadStream>& pFile,
-    std::unique_ptr<CPDF_Dictionary> pDict) {
-  m_pDict = std::move(pDict);
+    RetainPtr<CPDF_Dictionary> pDict) {
   m_bMemoryBased = false;
   m_pDataBuf.reset();
   m_pFile = pFile;
   m_dwSize = pdfium::base::checked_cast<uint32_t>(pFile->GetSize());
-  if (m_pDict)
-    m_pDict->SetNewFor<CPDF_Number>("Length", static_cast<int>(m_dwSize));
+  m_pDict = std::move(pDict);
+  m_pDict->SetNewFor<CPDF_Number>("Length", static_cast<int>(m_dwSize));
 }
 
-std::unique_ptr<CPDF_Object> CPDF_Stream::Clone() const {
+RetainPtr<CPDF_Object> CPDF_Stream::Clone() const {
   return CloneObjectNonCyclic(false);
 }
 
-std::unique_ptr<CPDF_Object> CPDF_Stream::CloneNonCyclic(
+RetainPtr<CPDF_Object> CPDF_Stream::CloneNonCyclic(
     bool bDirect,
     std::set<const CPDF_Object*>* pVisited) const {
   pVisited->insert(this);
@@ -100,13 +99,13 @@ std::unique_ptr<CPDF_Object> CPDF_Stream::CloneNonCyclic(
 
   uint32_t streamSize = pAcc->GetSize();
   const CPDF_Dictionary* pDict = GetDict();
-  std::unique_ptr<CPDF_Dictionary> pNewDict;
+  RetainPtr<CPDF_Dictionary> pNewDict;
   if (pDict && !pdfium::ContainsKey(*pVisited, pDict)) {
     pNewDict =
         ToDictionary(static_cast<const CPDF_Object*>(pDict)->CloneNonCyclic(
             bDirect, pVisited));
   }
-  return pdfium::MakeUnique<CPDF_Stream>(pAcc->DetachData(), streamSize,
+  return pdfium::MakeRetain<CPDF_Stream>(pAcc->DetachData(), streamSize,
                                          std::move(pNewDict));
 }
 
@@ -134,17 +133,17 @@ void CPDF_Stream::SetData(pdfium::span<const uint8_t> pData) {
     data_copy.reset(FX_Alloc(uint8_t, pData.size()));
     memcpy(data_copy.get(), pData.data(), pData.size());
   }
-  SetData(std::move(data_copy), pData.size());
+  TakeData(std::move(data_copy), pData.size());
 }
 
-void CPDF_Stream::SetData(std::unique_ptr<uint8_t, FxFreeDeleter> pData,
-                          uint32_t size) {
+void CPDF_Stream::TakeData(std::unique_ptr<uint8_t, FxFreeDeleter> pData,
+                           uint32_t size) {
   m_bMemoryBased = true;
   m_pFile = nullptr;
   m_pDataBuf = std::move(pData);
   m_dwSize = size;
   if (!m_pDict)
-    m_pDict = pdfium::MakeUnique<CPDF_Dictionary>();
+    m_pDict = pdfium::MakeRetain<CPDF_Dictionary>();
   m_pDict->SetNewFor<CPDF_Number>("Length", static_cast<int>(size));
 }
 
@@ -161,7 +160,7 @@ bool CPDF_Stream::ReadRawData(FX_FILESIZE offset,
                               uint8_t* buf,
                               uint32_t size) const {
   if (!m_bMemoryBased && m_pFile)
-    return m_pFile->ReadBlock(buf, offset, size);
+    return m_pFile->ReadBlockAtOffset(buf, offset, size);
 
   if (m_pDataBuf)
     memcpy(buf, m_pDataBuf.get() + offset, size);
@@ -176,7 +175,7 @@ bool CPDF_Stream::HasFilter() const {
 WideString CPDF_Stream::GetUnicodeText() const {
   auto pAcc = pdfium::MakeRetain<CPDF_StreamAcc>(this);
   pAcc->LoadAllDataFiltered();
-  return PDF_DecodeText(pAcc->GetData(), pAcc->GetSize());
+  return PDF_DecodeText(pAcc->GetSpan());
 }
 
 bool CPDF_Stream::WriteTo(IFX_ArchiveStream* archive,
@@ -192,20 +191,24 @@ bool CPDF_Stream::WriteTo(IFX_ArchiveStream* archive,
     data = encrypted_data;
   }
 
-  if (static_cast<uint32_t>(encoder.GetDict()->GetIntegerFor("Length")) !=
-      data.size()) {
+  size_t size = data.size();
+  if (static_cast<size_t>(encoder.GetDict()->GetIntegerFor("Length")) != size) {
     encoder.CloneDict();
-    encoder.GetClonedDict()->SetNewFor<CPDF_Number>(
-        "Length", static_cast<int>(data.size()));
+    encoder.GetClonedDict()->SetNewFor<CPDF_Number>("Length",
+                                                    static_cast<int>(size));
   }
 
   if (!encoder.GetDict()->WriteTo(archive, encryptor))
     return false;
 
-  if (!archive->WriteString("stream\r\n") ||
-      !archive->WriteBlock(data.data(), data.size()) ||
-      !archive->WriteString("\r\nendstream")) {
+  if (!archive->WriteString("stream\r\n"))
     return false;
-  }
+
+  if (size && !archive->WriteBlock(data.data(), size))
+    return false;
+
+  if (!archive->WriteString("\r\nendstream"))
+    return false;
+
   return true;
 }
