@@ -12,15 +12,17 @@
 
 #include "core/fpdfapi/page/cpdf_clippath.h"
 #include "core/fpdfapi/page/cpdf_graphicstates.h"
+#include "core/fpdfapi/page/cpdf_transparency.h"
+#include "core/fpdfapi/parser/cpdf_dictionary.h"
 #include "core/fpdfapi/render/cpdf_renderoptions.h"
-#include "core/fpdfapi/render/cpdf_transparency.h"
+#include "core/fxcrt/retain_ptr.h"
 #include "core/fxcrt/unowned_ptr.h"
 #include "core/fxge/fx_dib.h"
 
+class CFX_DIBitmap;
 class CFX_PathData;
 class CFX_RenderDevice;
 class CPDF_Color;
-class CPDF_Dictionary;
 class CPDF_Font;
 class CPDF_FormObject;
 class CPDF_ImageCacheEntry;
@@ -38,6 +40,7 @@ class CPDF_TransferFunc;
 class CPDF_Type3Cache;
 class CPDF_Type3Char;
 class CPDF_Type3Font;
+class PauseIndicatorIface;
 
 class CPDF_RenderStatus {
  public:
@@ -48,7 +51,9 @@ class CPDF_RenderStatus {
   void SetOptions(const CPDF_RenderOptions& options) { m_Options = options; }
   void SetDeviceMatrix(const CFX_Matrix& matrix) { m_DeviceMatrix = matrix; }
   void SetStopObject(const CPDF_PageObject* pStopObj) { m_pStopObj = pStopObj; }
-  void SetFormResource(const CPDF_Dictionary* pRes) { m_pFormResource = pRes; }
+  void SetFormResource(const CPDF_Dictionary* pRes) {
+    m_pFormResource.Reset(pRes);
+  }
   void SetType3Char(CPDF_Type3Char* pType3Char) { m_pType3Char = pType3Char; }
   void SetFillColor(FX_ARGB color) { m_T3FillColor = color; }
   void SetDropObjects(bool bDropObjects) { m_bDropObjects = bDropObjects; }
@@ -116,10 +121,14 @@ class CPDF_RenderStatus {
                          const CPDF_Transparency& transparency);
 
  private:
+  static std::unique_ptr<CPDF_GraphicStates> CloneObjStates(
+      const CPDF_GraphicStates* pSrcStates,
+      bool bStroke);
+
   FX_ARGB GetFillArgbInternal(CPDF_PageObject* pObj, bool bType3) const;
   bool ProcessTransparency(CPDF_PageObject* PageObj,
                            const CFX_Matrix& mtObj2Device);
-  void ProcessObjectNoClip(CPDF_PageObject* PageObj,
+  void ProcessObjectNoClip(CPDF_PageObject* pObj,
                            const CFX_Matrix& mtObj2Device);
   void DrawObjWithBackground(CPDF_PageObject* pObj,
                              const CFX_Matrix& mtObj2Device);
@@ -143,11 +152,6 @@ class CPDF_RenderStatus {
                     const CFX_Matrix& mtObj2Device);
   void ProcessShading(const CPDF_ShadingObject* pShadingObj,
                       const CFX_Matrix& mtObj2Device);
-  void DrawShading(const CPDF_ShadingPattern* pPattern,
-                   const CFX_Matrix& mtMatrix,
-                   const FX_RECT& clip_rect,
-                   int alpha,
-                   bool bAlphaMode);
   bool ProcessType3Text(CPDF_TextObject* textobj,
                         const CFX_Matrix& mtObj2Device);
   bool ProcessText(CPDF_TextObject* textobj,
@@ -174,16 +178,12 @@ class CPDF_RenderStatus {
   FX_ARGB GetBackColor(const CPDF_Dictionary* pSMaskDict,
                        const CPDF_Dictionary* pGroupDict,
                        int* pCSFamily);
-  static RetainPtr<CPDF_Type3Cache> GetCachedType3(CPDF_Type3Font* pFont);
-  static std::unique_ptr<CPDF_GraphicStates> CloneObjStates(
-      const CPDF_GraphicStates* pPathObj,
-      bool bStroke);
   FX_ARGB GetStrokeArgb(CPDF_PageObject* pObj) const;
   FX_RECT GetObjectClippedRect(const CPDF_PageObject* pObj,
                                const CFX_Matrix& mtObj2Device) const;
 
   CPDF_RenderOptions m_Options;
-  UnownedPtr<const CPDF_Dictionary> m_pFormResource;
+  RetainPtr<const CPDF_Dictionary> m_pFormResource;
   RetainPtr<CPDF_Dictionary> m_pPageResource;
   std::vector<CPDF_Type3Font*> m_Type3FontCache;
   UnownedPtr<CPDF_RenderContext> const m_pContext;
