@@ -8,34 +8,36 @@
 #define CORE_FPDFDOC_CPDF_AACTION_H_
 
 #include "core/fpdfdoc/cpdf_action.h"
+#include "core/fxcrt/retain_ptr.h"
 
 class CPDF_Dictionary;
 
 class CPDF_AAction {
  public:
   enum AActionType {
-    CursorEnter = 0,
-    CursorExit,
-    ButtonDown,
-    ButtonUp,
-    GetFocus,
-    LoseFocus,
-    PageOpen,
-    PageClose,
-    PageVisible,
-    PageInvisible,
-    OpenPage,
-    ClosePage,
-    KeyStroke,
-    Format,
-    Validate,
-    Calculate,
-    CloseDocument,
-    SaveDocument,
-    DocumentSaved,
-    PrintDocument,
-    DocumentPrinted,
-    NumberOfActions  // Must be last.
+    kCursorEnter = 0,
+    kCursorExit,
+    kButtonDown,
+    kButtonUp,
+    kGetFocus,
+    kLoseFocus,
+    kPageOpen,
+    kPageClose,
+    kPageVisible,
+    kPageInvisible,
+    kOpenPage,
+    kClosePage,
+    kKeyStroke,
+    kFormat,
+    kValidate,
+    kCalculate,
+    kCloseDocument,
+    kSaveDocument,
+    kDocumentSaved,
+    kPrintDocument,
+    kDocumentPrinted,
+    kDocumentOpen,
+    kNumberOfActions  // Must be last.
   };
 
   explicit CPDF_AAction(const CPDF_Dictionary* pDict);
@@ -46,8 +48,10 @@ class CPDF_AAction {
   CPDF_Action GetAction(AActionType eType) const;
   const CPDF_Dictionary* GetDict() const { return m_pDict.Get(); }
 
+  static bool IsUserClick(AActionType eType);
+
  private:
-  UnownedPtr<const CPDF_Dictionary> const m_pDict;
+  RetainPtr<const CPDF_Dictionary> const m_pDict;
 };
 
 #endif  // CORE_FPDFDOC_CPDF_AACTION_H_
