@@ -12,6 +12,8 @@
 
 #include "constants/form_fields.h"
 #include "constants/form_flags.h"
+#include "core/fpdfapi/font/cpdf_font.h"
+#include "core/fpdfapi/page/cpdf_docpagedata.h"
 #include "core/fpdfapi/parser/cfdf_document.h"
 #include "core/fpdfapi/parser/cpdf_array.h"
 #include "core/fpdfapi/parser/cpdf_dictionary.h"
@@ -20,6 +22,7 @@
 #include "core/fpdfapi/parser/cpdf_number.h"
 #include "core/fpdfapi/parser/cpdf_string.h"
 #include "core/fpdfapi/parser/fpdf_parser_decode.h"
+#include "core/fpdfapi/parser/fpdf_parser_utility.h"
 #include "core/fpdfdoc/cpdf_defaultappearance.h"
 #include "core/fpdfdoc/cpdf_formcontrol.h"
 #include "core/fpdfdoc/cpdf_interactiveform.h"
@@ -29,25 +32,6 @@
 
 namespace {
 
-// TODO(thestig): Move into constants/form_flags.h.
-const int kFormListMultiSelect = 0x100;
-
-const int kFormComboEdit = 0x100;
-
-const int kFormRadioNoToggleOff = 0x100;
-const int kFormRadioUnison = 0x200;
-
-const int kFormTextMultiLine = 0x100;
-const int kFormTextPassword = 0x200;
-const int kFormTextNoScroll = 0x400;
-const int kFormTextComb = 0x800;
-
-bool IsUnison(CPDF_FormField* pField) {
-  if (pField->GetType() == CPDF_FormField::kCheckBox)
-    return true;
-  return (pField->GetFieldFlags() & 0x2000000) != 0;
-}
-
 const CPDF_Object* FPDF_GetFieldAttrRecursive(const CPDF_Dictionary* pFieldDict,
                                               const char* name,
                                               int nLevel) {
@@ -109,66 +93,40 @@ CPDF_FormField::CPDF_FormField(CPDF_InteractiveForm* pForm,
   InitFieldFlags();
 }
 
-CPDF_FormField::~CPDF_FormField() {}
+CPDF_FormField::~CPDF_FormField() = default;
 
 void CPDF_FormField::InitFieldFlags() {
   const CPDF_Object* ft_attr =
       FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kFT);
   ByteString type_name = ft_attr ? ft_attr->GetString() : ByteString();
-  const CPDF_Object* ff_attr =
-      FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kFf);
-  uint32_t flags = ff_attr ? ff_attr->GetInteger() : 0;
-  if (flags & pdfium::form_flags::kReadOnly) {
-    m_Flags |= pdfium::form_flags::kReadOnly;
-    m_bReadOnly = true;
-  }
-  if (flags & pdfium::form_flags::kRequired) {
-    m_Flags |= pdfium::form_flags::kRequired;
-    m_bRequired = true;
-  }
-  if (flags & pdfium::form_flags::kNoExport) {
-    m_Flags |= pdfium::form_flags::kNoExport;
-    m_bNoExport = true;
-  }
+  uint32_t flags = GetFieldFlags();
+  m_bRequired = flags & pdfium::form_flags::kRequired;
+  m_bNoExport = flags & pdfium::form_flags::kNoExport;
 
   if (type_name == pdfium::form_fields::kBtn) {
-    if (flags & 0x8000) {
+    if (flags & pdfium::form_flags::kButtonRadio) {
       m_Type = kRadioButton;
-      if (flags & 0x4000)
-        m_Flags |= kFormRadioNoToggleOff;
-      if (flags & 0x2000000)
-        m_Flags |= kFormRadioUnison;
-    } else if (flags & 0x10000) {
+      m_bIsUnison = flags & pdfium::form_flags::kButtonRadiosInUnison;
+    } else if (flags & pdfium::form_flags::kButtonPushbutton) {
       m_Type = kPushButton;
     } else {
       m_Type = kCheckBox;
+      m_bIsUnison = true;
     }
   } else if (type_name == pdfium::form_fields::kTx) {
-    if (flags & 0x100000) {
+    if (flags & pdfium::form_flags::kTextFileSelect)
       m_Type = kFile;
-    } else if (flags & 0x2000000) {
+    else if (flags & pdfium::form_flags::kTextRichText)
       m_Type = kRichText;
-    } else {
+    else
       m_Type = kText;
-      if (flags & 0x1000)
-        m_Flags |= kFormTextMultiLine;
-      if (flags & 0x2000)
-        m_Flags |= kFormTextPassword;
-      if (flags & 0x800000)
-        m_Flags |= kFormTextNoScroll;
-      if (flags & 0x100000)
-        m_Flags |= kFormTextComb;
-    }
     LoadDA();
   } else if (type_name == pdfium::form_fields::kCh) {
-    if (flags & 0x20000) {
+    if (flags & pdfium::form_flags::kChoiceCombo) {
       m_Type = kComboBox;
-      if (flags & 0x40000)
-        m_Flags |= kFormComboEdit;
     } else {
       m_Type = kListBox;
-      if (flags & 0x200000)
-        m_Flags |= kFormListMultiSelect;
+      m_bIsMultiSelectListBox = flags & pdfium::form_flags::kChoiceMultiSelect;
     }
     LoadDA();
   } else if (type_name == pdfium::form_fields::kSig) {
@@ -186,7 +144,7 @@ bool CPDF_FormField::ResetField(NotificationOption notify) {
     case kRadioButton: {
       int iCount = CountControls();
       // TODO(weili): Check whether anything special needs to be done for
-      // unison field. (When IsUnison(this) returns true/false.)
+      // |m_bIsUnison|.
       for (int i = 0; i < iCount; i++) {
         CheckControl(i, GetControl(i)->IsDefaultChecked(),
                      NotificationOption::kDoNotNotify);
@@ -215,20 +173,21 @@ bool CPDF_FormField::ResetField(NotificationOption notify) {
     case kRichText:
     case kFile:
     default: {
-      const CPDF_Object* pDV =
-          FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kDV);
+      const CPDF_Object* pDV = GetDefaultValueObject();
       WideString csDValue;
       if (pDV)
         csDValue = pDV->GetUnicodeText();
 
-      const CPDF_Object* pV =
-          FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kV);
       WideString csValue;
-      if (pV)
-        csValue = pV->GetUnicodeText();
+      {
+        // Limit the scope of |pV| because it may get invalidated below.
+        const CPDF_Object* pV = GetValueObject();
+        if (pV)
+          csValue = pV->GetUnicodeText();
+      }
 
-      const CPDF_Object* pRV = FPDF_GetFieldAttr(m_pDict.Get(), "RV");
-      if (!pRV && (csDValue == csValue))
+      bool bHasRV = !!FPDF_GetFieldAttr(m_pDict.Get(), "RV");
+      if (!bHasRV && (csDValue == csValue))
         return false;
 
       if (notify == NotificationOption::kNotify &&
@@ -241,7 +200,7 @@ bool CPDF_FormField::ResetField(NotificationOption notify) {
           return false;
 
         m_pDict->SetFor(pdfium::form_fields::kV, std::move(pClone));
-        if (pRV) {
+        if (bHasRV) {
           m_pDict->SetFor("RV", pDV->Clone());
         }
       } else {
@@ -297,7 +256,7 @@ FormFieldType CPDF_FormField::GetFieldType() const {
 }
 
 CPDF_AAction CPDF_FormField::GetAdditionalAction() const {
-  const CPDF_Object* pObj =
+  CPDF_Object* pObj =
       FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kAA);
   return CPDF_AAction(pObj ? pObj->GetDict() : nullptr);
 }
@@ -334,15 +293,10 @@ WideString CPDF_FormField::GetValue(bool bDefault) const {
     return GetCheckValue(bDefault);
 
   const CPDF_Object* pValue =
-      FPDF_GetFieldAttr(m_pDict.Get(), bDefault ? pdfium::form_fields::kDV
-                                                : pdfium::form_fields::kV);
+      bDefault ? GetDefaultValueObject() : GetValueObject();
   if (!pValue) {
-    if (!bDefault) {
-      if (m_Type == kRichText)
-        pValue = FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kV);
-      if (!pValue && m_Type != kText)
-        pValue = FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kDV);
-    }
+    if (!bDefault && m_Type != kText)
+      pValue = GetDefaultValueObject();
     if (!pValue)
       return WideString();
   }
@@ -454,13 +408,9 @@ int CPDF_FormField::GetMaxLen() const {
 }
 
 int CPDF_FormField::CountSelectedItems() const {
-  const CPDF_Object* pValue =
-      FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kV);
-  if (!pValue) {
-    pValue = FPDF_GetFieldAttr(m_pDict.Get(), "I");
-    if (!pValue)
-      return 0;
-  }
+  const CPDF_Object* pValue = GetValueOrSelectedIndicesObject();
+  if (!pValue)
+    return 0;
 
   if (pValue->IsString() || pValue->IsNumber())
     return pValue->GetString().IsEmpty() ? 0 : 1;
@@ -469,13 +419,10 @@ int CPDF_FormField::CountSelectedItems() const {
 }
 
 int CPDF_FormField::GetSelectedIndex(int index) const {
-  const CPDF_Object* pValue =
-      FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kV);
-  if (!pValue) {
-    pValue = FPDF_GetFieldAttr(m_pDict.Get(), "I");
-    if (!pValue)
-      return -1;
-  }
+  const CPDF_Object* pValue = GetValueOrSelectedIndicesObject();
+  if (!pValue)
+    return -1;
+
   if (pValue->IsNumber())
     return pValue->GetInteger();
 
@@ -529,13 +476,9 @@ bool CPDF_FormField::IsItemSelected(int index) const {
     return true;
 
   WideString opt_value = GetOptionValue(index);
-  const CPDF_Object* pValue =
-      FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kV);
-  if (!pValue) {
-    pValue = FPDF_GetFieldAttr(m_pDict.Get(), "I");
-    if (!pValue)
-      return false;
-  }
+  const CPDF_Object* pValue = GetValueOrSelectedIndicesObject();
+  if (!pValue)
+    return false;
 
   if (pValue->IsString())
     return pValue->GetUnicodeText() == opt_value;
@@ -592,7 +535,7 @@ void CPDF_FormField::SetItemSelectionSelected(int index,
   }
 
   SelectOption(index, true, NotificationOption::kDoNotNotify);
-  if (!(m_Flags & kFormListMultiSelect)) {
+  if (!m_bIsMultiSelectListBox) {
     m_pDict->SetNewFor<CPDF_String>(pdfium::form_fields::kV, opt_value);
     return;
   }
@@ -606,8 +549,7 @@ void CPDF_FormField::SetItemSelectionSelected(int index,
 
 void CPDF_FormField::SetItemSelectionUnselected(int index,
                                                 const WideString& opt_value) {
-  const CPDF_Object* pValue =
-      FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kV);
+  const CPDF_Object* pValue = GetValueObject();
   if (!pValue)
     return;
 
@@ -634,7 +576,7 @@ void CPDF_FormField::SetItemSelectionUnselected(int index,
       pArray->AddNew<CPDF_String>(GetOptionValue(i));
   }
   if (pArray->size() > 0) {
-    m_pDict->SetFor(pdfium::form_fields::kV, std::move(pArray));
+    m_pDict->SetFor(pdfium::form_fields::kV, pArray);
   }
 }
 
@@ -648,8 +590,7 @@ bool CPDF_FormField::IsItemDefaultSelected(int index) const {
 
 int CPDF_FormField::GetDefaultSelectedItem() const {
   ASSERT(GetType() == kComboBox || GetType() == kListBox);
-  const CPDF_Object* pValue =
-      FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kDV);
+  const CPDF_Object* pValue = GetDefaultValueObject();
   if (!pValue)
     return -1;
   WideString csDV = pValue->GetUnicodeText();
@@ -710,10 +651,9 @@ bool CPDF_FormField::CheckControl(int iControlIndex,
 
   const WideString csWExport = pControl->GetExportValue();
   int iCount = CountControls();
-  bool bUnison = IsUnison(this);
   for (int i = 0; i < iCount; i++) {
     CPDF_FormControl* pCtrl = GetControl(i);
-    if (bUnison) {
+    if (m_bIsUnison) {
       WideString csEValue = pCtrl->GetExportValue();
       if (csEValue == csWExport) {
         if (pCtrl->GetOnStateName() == pControl->GetOnStateName())
@@ -738,8 +678,7 @@ bool CPDF_FormField::CheckControl(int iControlIndex,
       m_pDict->SetNewFor<CPDF_Name>(pdfium::form_fields::kV, csBExport);
     } else {
       ByteString csV;
-      const CPDF_Object* pV =
-          FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kV);
+      const CPDF_Object* pV = GetValueObject();
       if (pV)
         csV = pV->GetString();
       if (csV == csBExport)
@@ -797,12 +736,12 @@ int CPDF_FormField::GetTopVisibleIndex() const {
 }
 
 int CPDF_FormField::CountSelectedOptions() const {
-  const CPDF_Array* pArray = ToArray(FPDF_GetFieldAttr(m_pDict.Get(), "I"));
+  const CPDF_Array* pArray = ToArray(GetSelectedIndicesObject());
   return pArray ? pArray->size() : 0;
 }
 
 int CPDF_FormField::GetSelectedOptionIndex(int index) const {
-  const CPDF_Array* pArray = ToArray(FPDF_GetFieldAttr(m_pDict.Get(), "I"));
+  const CPDF_Array* pArray = ToArray(GetSelectedIndicesObject());
   if (!pArray)
     return -1;
 
@@ -813,7 +752,7 @@ int CPDF_FormField::GetSelectedOptionIndex(int index) const {
 }
 
 bool CPDF_FormField::IsOptionSelected(int iOptIndex) const {
-  const CPDF_Array* pArray = ToArray(FPDF_GetFieldAttr(m_pDict.Get(), "I"));
+  const CPDF_Array* pArray = ToArray(GetSelectedIndicesObject());
   if (!pArray)
     return false;
 
@@ -899,7 +838,7 @@ void CPDF_FormField::LoadDA() {
     return;
 
   CPDF_Dictionary* pFont = pDR->GetDictFor("Font");
-  if (!pFont)
+  if (!ValidateFontResourceDict(pFont))
     return;
 
   CPDF_DefaultAppearance appearance(DA);
@@ -911,7 +850,8 @@ void CPDF_FormField::LoadDA() {
   if (!pFontDict)
     return;
 
-  m_pFont = m_pForm->GetDocument()->LoadFont(pFontDict);
+  auto* pData = CPDF_DocPageData::FromDocument(m_pForm->GetDocument());
+  m_pFont = pData->GetFont(pFontDict);
 }
 
 bool CPDF_FormField::NotifyBeforeSelectionChange(const WideString& value) {
@@ -960,6 +900,25 @@ void CPDF_FormField::NotifyListOrComboBoxAfterChange() {
   }
 }
 
+const CPDF_Object* CPDF_FormField::GetDefaultValueObject() const {
+  return FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kDV);
+}
+
+const CPDF_Object* CPDF_FormField::GetValueObject() const {
+  return FPDF_GetFieldAttr(m_pDict.Get(), pdfium::form_fields::kV);
+}
+
+const CPDF_Object* CPDF_FormField::GetSelectedIndicesObject() const {
+  ASSERT(GetType() == kComboBox || GetType() == kListBox);
+  return FPDF_GetFieldAttr(m_pDict.Get(), "I");
+}
+
+const CPDF_Object* CPDF_FormField::GetValueOrSelectedIndicesObject() const {
+  ASSERT(GetType() == kComboBox || GetType() == kListBox);
+  const CPDF_Object* pValue = GetValueObject();
+  return pValue ? pValue : GetSelectedIndicesObject();
+}
+
 const std::vector<UnownedPtr<CPDF_FormControl>>& CPDF_FormField::GetControls()
     const {
   return m_pForm->GetControlsForField(this);
