@@ -16,7 +16,6 @@
 #include "core/fxcrt/fx_memory.h"
 #include "core/fxcrt/fx_safe_types.h"
 #include "third_party/base/ptr_util.h"
-#include "third_party/base/stl_util.h"
 
 #define JBIG2_GETDWORD(buf)                  \
   ((static_cast<uint32_t>((buf)[0]) << 24) | \
@@ -44,7 +43,7 @@ CJBig2_Image::CJBig2_Image(int32_t w, int32_t h) {
   if (w <= 0 || h <= 0 || w > kMaxImagePixels)
     return;
 
-  int32_t stride_pixels = (w + 31) & ~31;
+  int32_t stride_pixels = FxAlignToBoundary<32>(w);
   if (h > kMaxImagePixels / stride_pixels)
     return;
 
