@@ -16,7 +16,6 @@
 class CFX_BinaryBuf {
  public:
   CFX_BinaryBuf();
-  explicit CFX_BinaryBuf(size_t size);
   virtual ~CFX_BinaryBuf();
 
   uint8_t* GetBuffer() const { return m_pBuffer.get(); }
@@ -25,7 +24,8 @@ class CFX_BinaryBuf {
   bool IsEmpty() const { return GetLength() == 0; }
 
   void Clear();
-  void EstimateSize(size_t size, size_t alloc_step = 0);
+  void SetAllocStep(size_t step) { m_AllocStep = step; }
+  void EstimateSize(size_t size);
   void AppendBlock(const void* pBuf, size_t size);
   void AppendString(const ByteString& str) {
     AppendBlock(str.c_str(), str.GetLength());
@@ -36,7 +36,6 @@ class CFX_BinaryBuf {
     m_pBuffer.get()[m_DataSize++] = byte;
   }
 
-  void InsertBlock(size_t pos, const void* pBuf, size_t size);
   void Delete(size_t start_index, size_t count);
 
   // Releases ownership of |m_pBuffer| and returns it.
@@ -45,9 +44,9 @@ class CFX_BinaryBuf {
  protected:
   void ExpandBuf(size_t size);
 
-  size_t m_AllocStep;
-  size_t m_AllocSize;
-  size_t m_DataSize;
+  size_t m_AllocStep = 0;
+  size_t m_AllocSize = 0;
+  size_t m_DataSize = 0;
   std::unique_ptr<uint8_t, FxFreeDeleter> m_pBuffer;
 };
 
