@@ -8,6 +8,15 @@
 
 #include "testing/gtest/include/gtest/gtest.h"
 
+TEST(CFX_FloatRect, FromFXRect) {
+  FX_RECT downwards(10, 20, 30, 40);
+  CFX_FloatRect rect(downwards);
+  EXPECT_FLOAT_EQ(rect.left, 10.0f);
+  EXPECT_FLOAT_EQ(rect.bottom, 20.0f);
+  EXPECT_FLOAT_EQ(rect.right, 30.0f);
+  EXPECT_FLOAT_EQ(rect.top, 40.0f);
+}
+
 TEST(CFX_FloatRect, GetBBox) {
   CFX_FloatRect rect = CFX_FloatRect::GetBBox(nullptr, 0);
   EXPECT_FLOAT_EQ(0.0f, rect.left);
@@ -229,9 +238,7 @@ TEST(CFX_RectF, Print) {
 #endif  // NDEBUG
 
 TEST(CFX_Matrix, ReverseIdentity) {
-  CFX_Matrix m;
-  m.SetIdentity();
-  CFX_Matrix rev = m.GetInverse();
+  CFX_Matrix rev = CFX_Matrix().GetInverse();
 
   EXPECT_FLOAT_EQ(1.0, rev.a);
   EXPECT_FLOAT_EQ(0.0, rev.b);
@@ -241,34 +248,45 @@ TEST(CFX_Matrix, ReverseIdentity) {
   EXPECT_FLOAT_EQ(0.0, rev.f);
 
   CFX_PointF expected(2, 3);
-  CFX_PointF result = rev.Transform(m.Transform(CFX_PointF(2, 3)));
+  CFX_PointF result = rev.Transform(CFX_Matrix().Transform(CFX_PointF(2, 3)));
   EXPECT_FLOAT_EQ(expected.x, result.x);
   EXPECT_FLOAT_EQ(expected.y, result.y);
 }
 
 TEST(CFX_Matrix, SetIdentity) {
   CFX_Matrix m;
-  EXPECT_FLOAT_EQ(1.0, m.a);
-  EXPECT_FLOAT_EQ(0.0, m.b);
-  EXPECT_FLOAT_EQ(0.0, m.c);
-  EXPECT_FLOAT_EQ(1.0, m.d);
-  EXPECT_FLOAT_EQ(0.0, m.e);
-  EXPECT_FLOAT_EQ(0.0, m.f);
+  EXPECT_FLOAT_EQ(1.0f, m.a);
+  EXPECT_FLOAT_EQ(0.0f, m.b);
+  EXPECT_FLOAT_EQ(0.0f, m.c);
+  EXPECT_FLOAT_EQ(1.0f, m.d);
+  EXPECT_FLOAT_EQ(0.0f, m.e);
+  EXPECT_FLOAT_EQ(0.0f, m.f);
   EXPECT_TRUE(m.IsIdentity());
 
   m.a = -1;
   EXPECT_FALSE(m.IsIdentity());
 
-  m.SetIdentity();
-  EXPECT_FLOAT_EQ(1.0, m.a);
-  EXPECT_FLOAT_EQ(0.0, m.b);
-  EXPECT_FLOAT_EQ(0.0, m.c);
-  EXPECT_FLOAT_EQ(1.0, m.d);
-  EXPECT_FLOAT_EQ(0.0, m.e);
-  EXPECT_FLOAT_EQ(0.0, m.f);
+  m = CFX_Matrix();
+  EXPECT_FLOAT_EQ(1.0f, m.a);
+  EXPECT_FLOAT_EQ(0.0f, m.b);
+  EXPECT_FLOAT_EQ(0.0f, m.c);
+  EXPECT_FLOAT_EQ(1.0f, m.d);
+  EXPECT_FLOAT_EQ(0.0f, m.e);
+  EXPECT_FLOAT_EQ(0.0f, m.f);
   EXPECT_TRUE(m.IsIdentity());
 }
 
+TEST(CFX_Matrix, AsTuple) {
+  CFX_Matrix m(1, 2, 3, 4, 5, 6);
+  auto tuple = m.AsTuple();
+  EXPECT_FLOAT_EQ(1.0f, std::get<0>(tuple));
+  EXPECT_FLOAT_EQ(2.0f, std::get<1>(tuple));
+  EXPECT_FLOAT_EQ(3.0f, std::get<2>(tuple));
+  EXPECT_FLOAT_EQ(4.0f, std::get<3>(tuple));
+  EXPECT_FLOAT_EQ(5.0f, std::get<4>(tuple));
+  EXPECT_FLOAT_EQ(6.0f, std::get<5>(tuple));
+}
+
 TEST(CFX_Matrix, GetInverse) {
   static constexpr float data[6] = {3, 0, 2, 3, 1, 4};
   CFX_Matrix m(data);
@@ -436,10 +454,10 @@ TEST(CFX_Matrix, ComposeTransformations) {
   EXPECT_FLOAT_EQ(273.0f, p_10_20_transformed.y);
 
   // Now compose all transforms prepending.
-  m.SetIdentity();
-  m.ConcatPrepend(rotate_90);
-  m.ConcatPrepend(translate_23_11);
-  m.ConcatPrepend(scale_5_13);
+  m = CFX_Matrix();
+  m = rotate_90 * m;
+  m = translate_23_11 * m;
+  m = scale_5_13 * m;
   EXPECT_NEAR_FIVE_PLACES(0.0f, m.a);
   EXPECT_NEAR_FIVE_PLACES(5.0f, m.b);
   EXPECT_NEAR_FIVE_PLACES(-13.0f, m.c);
@@ -458,3 +476,45 @@ TEST(CFX_Matrix, ComposeTransformations) {
   EXPECT_FLOAT_EQ(-271.0f, p_10_20_transformed.x);
   EXPECT_FLOAT_EQ(73.0f, p_10_20_transformed.y);
 }
+
+TEST(CFX_Matrix, TransformRectForRectF) {
+  CFX_Matrix rotate_90;
+  rotate_90.Rotate(FX_PI / 2);
+
+  CFX_Matrix scale_5_13;
+  scale_5_13.Scale(5, 13);
+
+  CFX_RectF rect(10.5f, 20.5f, 4.25f, 3.25f);
+  rect = rotate_90.TransformRect(rect);
+  EXPECT_FLOAT_EQ(-23.75f, rect.Left());
+  EXPECT_FLOAT_EQ(10.5f, rect.Top());
+  EXPECT_FLOAT_EQ(3.25f, rect.Width());
+  EXPECT_FLOAT_EQ(4.25f, rect.Height());
+
+  rect = scale_5_13.TransformRect(rect);
+  EXPECT_FLOAT_EQ(-118.75f, rect.Left());
+  EXPECT_FLOAT_EQ(136.5f, rect.Top());
+  EXPECT_FLOAT_EQ(16.25f, rect.Width());
+  EXPECT_FLOAT_EQ(55.25f, rect.Height());
+}
+
+TEST(CFX_Matrix, TransformRectForFloatRect) {
+  CFX_Matrix rotate_90;
+  rotate_90.Rotate(FX_PI / 2);
+
+  CFX_Matrix scale_5_13;
+  scale_5_13.Scale(5, 13);
+
+  CFX_FloatRect rect(5.5f, 0.0f, 12.25f, 2.7f);
+  rect = rotate_90.TransformRect(rect);
+  EXPECT_FLOAT_EQ(-2.7f, rect.Left());
+  EXPECT_FLOAT_EQ(5.5f, rect.Bottom());
+  EXPECT_NEAR(0.0f, rect.Right(), 0.00001f);
+  EXPECT_FLOAT_EQ(12.25f, rect.Top());
+
+  rect = scale_5_13.TransformRect(rect);
+  EXPECT_FLOAT_EQ(-13.5f, rect.Left());
+  EXPECT_FLOAT_EQ(71.5f, rect.Bottom());
+  EXPECT_NEAR(0.0f, rect.Right(), 0.00001f);
+  EXPECT_FLOAT_EQ(159.25f, rect.Top());
+}
