@@ -11,9 +11,21 @@
 #include <limits>
 
 #include "third_party/base/compiler_specific.h"
-#include "third_party/base/ptr_util.h"
 
-time_t (*time_func)() = []() -> time_t { return time(nullptr); };
+namespace {
+
+time_t DefaultTimeFunction() {
+  return time(nullptr);
+}
+
+struct tm* DefaultLocaltimeFunction(const time_t* tp) {
+  return localtime(tp);
+}
+
+time_t (*g_time_func)() = DefaultTimeFunction;
+struct tm* (*g_localtime_func)(const time_t*) = DefaultLocaltimeFunction;
+
+}  // namespace
 
 float FXSYS_wcstof(const wchar_t* pwsStr, int32_t iLength, int32_t* pUsedLen) {
   ASSERT(pwsStr);
@@ -37,7 +49,7 @@ float FXSYS_wcstof(const wchar_t* pwsStr, int32_t iLength, int32_t* pUsedLen) {
   float fValue = 0.0f;
   while (iUsedLen < iLength) {
     wchar_t wch = pwsStr[iUsedLen];
-    if (!std::iswdigit(wch))
+    if (!FXSYS_IsDecimalDigit(wch))
       break;
 
     fValue = fValue * 10.0f + (wch - L'0');
@@ -48,7 +60,7 @@ float FXSYS_wcstof(const wchar_t* pwsStr, int32_t iLength, int32_t* pUsedLen) {
     float fPrecise = 0.1f;
     while (++iUsedLen < iLength) {
       wchar_t wch = pwsStr[iUsedLen];
-      if (!std::iswdigit(wch))
+      if (!FXSYS_IsDecimalDigit(wch))
         break;
 
       fValue += (wch - L'0') * fPrecise;
@@ -70,7 +82,7 @@ float FXSYS_wcstof(const wchar_t* pwsStr, int32_t iLength, int32_t* pUsedLen) {
     int32_t exp_value = 0;
     while (iUsedLen < iLength) {
       wchar_t wch = pwsStr[iUsedLen];
-      if (!std::iswdigit(wch))
+      if (!FXSYS_IsDecimalDigit(wch))
         break;
 
       exp_value = exp_value * 10.0f + (wch - L'0');
@@ -79,7 +91,8 @@ float FXSYS_wcstof(const wchar_t* pwsStr, int32_t iLength, int32_t* pUsedLen) {
            -exp_value < std::numeric_limits<float>::min_exponent10) ||
           (!negative_exponent &&
            exp_value > std::numeric_limits<float>::max_exponent10)) {
-        *pUsedLen = 0;
+        if (pUsedLen)
+          *pUsedLen = 0;
         return 0.0f;
       }
 
@@ -103,7 +116,10 @@ float FXSYS_wcstof(const wchar_t* pwsStr, int32_t iLength, int32_t* pUsedLen) {
 }
 
 wchar_t* FXSYS_wcsncpy(wchar_t* dstStr, const wchar_t* srcStr, size_t count) {
-  ASSERT(dstStr && srcStr && count > 0);
+  ASSERT(dstStr);
+  ASSERT(srcStr);
+  ASSERT(count > 0);
+
   for (size_t i = 0; i < count; ++i)
     if ((dstStr[i] = srcStr[i]) == L'\0')
       break;
@@ -111,7 +127,10 @@ wchar_t* FXSYS_wcsncpy(wchar_t* dstStr, const wchar_t* srcStr, size_t count) {
 }
 
 int32_t FXSYS_wcsnicmp(const wchar_t* s1, const wchar_t* s2, size_t count) {
-  ASSERT(s1 && s2 && count > 0);
+  ASSERT(s1);
+  ASSERT(s2);
+  ASSERT(count > 0);
+
   wchar_t wch1 = 0, wch2 = 0;
   while (count-- > 0) {
     wch1 = static_cast<wchar_t>(FXSYS_towlower(*s1++));
@@ -122,30 +141,6 @@ int32_t FXSYS_wcsnicmp(const wchar_t* s1, const wchar_t* s2, size_t count) {
   return wch1 - wch2;
 }
 
-uint32_t FX_HashCode_GetA(const ByteStringView& str, bool bIgnoreCase) {
-  uint32_t dwHashCode = 0;
-  if (bIgnoreCase) {
-    for (const auto& c : str)
-      dwHashCode = 31 * dwHashCode + tolower(c);
-  } else {
-    for (const auto& c : str)
-      dwHashCode = 31 * dwHashCode + c;
-  }
-  return dwHashCode;
-}
-
-uint32_t FX_HashCode_GetW(const WideStringView& str, bool bIgnoreCase) {
-  uint32_t dwHashCode = 0;
-  if (bIgnoreCase) {
-    for (const auto& c : str)
-      dwHashCode = 1313 * dwHashCode + FXSYS_towlower(c);
-  } else {
-    for (const auto& c : str)
-      dwHashCode = 1313 * dwHashCode + c;
-  }
-  return dwHashCode;
-}
-
 void FXSYS_IntToTwoHexChars(uint8_t n, char* buf) {
   static const char kHex[] = "0123456789ABCDEF";
   buf[0] = kHex[n / 16];
@@ -172,12 +167,20 @@ size_t FXSYS_ToUTF16BE(uint32_t unicode, char* buf) {
 }
 
 void FXSYS_SetTimeFunction(time_t (*func)()) {
-  time_func = func ? func : []() -> time_t { return time(nullptr); };
+  g_time_func = func ? func : DefaultTimeFunction;
+}
+
+void FXSYS_SetLocaltimeFunction(struct tm* (*func)(const time_t*)) {
+  g_localtime_func = func ? func : DefaultLocaltimeFunction;
 }
 
 time_t FXSYS_time(time_t* tloc) {
-  time_t ret_val = time_func();
+  time_t ret_val = g_time_func();
   if (tloc)
     *tloc = ret_val;
   return ret_val;
 }
+
+struct tm* FXSYS_localtime(const time_t* tp) {
+  return g_localtime_func(tp);
+}
