@@ -14,6 +14,10 @@
 
 namespace {
 
+#if !defined(OS_WIN)
+uint32_t g_last_error = 0;
+#endif
+
 template <typename IntType, typename CharType>
 IntType FXSYS_StrToInt(const CharType* str) {
   if (!str)
@@ -84,12 +88,22 @@ STR_T FXSYS_IntToStr(T value, STR_T str, int radix) {
 
 }  // namespace
 
-int FXSYS_round(float d) {
+int FXSYS_roundf(float f) {
+  if (std::isnan(f))
+    return 0;
+  if (f < static_cast<float>(std::numeric_limits<int>::min()))
+    return std::numeric_limits<int>::min();
+  if (f >= static_cast<float>(std::numeric_limits<int>::max()))
+    return std::numeric_limits<int>::max();
+  return static_cast<int>(round(f));
+}
+
+int FXSYS_round(double d) {
   if (std::isnan(d))
     return 0;
-  if (d < static_cast<float>(std::numeric_limits<int>::min()))
+  if (d < static_cast<double>(std::numeric_limits<int>::min()))
     return std::numeric_limits<int>::min();
-  if (d > static_cast<float>(std::numeric_limits<int>::max()))
+  if (d >= static_cast<double>(std::numeric_limits<int>::max()))
     return std::numeric_limits<int>::max();
   return static_cast<int>(round(d));
 }
@@ -181,26 +195,26 @@ wchar_t* FXSYS_wcsupr(wchar_t* str) {
   return s;
 }
 
-int FXSYS_stricmp(const char* dst, const char* src) {
+int FXSYS_stricmp(const char* str1, const char* str2) {
   int f;
   int l;
   do {
-    f = toupper(*dst);
-    l = toupper(*src);
-    ++dst;
-    ++src;
+    f = toupper(*str1);
+    l = toupper(*str2);
+    ++str1;
+    ++str2;
   } while (f && f == l);
   return f - l;
 }
 
-int FXSYS_wcsicmp(const wchar_t* dst, const wchar_t* src) {
+int FXSYS_wcsicmp(const wchar_t* str1, const wchar_t* str2) {
   wchar_t f;
   wchar_t l;
   do {
-    f = FXSYS_towupper(*dst);
-    l = FXSYS_towupper(*src);
-    ++dst;
-    ++src;
+    f = FXSYS_towupper(*str1);
+    l = FXSYS_towupper(*str2);
+    ++str1;
+    ++str2;
   } while (f && f == l);
   return f - l;
 }
@@ -243,4 +257,11 @@ int FXSYS_MultiByteToWideChar(uint32_t codepage,
   return wlen;
 }
 
+void FXSYS_SetLastError(uint32_t err) {
+  g_last_error = err;
+}
+
+uint32_t FXSYS_GetLastError() {
+  return g_last_error;
+}
 #endif  // defined(OS_WIN)
