@@ -9,13 +9,14 @@
 
 #include <functional>
 #include <iterator>
+#include <ostream>
 #include <utility>
 
-#include "core/fxcrt/fx_memory.h"
 #include "core/fxcrt/fx_system.h"
 #include "core/fxcrt/retain_ptr.h"
 #include "core/fxcrt/string_data_template.h"
 #include "core/fxcrt/string_view_template.h"
+#include "third_party/base/logging.h"
 #include "third_party/base/optional.h"
 #include "third_party/base/span.h"
 
@@ -34,7 +35,7 @@ class WideString {
   using const_iterator = const CharType*;
   using const_reverse_iterator = std::reverse_iterator<const_iterator>;
 
-  static WideString Format(const wchar_t* lpszFormat, ...) WARN_UNUSED_RESULT;
+  static WideString Format(const wchar_t* pFormat, ...) WARN_UNUSED_RESULT;
   static WideString FormatV(const wchar_t* lpszFormat,
                             va_list argList) WARN_UNUSED_RESULT;
 
@@ -52,16 +53,18 @@ class WideString {
   // NOLINTNEXTLINE(runtime/explicit)
   WideString(char) = delete;
 
-  WideString(const wchar_t* ptr, size_t len);
+  WideString(const wchar_t* pStr, size_t len);
 
-  explicit WideString(const WideStringView& str);
-  WideString(const WideStringView& str1, const WideStringView& str2);
+  explicit WideString(WideStringView str);
+  WideString(WideStringView str1, WideStringView str2);
   WideString(const std::initializer_list<WideStringView>& list);
 
   ~WideString();
 
-  static WideString FromLocal(const ByteStringView& str) WARN_UNUSED_RESULT;
-  static WideString FromUTF8(const ByteStringView& str) WARN_UNUSED_RESULT;
+  static WideString FromASCII(ByteStringView str) WARN_UNUSED_RESULT;
+  static WideString FromLatin1(ByteStringView str) WARN_UNUSED_RESULT;
+  static WideString FromDefANSI(ByteStringView str) WARN_UNUSED_RESULT;
+  static WideString FromUTF8(ByteStringView str) WARN_UNUSED_RESULT;
   static WideString FromUTF16LE(const unsigned short* str,
                                 size_t len) WARN_UNUSED_RESULT;
 
@@ -79,7 +82,7 @@ class WideString {
 
   // Explicit conversion to span.
   // Note: Any subsequent modification of |this| will invalidate the result.
-  pdfium::span<const wchar_t> AsSpan() const {
+  pdfium::span<const wchar_t> span() const {
     return pdfium::make_span(m_pData ? m_pData->m_String : nullptr,
                              GetLength());
   }
@@ -108,31 +111,31 @@ class WideString {
   bool IsValidIndex(size_t index) const { return index < GetLength(); }
   bool IsValidLength(size_t length) const { return length <= GetLength(); }
 
-  const WideString& operator=(const wchar_t* str);
-  const WideString& operator=(const WideStringView& stringSrc);
-  const WideString& operator=(const WideString& that);
-  const WideString& operator=(WideString&& that);
+  WideString& operator=(const wchar_t* str);
+  WideString& operator=(WideStringView str);
+  WideString& operator=(const WideString& that);
+  WideString& operator=(WideString&& that);
 
-  const WideString& operator+=(const wchar_t* str);
-  const WideString& operator+=(wchar_t ch);
-  const WideString& operator+=(const WideString& str);
-  const WideString& operator+=(const WideStringView& str);
+  WideString& operator+=(const wchar_t* str);
+  WideString& operator+=(wchar_t ch);
+  WideString& operator+=(const WideString& str);
+  WideString& operator+=(WideStringView str);
 
   bool operator==(const wchar_t* ptr) const;
-  bool operator==(const WideStringView& str) const;
+  bool operator==(WideStringView str) const;
   bool operator==(const WideString& other) const;
 
   bool operator!=(const wchar_t* ptr) const { return !(*this == ptr); }
-  bool operator!=(const WideStringView& str) const { return !(*this == str); }
+  bool operator!=(WideStringView str) const { return !(*this == str); }
   bool operator!=(const WideString& other) const { return !(*this == other); }
 
   bool operator<(const wchar_t* ptr) const;
-  bool operator<(const WideStringView& str) const;
+  bool operator<(WideStringView str) const;
   bool operator<(const WideString& other) const;
 
   CharType operator[](const size_t index) const {
-    ASSERT(IsValidIndex(index));
-    return m_pData ? m_pData->m_String[index] : 0;
+    CHECK(IsValidIndex(index));
+    return m_pData->m_String[index];
   }
 
   CharType First() const { return GetLength() ? (*this)[0] : 0; }
@@ -158,30 +161,30 @@ class WideString {
 
   void Trim();
   void Trim(wchar_t target);
-  void Trim(const WideStringView& targets);
+  void Trim(WideStringView targets);
 
   void TrimLeft();
   void TrimLeft(wchar_t target);
-  void TrimLeft(const WideStringView& targets);
+  void TrimLeft(WideStringView targets);
 
   void TrimRight();
   void TrimRight(wchar_t target);
-  void TrimRight(const WideStringView& targets);
+  void TrimRight(WideStringView targets);
 
   void Reserve(size_t len);
 
   // Note: any modification of the string (including ReleaseBuffer()) may
   // invalidate the span, which must not outlive its buffer.
-  pdfium::span<wchar_t> GetBuffer(size_t len);
-  void ReleaseBuffer(size_t len);
+  pdfium::span<wchar_t> GetBuffer(size_t nMinBufLength);
+  void ReleaseBuffer(size_t nNewLength);
 
   int GetInteger() const;
-  float GetFloat() const;
 
-  Optional<size_t> Find(const WideStringView& pSub, size_t start = 0) const;
+  Optional<size_t> Find(WideStringView subStr, size_t start = 0) const;
   Optional<size_t> Find(wchar_t ch, size_t start = 0) const;
+  Optional<size_t> ReverseFind(wchar_t ch) const;
 
-  bool Contains(const WideStringView& lpszSub, size_t start = 0) const {
+  bool Contains(WideStringView lpszSub, size_t start = 0) const {
     return Find(lpszSub, start).has_value();
   }
 
@@ -189,25 +192,35 @@ class WideString {
     return Find(ch, start).has_value();
   }
 
-  size_t Replace(const WideStringView& pOld, const WideStringView& pNew);
+  size_t Replace(WideStringView pOld, WideStringView pNew);
   size_t Remove(wchar_t ch);
 
+  bool IsASCII() const { return AsStringView().IsASCII(); }
+  bool EqualsASCII(ByteStringView that) const {
+    return AsStringView().EqualsASCII(that);
+  }
+  bool EqualsASCIINoCase(ByteStringView that) const {
+    return AsStringView().EqualsASCIINoCase(that);
+  }
+
+  ByteString ToASCII() const;
+  ByteString ToLatin1() const;
   ByteString ToDefANSI() const;
-  ByteString UTF8Encode() const;
+  ByteString ToUTF8() const;
 
   // This method will add \0\0 to the end of the string to represent the
   // wide string terminator. These values are in the string, not just the data,
   // so GetLength() will include them.
-  ByteString UTF16LE_Encode() const;
+  ByteString ToUTF16LE() const;
 
  protected:
   using StringData = StringDataTemplate<wchar_t>;
 
-  void ReallocBeforeWrite(size_t nLen);
-  void AllocBeforeWrite(size_t nLen);
+  void ReallocBeforeWrite(size_t nNewLength);
+  void AllocBeforeWrite(size_t nNewLength);
   void AllocCopy(WideString& dest, size_t nCopyLen, size_t nCopyIndex) const;
   void AssignCopy(const wchar_t* pSrcData, size_t nSrcLen);
-  void Concat(const wchar_t* lpszSrcData, size_t nSrcLen);
+  void Concat(const wchar_t* pSrcData, size_t nSrcLen);
   intptr_t ReferenceCountForTesting() const;
 
   RetainPtr<StringData> m_pData;
@@ -217,20 +230,19 @@ class WideString {
   friend StringPool_WideString_Test;
 };
 
-inline WideString operator+(const WideStringView& str1,
-                            const WideStringView& str2) {
+inline WideString operator+(WideStringView str1, WideStringView str2) {
   return WideString(str1, str2);
 }
-inline WideString operator+(const WideStringView& str1, const wchar_t* str2) {
+inline WideString operator+(WideStringView str1, const wchar_t* str2) {
   return WideString(str1, str2);
 }
-inline WideString operator+(const wchar_t* str1, const WideStringView& str2) {
+inline WideString operator+(const wchar_t* str1, WideStringView str2) {
   return WideString(str1, str2);
 }
-inline WideString operator+(const WideStringView& str1, wchar_t ch) {
+inline WideString operator+(WideStringView str1, wchar_t ch) {
   return WideString(str1, WideStringView(ch));
 }
-inline WideString operator+(wchar_t ch, const WideStringView& str2) {
+inline WideString operator+(wchar_t ch, WideStringView str2) {
   return WideString(ch, str2);
 }
 inline WideString operator+(const WideString& str1, const WideString& str2) {
@@ -248,24 +260,22 @@ inline WideString operator+(const WideString& str1, const wchar_t* str2) {
 inline WideString operator+(const wchar_t* str1, const WideString& str2) {
   return WideString(str1, str2.AsStringView());
 }
-inline WideString operator+(const WideString& str1,
-                            const WideStringView& str2) {
+inline WideString operator+(const WideString& str1, WideStringView str2) {
   return WideString(str1.AsStringView(), str2);
 }
-inline WideString operator+(const WideStringView& str1,
-                            const WideString& str2) {
+inline WideString operator+(WideStringView str1, const WideString& str2) {
   return WideString(str1, str2.AsStringView());
 }
 inline bool operator==(const wchar_t* lhs, const WideString& rhs) {
   return rhs == lhs;
 }
-inline bool operator==(const WideStringView& lhs, const WideString& rhs) {
+inline bool operator==(WideStringView lhs, const WideString& rhs) {
   return rhs == lhs;
 }
 inline bool operator!=(const wchar_t* lhs, const WideString& rhs) {
   return rhs != lhs;
 }
-inline bool operator!=(const WideStringView& lhs, const WideString& rhs) {
+inline bool operator!=(WideStringView lhs, const WideString& rhs) {
   return rhs != lhs;
 }
 inline bool operator<(const wchar_t* lhs, const WideString& rhs) {
@@ -274,14 +284,14 @@ inline bool operator<(const wchar_t* lhs, const WideString& rhs) {
 
 std::wostream& operator<<(std::wostream& os, const WideString& str);
 std::ostream& operator<<(std::ostream& os, const WideString& str);
-std::wostream& operator<<(std::wostream& os, const WideStringView& str);
-std::ostream& operator<<(std::ostream& os, const WideStringView& str);
+std::wostream& operator<<(std::wostream& os, WideStringView str);
+std::ostream& operator<<(std::ostream& os, WideStringView str);
 
 }  // namespace fxcrt
 
 using WideString = fxcrt::WideString;
 
-uint32_t FX_HashCode_GetW(const WideStringView& str, bool bIgnoreCase);
+uint32_t FX_HashCode_GetW(WideStringView str, bool bIgnoreCase);
 
 namespace std {
 
