@@ -25,7 +25,7 @@ TEST(WideString, ElementAccess) {
   EXPECT_DEATH({ abc[4]; }, ".*");
 #endif
 
-  pdfium::span<const wchar_t> abc_span = abc.AsSpan();
+  pdfium::span<const wchar_t> abc_span = abc.span();
   EXPECT_EQ(3u, abc_span.size());
   EXPECT_EQ(0, wmemcmp(abc_span.data(), L"abc", 3));
 
@@ -689,6 +689,40 @@ TEST(WideString, Find) {
   EXPECT_EQ(2u, result.value());
 }
 
+TEST(WideString, ReverseFind) {
+  WideString null_string;
+  EXPECT_FALSE(null_string.ReverseFind(L'a').has_value());
+  EXPECT_FALSE(null_string.ReverseFind(L'\0').has_value());
+
+  WideString empty_string(L"");
+  EXPECT_FALSE(empty_string.ReverseFind(L'a').has_value());
+  EXPECT_FALSE(empty_string.ReverseFind(L'\0').has_value());
+
+  Optional<size_t> result;
+  WideString single_string(L"a");
+  result = single_string.ReverseFind(L'a');
+  ASSERT_TRUE(result.has_value());
+  EXPECT_EQ(0u, result.value());
+  EXPECT_FALSE(single_string.ReverseFind(L'b').has_value());
+  EXPECT_FALSE(single_string.ReverseFind(L'\0').has_value());
+
+  WideString longer_string(L"abccc");
+  result = longer_string.ReverseFind(L'a');
+  ASSERT_TRUE(result.has_value());
+  EXPECT_EQ(0u, result.value());
+  result = longer_string.ReverseFind(L'c');
+  ASSERT_TRUE(result.has_value());
+  EXPECT_EQ(4u, result.value());
+  EXPECT_FALSE(longer_string.ReverseFind(L'\0').has_value());
+
+  WideString hibyte_string(
+      L"ab\xff8c"
+      L"def");
+  result = hibyte_string.ReverseFind(L'\xff8c');
+  ASSERT_TRUE(result.has_value());
+  EXPECT_EQ(2u, result.value());
+}
+
 TEST(WideString, UpperLower) {
   WideString fred(L"F-Re.42D");
   fred.MakeLower();
@@ -958,46 +992,64 @@ TEST(WideString, OneCharReverseIterator) {
 TEST(WideString, MultiCharReverseIterator) {
   WideString multi_str(L"abcd");
   auto iter = multi_str.rbegin();
-  EXPECT_FALSE(iter == multi_str.rend());
+  EXPECT_NE(iter, multi_str.rend());
+  EXPECT_EQ(4, multi_str.rend() - iter);
+  EXPECT_EQ(0, iter - multi_str.rbegin());
 
   char ch = *iter++;
   EXPECT_EQ('d', ch);
   EXPECT_EQ('c', *iter);
-  EXPECT_FALSE(iter == multi_str.rend());
+  EXPECT_NE(iter, multi_str.rend());
+  EXPECT_EQ(3, multi_str.rend() - iter);
+  EXPECT_EQ(1, iter - multi_str.rbegin());
 
   ch = *(++iter);
   EXPECT_EQ('b', ch);
   EXPECT_EQ('b', *iter);
-  EXPECT_FALSE(iter == multi_str.rend());
+  EXPECT_NE(iter, multi_str.rend());
+  EXPECT_EQ(2, multi_str.rend() - iter);
+  EXPECT_EQ(2, iter - multi_str.rbegin());
 
   ch = *iter++;
   EXPECT_EQ('b', ch);
   EXPECT_EQ('a', *iter);
-  EXPECT_FALSE(iter == multi_str.rend());
+  EXPECT_NE(iter, multi_str.rend());
+  EXPECT_EQ(1, multi_str.rend() - iter);
+  EXPECT_EQ(3, iter - multi_str.rbegin());
 
   ch = *iter++;
   EXPECT_EQ('a', ch);
-  EXPECT_TRUE(iter == multi_str.rend());
+  EXPECT_EQ(iter, multi_str.rend());
+  EXPECT_EQ(0, multi_str.rend() - iter);
+  EXPECT_EQ(4, iter - multi_str.rbegin());
 
   ch = *(--iter);
   EXPECT_EQ('a', ch);
   EXPECT_EQ('a', *iter);
-  EXPECT_FALSE(iter == multi_str.rend());
+  EXPECT_NE(iter, multi_str.rend());
+  EXPECT_EQ(1, multi_str.rend() - iter);
+  EXPECT_EQ(3, iter - multi_str.rbegin());
 
   ch = *iter--;
   EXPECT_EQ('a', ch);
   EXPECT_EQ('b', *iter);
-  EXPECT_FALSE(iter == multi_str.rend());
+  EXPECT_NE(iter, multi_str.rend());
+  EXPECT_EQ(2, multi_str.rend() - iter);
+  EXPECT_EQ(2, iter - multi_str.rbegin());
 
   ch = *iter--;
   EXPECT_EQ('b', ch);
   EXPECT_EQ('c', *iter);
-  EXPECT_FALSE(iter == multi_str.rend());
+  EXPECT_NE(iter, multi_str.rend());
+  EXPECT_EQ(3, multi_str.rend() - iter);
+  EXPECT_EQ(1, iter - multi_str.rbegin());
 
   ch = *(--iter);
   EXPECT_EQ('d', ch);
   EXPECT_EQ('d', *iter);
-  EXPECT_TRUE(iter == multi_str.rbegin());
+  EXPECT_EQ(iter, multi_str.rbegin());
+  EXPECT_EQ(4, multi_str.rend() - iter);
+  EXPECT_EQ(0, iter - multi_str.rbegin());
 }
 
 TEST(WideString, ToUTF16LE) {
@@ -1567,7 +1619,7 @@ TEST(WideString, Empty) {
   EXPECT_NE(nullptr, cstr);
   EXPECT_EQ(0u, wcslen(cstr));
 
-  pdfium::span<const wchar_t> cspan = empty_str.AsSpan();
+  pdfium::span<const wchar_t> cspan = empty_str.span();
   EXPECT_TRUE(cspan.empty());
   EXPECT_EQ(nullptr, cspan.data());
 }
