@@ -11,14 +11,15 @@
 
 #include "core/fxge/apple/apple_int.h"
 #include "core/fxge/cfx_cliprgn.h"
-#include "core/fxge/cfx_facecache.h"
 #include "core/fxge/cfx_font.h"
 #include "core/fxge/cfx_gemodule.h"
 #include "core/fxge/cfx_glyphbitmap.h"
+#include "core/fxge/cfx_glyphcache.h"
 #include "core/fxge/cfx_renderdevice.h"
 #include "core/fxge/cfx_substfont.h"
 #include "core/fxge/dib/cfx_dibitmap.h"
 #include "core/fxge/fx_freetype.h"
+#include "core/fxge/text_char_pos.h"
 #include "third_party/base/span.h"
 
 #ifndef _SKIA_SUPPORT_
@@ -35,7 +36,7 @@ bool CGDrawGlyphRun(CGContextRef pContext,
                     int nChars,
                     const TextCharPos* pCharPos,
                     CFX_Font* pFont,
-                    const CFX_Matrix* pObject2Device,
+                    const CFX_Matrix& mtObject2Device,
                     float font_size,
                     uint32_t argb) {
   if (nChars == 0)
@@ -45,12 +46,9 @@ bool CGDrawGlyphRun(CGContextRef pContext,
   if (bNegSize)
     font_size = -font_size;
 
-  CFX_Matrix new_matrix;
-  if (pObject2Device)
-    new_matrix.Concat(*pObject2Device);
-
+  CFX_Matrix new_matrix = mtObject2Device;
   CQuartz2D& quartz2d =
-      static_cast<CApplePlatform*>(CFX_GEModule::Get()->GetPlatformData())
+      static_cast<CApplePlatform*>(CFX_GEModule::Get()->GetPlatform())
           ->m_quartz2d;
   if (!pFont->GetPlatformFont()) {
     if (pFont->GetPsName() == "DFHeiStd-W5")
@@ -79,8 +77,8 @@ bool CGDrawGlyphRun(CGContextRef pContext,
     new_matrix.b = -new_matrix.b;
     new_matrix.d = -new_matrix.d;
   }
-  quartz2d.setGraphicsTextMatrix(pContext, &new_matrix);
-  return quartz2d.drawGraphicsString(pContext, pFont->GetPlatformFont(),
+  quartz2d.SetGraphicsTextMatrix(pContext, new_matrix);
+  return quartz2d.DrawGraphicsString(pContext, pFont->GetPlatformFont(),
                                      font_size, glyph_indices.data(),
                                      glyph_positions.data(), nChars, argb);
 }
@@ -89,17 +87,17 @@ bool CGDrawGlyphRun(CGContextRef pContext,
 
 void CFX_AggDeviceDriver::InitPlatform() {
   CQuartz2D& quartz2d =
-      static_cast<CApplePlatform*>(CFX_GEModule::Get()->GetPlatformData())
+      static_cast<CApplePlatform*>(CFX_GEModule::Get()->GetPlatform())
           ->m_quartz2d;
-  m_pPlatformGraphics = quartz2d.createGraphics(m_pBitmap);
+  m_pPlatformGraphics = quartz2d.CreateGraphics(m_pBitmap);
 }
 
 void CFX_AggDeviceDriver::DestroyPlatform() {
   CQuartz2D& quartz2d =
-      static_cast<CApplePlatform*>(CFX_GEModule::Get()->GetPlatformData())
+      static_cast<CApplePlatform*>(CFX_GEModule::Get()->GetPlatform())
           ->m_quartz2d;
   if (m_pPlatformGraphics) {
-    quartz2d.destroyGraphics(m_pPlatformGraphics);
+    quartz2d.DestroyGraphics(m_pPlatformGraphics);
     m_pPlatformGraphics = nullptr;
   }
 }
@@ -107,7 +105,7 @@ void CFX_AggDeviceDriver::DestroyPlatform() {
 bool CFX_AggDeviceDriver::DrawDeviceText(int nChars,
                                          const TextCharPos* pCharPos,
                                          CFX_Font* pFont,
-                                         const CFX_Matrix* pObject2Device,
+                                         const CFX_Matrix& mtObject2Device,
                                          float font_size,
                                          uint32_t argb) {
   if (!pFont)
@@ -155,7 +153,7 @@ bool CFX_AggDeviceDriver::DrawDeviceText(int nChars,
   else
     CGContextClipToRect(ctx, rect_cg);
 
-  bool ret = CGDrawGlyphRun(ctx, nChars, pCharPos, pFont, pObject2Device,
+  bool ret = CGDrawGlyphRun(ctx, nChars, pCharPos, pFont, mtObject2Device,
                             font_size, argb);
   if (pImageCG)
     CGImageRelease(pImageCG);
@@ -165,11 +163,11 @@ bool CFX_AggDeviceDriver::DrawDeviceText(int nChars,
 
 #endif  // _SKIA_SUPPORT_
 
-void CFX_FaceCache::InitPlatform() {}
+void CFX_GlyphCache::InitPlatform() {}
 
-void CFX_FaceCache::DestroyPlatform() {}
+void CFX_GlyphCache::DestroyPlatform() {}
 
-std::unique_ptr<CFX_GlyphBitmap> CFX_FaceCache::RenderGlyph_Nativetext(
+std::unique_ptr<CFX_GlyphBitmap> CFX_GlyphCache::RenderGlyph_Nativetext(
     const CFX_Font* pFont,
     uint32_t glyph_index,
     const CFX_Matrix& matrix,
@@ -181,7 +179,7 @@ std::unique_ptr<CFX_GlyphBitmap> CFX_FaceCache::RenderGlyph_Nativetext(
 void CFX_Font::ReleasePlatformResource() {
   if (m_pPlatformFont) {
     CQuartz2D& quartz2d =
-        static_cast<CApplePlatform*>(CFX_GEModule::Get()->GetPlatformData())
+        static_cast<CApplePlatform*>(CFX_GEModule::Get()->GetPlatform())
             ->m_quartz2d;
     quartz2d.DestroyFont(m_pPlatformFont);
     m_pPlatformFont = nullptr;
