@@ -17,20 +17,19 @@ CFX_GEModule* g_pGEModule = nullptr;
 
 }  // namespace
 
-CFX_GEModule::CFX_GEModule()
-    : m_pFontMgr(pdfium::MakeUnique<CFX_FontMgr>()),
-      m_pPlatformData(nullptr),
-      m_pUserFontPaths(nullptr) {}
+CFX_GEModule::CFX_GEModule(const char** pUserFontPaths)
+    : m_pPlatform(PlatformIface::Create()),
+      m_pFontMgr(pdfium::MakeUnique<CFX_FontMgr>()),
+      m_pFontCache(pdfium::MakeUnique<CFX_FontCache>()),
+      m_pUserFontPaths(pUserFontPaths) {}
 
-CFX_GEModule::~CFX_GEModule() {
-  DestroyPlatform();
-}
+CFX_GEModule::~CFX_GEModule() = default;
 
 // static
-CFX_GEModule* CFX_GEModule::Get() {
-  if (!g_pGEModule)
-    g_pGEModule = new CFX_GEModule();
-  return g_pGEModule;
+void CFX_GEModule::Create(const char** pUserFontPaths) {
+  ASSERT(!g_pGEModule);
+  g_pGEModule = new CFX_GEModule(pUserFontPaths);
+  g_pGEModule->m_pPlatform->Init();
 }
 
 // static
@@ -40,14 +39,8 @@ void CFX_GEModule::Destroy() {
   g_pGEModule = nullptr;
 }
 
-void CFX_GEModule::Init(const char** userFontPaths) {
+// static
+CFX_GEModule* CFX_GEModule::Get() {
   ASSERT(g_pGEModule);
-  m_pUserFontPaths = userFontPaths;
-  InitPlatform();
-}
-
-CFX_FontCache* CFX_GEModule::GetFontCache() {
-  if (!m_pFontCache)
-    m_pFontCache = pdfium::MakeUnique<CFX_FontCache>();
-  return m_pFontCache.get();
+  return g_pGEModule;
 }
