@@ -17,20 +17,20 @@ CFX_UnicodeEncoding::CFX_UnicodeEncoding(CFX_Font* pFont) : m_pFont(pFont) {}
 CFX_UnicodeEncoding::~CFX_UnicodeEncoding() {}
 
 uint32_t CFX_UnicodeEncoding::GlyphFromCharCode(uint32_t charcode) {
-  FXFT_Face face = m_pFont->GetFace();
+  FXFT_FaceRec* face = m_pFont->GetFaceRec();
   if (!face)
     return charcode;
 
-  if (FXFT_Select_Charmap(face, FXFT_ENCODING_UNICODE) == 0)
-    return FXFT_Get_Char_Index(face, charcode);
+  if (FT_Select_Charmap(face, FT_ENCODING_UNICODE) == 0)
+    return FT_Get_Char_Index(face, charcode);
 
   if (m_pFont->GetSubstFont() &&
       m_pFont->GetSubstFont()->m_Charset == FX_CHARSET_Symbol) {
     uint32_t index = 0;
-    if (FXFT_Select_Charmap(face, FXFT_ENCODING_MS_SYMBOL) == 0)
-      index = FXFT_Get_Char_Index(face, charcode);
-    if (!index && !FXFT_Select_Charmap(face, FXFT_ENCODING_APPLE_ROMAN))
-      return FXFT_Get_Char_Index(face, charcode);
+    if (FT_Select_Charmap(face, FT_ENCODING_MS_SYMBOL) == 0)
+      index = FT_Get_Char_Index(face, charcode);
+    if (!index && !FT_Select_Charmap(face, FT_ENCODING_APPLE_ROMAN))
+      return FT_Get_Char_Index(face, charcode);
   }
   return charcode;
 }
