@@ -57,27 +57,11 @@ const int16_t SDP_Table[513] = {
 
 FXDIB_ResampleOptions::FXDIB_ResampleOptions() = default;
 
-FXDIB_ResampleOptions::FXDIB_ResampleOptions(bool downsample,
-                                             bool bilinear,
-                                             bool bicubic,
-                                             bool halftone,
-                                             bool no_smoothing,
-                                             bool lossy)
-    : bInterpolateDownsample(downsample),
-      bInterpolateBilinear(bilinear),
-      bInterpolateBicubic(bicubic),
-      bHalftone(halftone),
-      bNoSmoothing(no_smoothing),
-      bLossy(lossy) {}
-
 bool FXDIB_ResampleOptions::HasAnyOptions() const {
   return bInterpolateDownsample || bInterpolateBilinear ||
          bInterpolateBicubic || bHalftone || bNoSmoothing || bLossy;
 }
 
-const FXDIB_ResampleOptions kBilinearInterpolation = {
-    false, /*bilinear=*/true, false, false, false, false};
-
 FX_RECT FXDIB_SwapClipBox(const FX_RECT& clip,
                           int width,
                           int height,
@@ -120,14 +104,14 @@ FX_ARGB AlphaAndColorRefToArgb(int a, FX_COLORREF colorref) {
                     FXSYS_GetBValue(colorref));
 }
 
-FX_ARGB StringToFXARGB(WideStringView wsValue) {
+FX_ARGB StringToFXARGB(WideStringView view) {
   static constexpr FX_ARGB kDefaultValue = 0xff000000;
-  if (wsValue.GetLength() == 0)
+  if (view.GetLength() == 0)
     return kDefaultValue;
 
   int cc = 0;
-  const wchar_t* str = wsValue.unterminated_c_str();
-  int len = wsValue.GetLength();
+  const wchar_t* str = view.unterminated_c_str();
+  int len = view.GetLength();
   while (cc < len && FXSYS_iswspace(str[cc]))
     cc++;
 
@@ -170,5 +154,5 @@ FX_ARGB StringToFXARGB(WideStringView wsValue) {
       }
     }
   }
-  return (0xff << 24) | (r << 16) | (g << 8) | b;
+  return (0xffU << 24) | (r << 16) | (g << 8) | b;
 }
