@@ -48,14 +48,6 @@ extern const int16_t SDP_Table[513];
 
 struct FXDIB_ResampleOptions {
   FXDIB_ResampleOptions();
-  // TODO(thestig): Remove this ctor once C++14 can be used for more flexible
-  // constexpr initialization.
-  FXDIB_ResampleOptions(bool downsample,
-                        bool bilinear,
-                        bool bicubic,
-                        bool halftone,
-                        bool no_smoothing,
-                        bool lossy);
 
   bool HasAnyOptions() const;
 
@@ -67,8 +59,6 @@ struct FXDIB_ResampleOptions {
   bool bLossy = false;
 };
 
-extern const FXDIB_ResampleOptions kBilinearInterpolation;
-
 // See PDF 1.7 spec, table 7.2 and 7.3. The enum values need to be in the same
 // order as listed in the spec.
 enum class BlendMode {
@@ -88,6 +78,7 @@ enum class BlendMode {
   kSaturation,
   kColor,
   kLuminosity,
+  kLast = kLuminosity,
 };
 
 constexpr uint32_t FXSYS_BGR(uint8_t b, uint8_t g, uint8_t r) {
@@ -106,6 +97,10 @@ constexpr uint8_t FXSYS_GetBValue(uint32_t bgr) {
   return (bgr >> 16) & 0xff;
 }
 
+constexpr unsigned int FXSYS_GetUnsignedAlpha(float alpha) {
+  return static_cast<unsigned int>(alpha * 255.f + 0.5f);
+}
+
 #define FXSYS_GetCValue(cmyk) ((uint8_t)((cmyk) >> 24) & 0xff)
 #define FXSYS_GetMValue(cmyk) ((uint8_t)((cmyk) >> 16) & 0xff)
 #define FXSYS_GetYValue(cmyk) ((uint8_t)((cmyk) >> 8) & 0xff)
