@@ -12,14 +12,14 @@
 #include "core/fxcrt/unowned_ptr.h"
 #include "public/fpdf_save.h"
 
-class CPDFSDK_FileWriteAdapter final : public IFX_WriteStream {
+class CPDFSDK_FileWriteAdapter final : public IFX_RetainableWriteStream {
  public:
   template <typename T, typename... Args>
   friend RetainPtr<T> pdfium::MakeRetain(Args&&... args);
 
   // IFX_WriteStream:
   bool WriteBlock(const void* data, size_t size) override;
-  bool WriteString(const ByteStringView& str) override;
+  bool WriteString(ByteStringView str) override;
 
  private:
   explicit CPDFSDK_FileWriteAdapter(FPDF_FILEWRITE* file_write);
