@@ -6,41 +6,45 @@
 
 #include "fpdfsdk/formfiller/cffl_listbox.h"
 
-#include "fpdfsdk/cpdfsdk_common.h"
+#include <utility>
+
+#include "constants/form_flags.h"
+#include "core/fpdfdoc/cba_fontmap.h"
 #include "fpdfsdk/cpdfsdk_formfillenvironment.h"
 #include "fpdfsdk/cpdfsdk_widget.h"
-#include "fpdfsdk/formfiller/cba_fontmap.h"
 #include "fpdfsdk/formfiller/cffl_interactiveformfiller.h"
 #include "fpdfsdk/pwl/cpwl_list_box.h"
 #include "third_party/base/ptr_util.h"
 
-#define FFL_DEFAULTLISTBOXFONTSIZE 12.0f
-
 CFFL_ListBox::CFFL_ListBox(CPDFSDK_FormFillEnvironment* pApp,
                            CPDFSDK_Widget* pWidget)
     : CFFL_TextObject(pApp, pWidget) {}
 
-CFFL_ListBox::~CFFL_ListBox() {}
+CFFL_ListBox::~CFFL_ListBox() = default;
 
 CPWL_Wnd::CreateParams CFFL_ListBox::GetCreateParam() {
   CPWL_Wnd::CreateParams cp = CFFL_TextObject::GetCreateParam();
   uint32_t dwFieldFlag = m_pWidget->GetFieldFlags();
-  if (dwFieldFlag & FIELDFLAG_MULTISELECT)
+  if (dwFieldFlag & pdfium::form_flags::kChoiceMultiSelect)
     cp.dwFlags |= PLBS_MULTIPLESEL;
 
   cp.dwFlags |= PWS_VSCROLL;
 
-  if (cp.dwFlags & PWS_AUTOFONTSIZE)
-    cp.fFontSize = FFL_DEFAULTLISTBOXFONTSIZE;
+  if (cp.dwFlags & PWS_AUTOFONTSIZE) {
+    constexpr float kDefaultListBoxFontSize = 12.0f;
+    cp.fFontSize = kDefaultListBoxFontSize;
+  }
 
   cp.pFontMap = MaybeCreateFontMap();
   return cp;
 }
 
-CPWL_Wnd* CFFL_ListBox::NewPDFWindow(const CPWL_Wnd::CreateParams& cp) {
-  auto* pWnd = new CPWL_ListBox();
+std::unique_ptr<CPWL_Wnd> CFFL_ListBox::NewPWLWindow(
+    const CPWL_Wnd::CreateParams& cp,
+    std::unique_ptr<IPWL_SystemHandler::PerWindowData> pAttachedData) {
+  auto pWnd = pdfium::MakeUnique<CPWL_ListBox>(cp, std::move(pAttachedData));
   pWnd->AttachFFLData(this);
-  pWnd->Create(cp);
+  pWnd->Realize();
   pWnd->SetFillerNotify(m_pFormFillEnv->GetInteractiveFormFiller());
 
   for (int32_t i = 0, sz = m_pWidget->CountOptions(); i < sz; i++)
@@ -70,8 +74,7 @@ CPWL_Wnd* CFFL_ListBox::NewPDFWindow(const CPWL_Wnd::CreateParams& cp) {
   }
 
   pWnd->SetTopVisibleIndex(m_pWidget->GetTopVisibleIndex());
-
-  return pWnd;
+  return std::move(pWnd);
 }
 
 bool CFFL_ListBox::OnChar(CPDFSDK_Annot* pAnnot,
@@ -81,11 +84,11 @@ bool CFFL_ListBox::OnChar(CPDFSDK_Annot* pAnnot,
 }
 
 bool CFFL_ListBox::IsDataChanged(CPDFSDK_PageView* pPageView) {
-  auto* pListBox = static_cast<CPWL_ListBox*>(GetPDFWindow(pPageView, false));
+  CPWL_ListBox* pListBox = GetListBox(pPageView);
   if (!pListBox)
     return false;
 
-  if (m_pWidget->GetFieldFlags() & FIELDFLAG_MULTISELECT) {
+  if (m_pWidget->GetFieldFlags() & pdfium::form_flags::kChoiceMultiSelect) {
     size_t nSelCount = 0;
     for (int32_t i = 0, sz = pListBox->GetCount(); i < sz; ++i) {
       if (pListBox->IsItemSelected(i)) {
@@ -102,14 +105,13 @@ bool CFFL_ListBox::IsDataChanged(CPDFSDK_PageView* pPageView) {
 }
 
 void CFFL_ListBox::SaveData(CPDFSDK_PageView* pPageView) {
-  CPWL_ListBox* pListBox =
-      static_cast<CPWL_ListBox*>(GetPDFWindow(pPageView, false));
+  CPWL_ListBox* pListBox = GetListBox(pPageView);
   if (!pListBox)
     return;
 
   int32_t nNewTopIndex = pListBox->GetTopVisibleIndex();
   m_pWidget->ClearSelection(NotificationOption::kDoNotNotify);
-  if (m_pWidget->GetFieldFlags() & FIELDFLAG_MULTISELECT) {
+  if (m_pWidget->GetFieldFlags() & pdfium::form_flags::kChoiceMultiSelect) {
     for (int32_t i = 0, sz = pListBox->GetCount(); i < sz; i++) {
       if (pListBox->IsItemSelected(i)) {
         m_pWidget->SetOptionSelection(i, true,
@@ -120,13 +122,13 @@ void CFFL_ListBox::SaveData(CPDFSDK_PageView* pPageView) {
     m_pWidget->SetOptionSelection(pListBox->GetCurSel(), true,
                                   NotificationOption::kDoNotNotify);
   }
-  CPDFSDK_Widget::ObservedPtr observed_widget(m_pWidget.Get());
-  CFFL_ListBox::ObservedPtr observed_this(this);
+  ObservedPtr<CPDFSDK_Widget> observed_widget(m_pWidget.Get());
+  ObservedPtr<CFFL_ListBox> observed_this(this);
   m_pWidget->SetTopVisibleIndex(nNewTopIndex);
   if (!observed_widget)
     return;
 
-  m_pWidget->ResetFieldAppearance(true);
+  m_pWidget->ResetFieldAppearance();
   if (!observed_widget)
     return;
 
@@ -141,12 +143,11 @@ void CFFL_ListBox::GetActionData(CPDFSDK_PageView* pPageView,
                                  CPDF_AAction::AActionType type,
                                  CPDFSDK_FieldAction& fa) {
   switch (type) {
-    case CPDF_AAction::Validate:
-      if (m_pWidget->GetFieldFlags() & FIELDFLAG_MULTISELECT) {
-        fa.sValue = L"";
+    case CPDF_AAction::kValidate:
+      if (m_pWidget->GetFieldFlags() & pdfium::form_flags::kChoiceMultiSelect) {
+        fa.sValue.clear();
       } else {
-        auto* pListBox =
-            static_cast<CPWL_ListBox*>(GetPDFWindow(pPageView, false));
+        CPWL_ListBox* pListBox = GetListBox(pPageView);
         if (pListBox) {
           int32_t nCurSel = pListBox->GetCurSel();
           if (nCurSel >= 0)
@@ -154,10 +155,10 @@ void CFFL_ListBox::GetActionData(CPDFSDK_PageView* pPageView,
         }
       }
       break;
-    case CPDF_AAction::LoseFocus:
-    case CPDF_AAction::GetFocus:
-      if (m_pWidget->GetFieldFlags() & FIELDFLAG_MULTISELECT) {
-        fa.sValue = L"";
+    case CPDF_AAction::kLoseFocus:
+    case CPDF_AAction::kGetFocus:
+      if (m_pWidget->GetFieldFlags() & pdfium::form_flags::kChoiceMultiSelect) {
+        fa.sValue.clear();
       } else {
         int32_t nCurSel = m_pWidget->GetSelectedIndex(0);
         if (nCurSel >= 0)
@@ -170,10 +171,7 @@ void CFFL_ListBox::GetActionData(CPDFSDK_PageView* pPageView,
 }
 
 void CFFL_ListBox::SaveState(CPDFSDK_PageView* pPageView) {
-  ASSERT(pPageView);
-
-  CPWL_ListBox* pListBox =
-      static_cast<CPWL_ListBox*>(GetPDFWindow(pPageView, false));
+  CPWL_ListBox* pListBox = GetListBox(pPageView);
   if (!pListBox)
     return;
 
@@ -184,11 +182,47 @@ void CFFL_ListBox::SaveState(CPDFSDK_PageView* pPageView) {
 }
 
 void CFFL_ListBox::RestoreState(CPDFSDK_PageView* pPageView) {
-  CPWL_ListBox* pListBox =
-      static_cast<CPWL_ListBox*>(GetPDFWindow(pPageView, false));
+  CPWL_ListBox* pListBox = GetListBox(pPageView);
   if (!pListBox)
     return;
 
   for (const auto& item : m_State)
     pListBox->Select(item);
 }
+
+bool CFFL_ListBox::SetIndexSelected(int index, bool selected) {
+  if (!IsValid())
+    return false;
+
+  if (index < 0 || index >= m_pWidget->CountOptions())
+    return false;
+
+  CPWL_ListBox* pListBox = GetListBox(GetCurPageView(true));
+  if (!pListBox)
+    return false;
+
+  if (selected) {
+    pListBox->Select(index);
+    pListBox->SetCaret(index);
+  } else {
+    pListBox->Deselect(index);
+    pListBox->SetCaret(index);
+  }
+
+  return true;
+}
+
+bool CFFL_ListBox::IsIndexSelected(int index) {
+  if (!IsValid())
+    return false;
+
+  if (index < 0 || index >= m_pWidget->CountOptions())
+    return false;
+
+  CPWL_ListBox* pListBox = GetListBox(GetCurPageView(true));
+  return pListBox && pListBox->IsItemSelected(index);
+}
+
+CPWL_ListBox* CFFL_ListBox::GetListBox(CPDFSDK_PageView* pPageView) {
+  return static_cast<CPWL_ListBox*>(GetPWLWindow(pPageView, /*bNew=*/false));
+}
