@@ -7,12 +7,14 @@
 #ifndef FPDFSDK_FORMFILLER_CFFL_LISTBOX_H_
 #define FPDFSDK_FORMFILLER_CFFL_LISTBOX_H_
 
+#include <memory>
 #include <set>
 #include <vector>
 
 #include "fpdfsdk/formfiller/cffl_textobject.h"
 
 class CBA_FontMap;
+class CPWL_ListBox;
 
 class CFFL_ListBox final : public CFFL_TextObject {
  public:
@@ -21,7 +23,10 @@ class CFFL_ListBox final : public CFFL_TextObject {
 
   // CFFL_TextObject:
   CPWL_Wnd::CreateParams GetCreateParam() override;
-  CPWL_Wnd* NewPDFWindow(const CPWL_Wnd::CreateParams& cp) override;
+  std::unique_ptr<CPWL_Wnd> NewPWLWindow(
+      const CPWL_Wnd::CreateParams& cp,
+      std::unique_ptr<IPWL_SystemHandler::PerWindowData> pAttachedData)
+      override;
   bool OnChar(CPDFSDK_Annot* pAnnot, uint32_t nChar, uint32_t nFlags) override;
   bool IsDataChanged(CPDFSDK_PageView* pPageView) override;
   void SaveData(CPDFSDK_PageView* pPageView) override;
@@ -30,8 +35,12 @@ class CFFL_ListBox final : public CFFL_TextObject {
                      CPDFSDK_FieldAction& fa) override;
   void SaveState(CPDFSDK_PageView* pPageView) override;
   void RestoreState(CPDFSDK_PageView* pPageView) override;
+  bool SetIndexSelected(int index, bool selected) override;
+  bool IsIndexSelected(int index) override;
 
  private:
+  CPWL_ListBox* GetListBox(CPDFSDK_PageView* pPageView);
+
   std::set<int> m_OriginSelections;
   std::vector<int> m_State;
 };
