@@ -6,10 +6,13 @@
 
 #include "fpdfsdk/formfiller/cffl_textfield.h"
 
-#include "fpdfsdk/cpdfsdk_common.h"
+#include <utility>
+
+#include "constants/form_flags.h"
+#include "core/fpdfdoc/cba_fontmap.h"
 #include "fpdfsdk/cpdfsdk_formfillenvironment.h"
 #include "fpdfsdk/cpdfsdk_widget.h"
-#include "fpdfsdk/formfiller/cba_fontmap.h"
+#include "public/fpdf_fwlevent.h"
 #include "third_party/base/ptr_util.h"
 
 CFFL_TextField::CFFL_TextField(CPDFSDK_FormFillEnvironment* pApp,
@@ -29,23 +32,23 @@ CFFL_TextField::~CFFL_TextField() {
 CPWL_Wnd::CreateParams CFFL_TextField::GetCreateParam() {
   CPWL_Wnd::CreateParams cp = CFFL_TextObject::GetCreateParam();
   int nFlags = m_pWidget->GetFieldFlags();
-  if (nFlags & FIELDFLAG_PASSWORD)
+  if (nFlags & pdfium::form_flags::kTextPassword)
     cp.dwFlags |= PES_PASSWORD;
 
-  if (nFlags & FIELDFLAG_MULTILINE) {
+  if (nFlags & pdfium::form_flags::kTextMultiline) {
     cp.dwFlags |= PES_MULTILINE | PES_AUTORETURN | PES_TOP;
-    if (!(nFlags & FIELDFLAG_DONOTSCROLL))
+    if (!(nFlags & pdfium::form_flags::kTextDoNotScroll))
       cp.dwFlags |= PWS_VSCROLL | PES_AUTOSCROLL;
   } else {
     cp.dwFlags |= PES_CENTER;
-    if (!(nFlags & FIELDFLAG_DONOTSCROLL))
+    if (!(nFlags & pdfium::form_flags::kTextDoNotScroll))
       cp.dwFlags |= PES_AUTOSCROLL;
   }
 
-  if (nFlags & FIELDFLAG_COMB)
+  if (nFlags & pdfium::form_flags::kTextComb)
     cp.dwFlags |= PES_CHARARRAY;
 
-  if (nFlags & FIELDFLAG_RICHTEXT)
+  if (nFlags & pdfium::form_flags::kTextRichText)
     cp.dwFlags |= PES_RICH;
 
   cp.dwFlags |= PES_UNDO;
@@ -67,15 +70,16 @@ CPWL_Wnd::CreateParams CFFL_TextField::GetCreateParam() {
   return cp;
 }
 
-CPWL_Wnd* CFFL_TextField::NewPDFWindow(const CPWL_Wnd::CreateParams& cp) {
-  auto* pWnd = new CPWL_Edit();
+std::unique_ptr<CPWL_Wnd> CFFL_TextField::NewPWLWindow(
+    const CPWL_Wnd::CreateParams& cp,
+    std::unique_ptr<IPWL_SystemHandler::PerWindowData> pAttachedData) {
+  auto pWnd = pdfium::MakeUnique<CPWL_Edit>(cp, std::move(pAttachedData));
   pWnd->AttachFFLData(this);
-  pWnd->Create(cp);
+  pWnd->Realize();
   pWnd->SetFillerNotify(m_pFormFillEnv->GetInteractiveFormFiller());
 
   int32_t nMaxLen = m_pWidget->GetMaxLen();
   WideString swValue = m_pWidget->GetValue();
-
   if (nMaxLen > 0) {
     if (pWnd->HasFlag(PES_CHARARRAY)) {
       pWnd->SetCharArray(nMaxLen);
@@ -84,9 +88,8 @@ CPWL_Wnd* CFFL_TextField::NewPDFWindow(const CPWL_Wnd::CreateParams& cp) {
       pWnd->SetLimitChar(nMaxLen);
     }
   }
-
   pWnd->SetText(swValue);
-  return pWnd;
+  return std::move(pWnd);
 }
 
 bool CFFL_TextField::OnChar(CPDFSDK_Annot* pAnnot,
@@ -94,7 +97,7 @@ bool CFFL_TextField::OnChar(CPDFSDK_Annot* pAnnot,
                             uint32_t nFlags) {
   switch (nChar) {
     case FWL_VKEY_Return: {
-      if (m_pWidget->GetFieldFlags() & FIELDFLAG_MULTILINE)
+      if (m_pWidget->GetFieldFlags() & pdfium::form_flags::kTextMultiline)
         break;
 
       CPDFSDK_PageView* pPageView = GetCurPageView(true);
@@ -104,7 +107,7 @@ bool CFFL_TextField::OnChar(CPDFSDK_Annot* pAnnot,
                                  pAnnot->GetRect().GetOuterRect());
 
       if (m_bValid) {
-        if (CPWL_Wnd* pWnd = GetPDFWindow(pPageView, true))
+        if (CPWL_Wnd* pWnd = GetPWLWindow(pPageView, true))
           pWnd->SetFocus();
         break;
       }
@@ -112,7 +115,7 @@ bool CFFL_TextField::OnChar(CPDFSDK_Annot* pAnnot,
       if (!CommitData(pPageView, nFlags))
         return false;
 
-      DestroyPDFWindow(pPageView);
+      DestroyPWLWindow(pPageView);
       return true;
     }
     case FWL_VKEY_Escape: {
@@ -138,13 +141,13 @@ void CFFL_TextField::SaveData(CPDFSDK_PageView* pPageView) {
 
   WideString sOldValue = m_pWidget->GetValue();
   WideString sNewValue = pWnd->GetText();
-  CPDFSDK_Widget::ObservedPtr observed_widget(m_pWidget.Get());
-  CFFL_TextField::ObservedPtr observed_this(this);
+  ObservedPtr<CPDFSDK_Widget> observed_widget(m_pWidget.Get());
+  ObservedPtr<CFFL_TextField> observed_this(this);
   m_pWidget->SetValue(sNewValue, NotificationOption::kDoNotNotify);
   if (!observed_widget)
     return;
 
-  m_pWidget->ResetFieldAppearance(true);
+  m_pWidget->ResetFieldAppearance();
   if (!observed_widget)
     return;
 
@@ -159,25 +162,25 @@ void CFFL_TextField::GetActionData(CPDFSDK_PageView* pPageView,
                                    CPDF_AAction::AActionType type,
                                    CPDFSDK_FieldAction& fa) {
   switch (type) {
-    case CPDF_AAction::KeyStroke:
+    case CPDF_AAction::kKeyStroke:
       if (CPWL_Edit* pWnd = GetEdit(pPageView, false)) {
         fa.bFieldFull = pWnd->IsTextFull();
 
         fa.sValue = pWnd->GetText();
 
         if (fa.bFieldFull) {
-          fa.sChange = L"";
-          fa.sChangeEx = L"";
+          fa.sChange.clear();
+          fa.sChangeEx.clear();
         }
       }
       break;
-    case CPDF_AAction::Validate:
+    case CPDF_AAction::kValidate:
       if (CPWL_Edit* pWnd = GetEdit(pPageView, false)) {
         fa.sValue = pWnd->GetText();
       }
       break;
-    case CPDF_AAction::LoseFocus:
-    case CPDF_AAction::GetFocus:
+    case CPDF_AAction::kLoseFocus:
+    case CPDF_AAction::kGetFocus:
       fa.sValue = m_pWidget->GetValue();
       break;
     default:
@@ -189,11 +192,11 @@ void CFFL_TextField::SetActionData(CPDFSDK_PageView* pPageView,
                                    CPDF_AAction::AActionType type,
                                    const CPDFSDK_FieldAction& fa) {
   switch (type) {
-    case CPDF_AAction::KeyStroke:
+    case CPDF_AAction::kKeyStroke:
       if (CPWL_Edit* pEdit = GetEdit(pPageView, false)) {
         pEdit->SetFocus();
         pEdit->SetSelection(fa.nSelStart, fa.nSelEnd);
-        pEdit->ReplaceSel(fa.sChange);
+        pEdit->ReplaceSelection(fa.sChange);
       }
       break;
     default:
@@ -205,7 +208,7 @@ bool CFFL_TextField::IsActionDataChanged(CPDF_AAction::AActionType type,
                                          const CPDFSDK_FieldAction& faOld,
                                          const CPDFSDK_FieldAction& faNew) {
   switch (type) {
-    case CPDF_AAction::KeyStroke:
+    case CPDF_AAction::kKeyStroke:
       return (!faOld.bFieldFull && faOld.nSelEnd != faNew.nSelEnd) ||
              faOld.nSelStart != faNew.nSelStart ||
              faOld.sChange != faNew.sChange;
@@ -217,8 +220,6 @@ bool CFFL_TextField::IsActionDataChanged(CPDF_AAction::AActionType type,
 }
 
 void CFFL_TextField::SaveState(CPDFSDK_PageView* pPageView) {
-  ASSERT(pPageView);
-
   CPWL_Edit* pWnd = GetEdit(pPageView, false);
   if (!pWnd)
     return;
@@ -228,8 +229,6 @@ void CFFL_TextField::SaveState(CPDFSDK_PageView* pPageView) {
 }
 
 void CFFL_TextField::RestoreState(CPDFSDK_PageView* pPageView) {
-  ASSERT(pPageView);
-
   CPWL_Edit* pWnd = GetEdit(pPageView, true);
   if (!pWnd)
     return;
@@ -251,11 +250,11 @@ void CFFL_TextField::OnSetFocus(CPWL_Edit* pEdit) {
 
   WideString wsText = pEdit->GetText();
   int nCharacters = wsText.GetLength();
-  ByteString bsUTFText = wsText.UTF16LE_Encode();
+  ByteString bsUTFText = wsText.ToUTF16LE();
   auto* pBuffer = reinterpret_cast<const unsigned short*>(bsUTFText.c_str());
   m_pFormFillEnv->OnSetFieldInputFocus(pBuffer, nCharacters, true);
 }
 
 CPWL_Edit* CFFL_TextField::GetEdit(CPDFSDK_PageView* pPageView, bool bNew) {
-  return static_cast<CPWL_Edit*>(GetPDFWindow(pPageView, bNew));
+  return static_cast<CPWL_Edit*>(GetPWLWindow(pPageView, bNew));
 }
