@@ -11,12 +11,14 @@
 
 #include "fpdfsdk/formfiller/cffl_formfiller.h"
 
+class CBA_FontMap;
+
 // Class to implement common functionality for CFFL_FormFiller sub-classes with
 // text fields.
 class CFFL_TextObject : public CFFL_FormFiller {
  public:
   // CFFL_FormFiller:
-  CPWL_Wnd* ResetPDFWindow(CPDFSDK_PageView* pPageView,
+  CPWL_Wnd* ResetPWLWindow(CPDFSDK_PageView* pPageView,
                            bool bRestoreValue) override;
 
  protected:
