@@ -86,7 +86,15 @@ TEST_F(FPDFAnnotEmbedderTest, RenderAnnotWithOnlyRolloverAP) {
   UnloadPage(page);
 }
 
-TEST_F(FPDFAnnotEmbedderTest, RenderMultilineMarkupAnnotWithoutAP) {
+// TODO(crbug.com/pdfium/11): Fix this test and enable.
+#if defined(_SKIA_SUPPORT_) || defined(_SKIA_SUPPORT_PATHS_)
+#define MAYBE_RenderMultilineMarkupAnnotWithoutAP \
+  DISABLED_RenderMultilineMarkupAnnotWithoutAP
+#else
+#define MAYBE_RenderMultilineMarkupAnnotWithoutAP \
+  RenderMultilineMarkupAnnotWithoutAP
+#endif
+TEST_F(FPDFAnnotEmbedderTest, MAYBE_RenderMultilineMarkupAnnotWithoutAP) {
   static const char kMd5[] = "76512832d88017668d9acc7aacd13dae";
   // Open a file with multiline markup annotations.
   ASSERT_TRUE(OpenDocument("annotation_markup_multiline_no_ap.pdf"));
@@ -183,7 +191,13 @@ TEST_F(FPDFAnnotEmbedderTest, ExtractHighlightLongContent) {
   UnloadPageNoEvents(page);
 }
 
-TEST_F(FPDFAnnotEmbedderTest, ExtractInkMultiple) {
+// TODO(crbug.com/pdfium/11): Fix this test and enable.
+#if defined(_SKIA_SUPPORT_) || defined(_SKIA_SUPPORT_PATHS_)
+#define MAYBE_ExtractInkMultiple DISABLED_ExtractInkMultiple
+#else
+#define MAYBE_ExtractInkMultiple ExtractInkMultiple
+#endif
+TEST_F(FPDFAnnotEmbedderTest, MAYBE_ExtractInkMultiple) {
   // Open a file with three annotations and load its first page.
   ASSERT_TRUE(OpenDocument("annotation_ink_multiple.pdf"));
   FPDF_PAGE page = LoadPageNoEvents(0);
@@ -327,7 +341,14 @@ TEST_F(FPDFAnnotEmbedderTest, AddFirstTextAnnotation) {
   UnloadPage(page);
 }
 
-TEST_F(FPDFAnnotEmbedderTest, AddAndSaveUnderlineAnnotation) {
+// TODO(crbug.com/pdfium/11): Fix this test and enable.
+#if defined(_SKIA_SUPPORT_) || defined(_SKIA_SUPPORT_PATHS_)
+#define MAYBE_AddAndSaveUnderlineAnnotation \
+  DISABLED_AddAndSaveUnderlineAnnotation
+#else
+#define MAYBE_AddAndSaveUnderlineAnnotation AddAndSaveUnderlineAnnotation
+#endif
+TEST_F(FPDFAnnotEmbedderTest, MAYBE_AddAndSaveUnderlineAnnotation) {
   // Open a file with one annotation and load its first page.
   ASSERT_TRUE(OpenDocument("annotation_highlight_long_content.pdf"));
   FPDF_PAGE page = LoadPage(0);
@@ -474,7 +495,13 @@ TEST_F(FPDFAnnotEmbedderTest, GetAndSetQuadPoints) {
   UnloadPage(page);
 }
 
-TEST_F(FPDFAnnotEmbedderTest, ModifyRectQuadpointsWithAP) {
+// TODO(crbug.com/pdfium/11): Fix this test and enable.
+#if defined(_SKIA_SUPPORT_) || defined(_SKIA_SUPPORT_PATHS_)
+#define MAYBE_ModifyRectQuadpointsWithAP DISABLED_ModifyRectQuadpointsWithAP
+#else
+#define MAYBE_ModifyRectQuadpointsWithAP ModifyRectQuadpointsWithAP
+#endif
+TEST_F(FPDFAnnotEmbedderTest, MAYBE_ModifyRectQuadpointsWithAP) {
 #if defined(OS_MACOSX)
   static const char kMd5Original[] = "63af8432fab95a67cdebb7cd0e514941";
   static const char kMd5ModifiedHighlight[] =
@@ -682,22 +709,28 @@ TEST_F(FPDFAnnotEmbedderTest, RemoveAnnotation) {
   FPDF_CloseDocument(new_doc);
 }
 
-TEST_F(FPDFAnnotEmbedderTest, AddAndModifyPath) {
+// TODO(crbug.com/pdfium/11): Fix this test and enable.
+#if defined(_SKIA_SUPPORT_) || defined(_SKIA_SUPPORT_PATHS_)
+#define MAYBE_AddAndModifyPath DISABLED_AddAndModifyPath
+#else
+#define MAYBE_AddAndModifyPath AddAndModifyPath
+#endif
+TEST_F(FPDFAnnotEmbedderTest, MAYBE_AddAndModifyPath) {
 #if defined(OS_MACOSX)
   static const char kMd5Original[] = "c35408717759562d1f8bf33d317483d2";
   static const char kMd5ModifiedPath[] = "9059723a045e17478753d2f0eb33bc03";
   static const char kMd5TwoPaths[] = "7eed0cfba780f1d4dd8068f717d3a6bf";
   static const char kMd5NewAnnot[] = "1de8212d43b7066a6df042095c2aca61";
 #elif defined(OS_WIN)
-  static const char kMd5Original[] = "6f3cc2dd37479ce7cc072bfb0c63c275";
-  static const char kMd5ModifiedPath[] = "c0c2eb2aba73ad15b3240e342fbe0d72";
-  static const char kMd5TwoPaths[] = "2306bf04915fe001b5f4726843d184c8";
-  static const char kMd5NewAnnot[] = "64a319f145768cb09944d2109efe394e";
+  static const char kMd5Original[] = "6aa001a77ec05d0f1b0d1d22e28744d4";
+  static const char kMd5ModifiedPath[] = "a7a8d675a6ddbcbdfecee65a33ba19e1";
+  static const char kMd5TwoPaths[] = "7c0bdd4552329704c47a7cce47edbbd6";
+  static const char kMd5NewAnnot[] = "3c48d492b4f62941fed0fb62f729f31e";
 #else
-  static const char kMd5Original[] = "964f89bbe8911e540a465cf1a64b7f7e";
-  static const char kMd5ModifiedPath[] = "9a38048fb3ac1b2c9a4b34139caa993c";
-  static const char kMd5TwoPaths[] = "ece3d4df54b3395d6a2bf7a29b17239c";
-  static const char kMd5NewAnnot[] = "4299493de84c249f42f220f30f2bbb67";
+  static const char kMd5Original[] = "b42cef463483e668eaf4055a65e4f1f5";
+  static const char kMd5ModifiedPath[] = "6ff77d6d1fec4ea571fabe0c7a19b517";
+  static const char kMd5TwoPaths[] = "ca37ad549e74ac5b359a055708f3e7b6";
+  static const char kMd5NewAnnot[] = "0d7a0e33fbf41ff7fa5d732ab2c5edff";
 #endif
 
   // Open a file with two annotations and load its first page.
@@ -727,7 +760,7 @@ TEST_F(FPDFAnnotEmbedderTest, AddAndModifyPath) {
     EXPECT_TRUE(path);
 
     // Modify the color of the path object.
-    EXPECT_TRUE(FPDFPath_SetStrokeColor(path, 0, 0, 0, 255));
+    EXPECT_TRUE(FPDFPageObj_SetStrokeColor(path, 0, 0, 0, 255));
     EXPECT_TRUE(FPDFAnnot_UpdateObject(annot.get(), path));
 
     // Check that the page with the modified annotation renders correctly.
@@ -739,8 +772,8 @@ TEST_F(FPDFAnnotEmbedderTest, AddAndModifyPath) {
     // Add a second path object to the same annotation.
     FPDF_PAGEOBJECT dot = FPDFPageObj_CreateNewPath(7, 84);
     EXPECT_TRUE(FPDFPath_BezierTo(dot, 9, 86, 10, 87, 11, 88));
-    EXPECT_TRUE(FPDFPath_SetStrokeColor(dot, 255, 0, 0, 100));
-    EXPECT_TRUE(FPDFPath_SetStrokeWidth(dot, 14));
+    EXPECT_TRUE(FPDFPageObj_SetStrokeColor(dot, 255, 0, 0, 100));
+    EXPECT_TRUE(FPDFPageObj_SetStrokeWidth(dot, 14));
     EXPECT_TRUE(FPDFPath_SetDrawMode(dot, 0, 1));
     EXPECT_TRUE(FPDFAnnot_AppendObject(annot.get(), dot));
     EXPECT_EQ(2, FPDFAnnot_GetObjectCount(annot.get()));
@@ -785,8 +818,8 @@ TEST_F(FPDFAnnotEmbedderTest, AddAndModifyPath) {
     EXPECT_TRUE(FPDFPath_LineTo(check, 500, 600));
     EXPECT_TRUE(FPDFPath_MoveTo(check, 350, 550));
     EXPECT_TRUE(FPDFPath_LineTo(check, 450, 450));
-    EXPECT_TRUE(FPDFPath_SetStrokeColor(check, 0, 255, 255, 180));
-    EXPECT_TRUE(FPDFPath_SetStrokeWidth(check, 8.35f));
+    EXPECT_TRUE(FPDFPageObj_SetStrokeColor(check, 0, 255, 255, 180));
+    EXPECT_TRUE(FPDFPageObj_SetStrokeWidth(check, 8.35f));
     EXPECT_TRUE(FPDFPath_SetDrawMode(check, 0, 1));
     EXPECT_TRUE(FPDFAnnot_AppendObject(annot.get(), check));
     EXPECT_EQ(1, FPDFAnnot_GetObjectCount(annot.get()));
@@ -884,19 +917,25 @@ TEST_F(FPDFAnnotEmbedderTest, ModifyAnnotationFlags) {
   UnloadPage(page);
 }
 
-TEST_F(FPDFAnnotEmbedderTest, AddAndModifyImage) {
+// TODO(crbug.com/pdfium/11): Fix this test and enable.
+#if defined(_SKIA_SUPPORT_) || defined(_SKIA_SUPPORT_PATHS_)
+#define MAYBE_AddAndModifyImage DISABLED_AddAndModifyImage
+#else
+#define MAYBE_AddAndModifyImage AddAndModifyImage
+#endif
+TEST_F(FPDFAnnotEmbedderTest, MAYBE_AddAndModifyImage) {
 #if defined(OS_MACOSX)
   static const char kMd5Original[] = "c35408717759562d1f8bf33d317483d2";
   static const char kMd5NewImage[] = "ff012f5697436dfcaec25b32d1333596";
   static const char kMd5ModifiedImage[] = "86cf8cb2755a7a2046a543e66d9c1e61";
 #elif defined(OS_WIN)
-  static const char kMd5Original[] = "6f3cc2dd37479ce7cc072bfb0c63c275";
-  static const char kMd5NewImage[] = "d19c6fcfd9a170802fcfb9adfa13557e";
-  static const char kMd5ModifiedImage[] = "1273cf2363570a50d1aa0c95b1318197";
+  static const char kMd5Original[] = "6aa001a77ec05d0f1b0d1d22e28744d4";
+  static const char kMd5NewImage[] = "3d77d06a971bcb9fb54db082f1082c8b";
+  static const char kMd5ModifiedImage[] = "dc4f4afc26c345418330d31c065020e1";
 #else
-  static const char kMd5Original[] = "964f89bbe8911e540a465cf1a64b7f7e";
-  static const char kMd5NewImage[] = "9ea8732dc9d579f68853f16892856208";
-  static const char kMd5ModifiedImage[] = "74239d2a8c55c9de1dbb9cd8781895aa";
+  static const char kMd5Original[] = "b42cef463483e668eaf4055a65e4f1f5";
+  static const char kMd5NewImage[] = "528e6243dc29d54f36b61e0d3287d935";
+  static const char kMd5ModifiedImage[] = "6d9e59f3e57a1ff82fb258356b7eb731";
 #endif
 
   // Open a file with two annotations and load its first page.
@@ -969,19 +1008,25 @@ TEST_F(FPDFAnnotEmbedderTest, AddAndModifyImage) {
   VerifySavedDocument(595, 842, kMd5ModifiedImage);
 }
 
-TEST_F(FPDFAnnotEmbedderTest, AddAndModifyText) {
+// TODO(crbug.com/pdfium/11): Fix this test and enable.
+#if defined(_SKIA_SUPPORT_) || defined(_SKIA_SUPPORT_PATHS_)
+#define MAYBE_AddAndModifyText DISABLED_AddAndModifyText
+#else
+#define MAYBE_AddAndModifyText AddAndModifyText
+#endif
+TEST_F(FPDFAnnotEmbedderTest, MAYBE_AddAndModifyText) {
 #if defined(OS_MACOSX)
   static const char kMd5Original[] = "c35408717759562d1f8bf33d317483d2";
   static const char kMd5NewText[] = "60031c1b0330cf1e1575f7d46687d429";
   static const char kMd5ModifiedText[] = "79f5cfb0b07caaf936f65f6a7a57ce77";
 #elif defined(OS_WIN)
-  static const char kMd5Original[] = "6f3cc2dd37479ce7cc072bfb0c63c275";
-  static const char kMd5NewText[] = "87d55e09f9096de7e6552f5ae79afd3b";
-  static const char kMd5ModifiedText[] = "26e94fbd3af4b1e65479327507600114";
+  static const char kMd5Original[] = "6aa001a77ec05d0f1b0d1d22e28744d4";
+  static const char kMd5NewText[] = "204cc01749a70b8afc246a4ca33c7eb6";
+  static const char kMd5ModifiedText[] = "641261a45e8dfd68c89b80bfd237660d";
 #else
-  static const char kMd5Original[] = "964f89bbe8911e540a465cf1a64b7f7e";
-  static const char kMd5NewText[] = "30f3f5b989612ca03827d95f184f0979";
-  static const char kMd5ModifiedText[] = "076c8f24a09ddc0e49f7e758edead6f0";
+  static const char kMd5Original[] = "b42cef463483e668eaf4055a65e4f1f5";
+  static const char kMd5NewText[] = "00197ad6206f763febad5719e5935306";
+  static const char kMd5ModifiedText[] = "85853bc0aaa5a4e3af04e58b9cbfff23";
 #endif
 
   // Open a file with two annotations and load its first page.
@@ -1014,7 +1059,7 @@ TEST_F(FPDFAnnotEmbedderTest, AddAndModifyText) {
     ScopedFPDFWideString text =
         GetFPDFWideString(L"I'm a translucent text laying on other text.");
     EXPECT_TRUE(FPDFText_SetText(text_object, text.get()));
-    EXPECT_TRUE(FPDFText_SetFillColor(text_object, 0, 0, 255, 150));
+    EXPECT_TRUE(FPDFPageObj_SetFillColor(text_object, 0, 0, 255, 150));
     FPDFPageObj_Transform(text_object, 1, 0, 0, 1, 200, 600);
     EXPECT_TRUE(FPDFAnnot_AppendObject(annot.get(), text_object));
   }
@@ -1055,7 +1100,13 @@ TEST_F(FPDFAnnotEmbedderTest, AddAndModifyText) {
   UnloadPage(page);
 }
 
-TEST_F(FPDFAnnotEmbedderTest, GetSetStringValue) {
+// TODO(crbug.com/pdfium/11): Fix this test and enable.
+#if defined(_SKIA_SUPPORT_) || defined(_SKIA_SUPPORT_PATHS_)
+#define MAYBE_GetSetStringValue DISABLED_GetSetStringValue
+#else
+#define MAYBE_GetSetStringValue GetSetStringValue
+#endif
+TEST_F(FPDFAnnotEmbedderTest, MAYBE_GetSetStringValue) {
   // Open a file with four annotations and load its first page.
   ASSERT_TRUE(OpenDocument("annotation_stamp_with_ap.pdf"));
   FPDF_PAGE page = LoadPage(0);
@@ -1113,9 +1164,9 @@ TEST_F(FPDFAnnotEmbedderTest, GetSetStringValue) {
 #if defined(OS_MACOSX)
   static const char kMd5[] = "4d64e61c9c0f8c60ab3cc3234bb73b1c";
 #elif defined(OS_WIN)
-  static const char kMd5[] = "9ee141f698c3fcb56c050dffd6c82624";
+  static const char kMd5[] = "20b612ebd46babcb44c48c903e2c5a48";
 #else
-  static const char kMd5[] = "c96ee1f316d7f5a1b154de9f9d467f01";
+  static const char kMd5[] = "1d7bea2042c6fea0558ff2aef05811b5";
 #endif
 
   // Open the saved annotation.
@@ -1427,7 +1478,7 @@ TEST_F(FPDFAnnotEmbedderTest, GetFormFieldFlagsTextField) {
     ASSERT_TRUE(annot);
 
     // Check that the flag values are as expected.
-    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), page, annot.get());
+    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), annot.get());
     EXPECT_FALSE(flags & FPDF_FORMFLAG_READONLY);
   }
 
@@ -1437,7 +1488,7 @@ TEST_F(FPDFAnnotEmbedderTest, GetFormFieldFlagsTextField) {
     ASSERT_TRUE(annot);
 
     // Check that the flag values are as expected.
-    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), page, annot.get());
+    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), annot.get());
     EXPECT_TRUE(flags & FPDF_FORMFLAG_READONLY);
   }
 
@@ -1456,7 +1507,7 @@ TEST_F(FPDFAnnotEmbedderTest, GetFormFieldFlagsComboBox) {
     ASSERT_TRUE(annot);
 
     // Check that the flag values are as expected.
-    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), page, annot.get());
+    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), annot.get());
     EXPECT_FALSE(flags & FPDF_FORMFLAG_READONLY);
     EXPECT_TRUE(flags & FPDF_FORMFLAG_CHOICE_COMBO);
     EXPECT_TRUE(flags & FPDF_FORMFLAG_CHOICE_EDIT);
@@ -1468,7 +1519,7 @@ TEST_F(FPDFAnnotEmbedderTest, GetFormFieldFlagsComboBox) {
     ASSERT_TRUE(annot);
 
     // Check that the flag values are as expected.
-    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), page, annot.get());
+    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), annot.get());
     EXPECT_FALSE(flags & FPDF_FORMFLAG_READONLY);
     EXPECT_TRUE(flags & FPDF_FORMFLAG_CHOICE_COMBO);
     EXPECT_FALSE(flags & FPDF_FORMFLAG_CHOICE_EDIT);
@@ -1480,7 +1531,7 @@ TEST_F(FPDFAnnotEmbedderTest, GetFormFieldFlagsComboBox) {
     ASSERT_TRUE(annot);
 
     // Check that the flag values are as expected.
-    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), page, annot.get());
+    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), annot.get());
     EXPECT_TRUE(flags & FPDF_FORMFLAG_READONLY);
     EXPECT_TRUE(flags & FPDF_FORMFLAG_CHOICE_COMBO);
     EXPECT_FALSE(flags & FPDF_FORMFLAG_CHOICE_EDIT);
@@ -1526,7 +1577,7 @@ TEST_F(FPDFAnnotEmbedderTest, GetFormAnnotAndCheckFlagsTextField) {
     ASSERT_TRUE(annot);
 
     // Check that interactive form annotation flag values are as expected.
-    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), page, annot.get());
+    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), annot.get());
     EXPECT_FALSE(flags & FPDF_FORMFLAG_READONLY);
   }
 
@@ -1537,7 +1588,7 @@ TEST_F(FPDFAnnotEmbedderTest, GetFormAnnotAndCheckFlagsTextField) {
     ASSERT_TRUE(annot);
 
     // Check that interactive form annotation flag values are as expected.
-    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), page, annot.get());
+    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), annot.get());
     EXPECT_TRUE(flags & FPDF_FORMFLAG_READONLY);
   }
 
@@ -1557,7 +1608,7 @@ TEST_F(FPDFAnnotEmbedderTest, GetFormAnnotAndCheckFlagsComboBox) {
     ASSERT_TRUE(annot);
 
     // Check that interactive form annotation flag values are as expected.
-    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), page, annot.get());
+    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), annot.get());
     EXPECT_FALSE(flags & FPDF_FORMFLAG_READONLY);
     EXPECT_TRUE(flags & FPDF_FORMFLAG_CHOICE_COMBO);
     EXPECT_TRUE(flags & FPDF_FORMFLAG_CHOICE_EDIT);
@@ -1570,7 +1621,7 @@ TEST_F(FPDFAnnotEmbedderTest, GetFormAnnotAndCheckFlagsComboBox) {
     ASSERT_TRUE(annot);
 
     // Check that interactive form annotation flag values are as expected.
-    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), page, annot.get());
+    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), annot.get());
     EXPECT_FALSE(flags & FPDF_FORMFLAG_READONLY);
     EXPECT_TRUE(flags & FPDF_FORMFLAG_CHOICE_COMBO);
     EXPECT_FALSE(flags & FPDF_FORMFLAG_CHOICE_EDIT);
@@ -1583,7 +1634,7 @@ TEST_F(FPDFAnnotEmbedderTest, GetFormAnnotAndCheckFlagsComboBox) {
     ASSERT_TRUE(annot);
 
     // Check that interactive form annotation flag values are as expected.
-    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), page, annot.get());
+    int flags = FPDFAnnot_GetFormFieldFlags(form_handle(), annot.get());
     EXPECT_TRUE(flags & FPDF_FORMFLAG_READONLY);
     EXPECT_TRUE(flags & FPDF_FORMFLAG_CHOICE_COMBO);
     EXPECT_FALSE(flags & FPDF_FORMFLAG_CHOICE_EDIT);
@@ -1592,7 +1643,13 @@ TEST_F(FPDFAnnotEmbedderTest, GetFormAnnotAndCheckFlagsComboBox) {
   UnloadPage(page);
 }
 
-TEST_F(FPDFAnnotEmbedderTest, BUG_1206) {
+// TODO(crbug.com/pdfium/11): Fix this test and enable.
+#if defined(_SKIA_SUPPORT_) || defined(_SKIA_SUPPORT_PATHS_)
+#define MAYBE_BUG_1206 DISABLED_BUG_1206
+#else
+#define MAYBE_BUG_1206 BUG_1206
+#endif
+TEST_F(FPDFAnnotEmbedderTest, MAYBE_BUG_1206) {
   static constexpr size_t kExpectedSize = 1609;
   static const char kExpectedBitmap[] = "0d9fc05c6762fd788bd23fd87a4967bc";
 
@@ -1655,6 +1712,8 @@ TEST_F(FPDFAnnotEmbedderTest, BUG_1212) {
   {
     ScopedFPDFAnnotation annot(FPDFPage_CreateAnnot(page, FPDF_ANNOT_STAMP));
     ASSERT_TRUE(annot);
+    FS_RECTF bounding_rect{206.0f, 753.0f, 339.0f, 709.0f};
+    EXPECT_TRUE(FPDFAnnot_SetRect(annot.get(), &bounding_rect));
     EXPECT_EQ(2, FPDFPage_GetAnnotCount(page));
     EXPECT_EQ(FPDF_ANNOT_STAMP, FPDFAnnot_GetSubtype(annot.get()));
     // Also do the same test for its appearance string.
@@ -2042,3 +2101,112 @@ TEST_F(FPDFAnnotEmbedderTest, GetFontSizeNegative) {
 
   UnloadPage(page);
 }
+
+TEST_F(FPDFAnnotEmbedderTest, IsCheckedCheckbox) {
+  // Open a file with checkbox and radiobuttons widget annotations and load its
+  // first page.
+  ASSERT_TRUE(OpenDocument("click_form.pdf"));
+  FPDF_PAGE page = LoadPage(0);
+  ASSERT_TRUE(page);
+
+  {
+    ScopedFPDFAnnotation annot(FPDFPage_GetAnnot(page, 1));
+    ASSERT_TRUE(annot);
+    ASSERT_FALSE(FPDFAnnot_IsChecked(form_handle(), annot.get()));
+  }
+
+  UnloadPage(page);
+}
+
+TEST_F(FPDFAnnotEmbedderTest, IsCheckedCheckboxReadOnly) {
+  // Open a file with checkbox and radiobutton widget annotations and load its
+  // first page.
+  ASSERT_TRUE(OpenDocument("click_form.pdf"));
+  FPDF_PAGE page = LoadPage(0);
+  ASSERT_TRUE(page);
+
+  {
+    ScopedFPDFAnnotation annot(FPDFPage_GetAnnot(page, 0));
+    ASSERT_TRUE(annot);
+    ASSERT_TRUE(FPDFAnnot_IsChecked(form_handle(), annot.get()));
+  }
+
+  UnloadPage(page);
+}
+
+TEST_F(FPDFAnnotEmbedderTest, IsCheckedRadioButton) {
+  // Open a file with checkbox and radiobutton widget annotations and load its
+  // first page.
+  ASSERT_TRUE(OpenDocument("click_form.pdf"));
+  FPDF_PAGE page = LoadPage(0);
+  ASSERT_TRUE(page);
+
+  {
+    ScopedFPDFAnnotation annot(FPDFPage_GetAnnot(page, 5));
+    ASSERT_TRUE(annot);
+    ASSERT_FALSE(FPDFAnnot_IsChecked(form_handle(), annot.get()));
+
+    annot.reset(FPDFPage_GetAnnot(page, 6));
+    ASSERT_FALSE(FPDFAnnot_IsChecked(form_handle(), annot.get()));
+
+    annot.reset(FPDFPage_GetAnnot(page, 7));
+    ASSERT_TRUE(FPDFAnnot_IsChecked(form_handle(), annot.get()));
+  }
+
+  UnloadPage(page);
+}
+
+TEST_F(FPDFAnnotEmbedderTest, IsCheckedRadioButtonReadOnly) {
+  // Open a file with checkbox and radiobutton widget annotations and load its
+  // first page.
+  ASSERT_TRUE(OpenDocument("click_form.pdf"));
+  FPDF_PAGE page = LoadPage(0);
+  ASSERT_TRUE(page);
+
+  {
+    ScopedFPDFAnnotation annot(FPDFPage_GetAnnot(page, 2));
+    ASSERT_TRUE(annot);
+    ASSERT_FALSE(FPDFAnnot_IsChecked(form_handle(), annot.get()));
+
+    annot.reset(FPDFPage_GetAnnot(page, 3));
+    ASSERT_FALSE(FPDFAnnot_IsChecked(form_handle(), annot.get()));
+
+    annot.reset(FPDFPage_GetAnnot(page, 4));
+    ASSERT_TRUE(FPDFAnnot_IsChecked(form_handle(), annot.get()));
+  }
+
+  UnloadPage(page);
+}
+
+TEST_F(FPDFAnnotEmbedderTest, IsCheckedInvalidArguments) {
+  // Open a file with checkbox and radiobuttons widget annotations and load its
+  // first page.
+  ASSERT_TRUE(OpenDocument("click_form.pdf"));
+  FPDF_PAGE page = LoadPage(0);
+  ASSERT_TRUE(page);
+
+  {
+    ScopedFPDFAnnotation annot(FPDFPage_GetAnnot(page, 0));
+    ASSERT_TRUE(annot);
+    ASSERT_FALSE(FPDFAnnot_IsChecked(nullptr, annot.get()));
+    ASSERT_FALSE(FPDFAnnot_IsChecked(form_handle(), nullptr));
+    ASSERT_FALSE(FPDFAnnot_IsChecked(nullptr, nullptr));
+  }
+
+  UnloadPage(page);
+}
+
+TEST_F(FPDFAnnotEmbedderTest, IsCheckedInvalidWidgetType) {
+  // Open a file with text widget annotations and load its first page.
+  ASSERT_TRUE(OpenDocument("text_form.pdf"));
+  FPDF_PAGE page = LoadPage(0);
+  ASSERT_TRUE(page);
+
+  {
+    ScopedFPDFAnnotation annot(FPDFPage_GetAnnot(page, 0));
+    ASSERT_TRUE(annot);
+    ASSERT_FALSE(FPDFAnnot_IsChecked(form_handle(), annot.get()));
+  }
+
+  UnloadPage(page);
+}
