@@ -15,8 +15,8 @@ TEST_F(FPDFEditPathEmbedderTest, VerifyCorrectColoursReturned) {
 
   for (size_t i = 0; i < 256; ++i) {
     FPDF_PAGEOBJECT path = FPDFPageObj_CreateNewPath(400, 100);
-    EXPECT_TRUE(FPDFPath_SetFillColor(path, i, i, i, i));
-    EXPECT_TRUE(FPDFPath_SetStrokeColor(path, i, i, i, i));
+    EXPECT_TRUE(FPDFPageObj_SetFillColor(path, i, i, i, i));
+    EXPECT_TRUE(FPDFPageObj_SetStrokeColor(path, i, i, i, i));
     EXPECT_TRUE(FPDFPath_SetDrawMode(path, FPDF_FILLMODE_ALTERNATE, 0));
     EXPECT_TRUE(FPDFPath_LineTo(path, 400, 200));
     EXPECT_TRUE(FPDFPath_LineTo(path, 300, 100));
@@ -44,13 +44,13 @@ TEST_F(FPDFEditPathEmbedderTest, VerifyCorrectColoursReturned) {
     unsigned int g;
     unsigned int b;
     unsigned int a;
-    FPDFPath_GetFillColor(path, &r, &g, &b, &a);
+    FPDFPageObj_GetFillColor(path, &r, &g, &b, &a);
     EXPECT_EQ(i, r);
     EXPECT_EQ(i, g);
     EXPECT_EQ(i, b);
     EXPECT_EQ(i, a);
 
-    FPDFPath_GetStrokeColor(path, &r, &g, &b, &a);
+    FPDFPageObj_GetStrokeColor(path, &r, &g, &b, &a);
     EXPECT_EQ(i, r);
     EXPECT_EQ(i, g);
     EXPECT_EQ(i, b);
