@@ -7,21 +7,23 @@
 #ifndef FPDFSDK_PWL_CPWL_BUTTON_H_
 #define FPDFSDK_PWL_CPWL_BUTTON_H_
 
+#include <memory>
+
 #include "fpdfsdk/pwl/cpwl_wnd.h"
+#include "fpdfsdk/pwl/ipwl_systemhandler.h"
 
 class CPWL_Button : public CPWL_Wnd {
  public:
-  CPWL_Button();
+  CPWL_Button(const CreateParams& cp,
+              std::unique_ptr<IPWL_SystemHandler::PerWindowData> pAttachedData);
   ~CPWL_Button() override;
 
   // CPWL_Wnd
-  ByteString GetClassName() const override;
-  void OnCreate(CreateParams* pParamsToAdjust) override;
   bool OnLButtonDown(const CFX_PointF& point, uint32_t nFlag) override;
   bool OnLButtonUp(const CFX_PointF& point, uint32_t nFlag) override;
 
  protected:
-  bool m_bMouseDown;
+  bool m_bMouseDown = false;
 };
 
 #endif  // FPDFSDK_PWL_CPWL_BUTTON_H_
