@@ -25,9 +25,9 @@ class CPWL_EditImpl;
 class CPWL_EditImpl_Iterator;
 class CPWL_EditImpl_Provider;
 class CFX_RenderDevice;
-class CFX_SystemHandler;
 class CPWL_Edit;
 class CPWL_EditCtrl;
+class IPWL_SystemHandler;
 class IFX_Edit_UndoItem;
 
 struct CPWL_EditImpl_LineRect {
@@ -96,10 +96,18 @@ class CPWL_EditImpl_Undo {
 
 class IFX_Edit_UndoItem {
  public:
-  virtual ~IFX_Edit_UndoItem() {}
+  virtual ~IFX_Edit_UndoItem() = default;
 
-  virtual void Undo() = 0;
-  virtual void Redo() = 0;
+  // Undo/Redo the current undo item and returns the number of additional items
+  // to be processed in |m_UndoItemStack| to fully undo/redo the action. (An
+  // example is CFXEU_ReplaceSelection::Undo(), if CFXEU_ReplaceSelection marks
+  // the end of a replace action, CFXEU_ReplaceSelection::Undo() returns 3
+  // because 3 more undo items need to be processed to revert the replace
+  // action: insert text, clear selection and the CFXEU_ReplaceSelection which
+  // marks the beginning of replace action.) Implementations should return 0 by
+  // default.
+  virtual int Undo() = 0;
+  virtual int Redo() = 0;
 };
 
 class CFXEU_InsertWord final : public IFX_Edit_UndoItem {
@@ -112,8 +120,8 @@ class CFXEU_InsertWord final : public IFX_Edit_UndoItem {
   ~CFXEU_InsertWord() override;
 
   // IFX_Edit_UndoItem:
-  void Redo() override;
-  void Undo() override;
+  int Redo() override;
+  int Undo() override;
 
  private:
   UnownedPtr<CPWL_EditImpl> m_pEdit;
@@ -132,8 +140,8 @@ class CFXEU_InsertReturn final : public IFX_Edit_UndoItem {
   ~CFXEU_InsertReturn() override;
 
   // IFX_Edit_UndoItem:
-  void Redo() override;
-  void Undo() override;
+  int Redo() override;
+  int Undo() override;
 
  private:
   UnownedPtr<CPWL_EditImpl> m_pEdit;
@@ -142,6 +150,22 @@ class CFXEU_InsertReturn final : public IFX_Edit_UndoItem {
   CPVT_WordPlace m_wpNew;
 };
 
+class CFXEU_ReplaceSelection final : public IFX_Edit_UndoItem {
+ public:
+  CFXEU_ReplaceSelection(CPWL_EditImpl* pEdit, bool bIsEnd);
+  ~CFXEU_ReplaceSelection() override;
+
+  // IFX_Edit_UndoItem:
+  int Redo() override;
+  int Undo() override;
+
+ private:
+  bool IsEnd() const { return m_bEnd; }
+
+  UnownedPtr<CPWL_EditImpl> m_pEdit;
+  const bool m_bEnd;  // indicate whether this is the end of replace action
+};
+
 class CFXEU_Backspace final : public IFX_Edit_UndoItem {
  public:
   CFXEU_Backspace(CPWL_EditImpl* pEdit,
@@ -152,8 +176,8 @@ class CFXEU_Backspace final : public IFX_Edit_UndoItem {
   ~CFXEU_Backspace() override;
 
   // IFX_Edit_UndoItem:
-  void Redo() override;
-  void Undo() override;
+  int Redo() override;
+  int Undo() override;
 
  private:
   UnownedPtr<CPWL_EditImpl> m_pEdit;
@@ -175,8 +199,8 @@ class CFXEU_Delete final : public IFX_Edit_UndoItem {
   ~CFXEU_Delete() override;
 
   // IFX_Edit_UndoItem:
-  void Redo() override;
-  void Undo() override;
+  int Redo() override;
+  int Undo() override;
 
  private:
   UnownedPtr<CPWL_EditImpl> m_pEdit;
@@ -196,8 +220,8 @@ class CFXEU_Clear final : public IFX_Edit_UndoItem {
   ~CFXEU_Clear() override;
 
   // IFX_Edit_UndoItem:
-  void Redo() override;
-  void Undo() override;
+  int Redo() override;
+  int Undo() override;
 
  private:
   UnownedPtr<CPWL_EditImpl> m_pEdit;
@@ -216,8 +240,8 @@ class CFXEU_InsertText final : public IFX_Edit_UndoItem {
   ~CFXEU_InsertText() override;
 
   // IFX_Edit_UndoItem:
-  void Redo() override;
-  void Undo() override;
+  int Redo() override;
+  int Undo() override;
 
  private:
   UnownedPtr<CPWL_EditImpl> m_pEdit;
@@ -237,7 +261,7 @@ class CPWL_EditImpl {
                        const CFX_FloatRect& rcClip,
                        const CFX_PointF& ptOffset,
                        const CPVT_WordRange* pRange,
-                       CFX_SystemHandler* pSystemHandler,
+                       IPWL_SystemHandler* pSystemHandler,
                        CFFL_FormFiller* pFFLData);
 
   CPWL_EditImpl();
@@ -289,6 +313,7 @@ class CPWL_EditImpl {
   bool Delete();
   bool ClearSelection();
   bool InsertText(const WideString& sText, int32_t charset);
+  void ReplaceSelection(const WideString& text);
   bool Redo();
   bool Undo();
   CPVT_WordPlace WordIndexToWordPlace(int32_t index) const;
@@ -303,7 +328,6 @@ class CPWL_EditImpl {
   int32_t GetCharArray() const;
   CFX_FloatRect GetContentRect() const;
   WideString GetRangeText(const CPVT_WordRange& range) const;
-  int32_t GetHorzScale() const;
   float GetCharSpace() const;
   void SetSelection(int32_t nStartChar, int32_t nEndChar);
   void GetSelection(int32_t& nStartChar, int32_t& nEndChar) const;
@@ -322,7 +346,7 @@ class CPWL_EditImpl {
   bool CanRedo() const;
   CPVT_WordRange GetVisibleWordRange() const;
 
-  bool Empty();
+  bool Clear();
 
   CPVT_WordPlace DoInsertText(const CPVT_WordPlace& place,
                               const WideString& sText,
