@@ -31,12 +31,8 @@ CBC_Code128::CBC_Code128(BC_TYPE type)
 
 CBC_Code128::~CBC_Code128() {}
 
-bool CBC_Code128::SetTextLocation(BC_TEXT_LOC location) {
-  return GetOnedCode128Writer()->SetTextLocation(location);
-}
-
-bool CBC_Code128::Encode(const WideStringView& contents) {
-  if (contents.IsEmpty())
+bool CBC_Code128::Encode(WideStringView contents) {
+  if (contents.IsEmpty() || contents.GetLength() > kMaxInputLengthBytes)
     return false;
 
   BCFORMAT format = BCFORMAT_CODE_128;
@@ -48,7 +44,7 @@ bool CBC_Code128::Encode(const WideStringView& contents) {
     content += '0';
 
   m_renderContents = pWriter->FilterContents(content.AsStringView());
-  ByteString byteString = m_renderContents.UTF8Encode();
+  ByteString byteString = m_renderContents.ToUTF8();
   std::unique_ptr<uint8_t, FxFreeDeleter> data(
       pWriter->Encode(byteString, format, outWidth, outHeight));
   return data && pWriter->RenderResult(m_renderContents.AsStringView(),
