@@ -9,29 +9,25 @@
 
 #include "core/fxcrt/fx_coordinates.h"
 #include "core/fxcrt/fx_system.h"
-#include "core/fxge/fx_dib.h"
 #include "fxbarcode/cbc_onecode.h"
 
-class CBC_Code128 : public CBC_OneCode {
+class CBC_OnedCode128Writer;
+
+class CBC_Code128 final : public CBC_OneCode {
  public:
   explicit CBC_Code128(BC_TYPE type);
   ~CBC_Code128() override;
 
   // CBC_OneCode:
-  bool Encode(const CFX_WideStringC& contents,
-              bool isDevice,
-              int32_t& e) override;
-  bool RenderDevice(CFX_RenderDevice* device,
-                    const CFX_Matrix* matrix,
-                    int32_t& e) override;
-  bool RenderBitmap(CFX_RetainPtr<CFX_DIBitmap>& pOutBitmap,
-                    int32_t& e) override;
   BC_TYPE GetType() override;
-
-  bool SetTextLocation(BC_TEXT_LOC loction);
+  bool Encode(WideStringView contents) override;
+  bool RenderDevice(CFX_RenderDevice* device,
+                    const CFX_Matrix* matrix) override;
 
  private:
-  CFX_WideString m_renderContents;
+  CBC_OnedCode128Writer* GetOnedCode128Writer();
+
+  WideString m_renderContents;
 };
 
 #endif  // FXBARCODE_CBC_CODE128_H_
