@@ -9,31 +9,32 @@
 
 #include <memory>
 
-#include "core/fxcrt/fx_coordinates.h"
+#include "core/fxcrt/fx_string.h"
 #include "core/fxcrt/fx_system.h"
 #include "core/fxge/fx_dib.h"
 #include "fxbarcode/BC_Library.h"
 
-class CBC_Writer;
 class CBC_Reader;
+class CBC_Writer;
 class CFX_DIBitmap;
+class CFX_Matrix;
 class CFX_RenderDevice;
 
 class CBC_CodeBase {
  public:
-  explicit CBC_CodeBase(CBC_Writer* pWriter);
+  explicit CBC_CodeBase(std::unique_ptr<CBC_Writer> pWriter);
   virtual ~CBC_CodeBase();
 
   virtual BC_TYPE GetType() = 0;
-  virtual bool Encode(const CFX_WideStringC& contents,
-                      bool isDevice,
-                      int32_t& e) = 0;
+  virtual bool Encode(WideStringView contents) = 0;
   virtual bool RenderDevice(CFX_RenderDevice* device,
-                            const CFX_Matrix* matrix,
-                            int32_t& e) = 0;
-  virtual bool RenderBitmap(CFX_RetainPtr<CFX_DIBitmap>& pOutBitmap,
-                            int32_t& e) = 0;
+                            const CFX_Matrix* matrix) = 0;
 
+  bool SetTextLocation(BC_TEXT_LOC location);
+  bool SetWideNarrowRatio(int8_t ratio);
+  bool SetStartChar(char start);
+  bool SetEndChar(char end);
+  bool SetErrorCorrectionLevel(int32_t level);
   bool SetCharEncoding(int32_t encoding);
   bool SetModuleHeight(int32_t moduleHeight);
   bool SetModuleWidth(int32_t moduleWidth);
