@@ -7,30 +7,21 @@
 #ifndef FXBARCODE_CBC_EAN13_H_
 #define FXBARCODE_CBC_EAN13_H_
 
-#include "core/fxcrt/fx_coordinates.h"
 #include "core/fxcrt/fx_string.h"
 #include "core/fxcrt/fx_system.h"
-#include "core/fxge/fx_dib.h"
-#include "fxbarcode/cbc_onecode.h"
+#include "fxbarcode/cbc_eancode.h"
 
 class CBC_OnedEAN13Writer;
 
-class CBC_EAN13 final : public CBC_OneCode {
+class CBC_EAN13 final : public CBC_EANCode {
  public:
   CBC_EAN13();
   ~CBC_EAN13() override;
 
-  // CBC_OneCode:
-  bool Encode(const WideStringView& contents) override;
-  bool RenderDevice(CFX_RenderDevice* device,
-                    const CFX_Matrix* matrix) override;
+  // CBC_EANCode:
   BC_TYPE GetType() override;
-
- private:
-  CBC_OnedEAN13Writer* GetOnedEAN13Writer();
-  WideString Preprocess(const WideStringView& contents);
-
-  WideString m_renderContents;
+  BCFORMAT GetFormat() const override;
+  size_t GetMaxLength() const override;
 };
 
 #endif  // FXBARCODE_CBC_EAN13_H_
