@@ -9,14 +9,12 @@
 
 #include <vector>
 
-#include "core/fxcrt/fx_basic.h"
+#include "core/fxcrt/fx_string.h"
 
-class CBC_DefaultPlacement {
+class CBC_DefaultPlacement final {
  public:
-  CBC_DefaultPlacement(CFX_WideString codewords,
-                       int32_t numcols,
-                       int32_t numrows);
-  virtual ~CBC_DefaultPlacement();
+  CBC_DefaultPlacement(WideString codewords, int32_t numcols, int32_t numrows);
+  ~CBC_DefaultPlacement();
 
   int32_t getNumrows();
   int32_t getNumcols();
@@ -27,7 +25,7 @@ class CBC_DefaultPlacement {
   void place();
 
  private:
-  CFX_WideString m_codewords;
+  WideString m_codewords;
   int32_t m_numrows;
   int32_t m_numcols;
   std::vector<uint8_t> m_bits;
