@@ -26,29 +26,27 @@
 #include <utility>
 
 #include "fxbarcode/BC_TwoDimWriter.h"
-#include "fxbarcode/common/BC_CommonBitArray.h"
 #include "fxbarcode/common/BC_CommonBitMatrix.h"
 #include "fxbarcode/pdf417/BC_PDF417.h"
 #include "fxbarcode/pdf417/BC_PDF417BarcodeMatrix.h"
-#include "fxbarcode/pdf417/BC_PDF417Compaction.h"
+#include "third_party/base/stl_util.h"
 
-CBC_PDF417Writer::CBC_PDF417Writer() {
-  m_bFixedSize = false;
-}
+CBC_PDF417Writer::CBC_PDF417Writer() : CBC_TwoDimWriter(false) {}
 
-CBC_PDF417Writer::~CBC_PDF417Writer() {}
+CBC_PDF417Writer::~CBC_PDF417Writer() = default;
 
 bool CBC_PDF417Writer::SetErrorCorrectionLevel(int32_t level) {
   if (level < 0 || level > 8) {
     return false;
   }
-  m_iCorrectLevel = level;
+  set_error_correction_level(level);
   return true;
 }
 
-uint8_t* CBC_PDF417Writer::Encode(const WideString& contents,
-                                  int32_t* outWidth,
-                                  int32_t* outHeight) {
+std::vector<uint8_t> CBC_PDF417Writer::Encode(WideStringView contents,
+                                              int32_t* pOutWidth,
+                                              int32_t* pOutHeight) {
+  std::vector<uint8_t> results;
   CBC_PDF417 encoder;
   int32_t col = (m_Width / m_ModuleWidth - 69) / 17;
   int32_t row = m_Height / (m_ModuleWidth * 20);
@@ -58,8 +56,8 @@ uint8_t* CBC_PDF417Writer::Encode(const WideString& contents,
     encoder.setDimensions(col, col, 90, 3);
   else if (row >= 3 && row <= 90)
     encoder.setDimensions(30, 1, row, row);
-  if (!encoder.generateBarcodeLogic(contents, m_iCorrectLevel))
-    return nullptr;
+  if (!encoder.GenerateBarcodeLogic(contents, error_correction_level()))
+    return results;
 
   CBC_BarcodeMatrix* barcodeMatrix = encoder.getBarcodeMatrix();
   std::vector<uint8_t> matrixData = barcodeMatrix->toBitArray();
@@ -67,24 +65,24 @@ uint8_t* CBC_PDF417Writer::Encode(const WideString& contents,
   int32_t matrixHeight = barcodeMatrix->getHeight();
 
   if (matrixWidth < matrixHeight) {
-    rotateArray(matrixData, matrixHeight, matrixWidth);
+    RotateArray(&matrixData, matrixHeight, matrixWidth);
     std::swap(matrixWidth, matrixHeight);
   }
-  uint8_t* result = FX_Alloc2D(uint8_t, matrixHeight, matrixWidth);
-  memcpy(result, matrixData.data(), matrixHeight * matrixWidth);
-  *outWidth = matrixWidth;
-  *outHeight = matrixHeight;
-  return result;
+  *pOutWidth = matrixWidth;
+  *pOutHeight = matrixHeight;
+  results = pdfium::Vector2D<uint8_t>(*pOutWidth, *pOutHeight);
+  memcpy(results.data(), matrixData.data(), *pOutWidth * *pOutHeight);
+  return results;
 }
 
-void CBC_PDF417Writer::rotateArray(std::vector<uint8_t>& bitarray,
+void CBC_PDF417Writer::RotateArray(std::vector<uint8_t>* bitarray,
                                    int32_t height,
                                    int32_t width) {
-  std::vector<uint8_t> temp = bitarray;
+  std::vector<uint8_t> temp = *bitarray;
   for (int32_t ii = 0; ii < height; ii++) {
     int32_t inverseii = height - ii - 1;
     for (int32_t jj = 0; jj < width; jj++) {
-      bitarray[jj * height + inverseii] = temp[ii * width + jj];
+      (*bitarray)[jj * height + inverseii] = temp[ii * width + jj];
     }
   }
 }
