@@ -13,26 +13,22 @@
 #include "core/fxcrt/fx_system.h"
 #include "fxbarcode/BC_TwoDimWriter.h"
 
-class CBC_PDF417Writer : public CBC_TwoDimWriter {
+class CBC_PDF417Writer final : public CBC_TwoDimWriter {
  public:
   CBC_PDF417Writer();
   ~CBC_PDF417Writer() override;
 
-  uint8_t* Encode(const CFX_WideString& contents,
-                  int32_t& outWidth,
-                  int32_t& outHeight,
-                  int32_t& e);
+  std::vector<uint8_t> Encode(WideStringView contents,
+                              int32_t* pOutWidth,
+                              int32_t* pOutHeight);
 
   // CBC_TwoDimWriter
   bool SetErrorCorrectionLevel(int32_t level) override;
 
-  void SetTruncated(bool truncated);
-
  private:
-  void rotateArray(std::vector<uint8_t>& bitarray,
+  void RotateArray(std::vector<uint8_t>* bitarray,
                    int32_t width,
                    int32_t height);
-  bool m_bTruncated;
 };
 
 #endif  // FXBARCODE_PDF417_BC_PDF417WRITER_H_
