@@ -17,18 +17,12 @@
 #include "third_party/base/ptr_util.h"
 #include "third_party/base/stl_util.h"
 
-#ifdef PDF_ENABLE_XFA
-#include "fpdfsdk/fpdfxfa/cpdfxfa_context.h"
-#endif  // PDF_ENABLE_XFA
-
 #define JS_STR_VIEWERTYPE L"pdfium"
 #define JS_STR_VIEWERVARIATION L"Full"
 #define JS_STR_PLATFORM L"WIN"
 #define JS_STR_LANGUAGE L"ENU"
 #define JS_NUM_VIEWERVERSION 8
-#ifdef PDF_ENABLE_XFA
 #define JS_NUM_VIEWERVERSION_XFA 11
-#endif  // PDF_ENABLE_XFA
 #define JS_NUM_FORMSVERSION 7
 
 const JSPropertySpec CJS_App::PropertySpecs[] = {
@@ -150,12 +144,12 @@ CJS_Result CJS_App::set_viewer_variation(CJS_Runtime* pRuntime,
 }
 
 CJS_Result CJS_App::get_viewer_version(CJS_Runtime* pRuntime) {
-#ifdef PDF_ENABLE_XFA
-  CPDFXFA_Context* pXFAContext = pRuntime->GetFormFillEnv()->GetXFAContext();
-  if (pXFAContext->ContainsXFAForm())
-    return CJS_Result::Success(pRuntime->NewNumber(JS_NUM_VIEWERVERSION_XFA));
-#endif  // PDF_ENABLE_XFA
-  return CJS_Result::Success(pRuntime->NewNumber(JS_NUM_VIEWERVERSION));
+  CPDF_Document::Extension* pContext =
+      pRuntime->GetFormFillEnv()->GetDocExtension();
+  int version = pContext && pContext->ContainsExtensionForm()
+                    ? JS_NUM_VIEWERVERSION_XFA
+                    : JS_NUM_VIEWERVERSION;
+  return CJS_Result::Success(pRuntime->NewNumber(version));
 }
 
 CJS_Result CJS_App::set_viewer_version(CJS_Runtime* pRuntime,
@@ -164,15 +158,12 @@ CJS_Result CJS_App::set_viewer_version(CJS_Runtime* pRuntime,
 }
 
 CJS_Result CJS_App::get_platform(CJS_Runtime* pRuntime) {
-#ifdef PDF_ENABLE_XFA
   CPDFSDK_FormFillEnvironment* pFormFillEnv = pRuntime->GetFormFillEnv();
-  if (!pFormFillEnv)
-    return CJS_Result::Failure(JSMessage::kBadObjectError);
-
-  WideString platform = pFormFillEnv->GetPlatform();
-  if (!platform.IsEmpty())
-    return CJS_Result::Success(pRuntime->NewString(platform.AsStringView()));
-#endif
+  if (pFormFillEnv) {
+    WideString platform = pFormFillEnv->GetPlatform();
+    if (!platform.IsEmpty())
+      return CJS_Result::Success(pRuntime->NewString(platform.AsStringView()));
+  }
   return CJS_Result::Success(pRuntime->NewString(JS_STR_PLATFORM));
 }
 
@@ -182,15 +173,12 @@ CJS_Result CJS_App::set_platform(CJS_Runtime* pRuntime,
 }
 
 CJS_Result CJS_App::get_language(CJS_Runtime* pRuntime) {
-#ifdef PDF_ENABLE_XFA
   CPDFSDK_FormFillEnvironment* pFormFillEnv = pRuntime->GetFormFillEnv();
-  if (!pFormFillEnv)
-    return CJS_Result::Failure(JSMessage::kBadObjectError);
-
-  WideString language = pFormFillEnv->GetLanguage();
-  if (!language.IsEmpty())
-    return CJS_Result::Success(pRuntime->NewString(language.AsStringView()));
-#endif
+  if (pFormFillEnv) {
+    WideString language = pFormFillEnv->GetLanguage();
+    if (!language.IsEmpty())
+      return CJS_Result::Success(pRuntime->NewString(language.AsStringView()));
+  }
   return CJS_Result::Success(pRuntime->NewString(JS_STR_LANGUAGE));
 }
 
@@ -315,8 +303,7 @@ CJS_Result CJS_App::setInterval(
 
   uint32_t dwInterval = params.size() > 1 ? pRuntime->ToInt32(params[1]) : 1000;
   auto timerRef = pdfium::MakeUnique<GlobalTimer>(
-      this, pRuntime->GetFormFillEnv(), pRuntime, GlobalTimer::Type::kRepeating,
-      script, dwInterval, 0);
+      this, pRuntime, GlobalTimer::Type::kRepeating, script, dwInterval, 0);
   GlobalTimer* pTimerRef = timerRef.get();
   m_Timers.insert(std::move(timerRef));
 
@@ -343,9 +330,9 @@ CJS_Result CJS_App::setTimeOut(
     return CJS_Result::Failure(JSMessage::kInvalidInputError);
 
   uint32_t dwTimeOut = params.size() > 1 ? pRuntime->ToInt32(params[1]) : 1000;
-  auto timerRef = pdfium::MakeUnique<GlobalTimer>(
-      this, pRuntime->GetFormFillEnv(), pRuntime, GlobalTimer::Type::kOneShot,
-      script, dwTimeOut, dwTimeOut);
+  auto timerRef = pdfium::MakeUnique<GlobalTimer>(this, pRuntime,
+                                                  GlobalTimer::Type::kOneShot,
+                                                  script, dwTimeOut, dwTimeOut);
   GlobalTimer* pTimerRef = timerRef.get();
   m_Timers.insert(std::move(timerRef));
 
