@@ -20,8 +20,3 @@ void CJS_Display::DefineJSObjects(CFXJS_Engine* pEngine) {
       pEngine->DefineObj("display", FXJSOBJTYPE_STATIC, nullptr, nullptr);
   DefineConsts(pEngine, ObjDefnID, ConstSpecs);
 }
-
-CJS_Display::CJS_Display(v8::Local<v8::Object> pObject, CJS_Runtime* pRuntime)
-    : CJS_Object(pObject, pRuntime) {}
-
-CJS_Display::~CJS_Display() = default;
