@@ -7,14 +7,13 @@
 #ifndef FXJS_CJS_POSITION_H_
 #define FXJS_CJS_POSITION_H_
 
-#include "fxjs/js_define.h"
+#include "fxjs/cjs_object.h"
 
 class CJS_Position final : public CJS_Object {
  public:
   static void DefineJSObjects(CFXJS_Engine* pEngine);
 
-  CJS_Position(v8::Local<v8::Object> pObject, CJS_Runtime* pRuntime);
-  ~CJS_Position() override;
+  CJS_Position() = delete;
 
  private:
   static int ObjDefnID;
