@@ -26,7 +26,7 @@
 #include "fpdfsdk/cpdfsdk_interactiveform.h"
 #include "fxjs/cjs_color.h"
 #include "fxjs/cjs_event_context.h"
-#include "fxjs/cjs_eventhandler.h"
+#include "fxjs/cjs_eventrecorder.h"
 #include "fxjs/cjs_field.h"
 #include "fxjs/cjs_object.h"
 #include "fxjs/cjs_runtime.h"
@@ -121,7 +121,7 @@ ByteString CalculateString(double dValue,
 }
 #endif
 
-WideString CalcMergedString(const CJS_EventHandler* event,
+WideString CalcMergedString(const CJS_EventRecorder* event,
                             const WideString& value,
                             const WideString& change) {
   WideString prefix = value.Left(event->SelStart());
@@ -587,9 +587,9 @@ CJS_Result CJS_PublicMethods::AFNumber_Format(
   if (params.size() != 6)
     return CJS_Result::Failure(JSMessage::kParamError);
 
-  CJS_EventHandler* pEvent =
-      pRuntime->GetCurrentEventContext()->GetEventHandler();
-  if (!pEvent->m_pValue)
+  CJS_EventContext* pEventContext = pRuntime->GetCurrentEventContext();
+  CJS_EventRecorder* pEvent = pEventContext->GetEventRecorder();
+  if (!pEvent->HasValue())
     return CJS_Result::Failure(WideString::FromASCII("No event handler"));
 
   WideString& Value = pEvent->Value();
@@ -654,7 +654,7 @@ CJS_Result CJS_PublicMethods::AFNumber_Format(
       Value += L')';
     }
     if (iNegStyle == 1 || iNegStyle == 3) {
-      if (CJS_Field* fTarget = pEvent->Target_Field()) {
+      if (CJS_Field* fTarget = pEventContext->TargetField()) {
         v8::Local<v8::Array> arColor = pRuntime->NewArray();
         pRuntime->PutArrayElement(arColor, 0, pRuntime->NewString("RGB"));
         pRuntime->PutArrayElement(arColor, 1, pRuntime->NewNumber(1));
@@ -665,7 +665,7 @@ CJS_Result CJS_PublicMethods::AFNumber_Format(
     }
   } else {
     if (iNegStyle == 1 || iNegStyle == 3) {
-      if (CJS_Field* fTarget = pEvent->Target_Field()) {
+      if (CJS_Field* fTarget = pEventContext->TargetField()) {
         v8::Local<v8::Array> arColor = pRuntime->NewArray();
         pRuntime->PutArrayElement(arColor, 0, pRuntime->NewString("RGB"));
         pRuntime->PutArrayElement(arColor, 1, pRuntime->NewNumber(0));
@@ -695,8 +695,8 @@ CJS_Result CJS_PublicMethods::AFNumber_Keystroke(
     return CJS_Result::Failure(JSMessage::kParamError);
 
   CJS_EventContext* pContext = pRuntime->GetCurrentEventContext();
-  CJS_EventHandler* pEvent = pContext->GetEventHandler();
-  if (!pEvent->m_pValue)
+  CJS_EventRecorder* pEvent = pContext->GetEventRecorder();
+  if (!pEvent->HasValue())
     return CJS_Result::Failure(JSMessage::kBadObjectError);
 
   WideString& val = pEvent->Value();
@@ -783,9 +783,9 @@ CJS_Result CJS_PublicMethods::AFPercent_Format(
   if (params.size() < 2)
     return CJS_Result::Failure(JSMessage::kParamError);
 
-  CJS_EventHandler* pEvent =
-      pRuntime->GetCurrentEventContext()->GetEventHandler();
-  if (!pEvent->m_pValue)
+  CJS_EventRecorder* pEvent =
+      pRuntime->GetCurrentEventContext()->GetEventRecorder();
+  if (!pEvent->HasValue())
     return CJS_Result::Failure(JSMessage::kBadObjectError);
 
   // Acrobat will accept this. Anything larger causes it to throw an error.
@@ -880,8 +880,8 @@ CJS_Result CJS_PublicMethods::AFDate_FormatEx(
     return CJS_Result::Failure(JSMessage::kParamError);
 
   CJS_EventContext* pContext = pRuntime->GetCurrentEventContext();
-  CJS_EventHandler* pEvent = pContext->GetEventHandler();
-  if (!pEvent->m_pValue)
+  CJS_EventRecorder* pEvent = pContext->GetEventRecorder();
+  if (!pEvent->HasValue())
     return CJS_Result::Failure(JSMessage::kBadObjectError);
 
   WideString& val = pEvent->Value();
@@ -955,11 +955,11 @@ CJS_Result CJS_PublicMethods::AFDate_KeystrokeEx(
   }
 
   CJS_EventContext* pContext = pRuntime->GetCurrentEventContext();
-  CJS_EventHandler* pEvent = pContext->GetEventHandler();
+  CJS_EventRecorder* pEvent = pContext->GetEventRecorder();
   if (!pEvent->WillCommit())
     return CJS_Result::Success();
 
-  if (!pEvent->m_pValue)
+  if (!pEvent->HasValue())
     return CJS_Result::Failure(JSMessage::kBadObjectError);
 
   const WideString& strValue = pEvent->Value();
@@ -1051,9 +1051,9 @@ CJS_Result CJS_PublicMethods::AFSpecial_Format(
   if (params.size() != 1)
     return CJS_Result::Failure(JSMessage::kParamError);
 
-  CJS_EventHandler* pEvent =
-      pRuntime->GetCurrentEventContext()->GetEventHandler();
-  if (!pEvent->m_pValue)
+  CJS_EventRecorder* pEvent =
+      pRuntime->GetCurrentEventContext()->GetEventRecorder();
+  if (!pEvent->HasValue())
     return CJS_Result::Failure(JSMessage::kBadObjectError);
 
   const WideString& wsSource = pEvent->Value();
@@ -1088,8 +1088,8 @@ CJS_Result CJS_PublicMethods::AFSpecial_KeystrokeEx(
     return CJS_Result::Failure(JSMessage::kParamError);
 
   CJS_EventContext* pContext = pRuntime->GetCurrentEventContext();
-  CJS_EventHandler* pEvent = pContext->GetEventHandler();
-  if (!pEvent->m_pValue)
+  CJS_EventRecorder* pEvent = pContext->GetEventRecorder();
+  if (!pEvent->HasValue())
     return CJS_Result::Failure(JSMessage::kBadObjectError);
 
   const WideString& valEvent = pEvent->Value();
@@ -1169,9 +1169,9 @@ CJS_Result CJS_PublicMethods::AFSpecial_Keystroke(
   if (params.size() != 1)
     return CJS_Result::Failure(JSMessage::kParamError);
 
-  CJS_EventHandler* pEvent =
-      pRuntime->GetCurrentEventContext()->GetEventHandler();
-  if (!pEvent->m_pValue)
+  CJS_EventRecorder* pEvent =
+      pRuntime->GetCurrentEventContext()->GetEventRecorder();
+  if (!pEvent->HasValue())
     return CJS_Result::Failure(JSMessage::kBadObjectError);
 
   const char* cFormat = "";
@@ -1204,18 +1204,18 @@ CJS_Result CJS_PublicMethods::AFMergeChange(
   if (params.size() != 1)
     return CJS_Result::Failure(JSMessage::kParamError);
 
-  CJS_EventHandler* pEventHandler =
-      pRuntime->GetCurrentEventContext()->GetEventHandler();
+  CJS_EventRecorder* pEventRecorder =
+      pRuntime->GetCurrentEventContext()->GetEventRecorder();
 
   WideString swValue;
-  if (pEventHandler->m_pValue)
-    swValue = pEventHandler->Value();
+  if (pEventRecorder->HasValue())
+    swValue = pEventRecorder->Value();
 
-  if (pEventHandler->WillCommit())
+  if (pEventRecorder->WillCommit())
     return CJS_Result::Success(pRuntime->NewString(swValue.AsStringView()));
 
   return CJS_Result::Success(pRuntime->NewString(
-      CalcMergedString(pEventHandler, swValue, pEventHandler->Change())
+      CalcMergedString(pEventRecorder, swValue, pEventRecorder->Change())
           .AsStringView()));
 }
 
@@ -1312,7 +1312,7 @@ CJS_Result CJS_PublicMethods::AFSimple_Calculate(
           WideString trimmed = pFormField->GetValue();
           trimmed.TrimRight();
           trimmed.TrimLeft();
-          dTemp = StringToFloat(trimmed.AsStringView());
+          dTemp = StringToDouble(trimmed.AsStringView());
           break;
         }
         case FormFieldType::kPushButton:
@@ -1364,8 +1364,8 @@ CJS_Result CJS_PublicMethods::AFSimple_Calculate(
   dValue = floor(dValue * FXSYS_pow(10, 6) + 0.49) / FXSYS_pow(10, 6);
 
   CJS_EventContext* pContext = pRuntime->GetCurrentEventContext();
-  if (pContext->GetEventHandler()->m_pValue) {
-    pContext->GetEventHandler()->Value() =
+  if (pContext->GetEventRecorder()->HasValue()) {
+    pContext->GetEventRecorder()->Value() =
         pRuntime->ToWideString(pRuntime->NewNumber(dValue));
   }
 
@@ -1381,8 +1381,8 @@ CJS_Result CJS_PublicMethods::AFRange_Validate(
     return CJS_Result::Failure(JSMessage::kParamError);
 
   CJS_EventContext* pContext = pRuntime->GetCurrentEventContext();
-  CJS_EventHandler* pEvent = pContext->GetEventHandler();
-  if (!pEvent->m_pValue)
+  CJS_EventRecorder* pEvent = pContext->GetEventRecorder();
+  if (!pEvent->HasValue())
     return CJS_Result::Failure(JSMessage::kBadObjectError);
 
   if (pEvent->Value().IsEmpty())
