@@ -7,20 +7,27 @@
 #ifndef FXJS_XFA_CJX_BOOLEAN_H_
 #define FXJS_XFA_CJX_BOOLEAN_H_
 
-#include "fxjs/jse_define.h"
-#include "fxjs/xfa/cjx_content.h"
+#include "fxjs/xfa/cjx_object.h"
+#include "fxjs/xfa/jse_define.h"
 
 class CXFA_Boolean;
 
-class CJX_Boolean final : public CJX_Content {
+class CJX_Boolean final : public CJX_Object {
  public:
   explicit CJX_Boolean(CXFA_Boolean* node);
   ~CJX_Boolean() override;
 
-  JSE_PROP(use);
+  // CJX_Object:
+  bool DynamicTypeIs(TypeTag eType) const override;
+
   JSE_PROP(defaultValue); /* {default} */
-  JSE_PROP(usehref);
   JSE_PROP(value);
+
+ private:
+  using Type__ = CJX_Boolean;
+  using ParentType__ = CJX_Object;
+
+  static const TypeTag static_type__ = TypeTag::Boolean;
 };
 
 #endif  // FXJS_XFA_CJX_BOOLEAN_H_
