@@ -7,8 +7,8 @@
 #ifndef FXJS_XFA_CJX_DELTA_H_
 #define FXJS_XFA_CJX_DELTA_H_
 
-#include "fxjs/jse_define.h"
 #include "fxjs/xfa/cjx_object.h"
+#include "fxjs/xfa/jse_define.h"
 
 class CXFA_Delta;
 
@@ -17,13 +17,20 @@ class CJX_Delta final : public CJX_Object {
   explicit CJX_Delta(CXFA_Delta* delta);
   ~CJX_Delta() override;
 
-  JSE_METHOD(restore, CJX_Delta);
+  // CJX_Object:
+  bool DynamicTypeIs(TypeTag eType) const override;
+
+  JSE_METHOD(restore);
 
   JSE_PROP(currentValue);
   JSE_PROP(savedValue);
   JSE_PROP(target);
 
  private:
+  using Type__ = CJX_Delta;
+  using ParentType__ = CJX_Object;
+
+  static const TypeTag static_type__ = TypeTag::Delta;
   static const CJX_MethodSpec MethodSpecs[];
 };
 
