@@ -7,8 +7,8 @@
 #ifndef FXJS_XFA_CJX_ENCRYPT_H_
 #define FXJS_XFA_CJX_ENCRYPT_H_
 
-#include "fxjs/jse_define.h"
 #include "fxjs/xfa/cjx_node.h"
+#include "fxjs/xfa/jse_define.h"
 
 class CXFA_Encrypt;
 
@@ -17,9 +17,16 @@ class CJX_Encrypt final : public CJX_Node {
   explicit CJX_Encrypt(CXFA_Encrypt* node);
   ~CJX_Encrypt() override;
 
+  // CJX_Object:
+  bool DynamicTypeIs(TypeTag eType) const override;
+
   JSE_PROP(format);
-  JSE_PROP(use);
-  JSE_PROP(usehref);
+
+ private:
+  using Type__ = CJX_Encrypt;
+  using ParentType__ = CJX_Node;
+
+  static const TypeTag static_type__ = TypeTag::Encrypt;
 };
 
 #endif  // FXJS_XFA_CJX_ENCRYPT_H_
