@@ -7,22 +7,29 @@
 #ifndef FXJS_XFA_CJX_SIGNATUREPSEUDOMODEL_H_
 #define FXJS_XFA_CJX_SIGNATUREPSEUDOMODEL_H_
 
-#include "fxjs/CJX_Define.h"
 #include "fxjs/xfa/cjx_object.h"
+#include "fxjs/xfa/jse_define.h"
 
 class CScript_SignaturePseudoModel;
 
-class CJX_SignaturePseudoModel : public CJX_Object {
+class CJX_SignaturePseudoModel final : public CJX_Object {
  public:
   explicit CJX_SignaturePseudoModel(CScript_SignaturePseudoModel* model);
   ~CJX_SignaturePseudoModel() override;
 
-  JS_METHOD(verifySignature /*verify*/, CJX_SignaturePseudoModel);
-  JS_METHOD(sign, CJX_SignaturePseudoModel);
-  JS_METHOD(enumerate, CJX_SignaturePseudoModel);
-  JS_METHOD(clear, CJX_SignaturePseudoModel);
+  // CJX_Object:
+  bool DynamicTypeIs(TypeTag eType) const override;
+
+  JSE_METHOD(verifySignature /*verify*/);
+  JSE_METHOD(sign);
+  JSE_METHOD(enumerate);
+  JSE_METHOD(clear);
 
  private:
+  using Type__ = CJX_SignaturePseudoModel;
+  using ParentType__ = CJX_Object;
+
+  static const TypeTag static_type__ = TypeTag::SignaturePesudoModel;
   static const CJX_MethodSpec MethodSpecs[];
 };
 
