@@ -7,8 +7,8 @@
 #ifndef FXJS_XFA_CJX_TEXTNODE_H_
 #define FXJS_XFA_CJX_TEXTNODE_H_
 
-#include "fxjs/jse_define.h"
 #include "fxjs/xfa/cjx_node.h"
+#include "fxjs/xfa/jse_define.h"
 
 class CXFA_Node;
 
@@ -17,8 +17,17 @@ class CJX_TextNode : public CJX_Node {
   explicit CJX_TextNode(CXFA_Node* node);
   ~CJX_TextNode() override;
 
+  // CJX_Object:
+  bool DynamicTypeIs(TypeTag eType) const override;
+
   JSE_PROP(defaultValue); /* {default} */
   JSE_PROP(value);
+
+ private:
+  using Type__ = CJX_TextNode;
+  using ParentType__ = CJX_Node;
+
+  static const TypeTag static_type__ = TypeTag::TextNode;
 };
 
 #endif  // FXJS_XFA_CJX_TEXTNODE_H_
