@@ -15,7 +15,7 @@
 //
 // Copyright (C) 2005 Martin Kretzschmar <martink@gnome.org>
 // Copyright (C) 2005, 2006 Kristian Høgsberg <krh@redhat.com>
-// Copyright (C) 2005, 2007-2010, 2012, 2015, 2017, 2018 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005, 2007-2010, 2012, 2015, 2017-2019 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2005 Jonathan Blandford <jrb@redhat.com>
 // Copyright (C) 2006, 2007 Jeff Muizelaar <jeff@infidigm.net>
 // Copyright (C) 2006 Takashi Iwai <tiwai@suse.de>
@@ -40,6 +40,7 @@
 // Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2019 Christian Persch <chpe@src.gnome.org>
+// Copyright (C) 2019 Oliver Sander <oliver.sander@tu-dresden.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -58,7 +59,6 @@
 #include "goo/glibc.h"
 #include "goo/gmem.h"
 #include "goo/GooString.h"
-#include "goo/GooList.h"
 #include "goo/gfile.h"
 #include "goo/gdir.h"
 #include "Error.h"
@@ -246,37 +246,41 @@ private:
 			       const char *path);
 #endif
 
-  GooList *fonts;			// [SysFontInfo]
+  std::vector<SysFontInfo*> *fonts;
 };
 
 SysFontList::SysFontList() {
-  fonts = new GooList();
+  fonts = new std::vector<SysFontInfo*>();
 }
 
 SysFontList::~SysFontList() {
-  deleteGooList<SysFontInfo>(fonts);
+  for (auto entry : *fonts) {
+    delete entry;
+  }
+  delete fonts;
 }
 
 SysFontInfo *SysFontList::find(const GooString *name, bool fixedWidth, bool exact) {
   GooString *name2;
   bool bold, italic, oblique;
   SysFontInfo *fi;
-  char c;
-  int n, i;
+  int n;
 
   name2 = name->copy();
 
   // remove space, comma, dash chars
-  i = 0;
-  while (i < name2->getLength()) {
-    c = name2->getChar(i);
-    if (c == ' ' || c == ',' || c == '-') {
-      name2->del(i);
-    } else {
-      ++i;
+  {
+    int i = 0;
+    while (i < name2->getLength()) {
+        const char c = name2->getChar(i);
+        if (c == ' ' || c == ',' || c == '-') {
+        name2->del(i);
+        } else {
+        ++i;
+        }
     }
+    n = name2->getLength();
   }
-  n = name2->getLength();
 
   // remove trailing "MT" (Foo-MT, Foo-BoldMT, etc.)
   if (n > 2 && !strcmp(name2->c_str() + n - 2, "MT")) {
@@ -337,8 +341,8 @@ SysFontInfo *SysFontList::find(const GooString *name, bool fixedWidth, bool exac
 
   // search for the font
   fi = nullptr;
-  for (i = 0; i < fonts->getLength(); ++i) {
-    fi = (SysFontInfo *)fonts->get(i);
+  for (std::size_t i = 0; i < fonts->size(); ++i) {
+    fi = (*fonts)[i];
     if (fi->match(name2, bold, italic, oblique, fixedWidth)) {
       break;
     }
@@ -346,8 +350,8 @@ SysFontInfo *SysFontList::find(const GooString *name, bool fixedWidth, bool exac
   }
   if (!fi && !exact && bold) {
     // try ignoring the bold flag
-    for (i = 0; i < fonts->getLength(); ++i) {
-      fi = (SysFontInfo *)fonts->get(i);
+    for (std::size_t i = 0; i < fonts->size(); ++i) {
+      fi = (*fonts)[i];
       if (fi->match(name2, false, italic)) {
 	break;
       }
@@ -356,8 +360,8 @@ SysFontInfo *SysFontList::find(const GooString *name, bool fixedWidth, bool exac
   }
   if (!fi && !exact && (bold || italic)) {
     // try ignoring the bold and italic flags
-    for (i = 0; i < fonts->getLength(); ++i) {
-      fi = (SysFontInfo *)fonts->get(i);
+    for (std::size_t i = 0; i < fonts->size(); ++i) {
+      fi = (*fonts)[i];
       if (fi->match(name2, false, false)) {
 	break;
       }
@@ -394,7 +398,7 @@ GlobalParams::GlobalParams(const char *customPopplerDataDir)
 
   nameToUnicodeZapfDingbats = new NameToCharCode();
   nameToUnicodeText = new NameToCharCode();
-  toUnicodeDirs = new GooList();
+  toUnicodeDirs = new std::vector<GooString*>();
   sysFonts = new SysFontList();
   psExpandSmaller = false;
   psShrinkLarger = true;
@@ -402,8 +406,6 @@ GlobalParams::GlobalParams(const char *customPopplerDataDir)
   textEncoding = new GooString("UTF-8");
 #if defined(_WIN32)
   textEOL = eolDOS;
-#elif defined(MACOS)
-  textEOL = eolMac;
 #else
   textEOL = eolUnix;
 #endif
@@ -555,7 +557,10 @@ GlobalParams::~GlobalParams() {
 
   delete nameToUnicodeZapfDingbats;
   delete nameToUnicodeText;
-  deleteGooList<GooString>(toUnicodeDirs);
+  for (auto entry : *toUnicodeDirs) {
+    delete entry;
+  }
+  delete toUnicodeDirs;
   delete sysFonts;
   delete textEncoding;
 
@@ -616,8 +621,8 @@ FILE *GlobalParams::findCMapFile(const GooString *collection, const GooString *c
   FILE *file = nullptr;
 
   globalParamsLocker();
-  const auto cMapDirs = this->cMapDirs.equal_range(collection->toStr());
-  for (auto cMapDir = cMapDirs.first; cMapDir != cMapDirs.second; ++cMapDir) {
+  const auto collectionCMapDirs = cMapDirs.equal_range(collection->toStr());
+  for (auto cMapDir = collectionCMapDirs.first; cMapDir != collectionCMapDirs.second; ++cMapDir) {
     auto* const path = new GooString(cMapDir->second);
     appendToPath(path, cMapName->c_str());
     file = openFile(path->c_str(), "r");
@@ -633,11 +638,10 @@ FILE *GlobalParams::findCMapFile(const GooString *collection, const GooString *c
 FILE *GlobalParams::findToUnicodeFile(const GooString *name) {
   GooString *dir, *fileName;
   FILE *f;
-  int i;
 
   globalParamsLocker();
-  for (i = 0; i < toUnicodeDirs->getLength(); ++i) {
-    dir = (GooString *)toUnicodeDirs->get(i);
+  for (std::size_t i = 0; i < toUnicodeDirs->size(); ++i) {
+    dir = (*toUnicodeDirs)[i];
     fileName = appendToPath(dir->copy(), name->c_str());
     f = openFile(fileName->c_str(), "r");
     delete fileName;
@@ -1206,9 +1210,9 @@ UnicodeMap *GlobalParams::getTextEncoding() {
   return getUnicodeMap2(textEncoding);
 }
 
-GooList *GlobalParams::getEncodingNames()
+std::vector<GooString*> *GlobalParams::getEncodingNames()
 {
-  auto* const result = new GooList;
+  auto* const result = new std::vector<GooString*>;
   for (const auto& unicodeMap : residentUnicodeMaps) {
     result->push_back(new GooString(unicodeMap.first));
   }
