@@ -473,7 +473,7 @@ bool PDFDoc::checkEncryption(const GooString *ownerPassword, const GooString *us
 static PDFSubtypePart pdfPartFromString(PDFSubtype subtype, GooString *pdfSubtypeVersion) {
   const std::regex regex("PDF/(?:A|X|VT|E|UA)-([[:digit:]])(?:[[:alpha:]]{1,2})?:?([[:digit:]]{4})?");
   std::smatch match;
-  std::string pdfsubver = pdfSubtypeVersion->toStr();
+  const std::string &pdfsubver = pdfSubtypeVersion->toStr();
   PDFSubtypePart subtypePart = subtypePartNone;
 
   if (std::regex_search(pdfsubver, match, regex)) {
@@ -533,7 +533,7 @@ static PDFSubtypePart pdfPartFromString(PDFSubtype subtype, GooString *pdfSubtyp
 static PDFSubtypeConformance pdfConformanceFromString(GooString *pdfSubtypeVersion) {
   const std::regex regex("PDF/(?:A|X|VT|E|UA)-[[:digit:]]([[:alpha:]]+)");
   std::smatch match;
-  const std::string pdfsubver = pdfSubtypeVersion->toStr();
+  const std::string &pdfsubver = pdfSubtypeVersion->toStr();
   PDFSubtypeConformance pdfConf = subtypeConfNone;
 
   // match contains the PDF conformance (A, B, G, N, P, PG or U)
@@ -722,7 +722,7 @@ bool PDFDoc::checkLinearization() {
     }
 
     pageRef.gen = xref->getEntry(pageRef.num)->gen;
-    Object obj = xref->fetch(pageRef.num, pageRef.gen);
+    Object obj = xref->fetch(pageRef);
     if (!obj.isDict("Page")) {
       linearizationState = 2;
       return false;
@@ -862,7 +862,7 @@ Hints *PDFDoc::getHints()
   return hints;
 }
 
-int PDFDoc::savePageAs(GooString *name, int pageNo) 
+int PDFDoc::savePageAs(const GooString *name, int pageNo)
 {
   FILE *f;
   OutStream *outStr;
@@ -896,7 +896,7 @@ int PDFDoc::savePageAs(GooString *name, int pageNo)
     getCatalog()->getPage(pageNo)->getMediaBox(),
     cropBox);
   Ref *refPage = getCatalog()->getPageRef(pageNo);
-  Object page = getXRef()->fetch(refPage->num, refPage->gen);
+  Object page = getXRef()->fetch(*refPage);
 
   if (!(f = openFile(name->c_str(), "wb"))) {
     error(errIO, -1, "Couldn't open file '{0:t}'", name);
@@ -925,9 +925,9 @@ int PDFDoc::savePageAs(GooString *name, int pageNo)
     markPageObjects(infoDict, yRef, countRef, 0, refPage->num, rootNum + 2);
     if (trailerObj->isDict()) {
       Dict *trailerDict = trailerObj->getDict();
-      Object ref = trailerDict->lookupNF("Info");
+      const Object &ref = trailerDict->lookupNF("Info");
       if (ref.isRef()) {
-        yRef->add(ref.getRef().num, ref.getRef().gen, 0, true);
+        yRef->add(ref.getRef(), 0, true);
         if (getXRef()->getEntry(ref.getRef().num)->type == xrefEntryCompressed) {
           yRef->getEntry(ref.getRef().num)->type = xrefEntryCompressed;
         }
@@ -939,7 +939,7 @@ int PDFDoc::savePageAs(GooString *name, int pageNo)
   Object catObj = getXRef()->getCatalog();
   Dict *catDict = catObj.getDict();
   Object pagesObj = catDict->lookup("Pages");
-  Object afObj = catDict->lookupNF("AcroForm");
+  Object afObj = catDict->lookupNF("AcroForm").copy();
   if (!afObj.isNull()) {
     markAcroForm(&afObj, yRef, countRef, 0, refPage->num, rootNum + 2);
   }
@@ -958,7 +958,7 @@ int PDFDoc::savePageAs(GooString *name, int pageNo)
     }
   }
   markPageObjects(pageDict, yRef, countRef, 0, refPage->num, rootNum + 2);
-  Object annotsObj = pageDict->lookupNF("Annots");
+  Object annotsObj = pageDict->lookupNF("Annots").copy();
   if (!annotsObj.isNull()) {
     markAnnotations(&annotsObj, yRef, countRef, 0, refPage->num, rootNum + 2);
   }
@@ -975,7 +975,7 @@ int PDFDoc::savePageAs(GooString *name, int pageNo)
       strcmp(key, "Pages") != 0) 
     {
       if (j > 0) outStr->printf(" ");
-      Object value = catDict->getValNF(j);
+      Object value = catDict->getValNF(j).copy();
       outStr->printf("/%s ", key);
       writeObject(&value, outStr, getXRef(), 0, nullptr, cryptRC4, 0, 0, 0);
     }
@@ -998,7 +998,7 @@ int PDFDoc::savePageAs(GooString *name, int pageNo)
   for (int n = 0; n < pageDict->getLength(); n++) {
     if (n > 0) outStr->printf(" ");
     const char *key = pageDict->getKey(n);
-    Object value = pageDict->getValNF(n);
+    Object value = pageDict->getValNF(n).copy();
     if (strcmp(key, "Parent") == 0) {
       outStr->printf("/Parent %d 0 R", rootNum + 1);
     } else {
@@ -1026,7 +1026,7 @@ int PDFDoc::savePageAs(GooString *name, int pageNo)
   return errNone;
 }
 
-int PDFDoc::saveAs(GooString *name, PDFWriteMode mode) {
+int PDFDoc::saveAs(const GooString *name, PDFWriteMode mode) {
   FILE *f;
   OutStream *outStr;
   int res;
@@ -1058,7 +1058,7 @@ int PDFDoc::saveAs(OutStream *outStr, PDFWriteMode mode) {
   return errNone;
 }
 
-int PDFDoc::saveWithoutChangesAs(GooString *name) {
+int PDFDoc::saveWithoutChangesAs(const GooString *name) {
   FILE *f;
   OutStream *outStr;
   int res;
@@ -1125,13 +1125,13 @@ void PDFDoc::saveIncrementalUpdate (OutStream* outStr)
       ref.num = i;
       ref.gen = xref->getEntry(i)->type == xrefEntryCompressed ? 0 : xref->getEntry(i)->gen;
       if (xref->getEntry(i)->type != xrefEntryFree) {
-        Object obj1 = xref->fetch(ref.num, ref.gen, 1);
+        Object obj1 = xref->fetch(ref, 1 /* recursion */);
         Goffset offset = writeObjectHeader(&ref, outStr);
-        writeObject(&obj1, outStr, fileKey, encAlgorithm, keyLength, ref.num, ref.gen);
+        writeObject(&obj1, outStr, fileKey, encAlgorithm, keyLength, ref);
         writeObjectFooter(outStr);
-        uxref->add(ref.num, ref.gen, offset, true);
+        uxref->add(ref, offset, true);
       } else {
-        uxref->add(ref.num, ref.gen, 0, false);
+        uxref->add(ref, 0, false);
       }
     }
   }
@@ -1157,7 +1157,7 @@ void PDFDoc::saveIncrementalUpdate (OutStream* outStr)
     // Append an entry for the xref stream itself
     uxrefStreamRef.num = numobjects++;
     uxrefStreamRef.gen = 0;
-    uxref->add(uxrefStreamRef.num, uxrefStreamRef.gen, uxrefOffset, true);
+    uxref->add(uxrefStreamRef, uxrefOffset, true);
   }
 
   Object trailerDict = createTrailerDict(numobjects, true, getStartXRef(), &rootRef, getXRef(), fileNameA, uxrefOffset);
@@ -1194,33 +1194,33 @@ void PDFDoc::saveCompleteRewrite (OutStream* outStr)
       /* the XRef class adds a lot of irrelevant free entries, we only want the significant one
           and we don't want the one with num=0 because it has already been added (gen = 65535)*/
       if (ref.gen > 0 && ref.num > 0)
-        uxref->add(ref.num, ref.gen, 0, false);
+        uxref->add(ref, 0, false);
     } else if (xref->getEntry(i)->getFlag(XRefEntry::DontRewrite)) {
       // This entry must not be written, put a free entry instead (with incremented gen)
       ref.num = i;
       ref.gen = xref->getEntry(i)->gen + 1;
-      uxref->add(ref.num, ref.gen, 0, false);
+      uxref->add(ref, 0, false);
     } else if (type == xrefEntryUncompressed){ 
       ref.num = i;
       ref.gen = xref->getEntry(i)->gen;
-      Object obj1 = xref->fetch(ref.num, ref.gen, 1);
+      Object obj1 = xref->fetch(ref, 1 /* recursion */);
       Goffset offset = writeObjectHeader(&ref, outStr);
       // Write unencrypted objects in unencrypted form
       if (xref->getEntry(i)->getFlag(XRefEntry::Unencrypted)) {
         writeObject(&obj1, outStr, nullptr, cryptRC4, 0, 0, 0);
       } else {
-        writeObject(&obj1, outStr, fileKey, encAlgorithm, keyLength, ref.num, ref.gen);
+        writeObject(&obj1, outStr, fileKey, encAlgorithm, keyLength, ref);
       }
       writeObjectFooter(outStr);
-      uxref->add(ref.num, ref.gen, offset, true);
+      uxref->add(ref, offset, true);
     } else if (type == xrefEntryCompressed) {
       ref.num = i;
       ref.gen = 0; //compressed entries have gen == 0
-      Object obj1 = xref->fetch(ref.num, ref.gen, 1);
+      Object obj1 = xref->fetch(ref, 1 /* recursion */);
       Goffset offset = writeObjectHeader(&ref, outStr);
-      writeObject(&obj1, outStr, fileKey, encAlgorithm, keyLength, ref.num, ref.gen);
+      writeObject(&obj1, outStr, fileKey, encAlgorithm, keyLength, ref);
       writeObjectFooter(outStr);
-      uxref->add(ref.num, ref.gen, offset, true);
+      uxref->add(ref, offset, true);
     }
   }
   xref->unlock();
@@ -1231,7 +1231,7 @@ void PDFDoc::saveCompleteRewrite (OutStream* outStr)
 }
 
 void PDFDoc::writeDictionnary (Dict* dict, OutStream* outStr, XRef *xRef, unsigned int numOffset, unsigned char *fileKey,
-                               CryptAlgorithm encAlgorithm, int keyLength, int objNum, int objGen, std::set<Dict*> *alreadyWrittenDicts)
+                               CryptAlgorithm encAlgorithm, int keyLength, Ref ref, std::set<Dict*> *alreadyWrittenDicts)
 {
   bool deleteSet = false;
   if (!alreadyWrittenDicts) {
@@ -1253,8 +1253,8 @@ void PDFDoc::writeDictionnary (Dict* dict, OutStream* outStr, XRef *xRef, unsign
     GooString *keyNameToPrint = keyName.sanitizedName(false /* non ps mode */);
     outStr->printf("/%s ", keyNameToPrint->c_str());
     delete keyNameToPrint;
-    Object obj1 = dict->getValNF(i);
-    writeObject(&obj1, outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, objNum, objGen, alreadyWrittenDicts);
+    Object obj1 = dict->getValNF(i).copy();
+    writeObject(&obj1, outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, ref, alreadyWrittenDicts);
   }
   outStr->printf(">> ");
 
@@ -1302,13 +1302,13 @@ void PDFDoc::writeRawStream (Stream* str, OutStream* outStr)
 }
 
 void PDFDoc::writeString (const GooString* s, OutStream* outStr, const unsigned char *fileKey,
-                          CryptAlgorithm encAlgorithm, int keyLength, int objNum, int objGen)
+                          CryptAlgorithm encAlgorithm, int keyLength, Ref ref)
 {
   // Encrypt string if encryption is enabled
   GooString *sEnc = nullptr;
   if (fileKey) {
     EncryptStream *enc = new EncryptStream(new MemStream(s->c_str(), 0, s->getLength(), Object(objNull)),
-                                           fileKey, encAlgorithm, keyLength, objNum, objGen);
+                                           fileKey, encAlgorithm, keyLength, ref);
     sEnc = new GooString();
     int c;
     enc->reset();
@@ -1365,6 +1365,12 @@ Goffset PDFDoc::writeObjectHeader (Ref *ref, OutStream* outStr)
 
 void PDFDoc::writeObject (Object* obj, OutStream* outStr, XRef *xRef, unsigned int numOffset, unsigned char *fileKey,
                           CryptAlgorithm encAlgorithm, int keyLength, int objNum, int objGen, std::set<Dict*> *alreadyWrittenDicts)
+{
+  writeObject(obj, outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, {objNum, objGen}, alreadyWrittenDicts);
+}
+
+void PDFDoc::writeObject (Object* obj, OutStream* outStr, XRef *xRef, unsigned int numOffset, unsigned char *fileKey,
+                          CryptAlgorithm encAlgorithm, int keyLength, Ref ref, std::set<Dict*> *alreadyWrittenDicts)
 {
   Array *array;
 
@@ -1386,7 +1392,7 @@ void PDFDoc::writeObject (Object* obj, OutStream* outStr, XRef *xRef, unsigned i
       break;
     }
     case objString:
-      writeString(obj->getString(), outStr, fileKey, encAlgorithm, keyLength, objNum, objGen);
+      writeString(obj->getString(), outStr, fileKey, encAlgorithm, keyLength, ref);
       break;
     case objName:
     {
@@ -1403,13 +1409,13 @@ void PDFDoc::writeObject (Object* obj, OutStream* outStr, XRef *xRef, unsigned i
       array = obj->getArray();
       outStr->printf("[");
       for (int i=0; i<array->getLength(); i++) {
-	Object obj1 = array->getNF(i);
-        writeObject(&obj1, outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, objNum, objGen);
+	Object obj1 = array->getNF(i).copy();
+        writeObject(&obj1, outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, ref);
       }
       outStr->printf("] ");
       break;
     case objDict:
-      writeDictionnary (obj->getDict(), outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, objNum, objGen, alreadyWrittenDicts);
+      writeDictionnary (obj->getDict(), outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, ref, alreadyWrittenDicts);
       break;
     case objStream: 
       {
@@ -1434,12 +1440,12 @@ void PDFDoc::writeObject (Object* obj, OutStream* outStr, XRef *xRef, unsigned i
                   }
                 }
                 if (removeFilter) {
-                  encStream = new EncryptStream(stream, fileKey, encAlgorithm, keyLength, objNum, objGen);
+                  encStream = new EncryptStream(stream, fileKey, encAlgorithm, keyLength, ref);
                   encStream->setAutoDelete(false);
                   stream = encStream;
                 }
               } else {
-                encStream = new EncryptStream(stream, fileKey, encAlgorithm, keyLength, objNum, objGen);
+                encStream = new EncryptStream(stream, fileKey, encAlgorithm, keyLength, ref);
                 encStream->setAutoDelete(false);
                 stream = encStream;
               }
@@ -1447,7 +1453,7 @@ void PDFDoc::writeObject (Object* obj, OutStream* outStr, XRef *xRef, unsigned i
               removeFilter = false;
             }
           } else if (fileKey != nullptr) { // Encrypt stream
-            encStream = new EncryptStream(stream, fileKey, encAlgorithm, keyLength, objNum, objGen);
+            encStream = new EncryptStream(stream, fileKey, encAlgorithm, keyLength, ref);
             encStream->setAutoDelete(false);
             stream = encStream;
           }
@@ -1466,9 +1472,15 @@ void PDFDoc::writeObject (Object* obj, OutStream* outStr, XRef *xRef, unsigned i
           }
           stream->getDict()->remove("DecodeParms");
 
-          writeDictionnary (stream->getDict(),outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, objNum, objGen, alreadyWrittenDicts);
+          writeDictionnary (stream->getDict(),outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, ref, alreadyWrittenDicts);
           writeStream (stream,outStr);
           delete encStream;
+        } else if (fileKey != nullptr && stream->getKind() == strFile && static_cast<FileStream*>(stream)->getNeedsEncryptionOnSave()) {
+          EncryptStream *encStream = new EncryptStream(stream, fileKey, encAlgorithm, keyLength, ref);
+          encStream->setAutoDelete(false);
+          writeDictionnary (encStream->getDict(), outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, ref, alreadyWrittenDicts);
+          writeStream (encStream, outStr);
+          delete encStream;
         } else {
           //raw stream copy
           FilterStream *fs = dynamic_cast<FilterStream*>(stream);
@@ -1482,7 +1494,7 @@ void PDFDoc::writeObject (Object* obj, OutStream* outStr, XRef *xRef, unsigned i
                 }
               }
           }
-          writeDictionnary (stream->getDict(), outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, objNum, objGen, alreadyWrittenDicts);
+          writeDictionnary (stream->getDict(), outStr, xRef, numOffset, fileKey, encAlgorithm, keyLength, ref, alreadyWrittenDicts);
           writeRawStream (stream, outStr);
         }
         break;
@@ -1550,7 +1562,7 @@ Object PDFDoc::createTrailerDict(int uxrefSize, bool incrUpdate, Goffset startxR
 
   bool hasEncrypt = false;
   if (!xRef->getTrailerDict()->isNone()) {
-    Object obj2 = xRef->getTrailerDict()->dictLookupNF("Encrypt");
+    Object obj2 = xRef->getTrailerDict()->dictLookupNF("Encrypt").copy();
     if (!obj2.isNull()) {
       trailerDict->set("Encrypt", std::move(obj2));
       hasEncrypt = true;
@@ -1583,7 +1595,7 @@ Object PDFDoc::createTrailerDict(int uxrefSize, bool incrUpdate, Goffset startxR
     trailerDict->set("ID", Object(array));
   }
 
-  trailerDict->set("Root", Object(root->num, root->gen));
+  trailerDict->set("Root", Object(*root));
 
   if (incrUpdate) { 
     trailerDict->set("Prev", Object(startxRef));
@@ -1603,7 +1615,7 @@ void PDFDoc::writeXRefTableTrailer(Object &&trailerDict, XRef *uxref, bool write
 {
   uxref->writeTableToFile( outStr, writeAllEntries );
   outStr->printf( "trailer\r\n");
-  writeDictionnary(trailerDict.getDict(), outStr, xRef, 0, nullptr, cryptRC4, 0, 0, 0, nullptr);
+  writeDictionnary(trailerDict.getDict(), outStr, xRef, 0, nullptr, cryptRC4, 0, {0, 0}, nullptr);
   outStr->printf( "\r\nstartxref\r\n");
   outStr->printf( "%lli\r\n", uxrefOffset);
   outStr->printf( "%%%%EOF\r\n");
@@ -1673,10 +1685,10 @@ void PDFDoc::markDictionnary (Dict* dict, XRef * xRef, XRef *countRef, unsigned
   for (int i=0; i<dict->getLength(); i++) {
     const char *key = dict->getKey(i);
     if (strcmp(key, "Annots") != 0) {
-      Object obj1 = dict->getValNF(i);
+      Object obj1 = dict->getValNF(i).copy();
       markObject(&obj1, xRef, countRef, numOffset, oldRefNum, newRefNum, alreadyMarkedDicts);
     } else {
-      Object annotsObj = dict->getValNF(i);
+      Object annotsObj = dict->getValNF(i).copy();
       if (!annotsObj.isNull()) {
         markAnnotations(&annotsObj, xRef, countRef, 0, oldRefNum, newRefNum, alreadyMarkedDicts);
       }
@@ -1696,8 +1708,8 @@ void PDFDoc::markObject (Object* obj, XRef *xRef, XRef *countRef, unsigned int n
     case objArray:
       array = obj->getArray();
       for (int i=0; i<array->getLength(); i++) {
-        Object obj1 = array->getNF(i);
-        markObject(&obj1, xRef, countRef, numOffset, oldRefNum, newRefNum);
+        Object obj1 = array->getNF(i).copy();
+        markObject(&obj1, xRef, countRef, numOffset, oldRefNum, newRefNum, alreadyMarkedDicts);
       }
       break;
     case objDict:
@@ -1730,7 +1742,7 @@ void PDFDoc::markObject (Object* obj, XRef *xRef, XRef *countRef, unsigned int n
           if (entry->gen > 9)
             break;
         } 
-        Object obj1 = getXRef()->fetch(obj->getRef().num, obj->getRef().gen);
+        Object obj1 = getXRef()->fetch(obj->getRef());
         markObject(&obj1, xRef, countRef, numOffset, oldRefNum, newRefNum);
       }
       break;
@@ -1744,7 +1756,7 @@ void PDFDoc::replacePageDict(int pageNo, int rotate,
                              const PDFRectangle *cropBox)
 {
   Ref *refPage = getCatalog()->getPageRef(pageNo);
-  Object page = getXRef()->fetch(refPage->num, refPage->gen);
+  Object page = getXRef()->fetch(*refPage);
   Dict *pageDict = page.getDict();
   pageDict->remove("MediaBoxssdf");
   pageDict->remove("MediaBox");
@@ -1784,7 +1796,7 @@ void PDFDoc::markPageObjects(Dict *pageDict, XRef *xRef, XRef *countRef, unsigne
 
   for (int n = 0; n < pageDict->getLength(); n++) {
     const char *key = pageDict->getKey(n);
-    Object value  = pageDict->getValNF(n);
+    Object value  = pageDict->getValNF(n).copy();
     if (strcmp(key, "Parent") != 0 &&
 	      strcmp(key, "Pages") != 0 &&
 	      strcmp(key, "AcroForm") != 0 &&
@@ -1807,18 +1819,21 @@ bool PDFDoc::markAnnotations(Object *annotsObj, XRef *xRef, XRef *countRef, unsi
           Dict *dict = obj1.getDict();
           Object type = dict->lookup("Type");
           if (type.isName() && strcmp(type.getName(), "Annot") == 0) {
-            Object obj2 = dict->lookupNF("P");
+            const Object &obj2 = dict->lookupNF("P");
             if (obj2.isRef()) {
               if (obj2.getRef().num == oldPageNum) {
-                Object obj3 = array->getNF(i);
+                const Object &obj3 = array->getNF(i);
                 if (obj3.isRef()) {
-                  dict->set("P", Object(newPageNum, 0));
+                  Ref r;
+                  r.num = newPageNum;
+                  r.gen = 0;
+                  dict->set("P", Object(r));
                   getXRef()->setModifiedObject(&obj1, obj3.getRef());
                 }
               } else if (obj2.getRef().num == newPageNum) {
                 continue;
               } else {
-                Object page  = getXRef()->fetch(obj2.getRef().num, obj2.getRef().gen);
+                Object page  = getXRef()->fetch(obj2.getRef());
                 if (page.isDict()) {
                   Dict *pageDict = page.getDict();
                   Object pagetype = pageDict->lookup("Type");
@@ -1834,7 +1849,7 @@ bool PDFDoc::markAnnotations(Object *annotsObj, XRef *xRef, XRef *countRef, unsi
           }
           markPageObjects(dict, xRef, countRef, numOffset, oldPageNum, newPageNum, alreadyMarkedDicts);
         }
-        obj1 = array->getNF(i);
+        obj1 = array->getNF(i).copy();
         if (obj1.isRef()) {
           if (obj1.getRef().num + (int) numOffset >= xRef->getNumObjects() || xRef->getEntry(obj1.getRef().num + numOffset)->type == xrefEntryFree) {
             if (getXRef()->getEntry(obj1.getRef().num)->type == xrefEntryFree) {
@@ -1886,10 +1901,10 @@ void PDFDoc::markAcroForm(Object *afObj, XRef *xRef, XRef *countRef, unsigned in
       Dict *dict = acroform.getDict();
       for (int i=0; i < dict->getLength(); i++) {
         if (strcmp(dict->getKey(i), "Fields") == 0) {
-          Object fields = dict->getValNF(i);
+          Object fields = dict->getValNF(i).copy();
           modified = markAnnotations(&fields, xRef, countRef, numOffset, oldRefNum, newRefNum);
         } else {
-          Object obj = dict->getValNF(i);
+          Object obj = dict->getValNF(i).copy();
           markObject(&obj, xRef, countRef, numOffset, oldRefNum, newRefNum);
         }
       }
@@ -1940,10 +1955,10 @@ unsigned int PDFDoc::writePageObjects(OutStream *outStr, XRef *xRef, unsigned in
       } else if (xRef->getEntry(n)->getFlag(XRefEntry::Unencrypted)) {
         writeObject(&obj, outStr, nullptr, cryptRC4, 0, 0, 0);
       } else {
-        writeObject(&obj, outStr, fileKey, encAlgorithm, keyLength, ref.num, ref.gen);
+        writeObject(&obj, outStr, fileKey, encAlgorithm, keyLength, ref);
       }
       writeObjectFooter(outStr);
-      xRef->add(ref.num, ref.gen, offset, true);
+      xRef->add(ref, offset, true);
     }
   }
   return objectsCount;
@@ -2093,7 +2108,7 @@ Page *PDFDoc::parsePage(int page)
   }
 
   pageRef.gen = xref->getEntry(pageRef.num)->gen;
-  Object obj = xref->fetch(pageRef.num, pageRef.gen);
+  Object obj = xref->fetch(pageRef);
   if (!obj.isDict("Page")) {
     error(errSyntaxWarning, -1, "Object ({0:d} {1:d}) is not a pageDict", pageRef.num, pageRef.gen);
     return nullptr;
