@@ -14,7 +14,7 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2005 Brad Hards <bradh@frogmouth.net>
-// Copyright (C) 2005-2009, 2011, 2012, 2014, 2015, 2018 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005-2009, 2011, 2012, 2014, 2015, 2018, 2019 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2008, 2010 Pino Toscano <pino@kde.org>
 // Copyright (C) 2009, 2011 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2009 Petr Gajdos <pgajdos@novell.com>
@@ -844,7 +844,6 @@ bool ArthurOutputDev::axialShadedFill(GfxState *state, GfxAxialShading *shading,
     }
 
     // set the color
-    GfxRGB rgb;
     shading->getColorSpace()->getRGB(&color1, &rgb);
     qColor.setRgb(colToByte(rgb.r), colToByte(rgb.g), colToByte(rgb.b));
     gradient.setColorAt((ta[j] - tMin)/(tMax - tMin), qColor);
@@ -883,7 +882,7 @@ void ArthurOutputDev::eoClip(GfxState *state)
 void ArthurOutputDev::drawChar(GfxState *state, double x, double y,
 			       double dx, double dy,
 			       double originX, double originY,
-			       CharCode code, int nBytes, Unicode *u, int uLen) {
+			       CharCode code, int nBytes, const Unicode *u, int uLen) {
 
   // First handle type3 fonts
   GfxFont *gfxFont = state->getFont();
@@ -1024,7 +1023,7 @@ void ArthurOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 
   // TODO: Would using QImage::Format_Mono be more efficient here?
   QImage image(width, height, QImage::Format_ARGB32);
-  unsigned int *data = (unsigned int *)image.bits();
+  unsigned int *data = reinterpret_cast<unsigned int *>(image.bits());
   int stride = image.bytesPerLine()/4;
 
   QRgb fillColor = m_currentBrush.color().rgb();
@@ -1055,7 +1054,7 @@ void ArthurOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 void ArthurOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 				int width, int height,
 				GfxImageColorMap *colorMap,
-				bool interpolate, int *maskColors, bool inlineImg)
+				bool interpolate, const int *maskColors, bool inlineImg)
 {
   unsigned int *data;
   unsigned int *line;
@@ -1074,7 +1073,7 @@ void ArthurOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
   imgStr->reset();
   
   image = QImage(width, height, QImage::Format_ARGB32);
-  data = (unsigned int *)image.bits();
+  data = reinterpret_cast<unsigned int *>(image.bits());
   stride = image.bytesPerLine()/4;
   for (y = 0; y < height; y++) {
     pix = imgStr->getLine();
@@ -1149,7 +1148,7 @@ void ArthurOutputDev::drawSoftMaskedImage(GfxState *state, Object *ref, Stream *
   maskImageStr->reset();
 
   QImage image(width, height, QImage::Format_ARGB32);
-  unsigned int *data = (unsigned int *)image.bits();
+  unsigned int *data = reinterpret_cast<unsigned int *>(image.bits());
   int stride = image.bytesPerLine()/4;
 
   std::vector<unsigned char> maskLine(maskWidth);
