@@ -21,12 +21,6 @@
 #ifndef SPLASHFONTFILEID_H
 #define SPLASHFONTFILEID_H
 
-#ifdef USE_GCC_PRAGMAS
-#pragma interface
-#endif
-
-#include "goo/gtypes.h"
-
 //------------------------------------------------------------------------
 // SplashFontFileID
 //------------------------------------------------------------------------
@@ -38,7 +32,7 @@ public:
   virtual ~SplashFontFileID();
   SplashFontFileID(const SplashFontFileID &) = delete;
   SplashFontFileID& operator=(const SplashFontFileID &) = delete;
-  virtual GBool matches(SplashFontFileID *id) = 0;
+  virtual bool matches(SplashFontFileID *id) = 0;
 };
 
 #endif
