@@ -480,7 +480,7 @@ bool BeagleWksDRParser::readRSRCZones()
 
   auto &entryMap = rsrcParser->getEntriesMap();
   // the 1 zone
-  char const *(zNames[]) = {"wPos", "DMPF" };
+  char const *zNames[] = {"wPos", "DMPF" };
   for (int z = 0; z < 2; ++z) {
     auto it = entryMap.lower_bound(zNames[z]);
     while (it != entryMap.end()) {
