@@ -192,6 +192,8 @@ struct Section final : public MWAWSection {
     m_heights[0]=m_heights[1]=0;
     m_balanceText=true;
   }
+  Section(Section const &)=default;
+  Section &operator=(Section const &)=default;
   //! destructor
   ~Section() final;
   //! return the i^th entry
