@@ -269,6 +269,8 @@ struct Paragraph final : public MWAWParagraph {
     , m_addPageBreak(false)
   {
   }
+  Paragraph(Paragraph const &)=default;
+  Paragraph &operator=(Paragraph const &)=default;
   //! destructor
   ~Paragraph() final;
   //! operator<<
@@ -1956,7 +1958,7 @@ bool HanMacWrdJText::readParagraph(HanMacWrdJTextInternal::Paragraph &para, long
     if (val) f << "#f" << i+4 << "=" << std::hex << val << std::dec << ",";
   }
   // the borders
-  char const *(wh[5]) = { "T", "L", "B", "R", "VSep" };
+  char const *wh[5] = { "T", "L", "B", "R", "VSep" };
   MWAWBorder borders[5];
   for (auto &border : borders) border.m_width = double(input->readLong(4))/65536.;
   for (int d=0; d < 5; d++) {
@@ -1994,7 +1996,7 @@ bool HanMacWrdJText::readParagraph(HanMacWrdJTextInternal::Paragraph &para, long
   }
   // update the paragraph
   para.resizeBorders(6);
-  libmwaw::Position const(which[5]) = {
+  libmwaw::Position const which[5] = {
     libmwaw::Top, libmwaw::Left, libmwaw::Bottom, libmwaw::Right,
     libmwaw::VMiddle
   };
