@@ -68,10 +68,14 @@ struct Font final : public WPSFont
 		, m_type(type)
 	{
 	}
+	Font(Font const &)=default;
+	~Font() final;
 	//! font encoding type
 	libwps_tools_win::Font::Type m_type;
 };
-
+Font::~Font()
+{
+}
 //! the state of LotusParser
 struct State
 {
@@ -450,9 +454,9 @@ bool LotusParser::checkHeader(std::shared_ptr<WPSStream> stream, bool mainStream
 	}
 
 	input->seek(0,librevenge::RVNG_SEEK_SET);
-	auto firstOffset = (int) libwps::readU8(input);
-	auto type = (int) libwps::read8(input);
-	auto val=(int) libwps::read16(input);
+	auto firstOffset = int(libwps::readU8(input));
+	auto type = int(libwps::read8(input));
+	auto val=int(libwps::read16(input));
 	f << "FileHeader:";
 	if (firstOffset == 0 && type == 0 && val==0x1a)
 	{
@@ -464,7 +468,7 @@ bool LotusParser::checkHeader(std::shared_ptr<WPSStream> stream, bool mainStream
 		WPS_DEBUG_MSG(("LotusParser::checkHeader: find unexpected first data\n"));
 		return false;
 	}
-	val=(int) libwps::readU16(input);
+	val=int(libwps::readU16(input));
 	if (!mainStream)
 	{
 		if (val!=0x8007)
@@ -599,8 +603,8 @@ bool LotusParser::readZones(std::shared_ptr<WPSStream> stream)
 		long pos = input->tell();
 		if (!stream->checkFilePosition(pos+4))
 			break;
-		auto type = (int) libwps::readU16(input); // 1
-		auto length = (int) libwps::readU16(input);
+		auto type = int(libwps::readU16(input)); // 1
+		auto length = int(libwps::readU16(input));
 		if (type==1 && length==0)
 		{
 			ascFile.addPos(pos);
@@ -618,9 +622,9 @@ bool LotusParser::readZones(std::shared_ptr<WPSStream> stream)
 	{
 		long pos=input->tell();
 		if (pos>=stream->m_eof) break;
-		auto id = (int) libwps::readU8(input);
-		auto type = (int) libwps::readU8(input);
-		auto sz = (long) libwps::readU16(input);
+		auto id = int(libwps::readU8(input));
+		auto type = int(libwps::readU8(input));
+		auto sz = long(libwps::readU16(input));
 		if ((type>0x2a) || sz<0 || !stream->checkFilePosition(pos+4+sz))
 		{
 			input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -650,9 +654,9 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
-	auto type = (int) libwps::readU8(input);
-	auto sz = (long) libwps::readU16(input);
+	auto id = int(libwps::readU8(input));
+	auto type = int(libwps::readU8(input));
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if ((type>0x2a) || sz<0 || !stream->checkFilePosition(endPos))
 	{
@@ -681,7 +685,7 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			f.str("");
 			f << "Entries(BOF):";
-			val=(int) libwps::readU16(input);
+			val=int(libwps::readU16(input));
 			m_state->m_inMainContentBlock=false;
 			if (val==0x8007)
 				f << "FMT,";
@@ -694,11 +698,11 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 				f << "#version=" << std::hex << val << std::dec << ",";
 			for (int i=0; i<4; ++i)   // f0=4, f3 a small number
 			{
-				val=(int) libwps::read16(input);
+				val=int(libwps::read16(input));
 				if (val)
 					f << "f" << i << "=" << val << ",";
 			}
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			if (m_state->m_inMainContentBlock)
 			{
 				m_spreadsheetParser->setLastSpreadsheetId(val);
@@ -711,7 +715,7 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 
 			for (int i=0; i<7; ++i)   // g0/g1=0..fd, g2=0|4, g3=0|5|7|1e|20|30, g4=0|8c|3d, g5=1|10, g6=2|a
 			{
-				val=(int) libwps::readU8(input);
+				val=int(libwps::readU8(input));
 				if (val)
 					f << "g" << i << "=" << std::hex << val << std::dec << ",";
 			}
@@ -802,7 +806,7 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			for (int i=0; i<3; ++i)   // f0=1, f2=1|32
 			{
-				val=(int) libwps::read16(input);
+				val=int(libwps::read16(input));
 				if (val)
 					f << "f" << i << "=" << val << ",";
 			}
@@ -817,13 +821,13 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			for (int i=0; i<2; ++i)   // f0=1|3
 			{
-				val=(int) libwps::read8(input);
+				val=int(libwps::read8(input));
 				if (val!=1)
 					f << "f" << i << "=" << val << ",";
 			}
 			for (int i=0; i<2; ++i)   // f2=1-3, f1=0|1
 			{
-				val=(int) libwps::read16(input);
+				val=int(libwps::read16(input));
 				if (val)
 					f << "f" << i+1 << "=" << val << ",";
 			}
@@ -839,10 +843,10 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 				break;
 			}
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			if (val)
 				f << "sheet[id]=" << val << ",";
-			val=(int) libwps::read8(input); // always 0?
+			val=int(libwps::read8(input)); // always 0?
 			if (val)
 				f << "f0=" << val << ",";
 
@@ -858,12 +862,12 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 				break;
 			}
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			if (val)
 				f << "sheet[id]=" << val << ",";
 			for (int i=0; i<4; ++i)   // f0=0, f2=0|1, f3=7-9
 			{
-				val=(int) libwps::read8(input); // always 0?
+				val=int(libwps::read8(input)); // always 0?
 				if (val)
 					f << "f" << i << "=" << val << ",";
 			}
@@ -887,7 +891,7 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 				break;
 			}
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::read8(input);
+			val=int(libwps::read8(input));
 			if (val==1)
 				f << "true,";
 			else if (val)
@@ -897,7 +901,7 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			for (int i=0; i<sz; ++i)
 			{
-				val=(int) libwps::read8(input);
+				val=int(libwps::read8(input));
 				if (val==1) f << "f" << i << ",";
 				else if (val) f << "f" << i << "=" << val << ",";
 			}
@@ -912,12 +916,12 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			for (int i=0; i<30; ++i)   // f7=0|f, f8=0|60, f9=0|54, f17=80, f18=0|ff, f19=3f|40, f26=0|f8, f27=80|ff, f28=b|c,f29=40
 			{
-				val=(int) libwps::read8(input);
+				val=int(libwps::read8(input));
 				if (val) f << "f" << i << "=" << val << ",";
 			}
 			if (sz>=32)
 			{
-				val=(int) libwps::read16(input); // always 1?
+				val=int(libwps::read16(input)); // always 1?
 				if (val!=1) f << "f30=" << val << ",";
 			}
 			isParsed=needWriteInAscii=true;
@@ -929,7 +933,7 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 				break;
 			}
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::read8(input); // 1|2
+			val=int(libwps::read8(input)); // 1|2
 			if (val!=1) f << "f0=" << val << ",";
 			for (int i=0; i<3; ++i)
 			{
@@ -937,7 +941,7 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 				std::string name("");
 				for (int j=0; j<16; ++j)
 				{
-					auto c=(char) libwps::readU8(input);
+					auto c=char(libwps::readU8(input));
 					if (!c) break;
 					name += c;
 				}
@@ -947,12 +951,12 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 			}
 			for (int i=0; i<17; ++i)   // f2=f11=1,f15=0|1, f16=0|2, f17=0|1|2
 			{
-				val=(int) libwps::read8(input);
+				val=int(libwps::read8(input));
 				if (val) f << "f" << i+1 << "=" << val << ",";
 			}
 			for (int i=0; i<10; ++i)   // g0=0|1,g1=Ø|1, g2=4|a, g3=4c|50|80, g4=g5=0|2, g6=42, g7=41|4c, g8=3c|42|59
 			{
-				val=(int) libwps::read16(input);
+				val=int(libwps::read16(input));
 				if (val) f << "g" << i << "=" << val << ",";
 			}
 			isParsed=needWriteInAscii=true;
@@ -969,13 +973,13 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			for (int i=0; i<2; ++i)
 			{
-				val=(int) libwps::readU8(input);
+				val=int(libwps::readU8(input));
 				if (val) f << "f" << i << "=" << val << ",";
 			}
 			std::string data("");
 			for (int i=2; i<sz; ++i)
 			{
-				auto c=(char) libwps::readU8(input);
+				auto c=char(libwps::readU8(input));
 				if (!c) break;
 				data += c;
 			}
@@ -1007,11 +1011,11 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 				break;
 			}
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::read16(input); // small number 6-c maybe a style
+			val=int(libwps::read16(input)); // small number 6-c maybe a style
 			if (val) f << "f0=" << val << ",";
 			for (int i=0; i<2; ++i)   // zone15: f1=3, f2=2-5, zone 1d: always 0
 			{
-				val=(int) libwps::readU8(input);
+				val=int(libwps::readU8(input));
 				if (val) f << "f" << i+1 << "=" << val << ",";
 			}
 			isParsed=needWriteInAscii=true;
@@ -1040,7 +1044,7 @@ bool LotusParser::readZone(std::shared_ptr<WPSStream> &stream)
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			for (int i=0; i<6; ++i)   // some int
 			{
-				val=(int) libwps::readU8(input);
+				val=int(libwps::readU8(input));
 				if (val) f << "f" << i << "=" << std::hex << val << std::dec << ",";
 			}
 			isParsed=needWriteInAscii=true;
@@ -1332,8 +1336,8 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto type = (int) libwps::readU16(input);
-	auto sz = (long) libwps::readU16(input);
+	auto type = int(libwps::readU16(input));
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (type!=0x1b || sz<2)
 	{
@@ -1341,7 +1345,7 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
 		return false;
 	}
-	type = (int) libwps::readU16(input);
+	type = int(libwps::readU16(input));
 	f << "Entries(Data" << std::hex << type << std::dec << "E):";
 	bool isParsed=false, needWriteInAscii=false;
 	sz-=2;
@@ -1361,15 +1365,15 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 			f << "###";
 			break;
 		}
-		val=(int) libwps::readU8(input);
+		val=int(libwps::readU8(input));
 		if (val) f << "id=" << val << ",";
-		val=(int) libwps::read8(input); // find 0|2
+		val=int(libwps::read8(input)); // find 0|2
 		if (val) f << "f0=" << val << ",";
 		int dim[4];
 		for (int i=0; i<4; ++i)
 		{
-			dim[i]=(int) libwps::read16(input);
-			val=(int) libwps::read16(input);
+			dim[i]=int(libwps::read16(input));
+			val=int(libwps::read16(input));
 			if (!val) continue;
 			if (i)
 				f << "num[split]=" << val << ",";
@@ -1379,7 +1383,7 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 		f << "dim=" << WPSBox2i(Vec2i(dim[0],dim[1]),Vec2i(dim[2],dim[3])) << ",";
 		for (int i=0; i<8; ++i)   // small value or 100
 		{
-			val=(int) libwps::read8(input);
+			val=int(libwps::read8(input));
 			if (val) f << "f" << i+1 << "=" << val << ",";
 		}
 		isParsed=needWriteInAscii=true;
@@ -1387,7 +1391,7 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 		if (remain<=1) break;
 		std::string name("");
 		for (int i=0; i<remain; ++i)
-			name+=(char) libwps::readU8(input);
+			name+=char(libwps::readU8(input));
 		f << name << ",";
 		break;
 	}
@@ -1401,27 +1405,27 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 			f << "###";
 			break;
 		}
-		val=(int) libwps::readU8(input);
+		val=int(libwps::readU8(input));
 		if (val) f << "id=" << val << ",";
-		val=(int) libwps::readU8(input);
+		val=int(libwps::readU8(input));
 		if (val) f << "split[id]=" << val << ",";
 		for (int i=0; i<3; ++i)   // 0 or 1
 		{
-			val=(int) libwps::read8(input);
+			val=int(libwps::read8(input));
 			if (val) f << "f" << i+1 << "=" << val << ",";
 		}
 		int dim[4];
 		for (int i=0; i<4; ++i)
 		{
-			val=(int) libwps::read16(input);
-			dim[i]=(int) libwps::read16(input);
+			val=int(libwps::read16(input));
+			dim[i]=int(libwps::read16(input));
 			if (val) f << "dim" << i <<"[h]=" << val << ",";
 		}
 		f << "dim=" << WPSBox2i(Vec2i(dim[0],dim[1]),Vec2i(dim[2],dim[3])) << ",";
 		for (int i=0; i<3; ++i)
 		{
 			static int const expected[]= {0,-1,25};
-			val=(int) libwps::read8(input);
+			val=int(libwps::read8(input));
 			if (val!=expected[i]) f << "g" << i << "=" << val << ",";
 		}
 		isParsed=needWriteInAscii=true;
@@ -1439,7 +1443,7 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 		}
 		for (int i=0; i<4; ++i)   // always 2,1,1,2 ?
 		{
-			val=(int) libwps::read8(input);
+			val=int(libwps::read8(input));
 			if (val) f << "f" << i << "=" << val << ",";
 		}
 		isParsed=needWriteInAscii=true;
@@ -1447,7 +1451,7 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 		if (remain<=1) break;
 		std::string name("");
 		for (int i=0; i<remain; ++i) // always LMBCS 1.2?
-			name+=(char) libwps::readU8(input);
+			name+=char(libwps::readU8(input));
 		f << name << ",";
 		break;
 	}
@@ -1463,7 +1467,7 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 		// time to update the style manager state
 		m_styleManager->updateState();
 
-		val=(int) libwps::readU8(input);
+		val=int(libwps::readU8(input));
 		if (val) f << "sheet[id]=" << val << ",";
 		// then always 0a3fff00ffff508451ff ?
 		isParsed=needWriteInAscii=true;
@@ -1487,16 +1491,16 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 			f << "###";
 			break;
 		}
-		val=(int) libwps::readU8(input);
+		val=int(libwps::readU8(input));
 		if (val) f << "sheet[id]=" << val << ",";
-		val=(int) libwps::readU8(input); // always 0
+		val=int(libwps::readU8(input)); // always 0
 		if (val) f << "f0=" << val << ",";
 		f << "break=[";
 		auto N=int((sz-2)/dataSz);
 		for (int i=0; i<N; ++i)
 		{
 			if (dataSz==1)
-				f << (int) libwps::readU8(input) << ",";
+				f << int(libwps::readU8(input)) << ",";
 			else
 				f << libwps::readU16(input) << ",";
 		}
@@ -1520,14 +1524,14 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 		}
 		for (int i=0; i<3; ++i)   // f0=0, f1=f2=1
 		{
-			val=(int) libwps::read16(input);
+			val=int(libwps::read16(input));
 			if (val) f << "f" << i << "=" << val << ",";
 		}
 		for (int i=0; i<3; ++i)
 		{
-			auto row=(int) libwps::readU16(input);
-			auto sheet=(int) libwps::readU8(input);
-			auto col=(int) libwps::readU8(input);
+			auto row=int(libwps::readU16(input));
+			auto sheet=int(libwps::readU8(input));
+			auto col=int(libwps::readU8(input));
 			f << "C" << col << "-" << row;
 			if (sheet) f << "[" << sheet << "],";
 			else f << ",";
@@ -1677,7 +1681,7 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 		}
 		for (int i=0; i<3; ++i)
 		{
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			static int const expected[]= {0x1f, 0xe0, 0/*or 1*/};
 			if (val!=expected[i])
 				f << "f" << i << "=" << val << ",";
@@ -1694,12 +1698,12 @@ bool LotusParser::readDataZone(std::shared_ptr<WPSStream> stream)
 			f << "###";
 			break;
 		}
-		val=(int) libwps::read16(input);
+		val=int(libwps::read16(input));
 		if (val!=20) f << "f0=" << val << ",";
 		std::string name("");
 		for (int i=4; i<sz; ++i)
 		{
-			auto c=(char) libwps::readU8(input);
+			auto c=char(libwps::readU8(input));
 			if (!c) break;
 			name+=c;
 		}
@@ -1765,8 +1769,8 @@ bool LotusParser::readZoneV3(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto type = (int) libwps::readU16(input);
-	auto sz = (long) libwps::readU16(input);
+	auto type = int(libwps::readU16(input));
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (sz<0 || !stream->checkFilePosition(endPos))
 	{
@@ -1789,13 +1793,13 @@ bool LotusParser::readZone1(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
+	auto id = int(libwps::readU8(input));
 	if (libwps::readU8(input) != 1)
 	{
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (sz<0 || !stream->checkFilePosition(endPos))
 	{
@@ -1973,13 +1977,13 @@ bool LotusParser::readSheetZone(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
+	auto id = int(libwps::readU8(input));
 	if (libwps::readU8(input) != 2)
 	{
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (sz<0 || !stream->checkFilePosition(endPos))
 	{
@@ -2272,13 +2276,13 @@ bool LotusParser::readZone4(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
+	auto id = int(libwps::readU8(input));
 	if (libwps::readU8(input)!=4)
 	{
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (sz<0 || !stream->checkFilePosition(endPos))
 	{
@@ -2456,13 +2460,13 @@ bool LotusParser::readChartZone(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
+	auto id = int(libwps::readU8(input));
 	if (libwps::readU8(input)!=5)
 	{
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (sz<0 || !stream->checkFilePosition(endPos))
 	{
@@ -2556,13 +2560,13 @@ bool LotusParser::readRefZone(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
+	auto id = int(libwps::readU8(input));
 	if (libwps::readU8(input)!=6)
 	{
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (sz<0 || !stream->checkFilePosition(endPos))
 	{
@@ -2642,13 +2646,13 @@ bool LotusParser::readZone7(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
+	auto id = int(libwps::readU8(input));
 	if (libwps::readU8(input)!=7)
 	{
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (sz<0 || !stream->checkFilePosition(endPos))
 	{
@@ -2768,7 +2772,7 @@ bool LotusParser::readZone8(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
+	auto id = int(libwps::readU8(input));
 	if (id==1)
 	{
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -2783,7 +2787,7 @@ bool LotusParser::readZone8(std::shared_ptr<WPSStream> stream)
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (sz<0 || !stream->checkFilePosition(endPos))
 	{
@@ -2811,9 +2815,9 @@ bool LotusParser::readZone8(std::shared_ptr<WPSStream> stream)
 			f << "###";
 			break;
 		}
-		long count=(int)(sz>=4 ? libwps::readU32(input) : libwps::readU16(input));
+		long count=int(sz>=4 ? libwps::readU32(input) : libwps::readU16(input));
 		Vec2i &zone=m_state->m_actualLevels.back();
-		if ((int)(zone[1]+count)<0)
+		if (int(zone[1]+count)<0)
 		{
 			WPS_DEBUG_MSG(("LotusParser::readZone8: arg the delta bad\n"));
 			f << "###delta=" << count << ",";
@@ -2910,13 +2914,13 @@ bool LotusParser::readVersionZone(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
+	auto id = int(libwps::readU8(input));
 	if (libwps::readU8(input)!=0xa)
 	{
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (sz<0 || !stream->checkFilePosition(endPos))
 	{
@@ -2966,19 +2970,19 @@ bool LotusParser::readMacFontName(std::shared_ptr<WPSStream> stream, long endPos
 		if (m_state->m_fontType==libwps_tools_win::Font::UNKNOWN)
 			m_state->m_fontType=libwps_tools_win::Font::MAC_ROMAN;
 		m_state->m_isMacFile=true;
-		auto id=(int) libwps::readU16(input);
+		auto id=int(libwps::readU16(input));
 		f << "FN" << id << ",";
-		auto val=(int) libwps::readU16(input); // always 0?
+		auto val=int(libwps::readU16(input)); // always 0?
 		if (val)
 			f << "f0=" << val << ",";
-		val=(int) libwps::read16(input); // find -1, 30 (Geneva), 60 (Helvetica)
+		val=int(libwps::read16(input)); // find -1, 30 (Geneva), 60 (Helvetica)
 		if (val)
 			f << "f1=" << val << ",";
 		librevenge::RVNGString name("");
 		bool nameOk=true;
 		for (int i=0; i<sz-6; ++i)
 		{
-			auto c=(char) libwps::readU8(input);
+			auto c=char(libwps::readU8(input));
 			if (!c) break;
 			if (nameOk && !(c==' ' || (c>='0'&&c<='9') || (c>='a'&&c<='z') || (c>='A'&&c<='Z')))
 			{
@@ -3009,7 +3013,7 @@ bool LotusParser::readMacFontName(std::shared_ptr<WPSStream> stream, long endPos
 	int id=0;
 	for (int i=0; i<4; ++i)
 	{
-		auto val=(int) libwps::readU8(input); // 0|1
+		auto val=int(libwps::readU8(input)); // 0|1
 		if (i==1)
 		{
 			id=val;
@@ -3020,7 +3024,7 @@ bool LotusParser::readMacFontName(std::shared_ptr<WPSStream> stream, long endPos
 	}
 	for (int i=0; i<2; ++i)   // f1=0|1288
 	{
-		auto val=(int) libwps::read16(input);
+		auto val=int(libwps::read16(input));
 		if (val)
 			f << "f" << i << "=" << val << ",";
 	}
@@ -3028,7 +3032,7 @@ bool LotusParser::readMacFontName(std::shared_ptr<WPSStream> stream, long endPos
 	bool nameOk=true;
 	for (int i=0; i<8; ++i)
 	{
-		auto c=(char) libwps::read8(input);
+		auto c=char(libwps::read8(input));
 		if (!c) break;
 		if (nameOk && !(c==' ' || (c>='0'&&c<='9') || (c>='a'&&c<='z') || (c>='A'&&c<='Z')))
 		{
@@ -3068,13 +3072,13 @@ bool LotusParser::readFMTStyleName(std::shared_ptr<WPSStream> stream)
 	libwps::DebugStream f;
 
 	long pos = input->tell();
-	auto type = (int) libwps::read16(input);
+	auto type = int(libwps::read16(input));
 	if (type!=0xb6)
 	{
 		WPS_DEBUG_MSG(("LotusParser::readFMTStyleName: not a font name definition\n"));
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	if (sz<8)
 	{
@@ -3115,13 +3119,13 @@ bool LotusParser::readLinkZone(std::shared_ptr<WPSStream> stream)
 	libwps::DebugStream f;
 
 	long pos = input->tell();
-	auto type = (int) libwps::read16(input);
+	auto type = int(libwps::read16(input));
 	if (type!=0xa)
 	{
 		WPS_DEBUG_MSG(("LotusParser::readLinkZone: not a link definition\n"));
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	f << "Entries(Link):";
 	if (sz < 19)
 	{
@@ -3131,7 +3135,7 @@ bool LotusParser::readLinkZone(std::shared_ptr<WPSStream> stream)
 		ascFile.addNote(f.str().c_str());
 		return true;
 	}
-	type=(int) libwps::read16(input);
+	type=int(libwps::read16(input));
 	if (type==0) // fixme: find if this is a note, so that we can retrieve it
 		f << "chart/note/...,";
 	else if (type==1)
@@ -3145,7 +3149,7 @@ bool LotusParser::readLinkZone(std::shared_ptr<WPSStream> stream)
 		return true;
 	}
 	f << "ID=" << int(libwps::readU8(input)) << ","; // 0,19,42,53,ff
-	auto id = (int) libwps::readU8(input);
+	auto id = int(libwps::readU8(input));
 	f << "id=" << id << ",";
 
 	Link link;
@@ -3167,7 +3171,7 @@ bool LotusParser::readLinkZone(std::shared_ptr<WPSStream> stream)
 	// Q[0-2]: contains often <<XXX>>YYY: link to another sheetname?
 	for (int i=0; i<14; ++i)
 	{
-		auto c=(char) libwps::readU8(input);
+		auto c=char(libwps::readU8(input));
 		if (!c) break;
 		link.m_name += c;
 	}
@@ -3185,9 +3189,9 @@ bool LotusParser::readLinkZone(std::shared_ptr<WPSStream> stream)
 		}
 		for (int i=0; i<2; ++i)
 		{
-			auto row=(int) libwps::readU16(input);
-			auto table=(int) libwps::readU8(input);
-			auto col=(int) libwps::readU8(input);
+			auto row=int(libwps::readU16(input));
+			auto table=int(libwps::readU8(input));
+			auto col=int(libwps::readU8(input));
 			link.m_cells[i]=WPSVec3i(col,row,table);
 			f << "C" << col << "-" << row;
 			if (table) f << "[" << table << "]";
@@ -3199,13 +3203,8 @@ bool LotusParser::readLinkZone(std::shared_ptr<WPSStream> stream)
 		break;
 	case 1:
 	{
-		auto fontType=getDefaultFontType();
-		for (int i=18; i<sz; ++i)
-		{
-			auto c=(unsigned char) libwps::readU8(input);
-			if (!c) break;
-			libwps::appendUnicode((uint32_t)libwps_tools_win::Font::unicode(c,fontType), link.m_linkName);
-		}
+		if (sz>18)
+			link.m_linkName=libwps_tools_win::Font::unicodeString(input.get(), static_cast<unsigned long>(sz-18), getDefaultFontType());
 		f << "link=" << link.m_linkName.cstr() << ",";
 		break;
 	}
@@ -3249,17 +3248,17 @@ bool LotusParser::readDocumentInfoMac(std::shared_ptr<WPSStream> stream, long en
 	int dim[7];
 	for (int i=0; i<7; ++i)
 	{
-		auto val=(int) libwps::read8(input);
+		auto val=int(libwps::read8(input));
 		if (i==0)
 			f << "dim[unkn]=";
 		else if (i==1)
 			f << "margins=[";
 		else if (i==5)
 			f << "pagesize=[";
-		dim[i]=(int) libwps::read16(input);
+		dim[i]=int(libwps::read16(input));
 		f << dim[i];
 		if (val) f << "[" << val << "]";
-		val=(int) libwps::read8(input); // always 0
+		val=int(libwps::read8(input)); // always 0
 		if (val) f << "[" << val << "]";
 		f << ",";
 		if (i==4 || i==6) f << "],";
@@ -3279,7 +3278,7 @@ bool LotusParser::readDocumentInfoMac(std::shared_ptr<WPSStream> stream, long en
 	f << "unkn=[";
 	for (int i=0; i<5; ++i)   // 1,1,1,100|inf,1
 	{
-		auto val=(int) libwps::read16(input);
+		auto val=int(libwps::read16(input));
 		if (val==9999)
 			f << "inf,";
 		else if (val)
@@ -3290,7 +3289,7 @@ bool LotusParser::readDocumentInfoMac(std::shared_ptr<WPSStream> stream, long en
 	f << "],";
 	for (int i=0; i<13; ++i)   // always 0?
 	{
-		auto val=(int) libwps::read8(input);
+		auto val=int(libwps::read8(input));
 		if (val)
 			f << "g" << i << "=" << val << ",";
 	}
