@@ -79,6 +79,12 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (creator=="ARTX") { // Painter X
+      if (type=="RIFF") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CORELPAINTER, 10, MWAWDocument::MWAW_K_PAINT));
+        return res;
+      }
+    }
     else if (creator=="BOBO") {
       if (type=="CWDB" || type=="CWD2" || type=="sWDB") {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CLARISWORKS, 1, MWAWDocument::MWAW_K_DATABASE));
@@ -163,6 +169,18 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (creator=="FSPS" || creator=="FSDA") { // Fractal Design Painter or Dabbler
+      if (type=="RIFF") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CORELPAINTER, 1, MWAWDocument::MWAW_K_PAINT));
+        return res;
+      }
+    }
+    else if (creator=="FSX3") {
+      if (type=="RIFF") { // also FSFS list of uncompressed picture data for movie
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CORELPAINTER, 3, MWAWDocument::MWAW_K_PAINT));
+        return res;
+      }
+    }
     else if (creator=="FWRT") {
       if (type=="FWRM") { // 1.7 ?
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_FULLWRITE, 1));
@@ -436,8 +454,8 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
-    else if (creator==std::string("St")+char(0xd8)+"l") { //  argh, not standart character
-      std::string type1=std::string("TEd")+char(0xb6); // argh, not standart character
+    else if (creator==std::string("St")+'\xd8'+"l") { //  argh, not standart character
+      std::string type1=std::string("TEd")+'\xb6'; // argh, not standart character
       if (type==type1) {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_STYLE, 1));
         return res;
@@ -904,6 +922,9 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
   if (val[0]==0x4d44 && input->size()>=512) { // maybe a MacDraw 0 file, will be check later
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAW, 0, MWAWDocument::MWAW_K_DRAW));
   }
+  if (val[0]==2 && (val[1]&0xff)==0 && input->size() > 300) {
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CORELPAINTER, 1, MWAWDocument::MWAW_K_PAINT));
+  }
   if (val[0] == 3 || val[0] == 6) {
     // version will be print by MacWrtParser::check
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACWRITE, val[0]));
