@@ -745,7 +745,7 @@ bool MWAWInputStream::unMacMIME(MWAWInputStream *inp,
       "MSDosInfo", "AFPName", "AFPInfo", "AFPDirId"
     };
 #endif
-    for (int i = 0; i < numEntries; i++) {
+    for (long i = 0; i < numEntries; i++) {
       long pos = inp->tell();
       auto wh = static_cast<int>(inp->readULong(4));
       if (wh <= 0 || wh >= 16 || inp->isEnd()) {
