@@ -163,7 +163,7 @@ struct Pixmap {
         for (int x = 0, rPos = 0; x < W;) {
           unsigned char val = values[size_t(rPos++)];
           for (int v = numValuesByInt-1; v >=0; v--) {
-            int index = (val>>(v*m_pixelSize))&maxValues;
+            int index = int((val>>(v*m_pixelSize))&maxValues);
             if (index > maxColorsIndex) maxColorsIndex = index;
             m_indices[size_t(wPos++)] = index;
             if (++x >= W) break;
@@ -1001,7 +1001,6 @@ bool MacDraft5StyleManager::readResource(MWAWEntry &entry, bool inRsrc)
   if (entry.type()=="OPST") {
     if (inRsrc && !m_parserState->m_rsrcParser) return false;
     MWAWInputStreamPtr input = inRsrc ? m_parserState->m_rsrcParser->getInput() : m_parserState->m_input;
-    libmwaw::DebugFile &ascFile = inRsrc ? m_parserState->m_rsrcParser->ascii() : m_parserState->m_asciiFile;
     libmwaw::DebugStream f;
     f << "Entries(OPST)[" << entry.id() << "]:";
     entry.setParsed(true);
@@ -1016,6 +1015,7 @@ bool MacDraft5StyleManager::readResource(MWAWEntry &entry, bool inRsrc)
         f << "f0=" << val << ",";
     }
     if (input && entry.valid()) {
+      libmwaw::DebugFile &ascFile = inRsrc ? m_parserState->m_rsrcParser->ascii() : m_parserState->m_asciiFile;
       ascFile.addPos(entry.begin()-(inRsrc ? 4 : 16));
       ascFile.addNote(f.str().c_str());
       input->seek(entry.end(), librevenge::RVNG_SEEK_SET);
