@@ -62,7 +62,10 @@ namespace MindWrtParserInternal
 //! Internal: a field of a MindWrtParser
 struct Field {
   //! constructor
-  explicit Field(MWAWField::Type type=MWAWField::None) : m_type(type), m_pos(0,-1), m_extra("")
+  explicit Field(MWAWField::Type type=MWAWField::None)
+    : m_type(type)
+    , m_pos(0,-1)
+    , m_extra("")
   {
   }
   //! return true is the field is defined
@@ -124,8 +127,14 @@ struct Field {
 //! Internal: the list properties of a MindWrtParser
 struct ListProperties {
   //! constructor
-  ListProperties() : m_startListIndex(1), m_headingStyle(1), m_headingFullSubLevels(true),
-    m_headingListLevels(), m_listLevelsRepeatPos(1), m_showFirstLevel(false), m_useHeadingStyle(false)
+  ListProperties()
+    : m_startListIndex(1)
+    , m_headingStyle(1)
+    , m_headingFullSubLevels(true)
+    , m_headingListLevels()
+    , m_listLevelsRepeatPos(1)
+    , m_showFirstLevel(false)
+    , m_useHeadingStyle(false)
   {
   }
   //! updates the heading list
@@ -150,7 +159,7 @@ struct ListProperties {
 
 MWAWListLevel ListProperties::getLevel(int level) const
 {
-  int numLevels=int(m_headingListLevels.size());
+  auto numLevels=int(m_headingListLevels.size());
   if (level < 0 || numLevels <= 0) {
     MWAW_DEBUG_MSG(("ListProperties::updateHeadingList: can not find any level\n"));
     return MWAWListLevel();
@@ -243,10 +252,20 @@ void  ListProperties::updateHeadingList()
 ////////////////////////////////////////
 //! Internal: a line information of a MindWrtParser
 struct LineInfo {
-  LineInfo() : m_entry(), m_type(-1000), m_height(0), m_y(-1), m_page(-1),
-    m_paragraph(),  m_specialHeadingInterface(false), m_paragraphSet(false), m_listLevel(0), m_listType(0), m_extra("")
+  LineInfo()
+    : m_entry()
+    , m_type(-1000)
+    , m_height(0)
+    , m_y(-1)
+    , m_page(-1)
+    , m_paragraph()
+    , m_specialHeadingInterface(false)
+    , m_paragraphSet(false)
+    , m_listLevel(0)
+    , m_listType(0)
+    , m_extra("")
   {
-    for (int i = 0; i < 4; i++) m_flags[i] = 0;
+    for (auto &fl : m_flags) fl=0;
   }
 
   //! operator<<
@@ -360,7 +379,8 @@ struct LineInfo {
 ////////////////////////////////////////
 //! Internal: a zone information of a MindWrtParser
 struct ZoneInfo {
-  ZoneInfo() : m_linesList()
+  ZoneInfo()
+    : m_linesList()
   {
   }
 
@@ -373,7 +393,6 @@ struct ZoneInfo {
 
 void ZoneInfo::updateListId(ListProperties &prop, MWAWListManager &listManager)
 {
-  size_t numLines=m_linesList.size();
   bool headingStyle=prop.m_useHeadingStyle && prop.m_headingStyle>=1 && prop.m_headingStyle<=4;
 
   /* we have 4 list, we are almost independent: heading, unlabelled, decimal, bullet */
@@ -383,11 +402,10 @@ void ZoneInfo::updateListId(ListProperties &prop, MWAWListManager &listManager)
   levelsList[3].m_type = MWAWListLevel::BULLET;
   libmwaw::appendUnicode(0x2022, levelsList[3].m_bullet);
 
-  shared_ptr<MWAWList> listsList[4];
+  std::shared_ptr<MWAWList> listsList[4];
   int maxLevelSet=0;
 
-  for (size_t i = 0; i < numLines; i++) {
-    LineInfo &line = m_linesList[i];
+  for (auto &line : m_linesList) {
     if (line.m_height == 0 ||  // ruler or false graphic
         (line.m_type==-1 && line.m_entry.length()==0)) continue;
     MWAWParagraph &para = line.m_paragraph;
@@ -448,11 +466,11 @@ void ZoneInfo::updateListId(ListProperties &prop, MWAWListManager &listManager)
     else
       *(para.m_margins[0])=-0.2;
 
-    shared_ptr<MWAWList> &actList=listsList[line.m_listType];
+    auto &actList=listsList[line.m_listType];
     if (!actList) // we have a pb....
       continue;
-    for (int type=0; type<4; type++)
-      if (listsList[type]) listsList[type]->setLevel(level);
+    for (auto &type : listsList)
+      if (type) type->setLevel(level);
     if (line.m_listType==0)
       listsList[2]->setStartValueForNextElement(level==1 ? prop.m_startListIndex : 1);
 
@@ -467,12 +485,16 @@ void ZoneInfo::updateListId(ListProperties &prop, MWAWListManager &listManager)
 //! Internal: the state of a MindWrtParser
 struct State {
   //! constructor
-  State() : m_compressCorr(" etnroaisdlhcfp"), m_entryMap(), m_listProperties(),
-    m_actPage(0), m_numPages(0), m_headerHeight(0), m_footerHeight(0),
-    m_headerFieldList(), m_footerFieldList()
+  State()
+    : m_compressCorr(" etnroaisdlhcfp")
+    , m_entryMap()
+    , m_listProperties()
+    , m_actPage(0)
+    , m_numPages(0)
+    , m_headerFieldList()
+    , m_footerFieldList()
   {
-    for (int i = 0; i < 3; i++)
-      m_numLinesByZone[i] = 0;
+    for (auto &num : m_numLinesByZone) num = 0;
   }
 
   //! the correspondance between int compressed and char : must be 15 character
@@ -490,8 +512,6 @@ struct State {
   ListProperties m_listProperties;
   int m_actPage /** the actual page */, m_numPages /** the number of page of the final document */;
   //! the actual list id
-  int m_headerHeight /** the header height if known */,
-      m_footerHeight /** the footer height if known */;
   //! the header field list
   std::vector<Field> m_headerFieldList;
   //! the footer field list
@@ -500,34 +520,32 @@ struct State {
 
 ////////////////////////////////////////
 //! Internal: the subdocument of a MindWrtParser
-class SubDocument : public MWAWSubDocument
+class SubDocument final : public MWAWSubDocument
 {
 public:
-  SubDocument(MindWrtParser &pars, MWAWInputStreamPtr input, int zoneId, int step) :
-    MWAWSubDocument(&pars, input, MWAWEntry()), m_id(zoneId), m_step(step) {}
+  SubDocument(MindWrtParser &pars, MWAWInputStreamPtr const &input, int zoneId, int step)
+    : MWAWSubDocument(&pars, input, MWAWEntry())
+    , m_id(zoneId)
+    , m_step(step)
+  {
+  }
 
   //! destructor
-  virtual ~SubDocument() {}
+  ~SubDocument() final {}
 
   //! operator!=
-  virtual bool operator!=(MWAWSubDocument const &doc) const
+  bool operator!=(MWAWSubDocument const &doc) const final
   {
     if (MWAWSubDocument::operator!=(doc)) return true;
-    SubDocument const *sDoc = dynamic_cast<SubDocument const *>(&doc);
+    auto const *sDoc = dynamic_cast<SubDocument const *>(&doc);
     if (!sDoc) return true;
     if (m_id != sDoc->m_id) return true;
     if (m_step != sDoc->m_step) return true;
     return false;
   }
 
-  //! operator!==
-  virtual bool operator==(MWAWSubDocument const &doc) const
-  {
-    return !operator!=(doc);
-  }
-
   //! the parser function
-  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type);
+  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type) final;
 
 protected:
   //! the subdocument id
@@ -550,7 +568,7 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
     MWAW_DEBUG_MSG(("MindWrtParserInternal::SubDocument::parse: unknown step\n"));
     return;
   }
-  MindWrtParser *parser=dynamic_cast<MindWrtParser *>(m_parser);
+  auto *parser=dynamic_cast<MindWrtParser *>(m_parser);
   if (!parser) {
     MWAW_DEBUG_MSG(("MindWrtParserInternal::SubDocument::parse: no parser\n"));
     return;
@@ -567,8 +585,9 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-MindWrtParser::MindWrtParser(MWAWInputStreamPtr input, MWAWRSRCParserPtr rsrcParser, MWAWHeader *header) :
-  MWAWTextParser(input, rsrcParser, header), m_state()
+MindWrtParser::MindWrtParser(MWAWInputStreamPtr const &input, MWAWRSRCParserPtr const &rsrcParser, MWAWHeader *header)
+  : MWAWTextParser(input, rsrcParser, header)
+  , m_state()
 {
   init();
 }
@@ -621,14 +640,14 @@ MWAWEntry MindWrtParser::readEntry()
 ////////////////////////////////////////////////////////////
 void MindWrtParser::parse(librevenge::RVNGTextInterface *docInterface)
 {
-  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(nullptr))  throw(libmwaw::ParseException());
   bool ok = true;
   try {
     // create the asciiFile
     ascii().setStream(getInput());
     ascii().open(asciiName());
 
-    checkHeader(0L);
+    checkHeader(nullptr);
     if (getRSRCParser()) {
       MWAWEntry corrEntry = getRSRCParser()->getEntry("STR ", 700);
       std::string corrString("");
@@ -647,10 +666,8 @@ void MindWrtParser::parse(librevenge::RVNGTextInterface *docInterface)
     }
 
     libmwaw::DebugStream f;
-    std::multimap<std::string, MWAWEntry>::const_iterator it
-      = m_state->m_entryMap.begin();
-    while (it != m_state->m_entryMap.end()) {
-      MWAWEntry const &entry = (it++)->second;
+    for (auto it : m_state->m_entryMap) {
+      MWAWEntry const &entry = it.second;
       if (entry.isParsed())
         continue;
       f.str("");
@@ -686,8 +703,7 @@ void MindWrtParser::createDocument(librevenge::RVNGTextInterface *documentInterf
   m_state->m_actPage = 0;
 
   int numPage = 0, numBreakPage = 0;
-  for (size_t i = 0; i < m_state->m_zones[0].m_linesList.size(); i++) {
-    MindWrtParserInternal::LineInfo const &line = m_state->m_zones[0].m_linesList[i];
+  for (auto const &line : m_state->m_zones[0].m_linesList) {
     if (line.m_type == -2)
       numBreakPage++;
     if (line.m_page > numPage)
@@ -699,7 +715,7 @@ void MindWrtParser::createDocument(librevenge::RVNGTextInterface *documentInterf
   // create the page list
   MWAWPageSpan ps(getPageSpan());
   for (int i = 1; i <= 2; i++) {
-    if (!m_state->m_zones[i].m_linesList.size())
+    if (m_state->m_zones[i].m_linesList.empty())
       continue;
     MWAWHeaderFooter hF((i==1) ? MWAWHeaderFooter::HEADER : MWAWHeaderFooter::FOOTER, MWAWHeaderFooter::ALL);
     hF.m_subDocument.reset(new MindWrtParserInternal::SubDocument(*this, getInput(), i, 0));
@@ -787,9 +803,8 @@ bool MindWrtParser::createZones()
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
 
-  std::multimap<std::string, MWAWEntry>::iterator it;
   // first read print info
-  it = m_state->m_entryMap.find("PrintInfo");
+  auto it = m_state->m_entryMap.find("PrintInfo");
   if (it !=  m_state->m_entryMap.end())
     readPrintInfo(it->second);
   // now the heading properties
@@ -820,8 +835,8 @@ bool MindWrtParser::createZones()
   while (it !=  m_state->m_entryMap.end() && it->first == "LineInfo")
     readLinesInfo(it++->second);
 
-  for (int i = 0; i < 3; i++)
-    if (m_state->m_zones[i].m_linesList.size())
+  for (auto const &info : m_state->m_zones)
+    if (info.m_linesList.size())
       return true;
   return false;
 }
@@ -860,9 +875,9 @@ bool MindWrtParser::sendZone(int id)
   MWAWInputStreamPtr input = getInput();
   libmwaw::DebugStream f;
   if (id==0) m_state->m_actPage = 1;
-  MindWrtParserInternal::ZoneInfo &zone = m_state->m_zones[id];
+  auto &zone = m_state->m_zones[id];
   for (size_t i = 0; i < zone.m_linesList.size(); i++) {
-    MindWrtParserInternal::LineInfo &line = zone.m_linesList[i];
+    auto &line = zone.m_linesList[i];
     if (i==0) {
       ascii().addPos(line.m_entry.begin());
       ascii().addNote("Entries(Text)");
@@ -929,7 +944,7 @@ bool MindWrtParser::readGraphic(MindWrtParserInternal::LineInfo const &line)
 
   MWAWInputStreamPtr input = getInput();
   libmwaw::DebugStream f;
-  int sz = static_cast<int>(line.m_entry.length());
+  auto sz = static_cast<int>(line.m_entry.length());
   if (sz < 10) {
     MWAW_DEBUG_MSG(("MindWrtParser::readGraphic: zone size is two short or odd\n"));
     return false;
@@ -939,12 +954,12 @@ bool MindWrtParser::readGraphic(MindWrtParserInternal::LineInfo const &line)
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
   int dim[4];
-  for (int i = 0; i < 4; i++) dim[i] = static_cast<int>(input->readLong(2));
+  for (auto &d : dim) d = static_cast<int>(input->readLong(2));
   MWAWBox2f box(MWAWVec2f(static_cast<float>(dim[1]),static_cast<float>(dim[0])),
                 MWAWVec2f(static_cast<float>(dim[3]),static_cast<float>(dim[2])));
   f << "Entries(graphic): bdBox=" << box << ",";
 
-  shared_ptr<MWAWPict> pict(MWAWPictData::get(input, sz-8));
+  std::shared_ptr<MWAWPict> pict(MWAWPictData::get(input, sz-8));
   if (!pict) {
     MWAW_DEBUG_MSG(("MindWrtParser::readGraphic: can not read the picture\n"));
     return false;
@@ -987,7 +1002,7 @@ bool MindWrtParser::readRuler(MindWrtParserInternal::LineInfo &line)
     f << "#rightMargin=" << getPageWidth()*72.0-*(para.m_margins[2]);
     para.m_margins[2] = 0.0;
   }
-  long val = long(input->readULong(1));
+  auto val = long(input->readULong(1));
   switch (val) {
   case 0:
     para.m_justify = MWAWParagraph::JustificationLeft;
@@ -1005,7 +1020,7 @@ bool MindWrtParser::readRuler(MindWrtParserInternal::LineInfo &line)
     f << "#align=" << std::hex << val << std::dec << ",";
     break;
   }
-  int N = static_cast<int>(input->readULong(1));
+  auto N = static_cast<int>(input->readULong(1));
   if (line.m_entry.length() != 2*N+10) {
     MWAW_DEBUG_MSG(("MindWrtParser::readRuler: num tabs is incorrect\n"));
     line.m_paragraph = para;
@@ -1067,15 +1082,14 @@ void MindWrtParser::sendHeaderFooter(bool header)
     return;
   }
   int zId=header ? 1 : 2;
-  std::vector<MindWrtParserInternal::Field> const &fieldList=
-    header ? m_state->m_headerFieldList : m_state->m_footerFieldList;
-  if (fieldList.size()) {
+  auto const &fieldList= header ? m_state->m_headerFieldList : m_state->m_footerFieldList;
+  if (!fieldList.empty()) {
     /** field are separated from the main text
         -> we need to use an intermediate frame */
     MWAWPosition fPos(MWAWVec2f(0,0), MWAWVec2f(float(getPageWidth()),0), librevenge::RVNG_INCH);
     fPos.m_anchorTo = MWAWPosition::Paragraph;
     fPos.m_wrapping =  MWAWPosition::WBackground;
-    shared_ptr<MindWrtParserInternal::SubDocument> subDoc
+    std::shared_ptr<MindWrtParserInternal::SubDocument> subDoc
     (new MindWrtParserInternal::SubDocument(*this, getInput(), zId, 1));
     listener->insertTextBox(fPos, subDoc);
   }
@@ -1089,8 +1103,7 @@ void MindWrtParser::sendHeaderFooterFields(bool header)
     MWAW_DEBUG_MSG(("MindWrtParser::sendHeaderFooterFields: can note find the listener\n"));
     return;
   }
-  std::vector<MindWrtParserInternal::Field> fieldList=
-    header ? m_state->m_headerFieldList : m_state->m_footerFieldList;
+  auto fieldList=header ? m_state->m_headerFieldList : m_state->m_footerFieldList;
   size_t numFields=fieldList.size();
   if (!numFields) {
     MWAW_DEBUG_MSG(("MindWrtParser::sendHeaderFooterFields: called with empty field list\n"));
@@ -1103,9 +1116,7 @@ void MindWrtParser::sendHeaderFooterFields(bool header)
       if (diffLine > 0 ||
           (diffLine==0 && fieldList[j].m_pos[0]>fieldList[i].m_pos[0]))
         continue;
-      MindWrtParserInternal::Field tmp=fieldList[j];
-      fieldList[j]=fieldList[i];
-      fieldList[i]=tmp;
+      std::swap(fieldList[i], fieldList[j]);
     }
   }
   if (fieldList[numFields-1].m_pos[1]>10) {
@@ -1164,7 +1175,7 @@ bool MindWrtParser::readText(MindWrtParserInternal::LineInfo const &line)
   long pos=line.m_entry.begin();
   long endPos=line.m_entry.end();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
-  int num = static_cast<int>(input->readULong(2));
+  auto num = static_cast<int>(input->readULong(2));
   if (pos+num >= endPos) {
     MWAW_DEBUG_MSG(("MindWrtParser::readText: numChar is too long\n"));
     return false;
@@ -1193,7 +1204,7 @@ bool MindWrtParser::readText(MindWrtParserInternal::LineInfo const &line)
   }
   std::string text("");
   for (int n = 0; n < num; n++) {
-    char c = char(input->readULong(1));
+    auto c = char(input->readULong(1));
     if (!c) {
       MWAW_DEBUG_MSG(("MindWrtParser::readText: find 0 char\n"));
       continue;
@@ -1231,7 +1242,7 @@ bool MindWrtParser::readCompressedText(MindWrtParserInternal::LineInfo const &li
   long pos=line.m_entry.begin();
   long endPos=line.m_entry.end();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
-  int num = static_cast<int>(input->readULong(2));
+  auto num = static_cast<int>(input->readULong(2));
   if (pos+num/2 > endPos) {
     MWAW_DEBUG_MSG(("MindWrtParser::readCompressedText: numChar is too long\n"));
     return false;
@@ -1335,7 +1346,7 @@ bool MindWrtParser::readFonts(MWAWEntry const &entry, std::vector<MWAWFont> &fon
     MWAW_DEBUG_MSG(("MindWrtParser::readFonts: zone is too short\n"));
     return false;
   }
-  int sz = static_cast<int>(input->readULong(2));
+  auto sz = static_cast<int>(input->readULong(2));
   if (pos+2+sz > endPos || (sz%6)) {
     MWAW_DEBUG_MSG(("MindWrtParser::readFonts: sz is odd\n"));
     return false;
@@ -1349,12 +1360,12 @@ bool MindWrtParser::readFonts(MWAWEntry const &entry, std::vector<MWAWFont> &fon
     pos = input->tell();
     f.str("");
     f << "Text:Font" << i << ":";
-    int tPos = static_cast<int>(input->readULong(2));
+    auto tPos = static_cast<int>(input->readULong(2));
     textPos.push_back(tPos);
     f << "pos=" << tPos << ",";
     MWAWFont font;
     font.setSize(float(input->readULong(1)));
-    int flag = static_cast<int>(input->readULong(1));
+    auto flag = static_cast<int>(input->readULong(1));
     uint32_t flags = 0;
     // bit 1 = plain
     if (flag&0x1) flags |= MWAWFont::boldBit;
@@ -1403,8 +1414,7 @@ bool MindWrtParser::checkHeader(MWAWHeader *header, bool strict)
     header->reset(MWAWDocument::MWAW_T_MINDWRITE, 2);
 
   f << "Entries(Header):";
-  for (int i = 0; i < 3; i++)
-    m_state->m_numLinesByZone[i] = static_cast<int>(input->readULong(2));
+  for (auto &num : m_state->m_numLinesByZone) num = static_cast<int>(input->readULong(2));
   f << "nLines=" << m_state->m_numLinesByZone[0] << ",";
   if (m_state->m_numLinesByZone[1])
     f << "nLines[Head]=" << m_state->m_numLinesByZone[1] << ",";
@@ -1470,9 +1480,9 @@ bool MindWrtParser::checkHeader(MWAWHeader *header, bool strict)
   if (strict) {
     // check the line info block size
     input->seek(0x50, librevenge::RVNG_SEEK_SET);
-    for (int i = 0; i < 3; i++) {
+    for (int i : m_state->m_numLinesByZone) {
       input->seek(4, librevenge::RVNG_SEEK_CUR);
-      if (input->readLong(4) == 32*m_state->m_numLinesByZone[i])
+      if (input->readLong(4) == 32*i)
         continue;
       ok = false;
       break;
@@ -1513,7 +1523,7 @@ bool MindWrtParser::readLinesInfo(MWAWEntry &entry)
   int N = int(entry.length())/32;
 
   libmwaw::DebugStream f;
-  MindWrtParserInternal::ZoneInfo &textZone = m_state->m_zones[entry.id()];
+  auto &textZone = m_state->m_zones[entry.id()];
   textZone.m_linesList.clear();
   for (int n = 0; n < N; n++) {
     MindWrtParserInternal::LineInfo line;
@@ -1528,7 +1538,7 @@ bool MindWrtParser::readLinesInfo(MWAWEntry &entry)
     f << "f0=" << std::hex << val << std::dec << ",";
     for (int i=0; i < 2; i++)
       line.m_flags[i] = static_cast<int>(input->readULong(1));
-    long ptr = long(input->readULong(1));
+    auto ptr = long(input->readULong(1));
     line.m_entry.setBegin((ptr << 16) | long(input->readULong(2)));
     line.m_entry.setLength(long(input->readULong(2)));
     line.m_extra=f.str();
@@ -1551,8 +1561,7 @@ bool MindWrtParser::readLinesInfo(MWAWEntry &entry)
   // first, we must read and update the paragraph
   MWAWParagraph para;
   bool hasSpecialHeadInterface = false;
-  for (size_t n = 0; n < size_t(N); n++) {
-    MindWrtParserInternal::LineInfo &line=textZone.m_linesList[n];
+  for (auto &line : textZone.m_linesList) {
     if (line.m_height || line.m_type) {
       line.m_paragraph=para;
       line.m_specialHeadingInterface = hasSpecialHeadInterface;
@@ -1630,8 +1639,8 @@ bool MindWrtParser::readHeadingStates(MWAWEntry &entry)
   if (unkn!=2) f << "unkn=" << unkn << ",";
   int N=int(entry.length()/2)-1;
   for (int i = 0; i < N; i++) {
-    int fl=static_cast<int>(input->readULong(1));
-    int wh=static_cast<int>(input->readULong(1));
+    auto fl=static_cast<int>(input->readULong(1));
+    auto wh=static_cast<int>(input->readULong(1));
     if (fl==0 && wh==4)
       continue;
     f << "L" << i << "=[";
@@ -1664,7 +1673,7 @@ bool MindWrtParser::readHeadingCustom(MWAWEntry &entry)
 {
   if (!entry.valid())
     return false;
-  int len=int(entry.length());
+  auto len=int(entry.length());
   if (len<16) {
     MWAW_DEBUG_MSG(("MindWrtParser::readHeadingCustom: the size seems odd\n"));
     return false;
@@ -1682,7 +1691,7 @@ bool MindWrtParser::readHeadingCustom(MWAWEntry &entry)
     ascii().addNote(f.str().c_str());
     return false;
   }
-  int N=static_cast<int>(input->readULong(2));
+  auto N=static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   if (len < 16+2*N) {
     MWAW_DEBUG_MSG(("MindWrtParser::readHeadingCustom: N seems bads\n"));
@@ -1692,11 +1701,11 @@ bool MindWrtParser::readHeadingCustom(MWAWEntry &entry)
     return false;
   }
 
-  int val = static_cast<int>(input->readLong(2));
+  auto val = static_cast<int>(input->readLong(2));
   m_state->m_listProperties.m_listLevelsRepeatPos = val;
   if (val) f << "repeatPos=" << val << ",";
-  int debDeplPos = static_cast<int>(input->readLong(2));
-  int debStringPos = static_cast<int>(input->readLong(2));
+  auto debDeplPos = static_cast<int>(input->readLong(2));
+  auto debStringPos = static_cast<int>(input->readLong(2));
   if (debDeplPos+2*N >= len || debStringPos > len) {
     f << "##deplPos=" << debDeplPos << ","
       << "##stringPos=" << debStringPos << ",";
@@ -1730,7 +1739,7 @@ bool MindWrtParser::readHeadingCustom(MWAWEntry &entry)
     MWAWListLevel level;
     bool seeIndex=false;
     for (int s = 0; s < sSz; s++) {
-      char c = char(input->readULong(1));
+      auto c = char(input->readULong(1));
       int unicode;
       str+=c;
       if (c==0)
@@ -1882,7 +1891,7 @@ bool MindWrtParser::readZone8(MWAWEntry &entry)
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   libmwaw::DebugStream f;
   f << "Zone8:";
-  long val = long(input->readULong(1));// fl0=0|1|2c|2d,
+  auto val = long(input->readULong(1));// fl0=0|1|2c|2d,
   if (val)
     f << "fl0=" << std::hex << val << std::dec << ",";
   val = long(input->readULong(1)); // fl1=2b|6b|ab
@@ -1903,8 +1912,7 @@ bool MindWrtParser::readZone8(MWAWEntry &entry)
       f << "f" << i << "=" << val << ",";
   }
   int dim[2];
-  for (int i = 0; i < 2; i++)
-    dim[i] = static_cast<int>(input->readLong(2));
+  for (auto &d : dim) d = static_cast<int>(input->readLong(2));
   f << "dim=" << dim[0] << "x" << dim[1] << ",";
   for (int i = 0; i < 12; i++) { // always 0...
     val = input->readLong(2);
@@ -1936,13 +1944,13 @@ bool MindWrtParser::readHeadingFields(MWAWEntry &entry)
   long pos = entry.begin();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
-  int num = int(entry.length()/12);
+  auto num = int(entry.length()/12);
   if (num != 8) {
     MWAW_DEBUG_MSG(("MindWrtParser::readHeadingFields: the number of fields seems odd\n"));
   }
   libmwaw::DebugStream f;
   long val;
-  static MWAWField::Type const(listType[4])= {
+  static MWAWField::Type const listType[4] = {
     MWAWField::Date, MWAWField::Time, MWAWField::PageNumber, MWAWField::Title
   };
   for (int i = 0; i < num; i++) {
@@ -1950,10 +1958,9 @@ bool MindWrtParser::readHeadingFields(MWAWEntry &entry)
     MindWrtParserInternal::Field field(listType[i%4]);
     f.str("");
     int dim[2];
-    for (int j=0; j < 2; j++)
-      dim[j] = static_cast<int>(input->readLong(j==0 ? 2:4));
+    for (int j=0; j < 2; j++) dim[j] = static_cast<int>(input->readLong(j==0 ? 2:4));
     field.m_pos=MWAWVec2i(dim[1], dim[0]);
-    static int const(expectedVal[3])= {3,12,0};
+    static int const expectedVal[3] = {3,12,0};
     for (int j=0; j < 3; j++) {
       val = input->readLong(2);
       if (val!=expectedVal[j])
@@ -2059,7 +2066,7 @@ void MindWrtParser::sendText(std::string const &text, std::vector<MWAWFont> cons
   for (size_t c = 0; c < numChar; c++) {
     if (actFontId < numFonts && int(c) == textPos[actFontId])
       getTextListener()->setFont(fonts[actFontId++]);
-    unsigned char ch = static_cast<unsigned char>(text[c]);
+    auto ch = static_cast<unsigned char>(text[c]);
     switch (ch) {
     case 0x9:
       getTextListener()->insertTab();
