@@ -62,19 +62,28 @@
 namespace RagTime5LayoutInternal
 {
 //! the layout cluster ( 4001 zone)
-struct ClusterLayout : public RagTime5ClusterManager::Cluster {
+struct ClusterLayout final : public RagTime5ClusterManager::Cluster {
   //! constructor
-  ClusterLayout() : Cluster(C_Layout), m_zoneList(), m_pipelineLink(), m_listItemLink(), m_childIdSet(),
-    m_numChild(0), m_numMasterChild(0)
+  ClusterLayout()
+    : Cluster(C_Layout)
+    , m_zoneList()
+    , m_pipelineLink()
+    , m_listItemLink()
+    , m_childIdSet()
+    , m_numChild(0)
+    , m_numMasterChild(0)
   {
   }
   //! destructor
-  virtual ~ClusterLayout();
+  ~ClusterLayout() final;
 
   //! a zone of in a layout
   struct Zone {
     //! constructor
-    Zone() : m_mainId(0), m_masterId(0), m_dimension()
+    Zone()
+      : m_mainId(0)
+      , m_masterId(0)
+      , m_dimension()
     {
     }
     //! the main zone id
@@ -107,11 +116,17 @@ ClusterLayout::~ClusterLayout()
 //! Internal: the state of a RagTime5Layout
 struct State {
   //! constructor
-  State() : m_numPages(-1), m_idLayoutMap(), m_masterIdSet(), m_layoutIdToSendList() { }
+  State()
+    : m_numPages(-1)
+    , m_idLayoutMap()
+    , m_masterIdSet()
+    , m_layoutIdToSendList()
+  {
+  }
   //! the number of pages
   int m_numPages;
   //! map data id to text zone
-  std::map<int, shared_ptr<ClusterLayout> > m_idLayoutMap;
+  std::map<int, std::shared_ptr<ClusterLayout> > m_idLayoutMap;
   //! the list of master id
   std::set<int> m_masterIdSet;
   //! the list of layout to send
@@ -123,9 +138,11 @@ struct State {
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-RagTime5Layout::RagTime5Layout(RagTime5Parser &parser) :
-  m_mainParser(parser), m_structManager(m_mainParser.getStructManager()),
-  m_parserState(parser.getParserState()), m_state(new RagTime5LayoutInternal::State)
+RagTime5Layout::RagTime5Layout(RagTime5Parser &parser)
+  : m_mainParser(parser)
+  , m_structManager(m_mainParser.getStructManager())
+  , m_parserState(parser.getParserState())
+  , m_state(new RagTime5LayoutInternal::State)
 {
 }
 
@@ -154,21 +171,20 @@ bool RagTime5Layout::sendPageContents()
       MWAW_DEBUG_MSG(("RagTime5Layout::sendPageContents: can not find layout %d\n", lId));
       continue;
     }
-    RagTime5LayoutInternal::ClusterLayout &layout=*m_state->m_idLayoutMap.find(lId)->second;
+    auto &layout=*m_state->m_idLayoutMap.find(lId)->second;
     layout.m_isSent=true;
-    size_t n=layout.m_zoneList.size();
-    for (size_t p=0; p<n; ++p) {
+    for (auto &zone : layout.m_zoneList) {
       MWAWPosition position(MWAWVec2f(0,0), MWAWVec2f(100,100), librevenge::RVNG_POINT);
       position.m_anchorTo=MWAWPosition::Page;
       position.setPage(++page);
-      RagTime5LayoutInternal::ClusterLayout::Zone &zone=layout.m_zoneList[p];
       if (zone.m_masterId) {
         if (m_state->m_idLayoutMap.find(zone.m_masterId)==m_state->m_idLayoutMap.end() || !m_state->m_idLayoutMap.find(zone.m_masterId)->second) {
           MWAW_DEBUG_MSG(("RagTime5Layout::sendPageContents: can not find layout %d\n", zone.m_masterId));
         }
         else {
-          RagTime5LayoutInternal::ClusterLayout &master=*m_state->m_idLayoutMap.find(zone.m_masterId)->second;
+          auto &master=*m_state->m_idLayoutMap.find(zone.m_masterId)->second;
           int cId=0;
+          size_t n=layout.m_zoneList.size();
           if (master.m_zoneList.size()==1)
             cId=master.m_zoneList[0].m_mainId;
           else if (n<master.m_zoneList.size())
@@ -191,35 +207,32 @@ bool RagTime5Layout::sendPageContents()
 ////////////////////////////////////////////////////////////
 void RagTime5Layout::updateLayouts()
 {
-  std::map<int, shared_ptr<RagTime5LayoutInternal::ClusterLayout> >::iterator it;
-  for (it=m_state->m_idLayoutMap.begin(); it!=m_state->m_idLayoutMap.end(); ++it) {
-    if (!it->second) continue;
-    updateLayout(*it->second);
+  for (auto it : m_state->m_idLayoutMap) {
+    if (!it.second) continue;
+    updateLayout(*it.second);
   }
   // look for no master layout
   int nPages=0;
   std::multimap<int,int> numZonesToLayout;
-  for (it=m_state->m_idLayoutMap.begin(); it!=m_state->m_idLayoutMap.end(); ++it) {
-    if (!it->second || it->second->m_zoneList.empty() || m_state->m_masterIdSet.find(it->first)!=m_state->m_masterIdSet.end()) {
-      if (it->second)
-        it->second->m_isSent=true;
+  for (auto it : m_state->m_idLayoutMap) {
+    if (!it.second || it.second->m_zoneList.empty() || m_state->m_masterIdSet.find(it.first)!=m_state->m_masterIdSet.end()) {
+      if (it.second)
+        it.second->m_isSent=true;
       continue;
     }
-    numZonesToLayout.insert(std::multimap<int,int>::value_type(it->second->m_numChild,it->first));
-    nPages+=static_cast<int>(it->second->m_zoneList.size());
+    numZonesToLayout.insert(std::multimap<int,int>::value_type(it.second->m_numChild,it.first));
+    nPages+=static_cast<int>(it.second->m_zoneList.size());
   }
   m_state->m_numPages=nPages;
 
-  std::multimap<int,int>::iterator lIt;
-  for (lIt=numZonesToLayout.end(); lIt!=numZonesToLayout.begin();)
+  for (auto lIt=numZonesToLayout.end(); lIt!=numZonesToLayout.begin();)
     m_state->m_layoutIdToSendList.push_back((--lIt)->second);
 }
 
 void RagTime5Layout::updateLayout(RagTime5LayoutInternal::ClusterLayout &layout)
 {
   int numChild=0, numMasterChild=0;
-  for (size_t i=0; i<layout.m_zoneList.size(); ++i) {
-    RagTime5LayoutInternal::ClusterLayout::Zone &zone=layout.m_zoneList[i];
+  for (auto &zone : layout.m_zoneList) {
     if (zone.m_mainId) {
       if (m_mainParser.getClusterType(zone.m_mainId)==RagTime5ClusterManager::Cluster::C_GraphicZone)
         ++numChild;
@@ -281,10 +294,13 @@ bool RagTime5Layout::send(RagTime5LayoutInternal::ClusterLayout &/*cluster*/, MW
 namespace RagTime5LayoutInternal
 {
 //! Internal: the helper to read a clustList
-struct ClustListParser : public RagTime5StructManager::DataParser {
+struct ClustListParser final : public RagTime5StructManager::DataParser {
   //! constructor
-  ClustListParser(RagTime5ClusterManager &clusterManager, int fieldSize, std::string const &zoneName) :
-    RagTime5StructManager::DataParser(zoneName), m_fieldSize(fieldSize), m_linkList(), m_clusterManager(clusterManager)
+  ClustListParser(RagTime5ClusterManager &clusterManager, int fieldSize, std::string const &zoneName)
+    : RagTime5StructManager::DataParser(zoneName)
+    , m_fieldSize(fieldSize)
+    , m_linkList()
+    , m_clusterManager(clusterManager)
   {
     if (m_fieldSize<4) {
       MWAW_DEBUG_MSG(("RagTime5LayoutInternal::ClustListParser: bad field size\n"));
@@ -292,14 +308,14 @@ struct ClustListParser : public RagTime5StructManager::DataParser {
     }
   }
   //! destructor
-  ~ClustListParser();
+  ~ClustListParser() final;
   //! returns the not null list dataId list
   std::vector<int> getIdList() const
   {
     std::vector<int> res;
-    for (size_t i=0; i<m_linkList.size(); ++i) {
-      if (m_linkList[i].m_dataId>0)
-        res.push_back(m_linkList[i].m_dataId);
+    for (auto const &lnk : m_linkList) {
+      if (lnk.m_dataId>0)
+        res.push_back(lnk.m_dataId);
     }
     return res;
   }
@@ -310,7 +326,7 @@ struct ClustListParser : public RagTime5StructManager::DataParser {
   }
 
   //! try to parse a data
-  bool parseData(MWAWInputStreamPtr &input, long endPos, RagTime5Zone &/*zone*/, int /*n*/, libmwaw::DebugStream &f)
+  bool parseData(MWAWInputStreamPtr &input, long endPos, RagTime5Zone &/*zone*/, int /*n*/, libmwaw::DebugStream &f) final
   {
     long pos=input->tell();
     if (endPos-pos!=m_fieldSize) {
@@ -340,9 +356,9 @@ private:
   //! the main zone manager
   RagTime5ClusterManager &m_clusterManager;
   //! copy constructor, not implemented
-  ClustListParser(ClustListParser &orig);
+  ClustListParser(ClustListParser &orig) = delete;
   //! copy operator, not implemented
-  ClustListParser &operator=(ClustListParser &orig);
+  ClustListParser &operator=(ClustListParser &orig) = delete;
 };
 
 ClustListParser::~ClustListParser()
@@ -352,26 +368,32 @@ ClustListParser::~ClustListParser()
 //
 //! low level: parser of layout cluster
 //
-struct LayoutCParser : public RagTime5ClusterManager::ClusterParser {
+struct LayoutCParser final : public RagTime5ClusterManager::ClusterParser {
   //! constructor
-  LayoutCParser(RagTime5ClusterManager &parser, int zoneType) : ClusterParser(parser, zoneType, "ClustLayout"), m_cluster(new ClusterLayout),
-    m_actualZoneId(0), m_numZones(0), m_what(-1), m_linkId(-1), m_fieldName(""), m_actualZone()
+  LayoutCParser(RagTime5ClusterManager &parser, int zoneType)
+    : ClusterParser(parser, zoneType, "ClustLayout"), m_cluster(new ClusterLayout)
+    , m_actualZoneId(0)
+    , m_numZones(0)
+    , m_what(-1)
+    , m_linkId(-1)
+    , m_fieldName("")
+    , m_actualZone()
   {
   }
   //! destructor
-  virtual ~LayoutCParser();
+  ~LayoutCParser() final;
   //! return the current cluster
-  shared_ptr<RagTime5ClusterManager::Cluster> getCluster()
+  std::shared_ptr<RagTime5ClusterManager::Cluster> getCluster() final
   {
     return m_cluster;
   }
   //! return the current layout cluster
-  shared_ptr<ClusterLayout> getLayoutCluster()
+  std::shared_ptr<ClusterLayout> getLayoutCluster()
   {
     return m_cluster;
   }
   //! start a new zone
-  void startZone()
+  void startZone() final
   {
     if (m_what<=0)
       ++m_what;
@@ -382,7 +404,7 @@ struct LayoutCParser : public RagTime5ClusterManager::ClusterParser {
     }
   }
   //! end of a start zone call
-  void endZone()
+  void endZone() final
   {
     if (m_what==1)
       m_cluster->m_zoneList.push_back(m_actualZone);
@@ -412,7 +434,7 @@ struct LayoutCParser : public RagTime5ClusterManager::ClusterParser {
     }
   }
   //! parse a zone
-  bool parseZone(MWAWInputStreamPtr &input, long fSz, int N, int flag, libmwaw::DebugStream &f)
+  bool parseZone(MWAWInputStreamPtr &input, long fSz, int N, int flag, libmwaw::DebugStream &f) final
   {
     m_fieldName="";
     m_linkId=-1;
@@ -546,7 +568,7 @@ struct LayoutCParser : public RagTime5ClusterManager::ClusterParser {
         if (val) f << "f" << i << "=" << val << ",";
       }
       float dim[2];
-      for (int i=0; i<2; ++i) dim[i]=float(input->readLong(4))/65536.f;
+      for (auto &d : dim) d=float(input->readLong(4))/65536.f;
       f << "sz=" << MWAWVec2f(dim[0],dim[1]) << ",";
 
       std::vector<int> listIds;
@@ -617,23 +639,22 @@ struct LayoutCParser : public RagTime5ClusterManager::ClusterParser {
     return true;
   }
   //! parse a field
-  bool parseField(RagTime5StructManager::Field const &field, int /*m*/, libmwaw::DebugStream &f)
+  bool parseField(RagTime5StructManager::Field const &field, int /*m*/, libmwaw::DebugStream &f) final
   {
     if (!m_fieldName.empty())
       f << m_fieldName << ",";
     switch (m_what) {
     case 0: // main
       if (field.m_type==RagTime5StructManager::Field::T_FieldList && field.m_fileType==0x14b5815) {
-        for (size_t i=0; i<field.m_fieldList.size(); ++i) {
-          RagTime5StructManager::Field const &child=field.m_fieldList[i];
+        for (auto const &child : field.m_fieldList) {
           if (child.m_type==RagTime5StructManager::Field::T_LongList && child.m_fileType==0xcf042) {
             // TODO: storeme
             f << "ids=[";
-            for (size_t j=0; j<child.m_longList.size(); ++j) {
-              if (child.m_longList[j]==0)
+            for (auto val : child.m_longList) {
+              if (val==0)
                 f << "_,";
               else
-                f << child.m_longList[j] << ",";
+                f << val << ",";
             }
             continue;
           }
@@ -662,13 +683,13 @@ struct LayoutCParser : public RagTime5ClusterManager::ClusterParser {
     case 4: // list
       if (field.m_type==RagTime5StructManager::Field::T_LongList && field.m_fileType==0xce842) {
         f << "pos=[";
-        for (size_t i=0; i<field.m_longList.size(); ++i) {
-          if (field.m_longList[i]==0)
+        for (auto val : field.m_longList) {
+          if (val==0)
             f << "_,";
-          else if (field.m_longList[i]>1000)
-            f << std::hex << field.m_longList[i] << std::dec << ",";
+          else if (val>1000)
+            f << std::hex << val << std::dec << ",";
           else
-            f << field.m_longList[i] << ",";
+            f << val << ",";
         }
         f << "],";
         m_link.m_longList=field.m_longList;
@@ -676,11 +697,11 @@ struct LayoutCParser : public RagTime5ClusterManager::ClusterParser {
       }
       if (field.m_type==RagTime5StructManager::Field::T_LongList && field.m_fileType==0xcf042) { // with 3c052
         f << "unkn=[";
-        for (size_t j=0; j<field.m_longList.size(); ++j) {
-          if (field.m_longList[j]==0)
+        for (auto val : field.m_longList) {
+          if (val==0)
             f << "_,";
           else
-            f << field.m_longList[j] << ",";
+            f << val << ",";
         }
         break;
       }
@@ -694,15 +715,14 @@ struct LayoutCParser : public RagTime5ClusterManager::ClusterParser {
       break;
     case 5: // fSz=60
       if (field.m_type==RagTime5StructManager::Field::T_FieldList && field.m_fileType==0x14b4815) {
-        for (size_t i=0; i<field.m_fieldList.size(); ++i) {
-          RagTime5StructManager::Field const &child=field.m_fieldList[i];
+        for (auto const &child : field.m_fieldList) {
           if (child.m_type==RagTime5StructManager::Field::T_LongList && child.m_fileType==0xce842) {
             f << "unkn=[";
-            for (size_t j=0; j<child.m_longList.size(); ++j) {
-              if (child.m_longList[j]==0)
+            for (auto val : child.m_longList) {
+              if (val==0)
                 f << "_,";
               else
-                f << child.m_longList[j] << ",";
+                f << val << ",";
             }
             continue;
           }
@@ -744,7 +764,7 @@ protected:
         f << "f" << i << "=" << val << ",";
     }
     float dim[2];
-    for (int i=0; i<2; ++i) dim[i]=float(input->readLong(4))/65536.f;
+    for (auto &d : dim) d=float(input->readLong(4))/65536.f;
     m_actualZone.m_dimension=MWAWVec2f(dim[0],dim[1]);
     f << "sz=" << MWAWVec2f(dim[0],dim[1]) << ",";
     std::vector<int> listIds;
@@ -850,7 +870,7 @@ protected:
     m_numZones=static_cast<int>(input->readLong(4));
     if (m_numZones!=1)
       f << "num[zone1]=" << m_numZones << ",";
-    long fileType=long(input->readULong(4));
+    auto fileType=long(input->readULong(4));
     if (fileType!=0x14b6052) {
       MWAW_DEBUG_MSG(("RagTime5LayoutInternal::LayoutCParser::parseHeaderZone: find unexpected filetype\n"));
       f << "#fileType0=" << std::hex << fileType << std::dec << ",";
@@ -889,7 +909,7 @@ protected:
         f << "g" << i+1 << "=" << val << ",";
     }
     float dim[4];
-    for (int i=0; i<2; ++i) dim[i]=float(input->readLong(4))/65536.f;
+    for (auto &d : dim) d=float(input->readLong(4))/65536.f;
     MWAWVec2f frameSize(dim[0],dim[1]);
     f << "sz=" << frameSize << ",";
     for (int i=0; i<2; ++i) dim[i]=float(input->readLong(4))/65536.f;
@@ -900,7 +920,7 @@ protected:
       if (val)
         f << "g" << i+3 << "=" << val << ",";
     }
-    for (int i=0; i<4; ++i) dim[i]=float(input->readLong(4))/65536.f;
+    for (auto &d : dim) d=float(input->readLong(4))/65536.f;
     f << "dim=" << MWAWBox2f(MWAWVec2f(dim[0],dim[1]), MWAWVec2f(dim[2],dim[3])) << ",";
     for (int i=0; i<4; ++i) { // h3=0|1|3|6
       val=static_cast<int>(input->readLong(2));
@@ -918,7 +938,7 @@ protected:
   }
 
   //! the current cluster
-  shared_ptr<ClusterLayout> m_cluster;
+  std::shared_ptr<ClusterLayout> m_cluster;
   //! the actual zone id
   int m_actualZoneId;
   //! the number of zones
@@ -940,20 +960,20 @@ LayoutCParser::~LayoutCParser()
 
 }
 
-shared_ptr<RagTime5ClusterManager::Cluster> RagTime5Layout::readLayoutCluster(RagTime5Zone &zone, int zoneType)
+std::shared_ptr<RagTime5ClusterManager::Cluster> RagTime5Layout::readLayoutCluster(RagTime5Zone &zone, int zoneType)
 {
-  shared_ptr<RagTime5ClusterManager> clusterManager=m_mainParser.getClusterManager();
+  auto clusterManager=m_mainParser.getClusterManager();
   if (!clusterManager) {
     MWAW_DEBUG_MSG(("RagTime5Layout::readLayoutCluster: oops can not find the cluster manager\n"));
-    return shared_ptr<RagTime5ClusterManager::Cluster>();
+    return std::shared_ptr<RagTime5ClusterManager::Cluster>();
   }
   RagTime5LayoutInternal::LayoutCParser parser(*clusterManager, zoneType);
   if (!clusterManager->readCluster(zone, parser) || !parser.getLayoutCluster()) {
     MWAW_DEBUG_MSG(("RagTime5Layout::readLayoutCluster: oops can not find the cluster\n"));
-    return shared_ptr<RagTime5ClusterManager::Cluster>();
+    return std::shared_ptr<RagTime5ClusterManager::Cluster>();
   }
 
-  shared_ptr<RagTime5LayoutInternal::ClusterLayout> cluster=parser.getLayoutCluster();
+  auto cluster=parser.getLayoutCluster();
   if (!cluster->m_listItemLink.empty())
     m_mainParser.readFixedSizeZone(cluster->m_listItemLink, "LayoutUnknown0");
   if (!cluster->m_pipelineLink.empty() && cluster->m_pipelineLink.m_ids.size()==1) {
@@ -969,11 +989,11 @@ shared_ptr<RagTime5ClusterManager::Cluster> RagTime5Layout::readLayoutCluster(Ra
   }
   // can have some setting
   for (int wh=0; wh<2; ++wh) {
-    std::vector<RagTime5ClusterManager::Link> const &list=wh==0 ? cluster->m_conditionFormulaLinks : cluster->m_settingLinks;
-    for (size_t i=0; i<list.size(); ++i) {
-      if (list[i].empty()) continue;
+    auto const &list=wh==0 ? cluster->m_conditionFormulaLinks : cluster->m_settingLinks;
+    for (auto const &lnk : list) {
+      if (lnk.empty()) continue;
       RagTime5ClusterManager::Cluster unknCluster(RagTime5ClusterManager::Cluster::C_Unknown);
-      unknCluster.m_dataLink=list[i];
+      unknCluster.m_dataLink=lnk;
       RagTime5StructManager::FieldParser defaultParser(wh==0 ? "CondFormula" : "Settings");
       m_mainParser.readStructZone(unknCluster, defaultParser, 0);
     }
@@ -984,8 +1004,7 @@ shared_ptr<RagTime5ClusterManager::Cluster> RagTime5Layout::readLayoutCluster(Ra
     m_mainParser.readUnicodeStringList(cluster->m_nameLink, idToStringMap);
   }
 
-  for (size_t i=0; i<cluster->m_linksList.size(); ++i) {
-    RagTime5ClusterManager::Link const &lnk=cluster->m_linksList[i];
+  for (auto const &lnk : cluster->m_linksList) {
     if (lnk.m_type==RagTime5ClusterManager::Link::L_List) {
       m_mainParser.readListZone(lnk);
       continue;
