@@ -35,6 +35,7 @@
 #include <iomanip>
 #include <iostream>
 #include <limits>
+#include <memory>
 #include <sstream>
 
 #include <librevenge/librevenge.h>
@@ -47,6 +48,7 @@
 #include "StarObject.hxx"
 #include "StarObjectChart.hxx"
 #include "StarObjectDraw.hxx"
+#include "StarObjectMath.hxx"
 #include "StarObjectSpreadsheet.hxx"
 #include "StarObjectText.hxx"
 #include "StarItemPool.hxx"
@@ -296,7 +298,7 @@ StarFileManager::~StarFileManager()
 {
 }
 
-bool StarFileManager::readOLEDirectory(std::shared_ptr<STOFFOLEParser> oleParser, std::shared_ptr<STOFFOLEParser::OleDirectory> ole, STOFFEmbeddedObject &image)
+bool StarFileManager::readOLEDirectory(std::shared_ptr<STOFFOLEParser> oleParser, std::shared_ptr<STOFFOLEParser::OleDirectory> ole, STOFFEmbeddedObject &image, std::shared_ptr<StarObject> &res)
 {
   image=STOFFEmbeddedObject();
   if (!oleParser || !ole || ole->m_inUse) {
@@ -306,9 +308,10 @@ bool StarFileManager::readOLEDirectory(std::shared_ptr<STOFFOLEParser> oleParser
   ole->m_inUse=true;
   StarObject object(nullptr, oleParser, ole); // do we need password here ?
   if (object.getDocumentKind()==STOFFDocument::STOFF_K_CHART) {
-    // TODO: retrieve content
-    StarObjectChart chart(object, false);
-    chart.parse();
+    auto chart=std::make_shared<StarObjectChart>(object, false);
+    ole->m_parsed=true;
+    if (chart->parse())
+      res=chart;
   }
   else if (object.getDocumentKind()==STOFFDocument::STOFF_K_DRAW) {
     StarObjectDraw draw(object, false);
@@ -329,6 +332,12 @@ bool StarFileManager::readOLEDirectory(std::shared_ptr<STOFFOLEParser> oleParser
       graphicEncoder.getBinaryResult(image);
     }
   }
+  else if (object.getDocumentKind()==STOFFDocument::STOFF_K_MATH) {
+    auto math=std::make_shared<StarObjectMath>(object, false);
+    ole->m_parsed=true;
+    if (math->parse())
+      res=math;
+  }
   else if (object.getDocumentKind()==STOFFDocument::STOFF_K_SPREADSHEET) {
     StarObjectSpreadsheet spreadsheet(object, false);
     ole->m_parsed=true;
@@ -341,7 +350,8 @@ bool StarFileManager::readOLEDirectory(std::shared_ptr<STOFFOLEParser> oleParser
         ps.m_pageSpan=1;
         pageList.push_back(ps);
       }
-      STOFFSpreadsheetListenerPtr spreadsheetListener(new STOFFSpreadsheetListener(STOFFListManagerPtr(), pageList, &spreadsheetEncoder));
+      STOFFListManagerPtr listManager;
+      STOFFSpreadsheetListenerPtr spreadsheetListener(new STOFFSpreadsheetListener(listManager, pageList, &spreadsheetEncoder));
       spreadsheetListener->startDocument();
       spreadsheet.send(spreadsheetListener);
       spreadsheetListener->endDocument();
@@ -349,9 +359,10 @@ bool StarFileManager::readOLEDirectory(std::shared_ptr<STOFFOLEParser> oleParser
     }
   }
   else if (object.getDocumentKind()==STOFFDocument::STOFF_K_TEXT) {
-    // TODO: retrieve content
-    StarObjectText text(object, false);
-    text.parse();
+    auto text=std::make_shared<StarObjectText>(object, false);
+    ole->m_parsed=true;
+    if (text->parse())
+      res=text;
   }
   else {
     ole->m_parsed=true;
@@ -385,10 +396,6 @@ bool StarFileManager::readOLEDirectory(std::shared_ptr<STOFFOLEParser> oleParser
           image.add(data);
         continue;
       }
-      if (base=="StarMathDocument") {
-        readMathDocument(stream,name,object);
-        continue;
-      }
       // other
       if (base=="Ole-Object") {
         librevenge::RVNGBinaryData data;
@@ -446,6 +453,11 @@ void StarFileManager::checkUnparsed(STOFFInputStreamPtr input, std::shared_ptr<S
       draw.parse();
       continue;
     }
+    if (object.getDocumentKind()==STOFFDocument::STOFF_K_MATH) {
+      StarObjectMath math(object, false);
+      math.parse();
+      continue;
+    }
     if (object.getDocumentKind()==STOFFDocument::STOFF_K_SPREADSHEET) {
       StarObjectSpreadsheet spreadsheet(object, false);
       spreadsheet.parse();
@@ -481,10 +493,6 @@ void StarFileManager::checkUnparsed(STOFFInputStreamPtr input, std::shared_ptr<S
         readImageDocument(ole,data,name);
         continue;
       }
-      if (base=="StarMathDocument") {
-        readMathDocument(ole,name,object);
-        continue;
-      }
       if (base.compare(0,3,"Pic")==0) {
         librevenge::RVNGBinaryData data;
         std::string type;
@@ -592,9 +600,7 @@ try
       return false;
     }
     int32_t nType;
-    uint16_t nMapMode;
-    int32_t sizeX, sizeY, nOffsX, nOffsY, nScaleNumX, nScaleDenomX, nScaleNumY, nScaleDenomY;
-    bool mbSimple;
+    int32_t sizeX, sizeY;
     *input >> nType >> nLen >> sizeX>> sizeY;
     if (nType)
       f << "type=" << nType << ",";
@@ -605,6 +611,9 @@ try
       STOFF_DEBUG_MSG(("StarFileManager::readEmbeddedPicture: can not open the mapmod header\n"));
     }
     else {
+      uint16_t nMapMode;
+      int32_t nOffsX, nOffsY, nScaleNumX, nScaleDenomX, nScaleNumY, nScaleDenomY;
+      bool mbSimple;
       *input >> nMapMode >> nOffsX>> nOffsY;
       if (nMapMode) f << "mapMode=" << nMapMode << ",";
       if (nOffsX || nOffsY)
@@ -643,7 +652,7 @@ try
     return false;
   }
   long pictPos=input->tell();
-  int header=int(input->readULong(2));
+  auto header=int(input->readULong(2));
   input->seek(pictPos, librevenge::RVNG_SEEK_SET);
   std::string extension("pict");
   if (header==0x4142 || header==0x4d42) {
@@ -732,166 +741,6 @@ bool StarFileManager::readOleObject(STOFFInputStreamPtr input, librevenge::RVNGB
   return true;
 }
 
-bool StarFileManager::readMathDocument(STOFFInputStreamPtr input, std::string const &fileName, StarObject &/*doc*/)
-try
-{
-  // checkme this zone can not be encoded
-  StarZone zone(input, fileName, "MathDocument", nullptr); // use encoding RTL_TEXTENCODING_MS_1252
-  libstoff::DebugFile &ascii=zone.ascii();
-  ascii.open(fileName);
-
-  input->seek(0, librevenge::RVNG_SEEK_SET);
-  libstoff::DebugStream f;
-  f << "Entries(SMMathDocument):";
-  // starmath_document.cxx Try3x
-  uint32_t lIdent, lVersion;
-  *input >> lIdent;
-  if (lIdent==0x534d3330 || lIdent==0x30333034) {
-    input->setReadInverted(input->readInverted());
-    input->seek(0, librevenge::RVNG_SEEK_SET);
-    *input >> lIdent;
-  }
-  if (lIdent!=0x30334d53L && lIdent!=0x34303330L) {
-    STOFF_DEBUG_MSG(("StarFileManager::readMathDocument: find unexpected header\n"));
-    f << "###header";
-    ascii.addPos(0);
-    ascii.addNote(f.str().c_str());
-    return true;
-  }
-  *input >> lVersion;
-  f << "vers=" << std::hex << lVersion << std::dec << ",";
-  ascii.addPos(0);
-  ascii.addNote(f.str().c_str());
-  std::vector<uint32_t> text;
-  while (!input->isEnd()) {
-    long pos=input->tell();
-    int8_t cTag;
-    *input>>cTag;
-    f.str("");
-    f << "SMMathDocument[" << char(cTag) << "]:";
-    bool done=true, isEnd=false;;
-    switch (cTag) {
-    case 0:
-      isEnd=true;
-      break;
-    case 'T':
-      if (!zone.readString(text)) {
-        done=false;
-        break;
-      }
-      f << libstoff::getString(text).cstr();
-      break;
-    case 'D':
-      for (int i=0; i<4; ++i) {
-        if (!zone.readString(text)) {
-          STOFF_DEBUG_MSG(("StarFileManager::readMathDocument: can not read a string\n"));
-          f << "###string" << i << ",";
-          done=false;
-          break;
-        }
-        if (!text.empty()) f << "str" << i << "=" << libstoff::getString(text).cstr() << ",";
-        if (i==1 || i==2) {
-          uint32_t date, time;
-          *input >> date >> time;
-          f << "date" << i << "=" << date << ",";
-          f << "time" << i << "=" << time << ",";
-        }
-      }
-      break;
-    case 'F': {
-      // starmath_format.cxx operator>>
-      uint16_t n, vLeftSpace, vRightSpace, vTopSpace, vDist, vSize;
-      *input>>n>>vLeftSpace>>vRightSpace;
-      f << "baseHeight=" << (n&0xff) << ",";
-      if (n&0x100) f << "isTextMode,";
-      if (n&0x200) f << "bScaleNormalBracket,";
-      if (vLeftSpace) f << "vLeftSpace=" << vLeftSpace << ",";
-      if (vRightSpace) f << "vRightSpace=" << vRightSpace << ",";
-      f << "size=[";
-      for (int i=0; i<=4; ++i) {
-        *input>>vSize;
-        f << vSize << ",";
-      }
-      f << "],";
-      *input>>vTopSpace;
-      if (vTopSpace) f << "vTopSpace=" << vTopSpace << ",";
-      f << "fonts=[";
-      for (int i=0; i<=6; ++i) {
-        // starmath_utility.cxx operator>>(..., SmFace)
-        uint32_t nData1, nData2, nData3, nData4;
-        f << "[";
-        if (input->isEnd() || !zone.readString(text)) {
-          STOFF_DEBUG_MSG(("StarFileManager::readMathDocument: can not read a font string\n"));
-          f << "###string,";
-          done=false;
-          break;
-        }
-        f << libstoff::getString(text).cstr() << ",";
-        *input >> nData1 >> nData2 >> nData3 >> nData4;
-        if (nData1) f << "familly=" << nData1 << ",";
-        if (nData2) f << "encoding=" << nData2 << ",";
-        if (nData3) f << "weight=" << nData3 << ",";
-        if (nData4) f << "bold=" << nData4 << ",";
-        f << "],";
-      }
-      f << "],";
-      if (!done) break;
-      if (input->tell()+21*2 > input->size()) {
-        STOFF_DEBUG_MSG(("StarFileManager::readMathDocument: zone is too short\n"));
-        f << "###short,";
-        done=false;
-        break;
-      }
-      f << "vDist=[";
-      for (int i=0; i<=18; ++i) {
-        *input >> vDist;
-        if (vDist)
-          f << vDist << ",";
-        else
-          f << "_,";
-      }
-      f << "],";
-      *input >> n >> vDist;
-      f << "vers=" << (n>>8) << ",";
-      f << "eHorAlign=" << (n&0xFF) << ",";
-      if (vDist) f << "bottomSpace=" << vDist << ",";
-      break;
-    }
-    case 'S': {
-      if (!zone.readString(text)) {
-        STOFF_DEBUG_MSG(("StarFileManager::readMathDocument: can not read a string\n"));
-        f << "###string,";
-        done=false;
-        break;
-      }
-      f << libstoff::getString(text).cstr() << ",";
-      uint16_t n;
-      *input>>n;
-      if (n) f << "n=" << n << ",";
-      break;
-    }
-    default:
-      STOFF_DEBUG_MSG(("StarFileManager::readMathDocument: find unexpected type\n"));
-      f << "###type,";
-      done=false;
-      break;
-    }
-    ascii.addPos(pos);
-    ascii.addNote(f.str().c_str());
-    if (!done) {
-      ascii.addDelimiter(input->tell(),'|');
-      break;
-    }
-    if (isEnd)
-      break;
-  }
-  return true;
-}
-catch (...)
-{
-  return false;
-}
-
 bool StarFileManager::readOutPlaceObject(STOFFInputStreamPtr input, libstoff::DebugFile &ascii)
 {
   input->seek(0, librevenge::RVNG_SEEK_SET);
@@ -1007,7 +856,7 @@ bool StarFileManager::readJobSetUp(StarZone &zone, bool useJobLen)
   // sfx2_printer.cxx: SfxPrinter::Create
   // jobset.cxx: JobSetup operator>>
   long lastPos=zone.getRecordLastPosition();
-  int len=int(input->readULong(2));
+  auto len=int(input->readULong(2));
   f << "nLen=" << len << ",";
   if (len==0) {
     ascFile.addPos(pos);
@@ -1033,12 +882,12 @@ bool StarFileManager::readJobSetUp(StarZone &zone, bool useJobLen)
       int const length=(i==0 ? 64 : 32);
       std::string name("");
       for (int c=0; c<length; ++c) {
-        char ch=char(input->readULong(1));
+        auto ch=char(input->readULong(1));
         if (ch==0) break;
         name+=ch;
       }
       if (!name.empty()) {
-        static char const *(wh[])= { "printerName", "deviceName", "portName", "driverName" };
+        static char const *wh[]= { "printerName", "deviceName", "portName", "driverName" };
         f << wh[i] << "=" << name << ",";
       }
       input->seek(actPos+length, librevenge::RVNG_SEEK_SET);
@@ -1229,7 +1078,7 @@ bool StarFileManager::readSVGDI(StarZone &zone)
       f << (type==12 ? "polypoly" : type==1024 ? "transparent[comment]" :
             type==1025 ? "hatch[comment]" : "gradient[comment]") << ",";
       *input >> nTmp;
-      for (int poly=0; poly<int(nTmp); ++poly) {
+      for (int poly=0; poly<int(nTmp) && !input->isEnd(); ++poly) {
         *input >> nTmp1;
         if (nTmp1<0 || nTmp1>(endDataPos-input->tell())/8) {
           STOFF_DEBUG_MSG(("StarFileManager::readSVGDI: bad number of points\n"));
@@ -1382,7 +1231,7 @@ bool StarFileManager::readSVGDI(StarZone &zone)
       }
       std::string name("");
       for (int c=0; c<32; ++c) {
-        char ch=char(input->readULong(1));
+        auto ch=char(input->readULong(1));
         if (!ch) break;
         name+=ch;
       }
@@ -1452,7 +1301,7 @@ bool StarFileManager::readSVGDI(StarZone &zone)
         break;
       case 3:
         *input >> nTmp;
-        for (int poly=0; poly<int(nTmp); ++poly) {
+        for (int poly=0; poly<int(nTmp) && !input->isEnd(); ++poly) {
           *input >> nTmp1;
           if (nTmp1<0 || nTmp1>(endDataPos-input->tell())/8) {
             STOFF_DEBUG_MSG(("StarFileManager::readSVGDI: bad number of points\n"));
