@@ -47,6 +47,7 @@
 #include "StarGraphicStruct.hxx"
 #include "StarLanguage.hxx"
 #include "StarObject.hxx"
+#include "StarObjectModel.hxx"
 #include "StarState.hxx"
 #include "StarWriterStruct.hxx"
 #include "StarZone.hxx"
@@ -65,10 +66,10 @@ FormatDef::~FormatDef()
 
 void FormatDef::updateState(StarState &state) const
 {
-  for (size_t i=0; i<m_attributeList.size(); ++i) {
-    if (!m_attributeList[i].m_attribute)
+  for (auto const &attr : m_attributeList) {
+    if (!attr.m_attribute)
       continue;
-    m_attributeList[i].m_attribute->addTo(state);
+    attr.m_attribute->addTo(state);
   }
 }
 
@@ -78,17 +79,25 @@ bool FormatDef::send(STOFFListenerPtr listener, StarState &state) const
     STOFF_DEBUG_MSG(("StarFormatManagerInternal::FormatDef::send: call without listener\n"));
     return false;
   }
-  bool done=false;
-  for (size_t i=0; i<m_attributeList.size(); ++i) {
-    if (!m_attributeList[i].m_attribute)
+  std::shared_ptr<StarAttribute> attrib;
+  for (auto const &attr : m_attributeList) {
+    if (!attr.m_attribute)
       continue;
-    m_attributeList[i].m_attribute->addTo(state);
+    attr.m_attribute->addTo(state);
     if (!state.m_content) continue;
-    done=true;
-    m_attributeList[i].m_attribute->send(listener, state);
+    if (!attrib)
+      attrib=attr.m_attribute;
+    else {
+      STOFF_DEBUG_MSG(("StarFormatManagerInternal::FormatDef::send: oops find already a content attribute\n"));
+    }
+    state.m_content=false;
   }
-  if (!done) {
-    STOFF_DEBUG_MSG(("StarFormatManagerInternal::FormatDef::send: can not find and data to send\n"));
+  if (attrib)
+    attrib->send(listener, state);
+  else if (state.m_global->m_objectModel && m_values[2]>=0)
+    state.m_global->m_objectModel->sendObject(m_values[2], listener, state);
+  else {
+    STOFF_DEBUG_MSG(("StarFormatManagerInternal::FormatDef::send: can not find any data to send %d\n", m_values[2]));
   }
   return true;
 }
@@ -101,7 +110,7 @@ void FormatDef::printData(libstoff::DebugStream &o) const
   }
   if (m_values[0]) o << "derived=" << m_values[0] << ",";
   if (m_values[1]!=0xFFFF) o << "poolId=" << m_values[1] << ",";
-  if (m_values[2]) o << "objRef=" << m_values[2] << ",";
+  if (m_values[2]>=0) o << "objRef=" << m_values[2] << ",";
 }
 
 //------------------------------------------------------------
@@ -110,7 +119,9 @@ struct NumberFormatter {
   //! struct use to store small format item
   struct FormatItem {
     //! constructor
-    FormatItem() : m_text(""), m_type(0)
+    FormatItem()
+      : m_text("")
+      , m_type(0)
     {
     }
     //! try to update the cell's formating
@@ -122,16 +133,16 @@ struct NumberFormatter {
       if (!item.m_text.empty()) o << item.m_text.cstr();
       o << ":";
       if (item.m_type<0 && item.m_type>=-21) {
-        char const *(wh[])= {"", "string", "del", "blank", "star", "digit",
-                             "decimSep", "thSep", "exp", "frac", "empty",
-                             "fracBlank", "currency", "currDel", "currext",
-                             "calendar", "caldel", "dateSep", "timeSep",
-                             "time100secSep", "percent", "frac_fdiv"
-                            };
+        char const *wh[]= {"", "string", "del", "blank", "star", "digit",
+                           "decimSep", "thSep", "exp", "frac", "empty",
+                           "fracBlank", "currency", "currDel", "currext",
+                           "calendar", "caldel", "dateSep", "timeSep",
+                           "time100secSep", "percent", "frac_fdiv"
+                          };
         o << wh[-item.m_type];
       }
       else if (item.m_type>0 && item.m_type<50) {
-        char const *(wh[])= {
+        char const *wh[]= {
           "", "E", "AMPM", "AP", "MI", "MMI", "M", "MM", // exp, 2a/p, 2min, 2month
           "MMM", "MMMM", "H", "HH", "S", "SS", "Q", "QQ", // 2month, 2hours, 2sec, 2quarter
           "D", "DD", "DDD", "DDDD", "YY", "YYYY", "NN", "NNNN", //4 day, 2 year, 2 day of week
@@ -153,7 +164,15 @@ struct NumberFormatter {
   //! struct use to store different local format
   struct Format {
     //! constructor
-    Format() : m_itemList(), m_type(0), m_hasThousandSep(false), m_prefix(0), m_postfix(0), m_exponential(0), m_thousand(0), m_colorName("")
+    Format()
+      : m_itemList()
+      , m_type(0)
+      , m_hasThousandSep(false)
+      , m_prefix(0)
+      , m_postfix(0)
+      , m_exponential(0)
+      , m_thousand(0)
+      , m_colorName("")
     {
     }
     //! operator<<
@@ -167,8 +186,8 @@ struct NumberFormatter {
       if (form.m_hasThousandSep) o << "thousand[digits]=" << form.m_thousand << ",";
       if (!form.m_itemList.empty()) {
         o << "item=[";
-        for (size_t i=0; i<form.m_itemList.size(); ++i)
-          o << form.m_itemList[i] << ",";
+        for (const auto &i : form.m_itemList)
+          o << i << ",";
         o << "],";
       }
       if (!form.m_colorName.empty()) o << "color[name]=" << form.m_colorName.cstr() << ",";
@@ -194,7 +213,13 @@ struct NumberFormatter {
     librevenge::RVNGString m_colorName;
   };
   //! constructor
-  NumberFormatter() : m_format(""), m_language(0), m_type(0), m_isStandart(false), m_isUsed(false), m_extra("")
+  NumberFormatter()
+    : m_format("")
+    , m_language(0)
+    , m_type(0)
+    , m_isStandart(false)
+    , m_isUsed(false)
+    , m_extra("")
   {
     for (int i=0; i<2; ++i) {
       m_limits[i]=0;
@@ -235,9 +260,10 @@ struct NumberFormatter {
     if (!form.m_isUsed) o << "unused,";
     for (int i=0; i<2; ++i) {
       if (!form.m_limitsOp[i]) continue;
-      char const *(wh[])= {"none", "=", "<>", "<", "<=", ">", ">="};
-      if (form.m_limitsOp[i]>0 && form.m_limitsOp[i]<=6)
+      if (form.m_limitsOp[i]>0 && form.m_limitsOp[i]<=6) {
+        char const *wh[]= {"none", "=", "<>", "<", "<=", ">", ">="};
         o << "lim" << i << "=[" << wh[form.m_limitsOp[i]] << form.m_limits[i] << "],";
+      }
       else {
         STOFF_DEBUG_MSG(("StarFormatManagerInternal::NumberFormatter::operator<< unknown limit op\n"));
         o << "lim" << i << "=###" << form.m_limitsOp[i] << ":" << form.m_limits[i] << ",";
@@ -283,7 +309,7 @@ bool NumberFormatter::FormatItem::updateNumberingProperties(librevenge::RVNGProp
   switch (m_type) {
   case -3: {
     if (m_text.empty()) break;
-    int fChar=int(m_text.cstr()[0]);
+    auto fChar=int(m_text.cstr()[0]);
     if (fChar>=32) {
       static int cCharWidths[ 128-32 ] = {
         1,1,1,2,2,3,2,1,1,1,1,2,1,1,1,1,
@@ -372,7 +398,7 @@ bool NumberFormatter::FormatItem::updateNumberingProperties(librevenge::RVNGProp
   case 26: // nnn
   case 41: // aaa
     list.insert("number:style", "long");
-  // fall through expected
+    STOFF_FALLTHROUGH;
   case 19: // dddd
   case 22: // nn
   case 40: // aa
@@ -429,19 +455,18 @@ bool NumberFormatter::FormatItem::updateNumberingProperties(librevenge::RVNGProp
 bool NumberFormatter::Format::updateNumberingProperties(STOFFCell &cell, std::string const &language, std::string const &country) const
 {
   if (m_type==0 || m_type==0x800) return false;
-  librevenge::RVNGPropertyList &propList=cell.getNumberingStyle();
+  auto &propList=cell.getNumberingStyle();
   librevenge::RVNGPropertyListVector pVect;
   STOFFCell::Format format=cell.getFormat();
+  bool done=true;
   if (m_type&6) {
     format.m_format=(m_type&6)==6 ? STOFFCell::F_DATETIME : (m_type&6)==2 ? STOFFCell::F_DATE : STOFFCell::F_TIME;
-    for (size_t i=0; i<m_itemList.size(); ++i) {
-      if (!m_itemList[i].updateNumberingProperties(pVect))
+    for (auto const &item : m_itemList) {
+      if (!item.updateNumberingProperties(pVect))
         return false;
     }
     propList.insert("librevenge:value-type", (m_type&6)==4 ? "time" : "date");
     propList.insert("librevenge:format", pVect);
-    cell.setFormat(format);
-    return true;
   }
   else if (m_type&8) {
     format.m_format=STOFFCell::F_NUMBER;
@@ -449,9 +474,9 @@ bool NumberFormatter::Format::updateNumberingProperties(STOFFCell &cell, std::st
     librevenge::RVNGString currency("");
     // look for currency
     int nString=0;
-    for (size_t i=0; i<m_itemList.size(); ++i) {
-      if (m_itemList[i].m_type!=-1) continue;
-      currency=m_itemList[i].m_text;
+    for (auto const &item : m_itemList) {
+      if (item.m_type!=-1) continue;
+      currency=item.m_text;
       ++nString;
     }
     if (nString!=1 || currency.empty()) currency="$";
@@ -469,8 +494,6 @@ bool NumberFormatter::Format::updateNumberingProperties(STOFFCell &cell, std::st
     if (m_hasThousandSep) list.insert("number:grouping", true);
     pVect.append(list);
     propList.insert("librevenge:format", pVect);
-    cell.setFormat(format);
-    return true;
   }
   else if (m_type&0x10) {
     format.m_format=STOFFCell::F_NUMBER;
@@ -484,8 +507,6 @@ bool NumberFormatter::Format::updateNumberingProperties(STOFFCell &cell, std::st
       prefix-=3;
     }
     if (prefix>0) propList.insert("number:min-integer-digits", prefix);
-    cell.setFormat(format);
-    return true;
   }
   else if (m_type&0x20) {
     format.m_format=STOFFCell::F_NUMBER;
@@ -495,8 +516,6 @@ bool NumberFormatter::Format::updateNumberingProperties(STOFFCell &cell, std::st
     if (m_prefix) propList.insert("number:min-integer-digits", m_prefix);
     if (m_exponential) propList.insert("number:min-exponent-digits", m_exponential);
     if (m_hasThousandSep) propList.insert("number:grouping", true);
-    cell.setFormat(format);
-    return true;
   }
   else if (m_type&0x40) {
     format.m_format=STOFFCell::F_NUMBER;
@@ -506,8 +525,6 @@ bool NumberFormatter::Format::updateNumberingProperties(STOFFCell &cell, std::st
     propList.insert("number:min-denominator-digits", m_exponential ? m_exponential : 1);
     if (m_prefix) propList.insert("number:min-integer-digits", m_prefix);
     if (m_hasThousandSep) propList.insert("number:grouping", true);
-    cell.setFormat(format);
-    return true;
   }
   else if (m_type&0x80) {
     format.m_format=STOFFCell::F_NUMBER;
@@ -516,21 +533,19 @@ bool NumberFormatter::Format::updateNumberingProperties(STOFFCell &cell, std::st
     propList.insert("number:decimal-places", m_postfix);
     if (m_prefix) propList.insert("number:min-integer-digits", m_prefix);
     if (m_hasThousandSep) propList.insert("number:grouping", true);
-    cell.setFormat(format);
-    return true;
   }
-  else if (m_type&0x100)
+  else if (m_type&0x100) {
     format.m_format=STOFFCell::F_TEXT;
+    done = false;
+  }
   else if (m_type&0x400) {
     format.m_format=STOFFCell::F_BOOLEAN;
     propList.insert("librevenge:value-type", "boolean");
-    cell.setFormat(format);
-    return true;
   }
   else
     return false;
   cell.setFormat(format);
-  return false;
+  return done;
 }
 
 ////////////////////////////////////////
@@ -545,14 +560,15 @@ struct State {
   //! a map id to number format
   std::map<unsigned, NumberFormatter> m_idNumberFormatMap;
   //! a map name to format definition
-  std::map<librevenge::RVNGString, shared_ptr<StarFormatManagerInternal::FormatDef> > m_nameToFormatDefMap;
+  std::map<librevenge::RVNGString, std::shared_ptr<StarFormatManagerInternal::FormatDef> > m_nameToFormatDefMap;
 };
 }
 
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-StarFormatManager::StarFormatManager() : m_state(new StarFormatManagerInternal::State)
+StarFormatManager::StarFormatManager()
+  : m_state(new StarFormatManagerInternal::State)
 {
 }
 
@@ -560,7 +576,7 @@ StarFormatManager::~StarFormatManager()
 {
 }
 
-void StarFormatManager::storeSWFormatDef(librevenge::RVNGString const &name, shared_ptr<StarFormatManagerInternal::FormatDef> &format)
+void StarFormatManager::storeSWFormatDef(librevenge::RVNGString const &name, std::shared_ptr<StarFormatManagerInternal::FormatDef> &format)
 {
   if (m_state->m_nameToFormatDefMap.find(name)!=m_state->m_nameToFormatDefMap.end()) {
     STOFF_DEBUG_MSG(("StarFormatManager::getSWFormatDef: a format with name %s is already defined\n", name.cstr()));
@@ -569,14 +585,14 @@ void StarFormatManager::storeSWFormatDef(librevenge::RVNGString const &name, sha
     m_state->m_nameToFormatDefMap[name]=format;
 }
 
-shared_ptr<StarFormatManagerInternal::FormatDef> StarFormatManager::getSWFormatDef(librevenge::RVNGString const &name) const
+std::shared_ptr<StarFormatManagerInternal::FormatDef> StarFormatManager::getSWFormatDef(librevenge::RVNGString const &name) const
 {
   if (m_state->m_nameToFormatDefMap.find(name)!=m_state->m_nameToFormatDefMap.end())
     return m_state->m_nameToFormatDefMap.find(name)->second;
   STOFF_DEBUG_MSG(("StarFormatManager::getSWFormatDef: can not find any format corresponding to %s\n", name.cstr()));
-  return shared_ptr<StarFormatManagerInternal::FormatDef>();
+  return std::shared_ptr<StarFormatManagerInternal::FormatDef>();
 }
-bool StarFormatManager::readSWFormatDef(StarZone &zone, char kind, shared_ptr<StarFormatManagerInternal::FormatDef> &format, StarObject &doc)
+bool StarFormatManager::readSWFormatDef(StarZone &zone, char kind, std::shared_ptr<StarFormatManagerInternal::FormatDef> &format, StarObject &doc)
 {
   STOFFInputStreamPtr input=zone.input();
   libstoff::DebugFile &ascFile=zone.ascii();
@@ -821,9 +837,9 @@ bool StarFormatManager::readNumberFormatter(StarZone &zone)
   f.str("");
   f << "NumberFormatter-T:";
   input->seek(endDataPos, librevenge::RVNG_SEEK_SET);
-  int val=int(input->readULong(2));
+  auto val=int(input->readULong(2));
   if (val) f << "nId=" << val << ",";
-  long nSizeTableLen=long(input->readULong(4));
+  auto nSizeTableLen=long(input->readULong(4));
   f << "nTableSize=" << nSizeTableLen << ","; // copy in pMemStream
   long lastZonePos=input->tell()+nSizeTableLen;
   std::vector<long> fieldSize;
@@ -848,7 +864,7 @@ bool StarFormatManager::readNumberFormatter(StarZone &zone)
   input->seek(actPos, librevenge::RVNG_SEEK_SET);
   f.str("");
   f << "NumberFormatter:";
-  int nVers=int(input->readULong(2));
+  auto nVers=int(input->readULong(2));
   f << "vers=" << nVers << ",";
   val=int(input->readULong(2));
   if (val) f << "nSysOnStore=" << std::hex << val << std::dec << ",";
@@ -857,7 +873,7 @@ bool StarFormatManager::readNumberFormatter(StarZone &zone)
   ascFile.addPos(actPos);
   ascFile.addNote(f.str().c_str());
 
-  unsigned long id=static_cast<unsigned long>(input->readULong(4));
+  auto id=static_cast<unsigned long>(input->readULong(4));
   size_t n=0;
   while (id!=0xffffffff) {
     pos=input->tell();
@@ -902,16 +918,16 @@ bool StarFormatManager::readNumberFormatter(StarZone &zone)
         f << "##limit" << i << ",";
       }
     }
-    for (int i=0; i<2; ++i)
-      form.m_limitsOp[i]=int(input->readULong(2));
+    for (int &i : form.m_limitsOp)
+      i=int(input->readULong(2));
     *input >> form.m_isStandart;
     *input >> form.m_isUsed;
 
     bool ok=true;
-    for (int i=0; i<4; ++i) {
+    for (auto &subFormat : form.m_subFormats) {
       // ImpSvNumFor::Load
       StarFormatManagerInternal::NumberFormatter::Format subForm;
-      int N=int(input->readULong(2));
+      auto N=int(input->readULong(2));
       if (input->tell()+4*N>endFieldPos) break;
       for (int c=0; c<N; ++c) {
         StarFormatManagerInternal::NumberFormatter::FormatItem item;
@@ -944,7 +960,7 @@ bool StarFormatManager::readNumberFormatter(StarZone &zone)
         f << "###[" << subForm << "],";
         break;
       }
-      form.m_subFormats[i]=subForm;
+      subFormat=subForm;
     }
     form.m_extra=f.str();
     if (ok && m_state->m_idNumberFormatMap.find(unsigned(id))==m_state->m_idNumberFormatMap.end())
@@ -1000,7 +1016,7 @@ bool StarFormatManager::readNumberFormatter(StarZone &zone)
   return true;
 }
 
-bool StarFormatManager::readSWFlyFrameList(StarZone &zone, StarObject &doc)
+bool StarFormatManager::readSWFlyFrameList(StarZone &zone, StarObject &doc, std::vector<std::shared_ptr<StarFormatManagerInternal::FormatDef> > &listFormats)
 {
   STOFFInputStreamPtr input=zone.input();
   libstoff::DebugFile &ascFile=zone.ascii();
@@ -1012,20 +1028,22 @@ bool StarFormatManager::readSWFlyFrameList(StarZone &zone, StarObject &doc)
   }
   libstoff::DebugStream f;
   f << "Entries(SWFlyFrames)[" << zone.getRecordLevel() << "]:";
+  ascFile.addPos(pos);
+  ascFile.addNote(f.str().c_str());
   // sw_sw3fmts.cxx sub part of Sw3IoImp::InFlyFrames
   long lastPos=zone.getRecordLastPosition();
   while (input->tell()<lastPos) {
     pos=input->tell();
     int rType=input->peek();
-    shared_ptr<StarFormatManagerInternal::FormatDef> format;
-    if ((rType=='o' || rType=='l') && readSWFormatDef(zone, char(rType), format, doc))
+    std::shared_ptr<StarFormatManagerInternal::FormatDef> format;
+    if ((rType=='o' || rType=='l') && readSWFormatDef(zone, char(rType), format, doc)) {
+      if (format) listFormats.push_back(format);
       continue;
+    }
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     break;
   }
 
-  ascFile.addPos(pos);
-  ascFile.addNote(f.str().c_str());
   zone.closeSWRecord('F', "SWFlyFrames");
   return true;
 }
@@ -1061,7 +1079,7 @@ bool StarFormatManager::readSWPatternLCL(StarZone &zone)
     f.str("");
     f << "SWPatternLCL[token-" << zone.getRecordLevel() << "]:";
     zone.openFlagZone();
-    int nType=int(input->readULong(2));
+    auto nType=int(input->readULong(2));
     f << "nType=" << nType << ",";
     f << "nStrIdx=" << input->readULong(2) << ",";
     zone.closeFlagZone();
@@ -1084,8 +1102,8 @@ bool StarFormatManager::readSWPatternLCL(StarZone &zone)
         f << "tknEntry,";
         break;
       }
-    // #92986# some older versions created a wrong content index entry definition
-    // fall through expected
+      // #92986# some older versions created a wrong content index entry definition
+      STOFF_FALLTHROUGH;
     case 4: {
       f << "tknText,";
       if (!zone.readString(string)) {
@@ -1130,9 +1148,9 @@ bool StarFormatManager::readSWPatternLCL(StarZone &zone)
 
 void StarFormatManager::updateNumberingProperties(STOFFCell &cell) const
 {
-  STOFFCellStyle const &style=cell.getCellStyle();
-  librevenge::RVNGPropertyList &propList=cell.getNumberingStyle();
-  STOFFCell::Format format=cell.getFormat();
+  auto const &style=cell.getCellStyle();
+  auto &propList=cell.getNumberingStyle();
+  auto format=cell.getFormat();
   librevenge::RVNGPropertyListVector pVect;
   if (style.m_format &&
       m_state->m_idNumberFormatMap.find(unsigned(style.m_format))!=m_state->m_idNumberFormatMap.end() &&
@@ -1196,9 +1214,9 @@ void StarFormatManager::updateNumberingProperties(STOFFCell &cell) const
     cell.setFormat(format);
     propList.insert("librevenge:value-type", "date");
     propList.insert("number:automatic-order", "true");
-    char const *(wh[])= {"%m/%d/%y", "%a %d/%b %y", "%m/%y", "%b %d", "%B",
-                         "%Q Quarter %y", "%m/%d/%Y"
-                        };
+    char const *wh[]= {"%m/%d/%y", "%a %d/%b %y", "%m/%y", "%b %d", "%B",
+                       "%Q Quarter %y", "%m/%d/%Y"
+                      };
     if (format.convertDTFormat(wh[formatId-30], pVect))
       propList.insert("librevenge:format", pVect);
     return;
@@ -1213,7 +1231,7 @@ void StarFormatManager::updateNumberingProperties(STOFFCell &cell) const
     cell.setFormat(format);
     propList.insert("librevenge:value-type", "time");
     propList.insert("number:automatic-order", "true");
-    char const *(wh[])= {"%H:%M", "%H:%M:%S", "%I:%M %p","%I:%M:%S %p", "%H:%M:%S", "%M:%s" };
+    char const *wh[]= {"%H:%M", "%H:%M:%S", "%I:%M %p","%I:%M:%S %p", "%H:%M:%S", "%M:%s" };
     if (format.convertDTFormat(wh[formatId-40], pVect))
       propList.insert("librevenge:format", pVect);
     return;
