@@ -31,6 +31,7 @@
 * instead of those above.
 */
 
+#include <algorithm>
 #include <cstring>
 #include <iomanip>
 #include <iostream>
@@ -58,9 +59,20 @@ namespace StarItemPoolInternal
 //! Internal: a structure use to read SfxMultiRecord zone of a StarItemPool
 struct SfxMultiRecord {
   //! constructor
-  SfxMultiRecord() : m_zone(0), m_zoneType(0), m_zoneOpened(false), m_headerType(0), m_headerVersion(0), m_headerTag(0),
-    m_actualRecord(0), m_numRecord(0), m_contentSize(0),
-    m_startPos(0), m_endPos(0), m_offsetList(), m_extra("")
+  SfxMultiRecord()
+    : m_zone(nullptr)
+    , m_zoneType(0)
+    , m_zoneOpened(false)
+    , m_headerType(0)
+    , m_headerVersion(0)
+    , m_headerTag(0)
+    , m_actualRecord(0)
+    , m_numRecord(0)
+    , m_contentSize(0)
+    , m_startPos(0)
+    , m_endPos(0)
+    , m_offsetList()
+    , m_extra("")
   {
   }
   //! returns true if the record is opened
@@ -170,7 +182,7 @@ struct SfxMultiRecord {
     else if (input->tell()==m_endPos)
       input->seek(m_zone->getRecordLastPosition(), librevenge::RVNG_SEEK_SET);
     m_zone->closeSfxRecord(m_zoneType, wh);
-    m_zone=0;
+    m_zone=nullptr;
   }
   //! try to go to the new content positon
   bool getNewContent(std::string const &wh, int &id)
@@ -208,7 +220,8 @@ struct SfxMultiRecord {
       STOFF_DEBUG_MSG(("StarItemPoolInternal::SfxMultiRecord::getLastContentPosition: argh, find unexpected index\n"));
       return m_endPos;
     }
-    return m_startPos+long(m_offsetList[size_t(m_actualRecord)]>>8)-14;
+    const long pos = m_startPos+long(m_offsetList[size_t(m_actualRecord)]>>8)-14;
+    return m_zone->input()->checkPosition(pos) ? pos : m_endPos;
   }
 
   //! basic operator<< ; print header data
@@ -225,8 +238,7 @@ struct SfxMultiRecord {
     if (r.m_contentSize) o << "content[size/pos]=" << r.m_contentSize << ",";
     if (!r.m_offsetList.empty()) {
       o << "offset=[";
-      for (size_t i=0; i<r.m_offsetList.size(); ++i) {
-        uint32_t off=r.m_offsetList[i];
+      for (auto const &off : r.m_offsetList) {
         if (off&0xff)
           o << (off>>8) << ":" << (off&0xff) << ",";
         else
@@ -265,15 +277,18 @@ protected:
   //! extra data
   std::string m_extra;
 private:
-  SfxMultiRecord(SfxMultiRecord const &orig);
-  SfxMultiRecord &operator=(SfxMultiRecord const &orig);
+  SfxMultiRecord(SfxMultiRecord const &orig) = delete;
+  SfxMultiRecord &operator=(SfxMultiRecord const &orig) = delete;
 };
 
 //! small struct used to keep a list of version
 struct Version {
   //! constructor
-  Version(int vers, int start, std::vector<int> const &list) :
-    m_version(vers), m_start(start), m_list(list), m_invertListMap()
+  Version(int vers, int start, std::vector<int> const &list)
+    : m_version(vers)
+    , m_start(start)
+    , m_list(list)
+    , m_invertListMap()
   {
     for (size_t i=0; i<m_list.size(); ++i)
       m_invertListMap[list[i]]=int(i);
@@ -291,20 +306,24 @@ struct Version {
 //! internal: list of attribute corresponding to a slot id
 struct Values {
   //! constructor
-  Values() : m_default(), m_idValueMap()
+  Values()
+    : m_default()
+    , m_idValueMap()
   {
   }
   //! the default values
-  shared_ptr<StarAttribute> m_default;
+  std::shared_ptr<StarAttribute> m_default;
   //! the list of attribute
-  std::map<int,shared_ptr<StarAttribute> > m_idValueMap;
+  std::map<int,std::shared_ptr<StarAttribute> > m_idValueMap;
 };
 
 ////////////////////////////////////////
 //! Internal: a style of a StarItemPool
 struct StyleId {
   //! constructor
-  StyleId(librevenge::RVNGString const &name, int family) : m_name(name), m_family(family)
+  StyleId(librevenge::RVNGString const &name, int family)
+    : m_name(name)
+    , m_family(family)
   {
   }
   //! operator==
@@ -334,10 +353,26 @@ struct StyleId {
 //! Internal: the state of a StarItemPool
 struct State {
   //! constructor
-  State(StarObject &document, StarItemPool::Type type) :
-    m_document(document), m_type(StarItemPool::T_Unknown), m_majorVersion(0), m_minorVersion(0), m_loadingVersion(-1),
-    m_name(""), m_isSecondaryPool(false), m_secondaryPool(), m_currentVersion(0), m_verStart(0), m_verEnd(0), m_versionList(),
-    m_idToAttributeList(), m_slotIdToValuesMap(), m_styleIdToStyleMap(), m_simplifyNameToStyleNameMap(), m_idToDefaultMap(), m_delayedItemList()
+  State(StarObject &document, StarItemPool::Type type)
+    : m_document(document)
+    , m_type(StarItemPool::T_Unknown)
+    , m_majorVersion(0)
+    , m_minorVersion(0)
+    , m_loadingVersion(-1)
+    , m_name("")
+    , m_relativeUnit(0)
+    , m_isSecondaryPool(false)
+    , m_secondaryPool()
+    , m_currentVersion(0)
+    , m_verStart(0)
+    , m_verEnd(0)
+    , m_versionList()
+    , m_idToAttributeList()
+    , m_slotIdToValuesMap()
+    , m_styleIdToStyleMap()
+    , m_simplifyNameToStyleNameMap()
+    , m_idToDefaultMap()
+    , m_delayedItemList()
   {
     init(type);
   }
@@ -359,7 +394,7 @@ struct State {
   void setPoolName(librevenge::RVNGString const &name)
   {
     m_name=name;
-    StarItemPool::Type type=StarItemPool::T_Unknown;
+    auto type=StarItemPool::T_Unknown;
     if (m_name=="EditEngineItemPool")
       type=StarItemPool::T_EditEnginePool;
     else if (m_name=="SchItemPool")
@@ -394,10 +429,8 @@ struct State {
     m_currentVersion=nVers;
     Version const &vers=m_versionList.back();
     if (vers.m_invertListMap.empty()) return;
-    int min=vers.m_invertListMap.begin()->first;
-    if (m_verStart>min) m_verStart=min;
-    int max=(--vers.m_invertListMap.end())->first;
-    if (m_verEnd<max) m_verEnd=max;
+    m_verStart=std::min(m_verStart,vers.m_invertListMap.begin()->first);
+    m_verEnd=std::max(m_verEnd,(--vers.m_invertListMap.end())->first);
   }
   //! try to return ???
   int getWhich(int nFileWhich) const
@@ -420,8 +453,7 @@ struct State {
       }
     }
     else if (m_loadingVersion<m_currentVersion) {
-      for (size_t i=0; i<m_versionList.size(); ++i) {
-        Version const &vers=m_versionList[i];
+      for (auto const &vers : m_versionList) {
         if (vers.m_version<=m_loadingVersion)
           continue;
         if (nFileWhich<vers.m_start || nFileWhich>=vers.m_start+int(vers.m_list.size())) {
@@ -439,7 +471,7 @@ struct State {
   {
     if (which>=m_verStart&&which<=m_verEnd) return this;
     if (m_secondaryPool) return m_secondaryPool->m_state->getPoolStateFor(which);
-    return 0;
+    return nullptr;
   }
   //! returns a pointer to the values data
   Values *getValues(int id, bool create=false)
@@ -447,17 +479,17 @@ struct State {
     if (m_slotIdToValuesMap.find(id)!=m_slotIdToValuesMap.end())
       return &m_slotIdToValuesMap.find(id)->second;
     if (!create)
-      return 0;
+      return nullptr;
     m_slotIdToValuesMap[id]=Values();
     return &m_slotIdToValuesMap.find(id)->second;
   }
   //! try to return a default attribute corresponding to which
-  shared_ptr<StarAttribute> getDefaultAttribute(int which)
+  std::shared_ptr<StarAttribute> getDefaultAttribute(int which)
   {
     if (m_idToDefaultMap.find(which)!=m_idToDefaultMap.end() && m_idToDefaultMap.find(which)->second)
       return m_idToDefaultMap.find(which)->second;
-    shared_ptr<StarAttribute> res;
-    StarItemPoolInternal::State *state=getPoolStateFor(which);
+    std::shared_ptr<StarAttribute> res;
+    auto *state=getPoolStateFor(which);
     if (!state || which<state->m_verStart || which>=state->m_verStart+int(state->m_idToAttributeList.size()) ||
         !state->m_document.getAttributeManager()) {
       STOFF_DEBUG_MSG(("StarItemPoolInternal::State::getDefaultAttribute: find unknown attribute\n"));
@@ -480,10 +512,12 @@ struct State {
   int m_loadingVersion;
   //! the name
   librevenge::RVNGString m_name;
+  //! the relative unit
+  double m_relativeUnit;
   //! a flag to know if a pool is a secondary pool
   bool m_isSecondaryPool;
   //! the secondary pool
-  shared_ptr<StarItemPool> m_secondaryPool;
+  std::shared_ptr<StarItemPool> m_secondaryPool;
   //! the current version
   int m_currentVersion;
   //! the minimum version
@@ -501,12 +535,12 @@ struct State {
   //! map simplify style name to style name
   std::map<librevenge::RVNGString, librevenge::RVNGString> m_simplifyNameToStyleNameMap;
   //! map of created default attribute
-  std::map<int,shared_ptr<StarAttribute> > m_idToDefaultMap;
+  std::map<int,std::shared_ptr<StarAttribute> > m_idToDefaultMap;
   //! list of item which need to be read
-  std::vector<shared_ptr<StarItem> > m_delayedItemList;
+  std::vector<std::shared_ptr<StarItem> > m_delayedItemList;
 private:
-  State(State const &orig);
-  State operator=(State const &orig);
+  State(State const &orig) = delete;
+  State operator=(State const &orig) = delete;
 };
 
 void State::init(StarItemPool::Type type)
@@ -528,7 +562,7 @@ void State::init(StarItemPool::Type type)
     m_verStart=1; // SCHATTR_START
     m_verEnd=100; // SCHATTR_NONPERSISTENT_START
     // svx_eerdll.cxx GlobalEditData::GetDefItems
-    static int const(what[])= {
+    static int const what[]= {
       StarAttribute::ATTR_SCH_DATADESCR_DESCR, StarAttribute::ATTR_SCH_DATADESCR_SHOW_SYM, StarAttribute::ATTR_SCH_LEGEND_POS, StarAttribute::ATTR_SCH_TEXT_ORIENT,
       StarAttribute::ATTR_SCH_TEXT_ORDER, StarAttribute::ATTR_SCH_Y_AXIS_AUTO_MIN, StarAttribute::ATTR_SCH_Y_AXIS_MIN, StarAttribute::ATTR_SCH_Y_AXIS_AUTO_MAX,
       StarAttribute::ATTR_SCH_Y_AXIS_MAX, StarAttribute::ATTR_SCH_Y_AXIS_AUTO_STEP_MAIN, StarAttribute::ATTR_SCH_Y_AXIS_STEP_MAIN, StarAttribute::ATTR_SCH_Y_AXIS_AUTO_STEP_HELP,
@@ -560,8 +594,8 @@ void State::init(StarItemPool::Type type)
       StarAttribute::ATTR_SCH_STOCK_VOLUME, StarAttribute::ATTR_SCH_STOCK_UPDOWN, StarAttribute::ATTR_SCH_SYMBOL_SIZE, StarAttribute::ATTR_SCH_USER_DEFINED_ATTR
     };
 
-    for (int i=0; i<int(sizeof(what)/sizeof(int)); ++i)
-      m_idToAttributeList.push_back(what[i]);
+    for (auto wh : what)
+      m_idToAttributeList.push_back(wh);
     break;
   }
   case StarItemPool::T_EditEnginePool: {
@@ -588,7 +622,7 @@ void State::init(StarItemPool::Type type)
     for (int i=25; i<=28; ++i)  list.push_back(3994+i+15);
     addVersionMap(4, 3994, list);
     // svx_eerdll.cxx GlobalEditData::GetDefItems
-    static int const(what[])= {
+    static int const what[]= {
       StarAttribute::ATTR_SC_WRITINGDIR, StarAttribute::ATTR_EE_PARA_XMLATTRIBS, StarAttribute::ATTR_PARA_HANGINGPUNCTUATION, StarAttribute::ATTR_PARA_FORBIDDEN_RULES,
       StarAttribute::ATTR_EE_PARA_ASIANCJKSPACING, StarAttribute::ATTR_EE_PARA_NUMBULLET, StarAttribute::ATTR_SC_HYPHENATE, StarAttribute::ATTR_EE_PARA_BULLETSTATE,
       StarAttribute::ATTR_EE_PARA_OUTLLR_SPACE, StarAttribute::ATTR_EE_PARA_OUTLLEVEL, StarAttribute::ATTR_EE_PARA_BULLET, StarAttribute::ATTR_FRM_LR_SPACE,
@@ -605,8 +639,8 @@ void State::init(StarItemPool::Type type)
       StarAttribute::ATTR_EE_CHR_XMLATTRIBS, StarAttribute::ATTR_EE_FEATURE_TAB, StarAttribute::ATTR_EE_FEATURE_LINEBR, StarAttribute::ATTR_CHR_CHARSETCOLOR,
       StarAttribute::ATTR_EE_FEATURE_FIELD
     };
-    for (int i=0; i<int(sizeof(what)/sizeof(int)); ++i)
-      m_idToAttributeList.push_back(what[i]);
+    for (auto wh : what)
+      m_idToAttributeList.push_back(wh);
     break;
   }
   case StarItemPool::T_SpreadsheetPool: {
@@ -658,7 +692,7 @@ void State::init(StarItemPool::Type type)
     for (int i = 0; i <= 34; i++) list.push_back(100+i);
     for (int i = 35; i <= 82; i++) list.push_back(100+i+1);
     addVersionMap(9, 100, list);
-    static int const(what[])= {
+    static int const what[]= {
       StarAttribute::ATTR_CHR_FONT, StarAttribute::ATTR_CHR_FONTSIZE, StarAttribute::ATTR_CHR_WEIGHT, StarAttribute::ATTR_CHR_POSTURE,
       StarAttribute::ATTR_CHR_UNDERLINE, StarAttribute::ATTR_CHR_CROSSEDOUT, StarAttribute::ATTR_CHR_CONTOUR, StarAttribute::ATTR_CHR_SHADOWED,
       StarAttribute::ATTR_CHR_COLOR, StarAttribute::ATTR_CHR_LANGUAGE, StarAttribute::ATTR_CHR_CJK_FONT, StarAttribute::ATTR_CHR_CJK_FONTSIZE,
@@ -684,8 +718,8 @@ void State::init(StarItemPool::Type type)
       StarAttribute::ATTR_SC_PAGE_HEADERLEFT, StarAttribute::ATTR_SC_PAGE_FOOTERLEFT, StarAttribute::ATTR_SC_PAGE_HEADERRIGHT, StarAttribute::ATTR_SC_PAGE_FOOTERRIGHT,
       StarAttribute::ATTR_SC_PAGE_HEADERSET, StarAttribute::ATTR_SC_PAGE_FOOTERSET, StarAttribute::ATTR_SC_PAGE_FORMULAS, StarAttribute::ATTR_SC_PAGE_NULLVALS
     };
-    for (int i=0; i<int(sizeof(what)/sizeof(int)); ++i)
-      m_idToAttributeList.push_back(what[i]);
+    for (auto wh : what)
+      m_idToAttributeList.push_back(wh);
     break;
   }
   case StarItemPool::T_VCControlPool: // never seens with data, so
@@ -748,7 +782,7 @@ void State::init(StarItemPool::Type type)
     for (int i=1127; i<=1140; ++i) list.push_back(i+45);
     addVersionMap(4, 1000, list);
 
-    static int const(what[])= {
+    static int const what[]= {
       StarAttribute::XATTR_LINESTYLE, StarAttribute::XATTR_LINEDASH, StarAttribute::XATTR_LINEWIDTH, StarAttribute::XATTR_LINECOLOR,
       StarAttribute::XATTR_LINESTART, StarAttribute::XATTR_LINEEND, StarAttribute::XATTR_LINESTARTWIDTH, StarAttribute::XATTR_LINEENDWIDTH,
       StarAttribute::XATTR_LINESTARTCENTER, StarAttribute::XATTR_LINEENDCENTER, StarAttribute::XATTR_LINETRANSPARENCE, StarAttribute::XATTR_LINEJOINT,
@@ -852,8 +886,8 @@ void State::init(StarItemPool::Type type)
       StarAttribute::SDRATTR_3DSCENE_RESERVED_14, StarAttribute::SDRATTR_3DSCENE_RESERVED_15, StarAttribute::SDRATTR_3DSCENE_RESERVED_16, StarAttribute::SDRATTR_3DSCENE_RESERVED_17,
       StarAttribute::SDRATTR_3DSCENE_RESERVED_18, StarAttribute::SDRATTR_3DSCENE_RESERVED_19, StarAttribute::SDRATTR_3DSCENE_RESERVED_20 /* 1333*/
     };
-    for (int i=0; i<int(sizeof(what)/sizeof(int)); ++i)
-      m_idToAttributeList.push_back(what[i]);
+    for (auto wh : what)
+      m_idToAttributeList.push_back(wh);
     break;
   }
   case StarItemPool::T_Unknown:
@@ -866,8 +900,9 @@ void State::init(StarItemPool::Type type)
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-StarItemPool::StarItemPool(StarObject &doc, StarItemPool::Type type) :
-  m_isInside(false), m_state(new StarItemPoolInternal::State(doc, type))
+StarItemPool::StarItemPool(StarObject &doc, StarItemPool::Type type)
+  : m_isInside(false)
+  , m_state(new StarItemPoolInternal::State(doc, type))
 {
 }
 
@@ -875,6 +910,17 @@ StarItemPool::~StarItemPool()
 {
 }
 
+void StarItemPool::setRelativeUnit(double relUnit)
+{
+  m_state->m_relativeUnit=relUnit;
+}
+
+double StarItemPool::getRelativeUnit() const
+{
+  if (m_state->m_relativeUnit>0)
+    return m_state->m_relativeUnit;
+  return m_state->m_type==StarItemPool::T_XOutdevPool || m_state->m_type==StarItemPool::T_EditEnginePool ? 0.028346457 : 0.05;
+}
 void StarItemPool::clean()
 {
   m_state->clean();
@@ -885,7 +931,7 @@ bool StarItemPool::isSecondaryPool() const
   return m_state->m_isSecondaryPool;
 }
 
-void StarItemPool::addSecondaryPool(shared_ptr<StarItemPool> secondary)
+void StarItemPool::addSecondaryPool(std::shared_ptr<StarItemPool> secondary)
 {
   if (!secondary) {
     STOFF_DEBUG_MSG(("StarItemPool::addSecondaryPool: called without pool\n"));
@@ -908,21 +954,21 @@ StarItemPool::Type StarItemPool::getType() const
   return m_state->m_type;
 }
 
-shared_ptr<StarItem> StarItemPool::createItem(int which, int surrogateId, bool localId)
+std::shared_ptr<StarItem> StarItemPool::createItem(int which, int surrogateId, bool localId)
 {
-  shared_ptr<StarItem> res(new StarItem(which));
+  std::shared_ptr<StarItem> res(new StarItem(which));
   res->m_surrogateId=surrogateId;
   res->m_localId=localId;
   m_state->m_delayedItemList.push_back(res);
   return res;
 }
 
-shared_ptr<StarAttribute> StarItemPool::readAttribute(StarZone &zone, int which, int vers, long endPos)
+std::shared_ptr<StarAttribute> StarItemPool::readAttribute(StarZone &zone, int which, int vers, long endPos)
 {
   if (m_state->m_currentVersion!=m_state->m_loadingVersion)
     which=m_state->getWhich(which);
 
-  StarItemPoolInternal::State *state=m_state->getPoolStateFor(which);
+  auto *state=m_state->getPoolStateFor(which);
   if (!state || which<state->m_verStart || which>=state->m_verStart+int(state->m_idToAttributeList.size()) ||
       !state->m_document.getAttributeManager()) {
     STOFFInputStreamPtr input=zone.input();
@@ -937,8 +983,8 @@ shared_ptr<StarAttribute> StarItemPool::readAttribute(StarZone &zone, int which,
     return StarAttributeManager::getDummyAttribute();
   }
   zone.openDummyRecord();
-  shared_ptr<StarAttribute> attribute=state->m_document.getAttributeManager()->readAttribute
-                                      (zone, state->m_idToAttributeList[size_t(which-state->m_verStart)], vers, endPos, state->m_document);
+  auto attribute=state->m_document.getAttributeManager()->readAttribute
+                 (zone, state->m_idToAttributeList[size_t(which-state->m_verStart)], vers, endPos, state->m_document);
   zone.closeDummyRecord();
   return attribute;
 }
@@ -957,7 +1003,7 @@ bool StarItemPool::read(StarZone &zone)
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   if ((tag!=0x1111 && tag!=0xbbbb) || nMajorVers<1 || nMajorVers>2)
     return false;
-  StarItemPool *master=0, *pool=this;
+  StarItemPool *master=nullptr, *pool=this;
   // update the inside flag to true
   while (pool) {
     pool->m_isInside=true;
@@ -984,15 +1030,15 @@ bool StarItemPool::read(StarZone &zone)
   // update the delayed item
   pool=this;
   while (pool) {
-    for (size_t i=0; i<pool->m_state->m_delayedItemList.size(); ++i)
-      loadSurrogate(*(pool->m_state->m_delayedItemList[i]));
+    for (auto it : pool->m_state->m_delayedItemList)
+      loadSurrogate(*it);
     pool->m_state->m_delayedItemList.clear();
     pool=pool->m_state->m_secondaryPool.get();
   }
   return ok;
 }
 
-shared_ptr<StarItem> StarItemPool::readItem(StarZone &zone, bool isDirect, long endPos)
+std::shared_ptr<StarItem> StarItemPool::readItem(StarZone &zone, bool isDirect, long endPos)
 {
   // polio.cxx SfxItemPool::LoadItem
   STOFFInputStreamPtr input=zone.input();
@@ -1002,13 +1048,13 @@ shared_ptr<StarItem> StarItemPool::readItem(StarZone &zone, bool isDirect, long
   long pos=input->tell();
   if (pos+4>endPos) {
     STOFF_DEBUG_MSG(("StarItemPool::readItem: the zone seems too short\n"));
-    return shared_ptr<StarItem>();
+    return std::shared_ptr<StarItem>();
   }
   uint16_t nWhich, nSlot;
   *input>>nWhich >> nSlot;
   f << "which=" << nWhich << ",";
   if (nSlot) f << "slot=" << nSlot << ",";
-  shared_ptr<StarItem> pItem;
+  std::shared_ptr<StarItem> pItem;
   if (!m_state->isInRange(nWhich)) {
     uint16_t nSurro;
     *input >> nSurro;
@@ -1060,7 +1106,7 @@ shared_ptr<StarItem> StarItemPool::readItem(StarZone &zone, bool isDirect, long
     }
     else if (nLength) {
       long endAttrPos=input->tell()+long(nLength);
-      shared_ptr<StarAttribute> attribute=readAttribute(zone, int(nWhich), int(nVersion), endAttrPos);
+      auto attribute=readAttribute(zone, int(nWhich), int(nVersion), endAttrPos);
       pItem.reset(new StarItem(attribute, nWhich));
       if (!attribute) {
         STOFF_DEBUG_MSG(("StarItemPool::readItem: can not read an attribute\n"));
@@ -1094,7 +1140,7 @@ bool StarItemPool::loadSurrogate(StarItem &item)
     return m_state->m_secondaryPool->loadSurrogate(item);
   int aWhich=(item.m_localId && m_state->m_currentVersion!=m_state->m_loadingVersion) ?
              m_state->getWhich(item.m_which) : item.m_which;
-  StarItemPoolInternal::Values *values=m_state->getValues(aWhich);
+  auto *values=m_state->getValues(aWhich);
   if (item.m_surrogateId==0xfffe) {
     // default
     if (!values || !values->m_default)
@@ -1113,7 +1159,7 @@ bool StarItemPool::loadSurrogate(StarItem &item)
   return true;
 }
 
-shared_ptr<StarItem> StarItemPool::loadSurrogate(StarZone &zone, uint16_t &nWhich, bool localId, libstoff::DebugStream &f)
+std::shared_ptr<StarItem> StarItemPool::loadSurrogate(StarZone &zone, uint16_t &nWhich, bool localId, libstoff::DebugStream &f)
 {
   if ((nWhich<m_state->m_verStart||nWhich>m_state->m_verEnd)&&m_state->m_secondaryPool)
     return m_state->m_secondaryPool->loadSurrogate(zone, nWhich, localId, f);
@@ -1122,16 +1168,16 @@ shared_ptr<StarItem> StarItemPool::loadSurrogate(StarZone &zone, uint16_t &nWhic
   *zone.input()>>nSurrog;
   if (nSurrog==0xffff) {
     f << "direct,";
-    return shared_ptr<StarItem>();
+    return std::shared_ptr<StarItem>();
   }
   if (nSurrog==0xfff0) {
     f << "null,";
     nWhich=0;
-    return shared_ptr<StarItem>();
+    return std::shared_ptr<StarItem>();
   }
   if (m_state->m_loadingVersion<0) // the pool is not read, so we wait
     return createItem(int(nWhich), int(nSurrog), localId);
-  shared_ptr<StarItem> res(new StarItem(int(nWhich)));
+  std::shared_ptr<StarItem> res(new StarItem(int(nWhich)));
   int aWhich=(localId && m_state->m_currentVersion!=m_state->m_loadingVersion) ?
              m_state->getWhich(nWhich) : nWhich;
   StarItemPoolInternal::Values *values=m_state->getValues(aWhich);
@@ -1286,7 +1332,7 @@ bool StarItemPool::readV2(StarZone &zone, StarItemPool *master)
     }
     f << "pos=[";
     std::vector<int> listData;
-    for (int i=int(nStart); i<=int(nEnd); ++i) {
+    for (auto i=int(nStart); i<=int(nEnd); ++i) {
       uint16_t nPos;
       *input>>nPos;
       listData.push_back(int(nPos));
@@ -1343,7 +1389,7 @@ bool StarItemPool::readV2(StarZone &zone, StarItemPool *master)
       ascii.addPos(pos);
       ascii.addNote(f.str().c_str());
       pos=input->tell();
-      shared_ptr<StarAttribute> attribute;
+      std::shared_ptr<StarAttribute> attribute;
       int aWhich=m_state->m_currentVersion!=m_state->m_loadingVersion ? m_state->getWhich(which) : which;
       StarItemPoolInternal::Values *values=m_state->getValues(aWhich, true);
       if (step==0) {
@@ -1661,7 +1707,7 @@ bool StarItemPool::readV1(StarZone &zone, StarItemPool * /*master*/)
       ascii.addPos(pos);
       ascii.addNote(f.str().c_str());
       int aWhich=m_state->m_currentVersion!=m_state->m_loadingVersion ? m_state->getWhich(which) : which;
-      StarItemPoolInternal::Values *values=m_state->getValues(aWhich, true);
+      auto *values=m_state->getValues(aWhich, true);
       if (step==0 && nCount) {
         if (!values->m_idValueMap.empty()) {
           STOFF_DEBUG_MSG(("StarItemPool::readV1: the slot %d is already created\n", aWhich));
@@ -1688,7 +1734,7 @@ bool StarItemPool::readV1(StarZone &zone, StarItemPool * /*master*/)
           break;
         }
         long endAttPos=debAttPos+long(sizeAttr[n]);
-        shared_ptr<StarAttribute> attribute;
+        std::shared_ptr<StarAttribute> attribute;
         if (input->tell()==endAttPos) {
           ascii.addPos(pos);
           ascii.addNote(f.str().c_str());
@@ -1913,9 +1959,9 @@ bool StarItemPool::readStyles(StarZone &zone, StarObject &doc)
       long dataPos=input->tell(), endDataPos=dataPos+long(nSize);
       libstoff::DebugStream f2;
       f2 << "Entries(SwStyleSheet):sz=" << nSize << ",";
-      int nId=int(input->readULong(2));
+      auto nId=int(input->readULong(2));
       if (nId) f2 << "id=" << nId << ",";
-      int level=int(input->readULong(1));
+      auto level=int(input->readULong(1));
       if (level!=201) {
         // checkme: sometimes is more complicated
         if (m_state->m_styleIdToStyleMap.find(styleId)!=m_state->m_styleIdToStyleMap.end())
@@ -1924,8 +1970,8 @@ bool StarItemPool::readStyles(StarZone &zone, StarObject &doc)
       }
       bool dataOk=true;
       if (1<=nVer) {
-        if (nVer==1 && style.m_family==2 && nId==1)
-          nId=(1<<11)+1; // RES_POOLCOLL_TEXT
+        /*if (nVer==1 && style.m_family==2 && nId==1)
+          nId=(1<<11)+1; // RES_POOLCOLL_TEXT*/
         dataOk=input->tell()+2<=endDataPos;
         int val=dataOk ? int(input->readULong(2)) : 0;
         if (val==1) {
@@ -1939,7 +1985,7 @@ bool StarItemPool::readStyles(StarZone &zone, StarObject &doc)
               break;
             }
             f2 << "[" << libstoff::getString(text).cstr();
-            int cond=int(input->readULong(4));
+            auto cond=int(input->readULong(4));
             if (cond) f2 << "cond=" << std::hex << cond << std::dec << ",";
             if (cond & 0x8000) {
               if (!zone.readString(text, charSet) || input->tell()>endDataPos) {
@@ -1966,9 +2012,9 @@ bool StarItemPool::readStyles(StarZone &zone, StarObject &doc)
           long len=dataOk ? long(input->readULong(4)) : 0;
           dataOk=input->tell()+len<=endDataPos;
           if (dataOk && len>=2 && doc.getAttributeManager()) {
-            int nAttrVer=int(input->readULong(2));
-            shared_ptr<StarAttribute> attrib=doc.getAttributeManager()->readAttribute
-                                             (zone, StarAttribute::ATTR_FRM_LR_SPACE, nAttrVer, endDataPos, doc);
+            auto nAttrVer=int(input->readULong(2));
+            auto attrib=doc.getAttributeManager()->readAttribute
+                        (zone, StarAttribute::ATTR_FRM_LR_SPACE, nAttrVer, endDataPos, doc);
             f2 << "LR,";
             if (!attrib || input->tell()>endDataPos) {
               STOFF_DEBUG_MSG(("StarItemPool::readStyles: reading relative LR is not implemented\n"));
@@ -2024,47 +2070,45 @@ void StarItemPool::updateStyles()
 {
   std::set<StarItemPoolInternal::StyleId> done, toDo;
   std::multimap<StarItemPoolInternal::StyleId, StarItemPoolInternal::StyleId> childMap;
-  std::map<StarItemPoolInternal::StyleId,StarItemStyle>::iterator it;
-  std::multimap<StarItemPoolInternal::StyleId, StarItemPoolInternal::StyleId>::iterator cIt;
-  std::map<int, shared_ptr<StarItem> >::const_iterator iIt;
-  for (it=m_state->m_styleIdToStyleMap.begin(); it!=m_state->m_styleIdToStyleMap.end(); ++it) {
-    if (it->second.m_names[1].empty())
-      toDo.insert(it->first);
+  std::map<int, std::shared_ptr<StarItem> >::const_iterator iIt;
+  for (auto it : m_state->m_styleIdToStyleMap) {
+    if (it.second.m_names[1].empty())
+      toDo.insert(it.first);
     else {
-      StarItemPoolInternal::StyleId parentId(it->second.m_names[1], it->first.m_family);
+      StarItemPoolInternal::StyleId parentId(it.second.m_names[1], it.first.m_family);
       // reset the parent name, as it must no longer be used when creating a styles...
-      it->second.m_names[1].clear();
+      it.second.m_names[1].clear();
       if (m_state->m_styleIdToStyleMap.find(parentId)==m_state->m_styleIdToStyleMap.end()) {
         STOFF_DEBUG_MSG(("StarItemPool::updateStyles: can not find style %s-%d\n", parentId.m_name.cstr(), parentId.m_family));
       }
       else
-        childMap.insert(std::multimap<StarItemPoolInternal::StyleId, StarItemPoolInternal::StyleId>::value_type(parentId, it->first));
+        childMap.insert(std::multimap<StarItemPoolInternal::StyleId, StarItemPoolInternal::StyleId>::value_type(parentId, it.first));
     }
   }
   while (!toDo.empty()) {
-    StarItemPoolInternal::StyleId styleId=*toDo.begin();
+    auto styleId=*toDo.begin();
     toDo.erase(styleId);
     if (done.find(styleId)!=done.end()) {
       STOFF_DEBUG_MSG(("StarItemPool::updateStyles: find loop for style %s-%d\n", styleId.m_name.cstr(), styleId.m_family));
       continue;
     }
     done.insert(styleId);
-    it=m_state->m_styleIdToStyleMap.find(styleId);
+    auto it=m_state->m_styleIdToStyleMap.find(styleId);
     if (it==m_state->m_styleIdToStyleMap.end()) {
       STOFF_DEBUG_MSG(("StarItemPool::updateStyles: can not find style %s-%d\n", styleId.m_name.cstr(), styleId.m_family));
       continue;
     }
     StarItemSet const &parentItemSet=it->second.m_itemSet;
-    cIt=childMap.lower_bound(styleId);
+    auto cIt=childMap.lower_bound(styleId);
     while (cIt!=childMap.end() && cIt->first==styleId) {
-      StarItemPoolInternal::StyleId childId=cIt++->second;
+      auto childId=cIt++->second;
       it=m_state->m_styleIdToStyleMap.find(childId);
       if (it==m_state->m_styleIdToStyleMap.end() || done.find(childId)!=done.end()) {
         STOFF_DEBUG_MSG(("StarItemPool::updateStyles: pb with style %s-%d\n", childId.m_name.cstr(), childId.m_family));
         continue;
       }
       toDo.insert(childId);
-      StarItemSet &childItemSet=it->second.m_itemSet;
+      auto &childItemSet=it->second.m_itemSet;
       for (iIt=parentItemSet.m_whichToItemMap.begin(); iIt!=parentItemSet.m_whichToItemMap.end(); ++iIt) {
         if (!iIt->second || childItemSet.m_whichToItemMap.find(iIt->first)!=childItemSet.m_whichToItemMap.end())
           continue;
@@ -2081,13 +2125,13 @@ void StarItemPool::updateStyles()
      from a bad encoded style name...
    */
   std::set<librevenge::RVNGString> dupplicatedSimpName;
-  for (it=m_state->m_styleIdToStyleMap.begin(); it!=m_state->m_styleIdToStyleMap.end(); ++it) {
-    if (it->second.m_names[0].empty()) continue;
-    librevenge::RVNGString simpName=libstoff::simplifyString(it->second.m_names[0]);
-    if (it->second.m_names[0]==simpName || dupplicatedSimpName.find(simpName)!=dupplicatedSimpName.end()) continue;
+  for (auto it : m_state->m_styleIdToStyleMap) {
+    if (it.second.m_names[0].empty()) continue;
+    auto simpName=libstoff::simplifyString(it.second.m_names[0]);
+    if (it.second.m_names[0]==simpName || dupplicatedSimpName.find(simpName)!=dupplicatedSimpName.end()) continue;
     if (m_state->m_simplifyNameToStyleNameMap.find(simpName)==m_state->m_simplifyNameToStyleNameMap.end())
-      m_state->m_simplifyNameToStyleNameMap[simpName]=it->second.m_names[0];
-    else if (m_state->m_simplifyNameToStyleNameMap.find(simpName)->second!=it->second.m_names[0]) {
+      m_state->m_simplifyNameToStyleNameMap[simpName]=it.second.m_names[0];
+    else if (m_state->m_simplifyNameToStyleNameMap.find(simpName)->second!=it.second.m_names[0]) {
       dupplicatedSimpName.insert(simpName);
       m_state->m_simplifyNameToStyleNameMap.erase(simpName);
     }
@@ -2097,7 +2141,7 @@ void StarItemPool::updateStyles()
 StarItemStyle const *StarItemPool::findStyleWithFamily(librevenge::RVNGString const &style, int family) const
 {
   if (style.empty())
-    return 0;
+    return nullptr;
   for (int step=0; step<2; ++step) {
     librevenge::RVNGString name(style);
     if (step==1) {
@@ -2108,7 +2152,7 @@ StarItemStyle const *StarItemPool::findStyleWithFamily(librevenge::RVNGString co
       name=m_state->m_simplifyNameToStyleNameMap.find(simpName)->second;
     }
     StarItemPoolInternal::StyleId styleId(name, 0);
-    std::map<StarItemPoolInternal::StyleId,StarItemStyle>::const_iterator it=m_state->m_styleIdToStyleMap.lower_bound(styleId);
+    auto it=m_state->m_styleIdToStyleMap.lower_bound(styleId);
     while (it!=m_state->m_styleIdToStyleMap.end() && it->first.m_name==name) {
       if ((it->first.m_family&family)==family)
         return &it->second;
@@ -2116,7 +2160,7 @@ StarItemStyle const *StarItemPool::findStyleWithFamily(librevenge::RVNGString co
     }
   }
   STOFF_DEBUG_MSG(("StarItemPool::findStyleWithFamily: can not find with style %s-%d\n", style.cstr(), family));
-  return 0;
+  return nullptr;
 }
 
 void StarItemPool::defineGraphicStyle(STOFFListenerPtr listener, librevenge::RVNGString const &styleName, StarObject &object, std::set<librevenge::RVNGString> &done) const
@@ -2130,7 +2174,7 @@ void StarItemPool::defineGraphicStyle(STOFFListenerPtr listener, librevenge::RVN
     STOFF_DEBUG_MSG(("StarItemPool::defineGraphicStyle: can not find the listener"));
     return;
   }
-  StarItemStyle const *style=findStyleWithFamily(styleName, StarItemStyle::F_Paragraph);
+  auto const *style=findStyleWithFamily(styleName, StarItemStyle::F_Paragraph);
   if (!style) {
     STOFF_DEBUG_MSG(("StarItemPool::defineGraphicStyle: can not find graphic style with name %s", styleName.cstr()));
     return;
@@ -2147,10 +2191,9 @@ void StarItemPool::defineGraphicStyle(STOFFListenerPtr listener, librevenge::RVN
       state.m_graphic.m_propertyList.insert("librevenge:parent-display-name", style->m_names[1]);
     }
   }
-  for (std::map<int, shared_ptr<StarItem> >::const_iterator it=style->m_itemSet.m_whichToItemMap.begin();
-       it!=style->m_itemSet.m_whichToItemMap.end(); ++it) {
-    if (it->second && it->second->m_attribute)
-      it->second->m_attribute->addTo(state);
+  for (auto it : style->m_itemSet.m_whichToItemMap) {
+    if (it.second && it.second->m_attribute)
+      it.second->m_attribute->addTo(state);
   }
   listener->defineStyle(state.m_graphic);
 }
@@ -2166,7 +2209,7 @@ void StarItemPool::defineParagraphStyle(STOFFListenerPtr listener, librevenge::R
     STOFF_DEBUG_MSG(("StarItemPool::defineParagraphStyle: can not find the listener"));
     return;
   }
-  StarItemStyle const *style=findStyleWithFamily(styleName, StarItemStyle::F_Paragraph);
+  auto const *style=findStyleWithFamily(styleName, StarItemStyle::F_Paragraph);
   if (!style) {
     STOFF_DEBUG_MSG(("StarItemPool::defineParagraphStyle: can not find paragraph style with name %s", styleName.cstr()));
     return;
@@ -2186,24 +2229,22 @@ void StarItemPool::defineParagraphStyle(STOFFListenerPtr listener, librevenge::R
       state.m_paragraph.m_propertyList.insert("librevenge:parent-display-name", style->m_names[1]);
     }
   }
-  for (std::map<int, shared_ptr<StarItem> >::const_iterator it=style->m_itemSet.m_whichToItemMap.begin();
-       it!=style->m_itemSet.m_whichToItemMap.end(); ++it) {
-    if (it->second && it->second->m_attribute)
-      it->second->m_attribute->addTo(state);
+  for (auto it : style->m_itemSet.m_whichToItemMap) {
+    if (it.second && it.second->m_attribute)
+      it.second->m_attribute->addTo(state);
   }
   listener->defineStyle(state.m_paragraph);
 }
 
 void StarItemPool::updateUsingStyles(StarItemSet &itemSet) const
 {
-  StarItemStyle const *style=findStyleWithFamily(itemSet.m_style, itemSet.m_family);
+  auto const *style=findStyleWithFamily(itemSet.m_style, itemSet.m_family);
   if (!style) return;
-  StarItemSet const &parentItemSet=style->m_itemSet;
-  std::map<int, shared_ptr<StarItem> >::const_iterator iIt;
-  for (iIt=parentItemSet.m_whichToItemMap.begin(); iIt!=parentItemSet.m_whichToItemMap.end(); ++iIt) {
-    if (!iIt->second || itemSet.m_whichToItemMap.find(iIt->first)!=itemSet.m_whichToItemMap.end())
+  auto const &parentItemSet=style->m_itemSet;
+  for (auto it : parentItemSet.m_whichToItemMap) {
+    if (!it.second || itemSet.m_whichToItemMap.find(it.first)!=itemSet.m_whichToItemMap.end())
       continue;
-    itemSet.m_whichToItemMap[iIt->first]=iIt->second;
+    itemSet.m_whichToItemMap[it.first]=it.second;
   }
 }
 
