@@ -159,7 +159,7 @@ try
     switch (type) {
     case '+': { // extra outline
       zone.openFlagZone();
-      int N=int(input->readULong(1));
+      auto N=int(input->readULong(1));
       f << "N=" << N << ",";
       zone.closeFlagZone();
       if (input->tell()+3*N>zone.getRecordLastPosition()) {
@@ -233,15 +233,15 @@ bool StarObjectNumericRuler::readLevel(StarZone &zone, STOFFListLevel &level)
       return true;
     }
     if (string.empty()) continue;
-    static char const *(wh[])= {"prefix", "postfix", "fontname", "fontstyle"};
+    static char const *wh[]= {"prefix", "postfix", "fontname", "fontstyle"};
     f << wh[i] << "=" << libstoff::getString(string).cstr() << ",";
     if (i==0) level.m_propertyList.insert("style:num-prefix",libstoff::getString(string));
     else if (i==1) level.m_propertyList.insert("style:num-suffix",libstoff::getString(string));
     else if (i==2) fontName=libstoff::getString(string);
   }
-  int format=int(input->readULong(2));
-  int eType=int(input->readULong(1));
-  uint8_t cBullet=uint8_t(input->readULong(1));
+  auto format=int(input->readULong(2));
+  auto eType=int(input->readULong(1));
+  auto cBullet=uint8_t(input->readULong(1));
   int maxLevel=0;
   if (zone.isCompatibleWith(0x201))
     maxLevel=int(input->readULong(1));
@@ -251,12 +251,12 @@ bool StarObjectNumericRuler::readLevel(StarZone &zone, STOFFListLevel &level)
     if (hasLevel) maxLevel=10;
   }
   level.m_startValue=int(input->readULong(2));
-  int adjust=int(input->readULong(1));
+  auto adjust=int(input->readULong(1));
   unsigned long leftSpace=input->readULong(4);
   long firstLineOffset=input->readLong(4);
-  int fontFamily=int(input->readULong(1));
-  int fontPitch=int(input->readULong(1));
-  int charSet=int(input->readULong(1));
+  auto fontFamily=int(input->readULong(1));
+  auto fontPitch=int(input->readULong(1));
+  auto charSet=int(input->readULong(1));
   bool isSymbolFont=false;
   if (!fontName.empty()) {
     // todo find where other font characteristic are stored
@@ -270,7 +270,7 @@ bool StarObjectNumericRuler::readLevel(StarZone &zone, STOFFListLevel &level)
   }
   if (format!=0xFFFF) f << "nFmt=" << format << ",";
   if (eType<=4 || eType==9 || eType==10) {
-    char const *(wh[])= {"A", "a", "I", "i", "1", "", "", "", "", "A"/*UPPER_LETTER_N*/, "a"/*LOWER_LETTER_N*/};
+    char const *wh[]= {"A", "a", "I", "i", "1", "", "", "", "", "A"/*UPPER_LETTER_N*/, "a"/*LOWER_LETTER_N*/};
     level.m_propertyList.insert("style:num-format", wh[eType]);
     level.m_type=STOFFListLevel::NUMBER;
     f << "type=" << wh[eType] << ",";
@@ -303,7 +303,7 @@ bool StarObjectNumericRuler::readLevel(StarZone &zone, STOFFListLevel &level)
   if (maxLevel) f << "nUpperLevel=" << maxLevel << ",";
   if (level.m_startValue) f << "start=" << level.m_startValue << ",";
   if (adjust>=0 && adjust<=5) {
-    char const *(wh[])= {"left", "right", "justify", "center", "justify"/*block line*/, "end"};
+    char const *wh[]= {"left", "right", "justify", "center", "justify"/*block line*/, "end"};
     level.m_propertyList.insert("fo:text-align", wh[adjust]);
     f << wh[adjust] << ",";
   }
@@ -334,7 +334,7 @@ bool StarObjectNumericRuler::readLevel(StarZone &zone, STOFFListLevel &level)
     f << "nRelLSpace=" << input->readLong(4) << ",";
   }
   if (zone.isCompatibleWith(0x17,0x22, 0x101)) {
-    int charTextDistance=int(input->readULong(2));
+    auto charTextDistance=int(input->readULong(2));
     if (charTextDistance) {
       level.m_propertyList.insert("text:min-label-distance", 0.05*double(charTextDistance), librevenge::RVNG_POINT);
       f << "nTextOffset=" << charTextDistance << ",";
@@ -342,7 +342,7 @@ bool StarObjectNumericRuler::readLevel(StarZone &zone, STOFFListLevel &level)
     if (input->tell()<zone.getRecordLastPosition() && eType==8) {
       f << "width=" << input->readLong(4) << ",";
       f << "height=" << input->readLong(4) << ",";
-      int cF=int(input->readULong(1));
+      auto cF=int(input->readULong(1));
       if (cF&1) f << "nVer[brush]=" << input->readULong(2) << ",";
       if (cF&2) f << "nVer[vertOrient]=" << input->readULong(2) << ",";
     }
@@ -369,8 +369,8 @@ bool StarObjectNumericRuler::readList(StarZone &zone, std::shared_ptr<STOFFList>
   int val;
   librevenge::RVNGString name;
   if (zone.isCompatibleWith(0x201)) {
-    int cFlags=int(zone.openFlagZone());
-    int nStringId=int(input->readULong(2));
+    auto cFlags=int(zone.openFlagZone());
+    auto nStringId=int(input->readULong(2));
     if (nStringId==0xFFFF)
       ;
     else if (!zone.getPoolName(nStringId, name))
@@ -404,7 +404,7 @@ bool StarObjectNumericRuler::readList(StarZone &zone, std::shared_ptr<STOFFList>
   for (int i=0; i<10; ++i) list->set(i+1, defLevel);
   if (zone.isCompatibleWith(0x201))
     zone.closeFlagZone();
-  int nFormat=int(input->readULong(1));
+  auto nFormat=int(input->readULong(1));
   long lastPos=zone.getRecordLastPosition();
   f << "nFormat=" << nFormat << ",";
   if (input->tell()+nFormat>lastPos) {
@@ -569,7 +569,7 @@ bool StarObjectNumericRuler::readAttributeLevel(StarZone &zone, int vers, long l
     case 16: // bold dash-dot
     case 17: // bold dash-dot-dot
     case 18: { // bold wave
-      char const *(wh[])= {"solid", "dotted", "dash", "long-dash", "dot-dash", "dot-dot-dash", "wave"};
+      char const *wh[]= {"solid", "dotted", "dash", "long-dash", "dot-dash", "dot-dot-dash", "wave"};
       font.m_propertyList.insert("style:text-underline-type", "single");
       font.m_propertyList.insert("style:text-underline-style", wh[underline-12]);
       font.m_propertyList.insert("style:text-underline-width", "thick");
@@ -617,11 +617,11 @@ bool StarObjectNumericRuler::readAttributeLevel(StarZone &zone, int vers, long l
     font.m_propertyList.insert("style:text-outline", outline);
     font.m_propertyList.insert("fo:text-shadow", shadow ? "1pt 1pt" : "none");
   }
-  int width=int(input->readLong(4));
-  int start=int(input->readULong(2));
-  int justify=int(input->readULong(1));
-  int symbol=int(input->readULong(1));
-  int scale=int(input->readULong(2));
+  auto width=int(input->readLong(4));
+  auto start=int(input->readULong(2));
+  auto justify=int(input->readULong(1));
+  auto symbol=int(input->readULong(1));
+  auto scale=int(input->readULong(2));
   if (width) f << "width=" << width << ",";
   if (start) f << "start=" << start << ",";
   if (justify) f << "justify=" << justify << ",";
@@ -629,7 +629,7 @@ bool StarObjectNumericRuler::readAttributeLevel(StarZone &zone, int vers, long l
   if (scale) f << "scale=" << scale << ",";
 
   if (style<=4) {
-    char const *(wh[])= {"A", "a", "I", "i", "1"};
+    char const *wh[]= {"A", "a", "I", "i", "1"};
     level.m_propertyList.insert("style:num-format", wh[style]);
     level.m_type=STOFFListLevel::NUMBER;
   }
@@ -655,7 +655,7 @@ bool StarObjectNumericRuler::readAttributeLevel(StarZone &zone, int vers, long l
     level.m_propertyList.insert("text:bullet-char", bullet);
   }
   if (alignement>=0 && alignement<6) {
-    char const *(wh[])= {"left", "right", "justify", "center", "justify"/*block line*/, "end"};
+    char const *wh[]= {"left", "right", "justify", "center", "justify"/*block line*/, "end"};
     level.m_propertyList.insert("fo:text-align", wh[alignement]);
   }
   else if (alignement>=0) {
