@@ -58,10 +58,17 @@ struct Font final : public WPSFont
 	explicit Font(libwps_tools_win::Font::Type type) : WPSFont(), m_type(type)
 	{
 	}
+	Font(Font const &)=default;
+	//! destructor
+	~Font() final;
 	//! font encoding type
 	libwps_tools_win::Font::Type m_type;
 };
 
+Font::~Font()
+{
+}
+
 //! Internal: the subdocument of a WPS4Parser
 class SubDocument final : public WKSSubDocument
 {
@@ -138,6 +145,8 @@ struct State
 	//! return the default font style
 	libwps_tools_win::Font::Type getDefaultFontType() const
 	{
+		if (m_hasLICSCharacters && m_version<=2)
+			return libwps_tools_win::Font::LICS;
 		if (m_fontType != libwps_tools_win::Font::UNKNOWN)
 			return m_fontType;
 		if (m_version>2)
@@ -295,7 +304,7 @@ bool WKS4Parser::getColor(int id, WPSColor &color) const
 
 bool WKS4Parser::getFont(int id, WPSFont &font, libwps_tools_win::Font::Type &type) const
 {
-	if (id < 0 || id>=(int)m_state->m_fontsList.size())
+	if (id < 0 || id>=int(m_state->m_fontsList.size()))
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::getFont: can not find font %d\n", id));
 		return false;
@@ -306,13 +315,6 @@ bool WKS4Parser::getFont(int id, WPSFont &font, libwps_tools_win::Font::Type &ty
 	return true;
 }
 
-bool WKS4Parser::hasLICSCharacters() const
-{
-	if (version()<=2)
-		return m_state->m_hasLICSCharacters;
-	return false;
-}
-
 librevenge::RVNGString WKS4Parser::getSheetName(int id) const
 {
 	return m_spreadsheetParser->getSheetName(id);
@@ -421,8 +423,8 @@ bool WKS4Parser::checkHeader(WPSHeader *header, bool strict)
 	}
 
 	input->seek(0,librevenge::RVNG_SEEK_SET);
-	auto firstOffset = (int) libwps::readU8(input);
-	auto type = (int) libwps::read8(input);
+	auto firstOffset = int(libwps::readU8(input));
+	auto type = int(libwps::read8(input));
 	bool needEncoding=true;
 	f << "FileHeader:";
 	if ((firstOffset == 0 && type == 0) ||
@@ -458,11 +460,11 @@ bool WKS4Parser::checkHeader(WPSHeader *header, bool strict)
 		WPS_DEBUG_MSG(("WKS4Parser::checkHeader: find unexpected type file\n"));
 		return false;
 	}
-	auto val=(int) libwps::read16(input);
+	auto val=int(libwps::read16(input));
 	if (val==2)
 	{
 		// version
-		val=(int) libwps::readU16(input);
+		val=int(libwps::readU16(input));
 		if (isSpreadsheet)
 		{
 			if (val==0x404)
@@ -524,7 +526,7 @@ bool WKS4Parser::checkHeader(WPSHeader *header, bool strict)
 	m_state->m_isSpreadsheet=isSpreadsheet;
 	if (header)
 	{
-		header->setMajorVersion((uint8_t) m_state->m_version);
+		header->setMajorVersion(uint8_t(m_state->m_version));
 		header->setCreator(creatorId);
 		header->setKind(kind);
 		header->setNeedEncoding(needEncoding);
@@ -583,8 +585,8 @@ bool WKS4Parser::readZones()
 		WPS_DEBUG_MSG(("WKS4Parser::readZones: cell header is too short\n"));
 		return m_spreadsheetParser->getNumSpreadsheets()>0;
 	}
-	auto type = (int) libwps::readU16(input); // 1
-	auto length = (int) libwps::readU16(input);
+	auto type = int(libwps::readU16(input)); // 1
+	auto length = int(libwps::readU16(input));
 	if (length)
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::readZones: parse breaks before ending\n"));
@@ -610,9 +612,9 @@ bool WKS4Parser::readZone()
 	libwps::DebugStream f;
 	RVNGInputStreamPtr input = getInput();
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
-	auto type = (int) libwps::read8(input);
-	auto sz = (long) libwps::readU16(input);
+	auto id = int(libwps::readU8(input));
+	auto type = int(libwps::read8(input));
+	auto sz = long(libwps::readU16(input));
 	if (sz<0 || !checkFilePosition(pos+4+sz))
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::readZone: size is bad\n"));
@@ -672,7 +674,7 @@ bool WKS4Parser::readZone()
 			f << "Entries(Byte" << std::hex << id << std::dec << "Z):";
 			if (sz!=1) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			if (id==0x29)
 				f << "val=" << std::hex << val << std::dec << ",";
 			else if (id==0x31)
@@ -767,7 +769,7 @@ bool WKS4Parser::readZone()
 			if (sz!=expectedSz) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			int dim[4];
-			for (int &i : dim) i=(int) libwps::read16(input);
+			for (int &i : dim) i=int(libwps::read16(input));
 			// in a spreadsheet, the cell or the cells corresponding to the field
 			// in a database, col,0,col,0
 			if (dim[0]==-1 && dim[1]==dim[0] && dim[2]==dim[0] && dim[3]==dim[0])
@@ -784,7 +786,7 @@ bool WKS4Parser::readZone()
 				f << "col=" << dim[0] << ",";
 			if (expectedSz==9)
 			{
-				val=(int) libwps::readU8(input); // 0|1|ff
+				val=int(libwps::readU8(input)); // 0|1|ff
 				if (val==0xFF) f << "true,";
 				else if (val) f << "val=" << val << ",";
 			}
@@ -796,7 +798,7 @@ bool WKS4Parser::readZone()
 			f << "Entries(Protection):global,";
 			if (sz!=1) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			if (val==0)
 			{
 				f.str("");
@@ -817,7 +819,7 @@ bool WKS4Parser::readZone()
 			for (int i=0; i<5; ++i)   // f1=4c|96|ac|f0
 			{
 				static const int expected[]= {4, 0x4c, 0x42, 2, 2};
-				val=(int) libwps::read16(input);
+				val=int(libwps::read16(input));
 				if (val!=expected[i]) f << "f" << i << "=" << val << ",";
 			}
 			isParsed=needWriteInAscii=true;
@@ -831,7 +833,7 @@ bool WKS4Parser::readZone()
 			if (sz!=1) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			f.str("");
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			f << "Entries(ItCount):dos";
 			if (val!=1) f << "=" << val << ",";
 			if (m_state->m_version==2)
@@ -916,7 +918,7 @@ bool WKS4Parser::readZone()
 			f << "Entries(ByteA" << std::hex << id << std::dec << "Z):";
 			if (sz!=1) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			if (val==0xFF) f << "true,";
 			else if (val) f << "#val=" << val << ",";
 			isParsed=needWriteInAscii=true;
@@ -928,7 +930,7 @@ bool WKS4Parser::readZone()
 			f << "Entries(IntSmallA" << std::hex << id << std::dec << "Z):";
 			if (sz!=1) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			if (id==0x1a)
 			{
 				f.str("");
@@ -950,7 +952,7 @@ bool WKS4Parser::readZone()
 				f << "Entries(IntA" << std::hex << id << std::dec << "Z):";
 			if (sz!=2) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::read16(input);
+			val=int(libwps::read16(input));
 			if (val) f << "f0=" << val << ",";
 			isParsed=needWriteInAscii=true;
 			break;
@@ -962,7 +964,7 @@ bool WKS4Parser::readZone()
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			for (int i=0; i<2; ++i)
 			{
-				val=(int) libwps::read16(input);
+				val=int(libwps::read16(input));
 				if (val) f << "f" << i << "=" << val << ",";
 			}
 			isParsed=needWriteInAscii=true;
@@ -973,15 +975,15 @@ bool WKS4Parser::readZone()
 			f << "Entries(SelectCells):";
 			if (sz!=0xc) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val = (int) libwps::read16(input); // always 0?
+			val = int(libwps::read16(input)); // always 0?
 			if (val) f << "f0=" << val << ",";
 			int dim[4];
-			for (int &i : dim) i = (int) libwps::read16(input);
+			for (int &i : dim) i = int(libwps::read16(input));
 			if (dim[2]==dim[0]+1 && dim[3]==dim[1]+1) // almost always true
 				f << "cell?=" << dim[0] << "x" << dim[1] << ",";
 			else
 				f << "cells?=" << dim[0] << "x" << dim[1] << "<->" << dim[2] << "x" << dim[3] << ",";
-			val = (int) libwps::read16(input); // always 0|2
+			val = int(libwps::read16(input)); // always 0|2
 			if (val) f << "f1=" << val << ",";
 			isParsed = needWriteInAscii = true;
 			break;
@@ -1006,18 +1008,18 @@ bool WKS4Parser::readZone()
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			for (int i=0; i<6; ++i)   // f0=2|7, f2=0|1|2|4|5|7, f4=0|1|4|5|6|17|19|37|114, f5=3..40
 			{
-				val=(int) libwps::read16(input);
+				val=int(libwps::read16(input));
 				if (val) f << "f" << i << "=" << val << ",";
 			}
 			for (int i=0; i<4; ++i)   // g0=0|12|38|59|71, g1=0|1, g3=1|2|3
 			{
 				int const expected[]= {0,1,0,2};
-				val=(int) libwps::read8(input);
+				val=int(libwps::read8(input));
 				if (val!=expected[i]) f << "g" << i << "=" << val << ",";
 			}
 			for (int i=0; i<4; ++i)   // h0=0|28, h1=0|9
 			{
-				val=(int) libwps::read16(input);
+				val=int(libwps::read16(input));
 				if (val) f << "h" << i << "=" << val << ",";
 			}
 			isParsed = needWriteInAscii = true;
@@ -1103,7 +1105,7 @@ bool WKS4Parser::readZone()
 			if (sz!=4) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			f << "id=" << libwps::read16(input) << ",";
-			val=(int) libwps::readU16(input);
+			val=int(libwps::readU16(input));
 			if (val) f << "f0=" << std::hex << val << std::dec << ",";
 			isParsed=needWriteInAscii=true;
 			break;
@@ -1119,13 +1121,13 @@ bool WKS4Parser::readZone()
 			// find with 050300000005|058000000005|05000000, so maybe:
 			if (sz<4 || (sz%2)!=0) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::read8(input); // always 5?
+			val=int(libwps::read8(input)); // always 5?
 			if (val!=5) f << "f0=" << val << ",";
-			val=(int) libwps::readU8(input); // 0|80
+			val=int(libwps::readU8(input)); // 0|80
 			if (val) f << "f1=" << std::hex << val << ",";
 			for (int i=1; i<sz/2; ++i)
 			{
-				val=(int) libwps::read16(input);
+				val=int(libwps::read16(input));
 				if (val) f << "f" << i+2 << "=" << val << ",";
 			}
 			isParsed=needWriteInAscii=true;
@@ -1141,7 +1143,7 @@ bool WKS4Parser::readZone()
 			f << "Entries(FontDef):";
 			if (sz!=4) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::read16(input);
+			val=int(libwps::read16(input));
 			if (val) f << "fId=" << val << ",";
 			f << "fSz=" << libwps::read16(input)/2 << ",";
 			isParsed=needWriteInAscii=true;
@@ -1155,7 +1157,7 @@ bool WKS4Parser::readZone()
 			f << "Entries(Protection)[form]:";
 			if (sz!=1) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			if (val==0)
 			{
 				f.str("");
@@ -1186,7 +1188,7 @@ bool WKS4Parser::readZone()
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
 			if (id==0x57)
 			{
-				val=(int) libwps::read16(input); // always 0?
+				val=int(libwps::read16(input)); // always 0?
 				if (val) f << "f0=" << val << ",";
 			}
 			librevenge::RVNGString name;
@@ -1195,9 +1197,9 @@ bool WKS4Parser::readZone()
 			else if (!name.empty())
 				f << name.cstr() << ",";
 			input->seek(pos+36+headerSize, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::read16(input); // 10|20|30|50: some flags?
+			val=int(libwps::read16(input)); // 10|20|30|50: some flags?
 			if (val!=0x10) f << "f1=" << std::hex << val << std::dec << ",";
-			val=(int) libwps::read16(input); // 14|18
+			val=int(libwps::read16(input)); // 14|18
 			if (val!=0x18) f << "f2=" << std::hex << val << std::dec << ",";
 			isParsed=needWriteInAscii=true;
 			break;
@@ -1269,7 +1271,7 @@ bool WKS4Parser::readZone()
 		{
 			if (sz!=4) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			auto dataSz=(long) libwps::readU32(input);
+			auto dataSz=long(libwps::readU32(input));
 			if (!checkFilePosition(pos+8+dataSz)) break;
 			if (dataSz) f << "dSz=" << std::hex << dataSz << std::dec << ",";
 			ascii().addPos(pos);
@@ -1302,12 +1304,12 @@ bool WKS4Parser::readZone()
 			f << "Entries(FldSeries):";
 			if (sz!=8) break;
 			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			val=(int) libwps::read16(input);
+			val=int(libwps::read16(input));
 			if (val) f << "col=" << val << ",";
 			f << "act[val]=" << libwps::read16(input) << ",";
-			val=(int) libwps::read16(input); // always 0 first?
+			val=int(libwps::read16(input)); // always 0 first?
 			if (val) f << "first=" << val << ",";
-			val=(int) libwps::read16(input);
+			val=int(libwps::read16(input));
 			if (val!=1) f << "increm=" << val << ",";
 			isParsed=needWriteInAscii=true;
 			break;
@@ -1362,9 +1364,9 @@ bool WKS4Parser::readZoneQuattro()
 	libwps::DebugStream f;
 	RVNGInputStreamPtr input = getInput();
 	long pos = input->tell();
-	auto id = (int) libwps::readU8(input);
-	auto type = (int) libwps::readU8(input);
-	auto sz = (long) libwps::readU16(input);
+	auto id = int(libwps::readU8(input));
+	auto type = int(libwps::readU8(input));
+	auto sz = long(libwps::readU16(input));
 	if (type>5 || sz<0 || !checkFilePosition(pos+4+sz))
 	{
 		input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -1393,19 +1395,15 @@ bool WKS4Parser::readCString(librevenge::RVNGString &string, long maxSize)
 		WPS_DEBUG_MSG(("WKS4Parser::readCString: string's size seems bad\n"));
 		return false;
 	}
-	auto const fontType = getDefaultFontType();
-	bool const hasLICS=hasLICSCharacters();
+	std::string text;
 	for (long i=0; i<maxSize; ++i)
 	{
-		auto c = (unsigned char) libwps::readU8(input);
+		auto c = char(libwps::readU8(input));
 		if (c == '\0') break;
-		if (!hasLICS)
-			libwps::appendUnicode
-			((uint32_t)libwps_tools_win::Font::unicode((unsigned char)c,fontType), string);
-		else
-			libwps::appendUnicode
-			((uint32_t)libwps_tools_win::Font::LICSunicode((unsigned char)c), string);
+		text.push_back(c);
 	}
+	if (!text.empty())
+		string=libwps_tools_win::Font::unicodeString(text, getDefaultFontType());
 	return true;
 }
 
@@ -1414,14 +1412,14 @@ bool WKS4Parser::readFont()
 	libwps::DebugStream f;
 	RVNGInputStreamPtr input = getInput();
 	long pos = input->tell();
-	auto type = (int)libwps::read16(input);
+	auto type = int(libwps::read16(input));
 
 	if (type != 0x5456)
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::readFont: not a font zone\n"));
 		return false;
 	}
-	auto sz = (long)libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos = pos+4+sz;
 	if (sz < 32)
 	{
@@ -1432,7 +1430,7 @@ bool WKS4Parser::readFont()
 	}
 
 	WKS4ParserInternal::Font font(getDefaultFontType());
-	auto flags = (int)libwps::readU8(input);
+	auto flags = int(libwps::readU8(input));
 	uint32_t attributes = 0;
 	if (flags & 1) attributes |= WPS_BOLD_BIT;
 	if (flags & 2) attributes |= WPS_ITALICS_BIT;
@@ -1449,12 +1447,12 @@ bool WKS4Parser::readFont()
 		}
 	}
 
-	auto val=(int)libwps::readU8(input);
+	auto val=int(libwps::readU8(input));
 	if (val) f << "f0=" << std::hex << val << std::dec << ",";
 	librevenge::RVNGString name("");
 	while (int(input->tell()) < endPos-4)
 	{
-		auto c = (char) libwps::readU8(input);
+		auto c = char(libwps::readU8(input));
 		if (c == '\0') break;
 		name.append(c);
 	}
@@ -1465,7 +1463,7 @@ bool WKS4Parser::readFont()
 	font.m_name=name;
 
 	input->seek(endPos-4, librevenge::RVNG_SEEK_SET);
-	val = (int) libwps::readU16(input); // always 0x20
+	val = int(libwps::readU16(input)); // always 0x20
 	if (val!=0x20)  f << "f1=" << std::hex << val << std::dec << ",";
 	int fSize = libwps::read16(input)/2;
 	if (fSize >= 1 && fSize <= 50)
@@ -1507,20 +1505,20 @@ bool WKS4Parser::readHeaderFooter(bool header)
 	libwps::DebugStream f;
 	RVNGInputStreamPtr input = getInput();
 	long pos = input->tell();
-	auto type = (int) libwps::read16(input);
+	auto type = int(libwps::read16(input));
 	if (type != 0x0026 && type != 0x0025)
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::readHeaderFooter: not a header/footer\n"));
 		return false;
 	}
-	auto sz = (long)libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos = pos+4+sz;
 
 	f << "Entries(" << (header ? "HeaderText" : "FooterText") << "):";
 	if (sz==1)
 	{
 		// followed with 0
-		auto val=(int) libwps::read8(input);
+		auto val=int(libwps::read8(input));
 		if (val) f << "##f0=" << val << ",";
 		ascii().addPos(pos);
 		ascii().addNote(f.str().c_str());
@@ -1555,13 +1553,13 @@ bool WKS4Parser::readPrnt()
 	libwps::DebugStream f;
 	RVNGInputStreamPtr input = getInput();
 	long pos = input->tell();
-	auto type = (int) libwps::read16(input);
+	auto type = int(libwps::read16(input));
 	if (type != 0x5423 && type != 0x5437)
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::readPrnt: not a prnt zone\n"));
 		return false;
 	}
-	auto sz = (long)libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos = pos+4+sz;
 
 	f << "Entries(PRNT):";
@@ -1577,12 +1575,12 @@ bool WKS4Parser::readPrnt()
 		// check me
 		if (type==0x5423)
 		{
-			m_state->m_pageSpan.setFormWidth(dim[5]);
-			m_state->m_pageSpan.setFormLength(dim[4]);
-			m_state->m_pageSpan.setMarginLeft(dim[0]);
-			m_state->m_pageSpan.setMarginTop(dim[2]);
-			m_state->m_pageSpan.setMarginRight(dim[3]);
-			m_state->m_pageSpan.setMarginBottom(dim[1]);
+			m_state->m_pageSpan.setFormWidth(double(dim[5]));
+			m_state->m_pageSpan.setFormLength(double(dim[4]));
+			m_state->m_pageSpan.setMarginLeft(double(dim[0]));
+			m_state->m_pageSpan.setMarginTop(double(dim[2]));
+			m_state->m_pageSpan.setMarginRight(double(dim[3]));
+			m_state->m_pageSpan.setMarginBottom(double(dim[1]));
 		}
 	}
 	int val = libwps::read16(input);
@@ -1636,7 +1634,7 @@ bool WKS4Parser::readPrn2()
 	 */
 	for (int i = 0; i < numElt; i++)
 	{
-		auto val = (int) libwps::read16(input);
+		auto val = int(libwps::read16(input));
 		if (!val) continue;
 		f << "f" << i << "=" << std::hex << val << std::dec << ",";
 	}
@@ -1652,13 +1650,13 @@ bool WKS4Parser::readFieldName()
 	RVNGInputStreamPtr input = getInput();
 
 	long pos = input->tell();
-	auto type = (long) libwps::readU16(input);
+	auto type = long(libwps::readU16(input));
 	if (type != 0xb)
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::readFieldName: not a zoneB type\n"));
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	f << "Entries(FldNames):";
 	if (sz != 0x18 && sz != 0x1e)
 	{
@@ -1680,13 +1678,13 @@ bool WKS4Parser::readFieldName()
 	int dim[4];
 	if (sz==0x18)
 	{
-		for (int &i : dim) i=(int) libwps::read16(input);
+		for (int &i : dim) i=int(libwps::read16(input));
 	}
 	else
 	{
 		for (int i=0; i<7; ++i)
 		{
-			auto val=(int) libwps::read16(input);
+			auto val=int(libwps::read16(input));
 			if (i<2) dim[i]=val;
 			else if (i>=3 && i<5) dim[i-1]=val;
 			else if (val) f << "f" << i << "=" << val << ",";
@@ -1720,13 +1718,13 @@ bool WKS4Parser::readWindowRecord()
 	RVNGInputStreamPtr input = getInput();
 
 	long pos = input->tell();
-	auto type = (long) libwps::read16(input);
+	auto type = long(libwps::read16(input));
 	if (type != 7 && type != 9)
 	{
 		WPS_DEBUG_MSG(("WKS4Parser::readWindowRecord: unknown type\n"));
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 
 	// normally size=0x1f but one time 0x1e
 	if (sz < 0x1e)
@@ -1743,15 +1741,15 @@ bool WKS4Parser::readWindowRecord()
 	// f8=0|1, f9=0|2, f10=f11=0
 	for (int i=0; i<12; ++i)
 	{
-		auto val=(int) libwps::read16(input);
+		auto val=int(libwps::read16(input));
 		if (val) f << "f" << i << "=" << val << ",";
 	}
 	for (int i=0; i<2; ++i)   // g0=4, g1=4|a|b|10
 	{
-		auto val=(int) libwps::read16(input);
+		auto val=int(libwps::read16(input));
 		if (val!=4) f << "g" << i << "=" << val << ",";
 	}
-	auto val=(int) libwps::read16(input); // number between -5 and ad
+	auto val=int(libwps::read16(input)); // number between -5 and ad
 	f << "g2=" << val << ",";
 
 	if (sz!=0x1e)
@@ -1769,7 +1767,7 @@ bool WKS4Parser::readUnknown1()
 	RVNGInputStreamPtr input = getInput();
 
 	long pos = input->tell();
-	auto type = (long) libwps::read16(input);
+	auto type = long(libwps::read16(input));
 	int expectedSize=0, extraSize=0;
 	switch (type)
 	{
@@ -1789,7 +1787,7 @@ bool WKS4Parser::readUnknown1()
 		WPS_DEBUG_MSG(("WKS4Parser::readUnknown1: unexpected type ???\n"));
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 
 	f << "Entries(Flags" << std::hex << type << std::dec << ")]:";
 	if (sz != expectedSize+extraSize)
@@ -1797,7 +1795,7 @@ bool WKS4Parser::readUnknown1()
 		// find also 270001000[01]
 		if (type==0x27 && sz==1)
 		{
-			f << "f0=" << (int) libwps::read8(input) << ",";
+			f << "f0=" << int(libwps::read8(input)) << ",";
 			ascii().addPos(pos);
 			ascii().addNote(f.str().c_str());
 			return true;
@@ -1812,17 +1810,17 @@ bool WKS4Parser::readUnknown1()
 	// find always 0xff, excepted for zone 18(f0=0), zone 19(f24=0|3), zone 27(f0=..=f23=0|ff, f24=0|3)
 	for (int i=0; i<expectedSize; ++i)
 	{
-		auto val=(int) libwps::read8(input);
+		auto val=int(libwps::read8(input));
 		if (val!=-1) f << "f" << i << "=" << val << ",";
 	}
 
 	if (type==0x27)
 	{
-		auto val=(int) libwps::read8(input); // always 0
+		auto val=int(libwps::read8(input)); // always 0
 		if (val) f << "g0=" << val << ",";
 		for (int i=0; i<7; ++i)   // g1=0|4, g2=0|72, g4=0|20, g5=0|1|4, g6=0|1, g7=0|-1|205|80d8|e9f
 		{
-			val=(int) libwps::read16(input);
+			val=int(libwps::read16(input));
 			if (val) f << "g" << i+1 << "=" << val << ",";
 		}
 	}
