@@ -551,13 +551,14 @@ bool WPSOLE1Parser::updateMetaData(librevenge::RVNGPropertyList &metadata, libwp
 			case 5:
 			case 9:
 			{
-				librevenge::RVNGString name("");
+				std::string text;
 				for (int i=0; i<dSz; ++i)
 				{
-					auto ch=static_cast<unsigned char>(libwps::readU8(input));
+					auto ch=char(libwps::readU8(input));
 					if (!ch) break;
-					libwps::appendUnicode((uint32_t)libwps_tools_win::Font::unicode(ch,encoding), name);
+					text.push_back(ch);
 				}
+				auto name=libwps_tools_win::Font::unicodeString(text, encoding);
 				ok=true;
 				if (name.empty()) break;
 				if (wh==9)
