@@ -25,7 +25,7 @@
  *
  ************************************************************************/
 
-#include "test_global.hpp"
+#include "test_global.hpp" // This must be the first header to be included.
 
 #define MDDS_MULTI_TYPE_VECTOR_DEBUG 1
 #include <mdds/multi_type_vector.hpp>
@@ -42,29 +42,29 @@ struct event_block_counter
     size_t block_count;  // number of element (data) blocks
     size_t block_count_numeric;
     size_t block_count_string;
-    size_t block_count_short;
-    size_t block_count_ushort;
-    size_t block_count_int;
-    size_t block_count_uint;
-    size_t block_count_long;
-    size_t block_count_ulong;
+    size_t block_count_int16;
+    size_t block_count_uint16;
+    size_t block_count_int32;
+    size_t block_count_uint32;
+    size_t block_count_int64;
+    size_t block_count_uint64;
     size_t block_count_boolean;
-    size_t block_count_char;
-    size_t block_count_uchar;
+    size_t block_count_int8;
+    size_t block_count_uint8;
 
     event_block_counter() :
         block_count(0),
         block_count_numeric(0),
         block_count_string(0),
-        block_count_short(0),
-        block_count_ushort(0),
-        block_count_int(0),
-        block_count_uint(0),
-        block_count_long(0),
-        block_count_ulong(0),
+        block_count_int16(0),
+        block_count_uint16(0),
+        block_count_int32(0),
+        block_count_uint32(0),
+        block_count_int64(0),
+        block_count_uint64(0),
         block_count_boolean(0),
-        block_count_char(0),
-        block_count_uchar(0)
+        block_count_int8(0),
+        block_count_uint8(0)
     {}
 
     void element_block_acquired(const mtv::base_element_block* block)
@@ -73,38 +73,38 @@ struct event_block_counter
 
         switch (mtv::get_block_type(*block))
         {
-            case mtv::element_type_numeric:
+            case mtv::element_type_double:
                 ++block_count_numeric;
             break;
             case mtv::element_type_string:
                 ++block_count_string;
             break;
-            case mtv::element_type_short:
-                ++block_count_short;
+            case mtv::element_type_int16:
+                ++block_count_int16;
             break;
-            case mtv::element_type_ushort:
-                ++block_count_ushort;
+            case mtv::element_type_uint16:
+                ++block_count_uint16;
             break;
-            case mtv::element_type_int:
-                ++block_count_int;
+            case mtv::element_type_int32:
+                ++block_count_int32;
             break;
-            case mtv::element_type_uint:
-                ++block_count_uint;
+            case mtv::element_type_uint32:
+                ++block_count_uint32;
             break;
-            case mtv::element_type_long:
-                ++block_count_long;
+            case mtv::element_type_int64:
+                ++block_count_int64;
             break;
-            case mtv::element_type_ulong:
-                ++block_count_ulong;
+            case mtv::element_type_uint64:
+                ++block_count_uint64;
             break;
             case mtv::element_type_boolean:
                 ++block_count_boolean;
             break;
-            case mtv::element_type_char:
-                ++block_count_char;
+            case mtv::element_type_int8:
+                ++block_count_int8;
             break;
-            case mtv::element_type_uchar:
-                ++block_count_uchar;
+            case mtv::element_type_uint8:
+                ++block_count_uint8;
             break;
             default:
                 ;
@@ -117,38 +117,38 @@ struct event_block_counter
 
         switch (mtv::get_block_type(*block))
         {
-            case mtv::element_type_numeric:
+            case mtv::element_type_double:
                 --block_count_numeric;
             break;
             case mtv::element_type_string:
                 --block_count_string;
             break;
-            case mtv::element_type_short:
-                --block_count_short;
+            case mtv::element_type_int16:
+                --block_count_int16;
             break;
-            case mtv::element_type_ushort:
-                --block_count_ushort;
+            case mtv::element_type_uint16:
+                --block_count_uint16;
             break;
-            case mtv::element_type_int:
-                --block_count_int;
+            case mtv::element_type_int32:
+                --block_count_int32;
             break;
-            case mtv::element_type_uint:
-                --block_count_uint;
+            case mtv::element_type_uint32:
+                --block_count_uint32;
             break;
-            case mtv::element_type_long:
-                --block_count_long;
+            case mtv::element_type_int64:
+                --block_count_int64;
             break;
-            case mtv::element_type_ulong:
-                --block_count_ulong;
+            case mtv::element_type_uint64:
+                --block_count_uint64;
             break;
             case mtv::element_type_boolean:
                 --block_count_boolean;
             break;
-            case mtv::element_type_char:
-                --block_count_char;
+            case mtv::element_type_int8:
+                --block_count_int8;
             break;
-            case mtv::element_type_uchar:
-                --block_count_uchar;
+            case mtv::element_type_uint8:
+                --block_count_uint8;
             break;
             default:
                 ;
@@ -395,16 +395,16 @@ void mtv_test_block_counter()
         db4.insert(0, vals.begin(), vals.end());
         assert(db4.event_handler().block_count == 1);
 
-        mtv_type db5(3, long(10));
+        mtv_type db5(3, int64_t(10));
         assert(db5.event_handler().block_count == 1);
         db5.insert(0, vals.begin(), vals.end());
         assert(db5.event_handler().block_count == 2);
 
-        mtv_type db6(2, int(30));
+        mtv_type db6(2, int32_t(30));
         assert(db6.event_handler().block_count == 1);
         db6.insert(1, vals.begin(), vals.end()); // Insert to split the block.
         assert(db6.event_handler().block_count == 3);
-        assert(db6.event_handler().block_count_int == 2);
+        assert(db6.event_handler().block_count_int32 == 2);
         assert(db6.event_handler().block_count_numeric == 1);
     }
 
@@ -443,18 +443,18 @@ void mtv_test_block_counter()
 
         db.set(1, 1.1);
         assert(db.event_handler().block_count == 3);
-        db.set(2, long(10));
+        db.set(2, int64_t(10));
         db.set(1, true);
         assert(db.event_handler().block_count == 2);
         assert(db.event_handler().block_count_boolean == 1);
-        assert(db.event_handler().block_count_long == 1);
+        assert(db.event_handler().block_count_int64 == 1);
 
         db.set(1, 1.1);
         assert(db.event_handler().block_count == 3);
         assert(db.event_handler().block_count_boolean == 1);
-        assert(db.event_handler().block_count_long == 1);
+        assert(db.event_handler().block_count_int64 == 1);
         assert(db.event_handler().block_count_numeric == 1);
-        db.set(1, long(20));
+        db.set(1, int64_t(20));
         assert(db.event_handler().block_count == 2);
 
         db.release();
@@ -464,10 +464,10 @@ void mtv_test_block_counter()
     {
         mtv_type db;
         db.push_back(1.1);
-        db.push_back(long(10));
+        db.push_back(int64_t(10));
         db.push_back(string("foo"));
         assert(db.event_handler().block_count == 3);
-        assert(db.event_handler().block_count_long == 1);
+        assert(db.event_handler().block_count_int64 == 1);
         assert(db.event_handler().block_count_numeric == 1);
         assert(db.event_handler().block_count_string == 1);
 
@@ -520,29 +520,29 @@ void mtv_test_block_counter()
     {
         vector<double> vals = { 1.1, 1.2 };
         mtv_type db(4);
-        db.set(0, int(5));
-        db.set(1, int(10));
+        db.set(0, int32_t(5));
+        db.set(1, int32_t(10));
         assert(db.event_handler().block_count == 1);
-        assert(db.event_handler().block_count_int == 1);
+        assert(db.event_handler().block_count_int32 == 1);
         db.set(2, vals.begin(), vals.end()); // set to empty block.
         assert(db.event_handler().block_count == 2);
-        assert(db.event_handler().block_count_int == 1);
+        assert(db.event_handler().block_count_int32 == 1);
         assert(db.event_handler().block_count_numeric == 1);
     }
 
     {
         vector<double> vals = { 1.1, 1.2 };
         mtv_type db(4);
-        db.set(0, int(5));
-        db.set(1, int(10));
+        db.set(0, int32_t(5));
+        db.set(1, int32_t(10));
         db.set(2, string("foo"));
         db.set(3, string("bar"));
         assert(db.event_handler().block_count == 2);
-        assert(db.event_handler().block_count_int == 1);
+        assert(db.event_handler().block_count_int32 == 1);
         assert(db.event_handler().block_count_string == 1);
         db.set(2, vals.begin(), vals.end()); // replace a block.
         assert(db.event_handler().block_count == 2);
-        assert(db.event_handler().block_count_int == 1);
+        assert(db.event_handler().block_count_int32 == 1);
         assert(db.event_handler().block_count_numeric == 1);
     }
 
@@ -568,12 +568,12 @@ void mtv_test_block_counter()
     {
         vector<double> vals = { 1.1, 1.2 };
         mtv_type db(4, string("foo"));
-        db.push_back(long(100));
+        db.push_back(int64_t(100));
         assert(db.event_handler().block_count == 2);
         db.set(2, vals.begin(), vals.end()); // replace the lower part of a block.
         assert(db.event_handler().block_count == 3);
         assert(db.event_handler().block_count_string == 1);
-        assert(db.event_handler().block_count_long == 1);
+        assert(db.event_handler().block_count_int64 == 1);
         assert(db.event_handler().block_count_numeric == 1);
     }
 
@@ -589,11 +589,11 @@ void mtv_test_block_counter()
 
     {
         mtv_type db(1, 0.1);
-        db.push_back(short(1));
-        db.push_back(int(20));
+        db.push_back(int16_t(1));
+        db.push_back(int32_t(20));
         assert(db.event_handler().block_count == 3);
-        assert(db.event_handler().block_count_short == 1);
-        assert(db.event_handler().block_count_int == 1);
+        assert(db.event_handler().block_count_int16 == 1);
+        assert(db.event_handler().block_count_int32 == 1);
         assert(db.event_handler().block_count_numeric == 1);
 
         vector<double> vals = { 1.1, 1.2, 1.3 }; // same type as the top block.
@@ -604,11 +604,11 @@ void mtv_test_block_counter()
 
     {
         mtv_type db(1, string("foo"));
-        db.push_back(short(1));
-        db.push_back(int(20));
+        db.push_back(int16_t(1));
+        db.push_back(int32_t(20));
         assert(db.event_handler().block_count == 3);
-        assert(db.event_handler().block_count_short == 1);
-        assert(db.event_handler().block_count_int == 1);
+        assert(db.event_handler().block_count_int16 == 1);
+        assert(db.event_handler().block_count_int32 == 1);
         assert(db.event_handler().block_count_string == 1);
 
         vector<double> vals = { 1.1, 1.2, 1.3 }; // differene type from that of the top block.
@@ -620,29 +620,29 @@ void mtv_test_block_counter()
     {
         mtv_type db(6);
         db.set(2, 1.1);
-        db.set(3, int(22));
+        db.set(3, int32_t(22));
         assert(db.event_handler().block_count == 2);
-        assert(db.event_handler().block_count_int == 1);
+        assert(db.event_handler().block_count_int32 == 1);
         assert(db.event_handler().block_count_numeric == 1);
         db.erase(2, 3);
         assert(db.event_handler().block_count == 0);
     }
 
     {
-        mtv_type db(6, char('a'));
+        mtv_type db(6, int8_t('a'));
         db.set(2, 1.1);
-        db.set(3, int(22));
+        db.set(3, int32_t(22));
         assert(db.event_handler().block_count == 4);
-        assert(db.event_handler().block_count_int == 1);
+        assert(db.event_handler().block_count_int32 == 1);
         assert(db.event_handler().block_count_numeric == 1);
-        assert(db.event_handler().block_count_char == 2);
+        assert(db.event_handler().block_count_int8 == 2);
         db.erase(2, 3);
         assert(db.event_handler().block_count == 1);
-        assert(db.event_handler().block_count_char == 1);
+        assert(db.event_handler().block_count_int8 == 1);
     }
 
     {
-        mtv_type src(6, char('a')), dst(6);
+        mtv_type src(6, int8_t('a')), dst(6);
         assert(src.event_handler().block_count == 1);
         assert(dst.event_handler().block_count == 0);
         src.transfer(0, 2, dst, 0);
@@ -656,14 +656,14 @@ void mtv_test_block_counter()
 
     {
         mtv_type src(6), dst(6);
-        src.set(0, char('z'));
-        src.set(1, int(10));
-        src.set(2, short(5));
+        src.set(0, int8_t('z'));
+        src.set(1, int32_t(10));
+        src.set(2, int16_t(5));
         dst.set(3, 1.1);
         assert(src.event_handler().block_count == 3);
-        assert(src.event_handler().block_count_char == 1);
-        assert(src.event_handler().block_count_int == 1);
-        assert(src.event_handler().block_count_short == 1);
+        assert(src.event_handler().block_count_int8 == 1);
+        assert(src.event_handler().block_count_int32 == 1);
+        assert(src.event_handler().block_count_int16 == 1);
         assert(dst.event_handler().block_count == 1);
         assert(dst.event_handler().block_count_numeric == 1);
 
@@ -671,9 +671,9 @@ void mtv_test_block_counter()
         assert(src.event_handler().block_count == 0);
         assert(dst.event_handler().block_count == 4);
         assert(dst.event_handler().block_count_numeric == 1);
-        assert(dst.event_handler().block_count_char == 1);
-        assert(dst.event_handler().block_count_int == 1);
-        assert(dst.event_handler().block_count_short == 1);
+        assert(dst.event_handler().block_count_int8 == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
+        assert(dst.event_handler().block_count_int16 == 1);
     }
 
     {
@@ -825,19 +825,19 @@ void mtv_test_block_counter()
         src.set(2, 1.3);
 
         dst.set(0, string("2.1"));
-        dst.set(1, int(100));
+        dst.set(1, int32_t(100));
         dst.set(2, string("2.3"));
 
         assert(src.event_handler().block_count == 1);
         assert(src.event_handler().block_count_numeric == 1);
         assert(dst.event_handler().block_count == 3);
         assert(dst.event_handler().block_count_string == 2);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
 
         src.swap(0, 2, dst, 0);
         assert(src.event_handler().block_count == 3);
         assert(src.event_handler().block_count_string == 2);
-        assert(src.event_handler().block_count_int == 1);
+        assert(src.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count == 1);
         assert(dst.event_handler().block_count_numeric == 1);
     }
@@ -852,7 +852,7 @@ void mtv_test_block_counter()
 
         dst.set(0, string("2.1"));
         dst.set(1, string("2.2"));
-        dst.set(2, int(100));
+        dst.set(2, int32_t(100));
         dst.set(3, string("2.3"));
         dst.set(4, string("2.4"));
 
@@ -860,13 +860,13 @@ void mtv_test_block_counter()
         assert(src.event_handler().block_count_numeric == 1);
         assert(dst.event_handler().block_count == 3);
         assert(dst.event_handler().block_count_string == 2);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
 
         src.swap(1, 3, dst, 1);
         assert(src.event_handler().block_count == 5);
         assert(src.event_handler().block_count_numeric == 2);
         assert(src.event_handler().block_count_string == 2);
-        assert(src.event_handler().block_count_int == 1);
+        assert(src.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count == 3);
         assert(dst.event_handler().block_count_string == 2);
         assert(dst.event_handler().block_count_numeric == 1);
@@ -879,19 +879,19 @@ void mtv_test_block_counter()
         src.set(2, 1.3);
 
         dst.set(0, 2.1);
-        dst.set(1, int(5));
-        dst.set(2, int(6));
-        dst.set(3, int(7));
+        dst.set(1, int32_t(5));
+        dst.set(2, int32_t(6));
+        dst.set(3, int32_t(7));
 
         assert(src.event_handler().block_count == 1);
         assert(src.event_handler().block_count_numeric == 1);
         assert(dst.event_handler().block_count == 2);
         assert(dst.event_handler().block_count_numeric == 1);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
 
         src.swap(0, 2, dst, 1);
         assert(src.event_handler().block_count == 1);
-        assert(src.event_handler().block_count_int == 1);
+        assert(src.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count == 1);
         assert(dst.event_handler().block_count_numeric == 1);
     }
@@ -903,20 +903,20 @@ void mtv_test_block_counter()
         src.set(2, 1.3);
 
         dst.set(0, 2.1);
-        dst.set(1, int(5));
-        dst.set(2, int(6));
-        dst.set(3, int(7));
+        dst.set(1, int32_t(5));
+        dst.set(2, int32_t(6));
+        dst.set(3, int32_t(7));
         dst.set(4, 2.2);
 
         assert(src.event_handler().block_count == 1);
         assert(src.event_handler().block_count_numeric == 1);
         assert(dst.event_handler().block_count == 3);
         assert(dst.event_handler().block_count_numeric == 2);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
 
         src.swap(0, 2, dst, 1);
         assert(src.event_handler().block_count == 1);
-        assert(src.event_handler().block_count_int == 1);
+        assert(src.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count == 1);
         assert(dst.event_handler().block_count_numeric == 1);
     }
@@ -927,20 +927,20 @@ void mtv_test_block_counter()
         src.set(1, 1.2);
         src.set(2, 1.3);
 
-        dst.set(0, int(5));
-        dst.set(1, int(6));
-        dst.set(2, int(7));
+        dst.set(0, int32_t(5));
+        dst.set(1, int32_t(6));
+        dst.set(2, int32_t(7));
         dst.set(3, 2.1);
 
         assert(src.event_handler().block_count == 1);
         assert(src.event_handler().block_count_numeric == 1);
         assert(dst.event_handler().block_count == 2);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count_numeric == 1);
 
         src.swap(0, 2, dst, 0);
         assert(src.event_handler().block_count == 1);
-        assert(src.event_handler().block_count_int == 1);
+        assert(src.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count == 1);
         assert(dst.event_handler().block_count_numeric == 1);
     }
@@ -951,10 +951,10 @@ void mtv_test_block_counter()
         src.set(1, 1.2);
         src.set(2, 1.3);
 
-        dst.set(0, int(5));
-        dst.set(1, int(6));
-        dst.set(2, int(7));
-        dst.set(3, int(8));
+        dst.set(0, int32_t(5));
+        dst.set(1, int32_t(6));
+        dst.set(2, int32_t(7));
+        dst.set(3, int32_t(8));
 
         assert(src.event_handler().block_count == 1);
         assert(dst.event_handler().block_count == 1);
@@ -962,7 +962,7 @@ void mtv_test_block_counter()
         src.swap(0, 2, dst, 0);
         assert(src.event_handler().block_count == 1);
         assert(dst.event_handler().block_count == 2);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count_numeric == 1);
     }
 
@@ -973,23 +973,23 @@ void mtv_test_block_counter()
         src.set(2, 1.3);
 
         dst.set(0, 2.1);
-        dst.set(1, int(5));
-        dst.set(2, int(6));
-        dst.set(3, int(7));
-        dst.set(4, int(8));
+        dst.set(1, int32_t(5));
+        dst.set(2, int32_t(6));
+        dst.set(3, int32_t(7));
+        dst.set(4, int32_t(8));
 
         assert(src.event_handler().block_count == 1);
         assert(src.event_handler().block_count_numeric == 1);
         assert(dst.event_handler().block_count == 2);
         assert(dst.event_handler().block_count_numeric == 1);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
 
         src.swap(0, 2, dst, 1);
         assert(src.event_handler().block_count == 1);
-        assert(src.event_handler().block_count_int == 1);
+        assert(src.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count == 2);
         assert(dst.event_handler().block_count_numeric == 1);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
     }
 
     {
@@ -998,20 +998,20 @@ void mtv_test_block_counter()
         src.set(1, 1.2);
         src.set(2, 1.3);
 
-        dst.set(0, int(4));
-        dst.set(1, int(5));
-        dst.set(2, int(6));
-        dst.set(3, int(7));
-        dst.set(4, int(8));
+        dst.set(0, int32_t(4));
+        dst.set(1, int32_t(5));
+        dst.set(2, int32_t(6));
+        dst.set(3, int32_t(7));
+        dst.set(4, int32_t(8));
 
         assert(src.event_handler().block_count == 1);
         assert(dst.event_handler().block_count == 1);
 
         src.swap(0, 2, dst, 1);
         assert(src.event_handler().block_count == 1);
-        assert(src.event_handler().block_count_int == 1);
+        assert(src.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count == 3);
-        assert(dst.event_handler().block_count_int == 2);
+        assert(dst.event_handler().block_count_int32 == 2);
         assert(dst.event_handler().block_count_numeric == 1);
     }
 
@@ -1021,21 +1021,21 @@ void mtv_test_block_counter()
         src.set(1, 1.2);
         src.set(2, 1.3);
 
-        dst.set(0, int(4));
-        dst.set(1, int(5));
-        dst.set(2, int(6));
-        dst.set(3, int(7));
+        dst.set(0, int32_t(4));
+        dst.set(1, int32_t(5));
+        dst.set(2, int32_t(6));
+        dst.set(3, int32_t(7));
 
         assert(src.event_handler().block_count == 1);
         assert(src.event_handler().block_count_numeric == 1);
         assert(dst.event_handler().block_count == 1);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
 
         src.swap(0, 2, dst, 1);
         assert(src.event_handler().block_count == 1);
-        assert(src.event_handler().block_count_int == 1);
+        assert(src.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count == 2);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count_numeric == 1);
     }
 
@@ -1045,23 +1045,23 @@ void mtv_test_block_counter()
         src.set(1, 1.2);
         src.set(2, 1.3);
 
-        dst.set(0, int(4));
-        dst.set(1, int(5));
-        dst.set(2, int(6));
-        dst.set(3, int(7));
+        dst.set(0, int32_t(4));
+        dst.set(1, int32_t(5));
+        dst.set(2, int32_t(6));
+        dst.set(3, int32_t(7));
         dst.set(4, 2.1);
 
         assert(src.event_handler().block_count == 1);
         assert(src.event_handler().block_count_numeric == 1);
         assert(dst.event_handler().block_count == 2);
         assert(dst.event_handler().block_count_numeric == 1);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
 
         src.swap(0, 2, dst, 1);
         assert(src.event_handler().block_count == 1);
-        assert(src.event_handler().block_count_int == 1);
+        assert(src.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count == 2);
-        assert(dst.event_handler().block_count_int == 1);
+        assert(dst.event_handler().block_count_int32 == 1);
         assert(dst.event_handler().block_count_numeric == 1);
     }
 
