@@ -7,10 +7,12 @@
 
 #include "orcus/parser_global.hpp"
 #include "orcus/cell_buffer.hpp"
+#include "orcus/global.hpp"
 
 #include <cassert>
 #include <cmath>
 #include <iostream>
+#include <limits>
 
 namespace orcus {
 
@@ -19,7 +21,7 @@ const size_t parse_quoted_string_state::error_illegal_escape_char = 2;
 
 bool is_blank(char c)
 {
-    return is_in(c, " \t\n\r");
+    return is_in(c, ORCUS_ASCII(" \t\n\r"));
 }
 
 bool is_alpha(char c)
@@ -33,7 +35,7 @@ bool is_alpha(char c)
 
 bool is_name_char(char c)
 {
-    return is_in(c, "-_");
+    return is_in(c, ORCUS_ASCII("-_"));
 }
 
 bool is_numeric(char c)
@@ -41,9 +43,16 @@ bool is_numeric(char c)
     return ('0' <= c && c <= '9');
 }
 
-bool is_in(char c, const char* allowed)
+bool is_in(char c, const char* allowed, size_t n_allowed)
 {
-    for (; *allowed != '\0'; ++allowed)
+#ifdef __ORCUS_DEBUG_UTILS
+    if (allowed && !n_allowed)
+        throw std::invalid_argument("'allowed' pointer is non-null but the value of 'n_allowed' is 0.");
+#endif
+
+    const char* p_end = allowed + n_allowed;
+
+    for (; allowed != p_end; ++allowed)
     {
         if (c == *allowed)
             return true;
@@ -71,7 +80,7 @@ namespace {
  */
 double parse_exponent(const char*& p, const char* p_end)
 {
-    const char* p0 = p; // original position to restore to in case of parsing failure.
+    const char* p0 = p - 1; // original position to restore to in case of parsing failure. The e needs to be added back as well.
     double exponent = 0.0;
     bool negative_sign = false;
     bool valid = false;
@@ -172,6 +181,9 @@ double parse_numeric(const char*& p, size_t max_length)
 
         if (*p < '0' || '9' < *p)
         {
+            if (!has_digit) // without a digit we have no numbers
+                return std::numeric_limits<double>::quiet_NaN();
+
             ret /= divisor;
             return negative_sign ? -ret : ret;
         }
@@ -183,6 +195,8 @@ double parse_numeric(const char*& p, size_t max_length)
         if (!before_decimal_pt)
             divisor *= 10.0;
     }
+    if (!has_digit) // without a digit we have no numbers
+        return std::numeric_limits<double>::quiet_NaN();
 
     ret /= divisor;
     return negative_sign ? -ret : ret;
@@ -492,7 +506,12 @@ parse_quoted_string_state parse_double_quoted_string(
     const char*& p, size_t max_length, cell_buffer& buffer)
 {
     assert(*p == '"');
+
     parse_quoted_string_state ret;
+    ret.str = nullptr;
+    ret.length = 0;
+    ret.transient = false;
+
     if (max_length == 0)
     {
         ret.str = nullptr;
