@@ -5,13 +5,13 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
+#include "test_global.hpp"
 #include "orcus/sax_token_parser.hpp"
 #include "orcus/tokens.hpp"
 #include "orcus/global.hpp"
 #include "orcus/xml_namespace.hpp"
 
 #include <cstring>
-#include <iostream>
 
 using namespace std;
 using namespace orcus;
@@ -119,8 +119,8 @@ void test_unicode_string()
     {
         orcus::pstring str;
     public:
-        handler(orcus::pstring str):
-            str(str)
+        handler(orcus::pstring _str):
+            str(_str)
             {}
 
         void declaration(const orcus::xml_declaration_t&) {}
@@ -142,6 +142,7 @@ void test_unicode_string()
     };
 
     const char* token_names[] = {
+        "???",
     };
     size_t token_count = ORCUS_N_ELEMENTS(token_names);
 
@@ -161,8 +162,6 @@ void test_unicode_string()
 
 void test_declaration()
 {
-    xml_declaration_t decl;
-
     class handler
     {
         xml_declaration_t& m_decl;
