@@ -5,13 +5,12 @@
 #include <cstdint>
 #include <memory>
 
-#include "core/fxcodec/codec/ccodec_basicmodule.h"
+#include "core/fxcodec/basic/basicmodule.h"
 #include "core/fxcrt/fx_memory.h"
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size) {
   std::unique_ptr<uint8_t, FxFreeDeleter> dest_buf;
   uint32_t dest_size = 0;
-  CCodec_BasicModule encoder_module;
-  encoder_module.A85Encode({data, size}, &dest_buf, &dest_size);
+  BasicModule::A85Encode({data, size}, &dest_buf, &dest_size);
   return 0;
 }
