@@ -8,6 +8,7 @@
 #include "core/fxcrt/fx_memory.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "testing/test_support.h"
 
 #ifdef PDF_ENABLE_V8
 #include "testing/v8_initializer.h"
@@ -27,15 +28,15 @@ int main(int argc, char** argv) {
 #ifdef PDF_ENABLE_V8
   std::unique_ptr<v8::Platform> platform;
 #ifdef V8_USE_EXTERNAL_STARTUP_DATA
-  static v8::StartupData* natives = new v8::StartupData;
   static v8::StartupData* snapshot = new v8::StartupData;
-  platform = InitializeV8ForPDFiumWithStartupData(argv[0], std::string(),
-                                                  natives, snapshot);
+  platform =
+      InitializeV8ForPDFiumWithStartupData(argv[0], std::string(), snapshot);
 #else  // V8_USE_EXTERNAL_STARTUP_DATA
   platform = InitializeV8ForPDFium(argv[0]);
 #endif  // V8_USE_EXTERNAL_STARTUP_DATA
 #endif  // PDF_ENABLE_V8
 
+  InitializePDFTestEnvironment();
 #ifdef PDF_ENABLE_XFA
   InitializeXFATestEnvironment();
 #endif  // PDF_ENABLE_XFA
@@ -45,6 +46,9 @@ int main(int argc, char** argv) {
 
   int ret_val = RUN_ALL_TESTS();
 
+  DestroyPDFTestEnvironment();
+  // NOTE: XFA test environment, if present, destroyed by gtest.
+
 #ifdef PDF_ENABLE_V8
   v8::V8::ShutdownPlatform();
 #endif  // PDF_ENABLE_V8
