@@ -13,7 +13,7 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2007-2008, 2010, 2012, 2015-2018 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2008, 2010, 2012, 2015-2019 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2010 Mike Slegeir <tehpola@yahoo.com>
 // Copyright (C) 2010, 2013 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
@@ -26,6 +26,8 @@
 // Copyright (C) 2015 William Bader <williambader@hotmail.com>
 // Copyright (C) 2017 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
+// Copyright (C) 2018 Thibaut Brard <thibaut.brard@gmail.com>
+// Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -44,6 +46,8 @@
 #include <time.h>
 #include "parseargs.h"
 #include "goo/GooString.h"
+#include "goo/gbase64.h"
+#include "goo/gbasename.h"
 #include "goo/gmem.h"
 #include "Object.h"
 #include "Stream.h"
@@ -66,31 +70,34 @@
 #include "DateInfo.h"
 #include "goo/gfile.h"
 #include "Win32Console.h"
+#include "InMemoryFile.h"
 
 static int firstPage = 1;
 static int lastPage = 0;
-static GBool rawOrder = gTrue;
-GBool printCommands = gTrue;
-static GBool printHelp = gFalse;
-GBool printHtml = gFalse;
-GBool complexMode=gFalse;
-GBool singleHtml=gFalse; // singleHtml
-GBool ignore=gFalse;
+static bool rawOrder = true;
+bool printCommands = true;
+static bool printHelp = false;
+bool printHtml = false;
+bool complexMode=false;
+bool singleHtml=false; // singleHtml
+bool dataUrls = false;
+bool ignore=false;
 static char extension[5]="png";
 static double scale=1.5;
-GBool noframes=gFalse;
-GBool stout=gFalse;
-GBool xml=gFalse;
-static GBool errQuiet=gFalse;
-static GBool noDrm=gFalse;
+bool noframes=false;
+bool stout=false;
+bool xml=false;
+bool noRoundedCoordinates = false;
+static bool errQuiet=false;
+static bool noDrm=false;
 double wordBreakThreshold=10;  // 10%, below converted into a coefficient - 0.1
 
-GBool showHidden = gFalse;
-GBool noMerge = gFalse;
-GBool fontFullName = gFalse;
+bool showHidden = false;
+bool noMerge = false;
+bool fontFullName = false;
 static char ownerPassword[33] = "";
 static char userPassword[33] = "";
-static GBool printVersion = gFalse;
+static bool printVersion = false;
 
 static GooString* getInfoString(Dict *infoDict, const char *key);
 static GooString* getInfoDate(Dict *infoDict, const char *key);
@@ -120,6 +127,10 @@ static const ArgDesc argDesc[] = {
    "generate complex document"},
   {"-s",      argFlag,     &singleHtml,          0,
    "generate single document that includes all pages"},
+#ifdef HAVE_IN_MEMORY_FILE
+  {"-dataurls", argFlag,   &dataUrls,      0,
+   "use data URLs instead of external images in HTML"},
+#endif
   {"-i",      argFlag,     &ignore,        0,
    "ignore images"},
   {"-noframes", argFlag,   &noframes,      0,
@@ -130,6 +141,8 @@ static const ArgDesc argDesc[] = {
    "zoom the pdf document (default 1.5)"},
   {"-xml",    argFlag,    &xml,         0,
    "output for XML post-processing"},
+  {"-noroundcoord", argFlag, &noRoundedCoordinates, 0,
+    "do not round coordinates (with XML output only)"},
   {"-hidden", argFlag,   &showHidden,   0,
    "output hidden text"},
   {"-nomerge", argFlag, &noMerge, 0,
@@ -157,22 +170,22 @@ static const ArgDesc argDesc[] = {
 class SplashOutputDevNoText : public SplashOutputDev {
 public:
   SplashOutputDevNoText(SplashColorMode colorModeA, int bitmapRowPadA,
-        GBool reverseVideoA, SplashColorPtr paperColorA,
-        GBool bitmapTopDownA = gTrue) : SplashOutputDev(colorModeA,
+        bool reverseVideoA, SplashColorPtr paperColorA,
+        bool bitmapTopDownA = true) : SplashOutputDev(colorModeA,
             bitmapRowPadA, reverseVideoA, paperColorA, bitmapTopDownA) { }
   virtual ~SplashOutputDevNoText() { }
   
   void drawChar(GfxState *state, double x, double y,
       double dx, double dy,
       double originX, double originY,
-      CharCode code, int nBytes, Unicode *u, int uLen) override { }
-  GBool beginType3Char(GfxState *state, double x, double y,
+      CharCode code, int nBytes, const Unicode *u, int uLen) override { }
+  bool beginType3Char(GfxState *state, double x, double y,
       double dx, double dy,
-      CharCode code, Unicode *u, int uLen) override { return false; }
+      CharCode code, const Unicode *u, int uLen) override { return false; }
   void endType3Char(GfxState *state) override { }
   void beginTextObject(GfxState *state) override { }
   void endTextObject(GfxState *state) override { }
-  GBool interpretType3Chars() override { return gFalse; }
+  bool interpretType3Chars() override { return false; }
 };
 #endif
 
@@ -186,9 +199,8 @@ int main(int argc, char *argv[]) {
 #ifdef HAVE_SPLASH
   SplashOutputDev *splashOut = nullptr;
 #endif
-  GBool doOutline;
-  GBool ok;
-  char *p;
+  bool doOutline;
+  bool ok;
   GooString *ownerPW, *userPW;
   Object info;
   int exit_status = EXIT_FAILURE;
@@ -215,7 +227,7 @@ int main(int argc, char *argv[]) {
 
   if (errQuiet) {
     globalParams->setErrQuiet(errQuiet);
-    printCommands = gFalse; // I'm not 100% what is the differecne between them
+    printCommands = false; // I'm not 100% what is the difference between them
   }
 
   if (textEncName[0]) {
@@ -273,16 +285,16 @@ int main(int argc, char *argv[]) {
     GooString* tmp = new GooString(argv[2]);
     if (!xml) {
       if (tmp->getLength() >= 5) {
-        p = tmp->getCString() + tmp->getLength() - 5;
+        const char *p = tmp->c_str() + tmp->getLength() - 5;
         if (!strcmp(p, ".html") || !strcmp(p, ".HTML")) {
-          htmlFileName = new GooString(tmp->getCString(), tmp->getLength() - 5);
+          htmlFileName = new GooString(tmp->c_str(), tmp->getLength() - 5);
         }
       }
     } else {
       if (tmp->getLength() >= 4) {
-        p = tmp->getCString() + tmp->getLength() - 4;
+        const char *p = tmp->c_str() + tmp->getLength() - 4;
         if (!strcmp(p, ".xml") || !strcmp(p, ".XML")) {
-          htmlFileName = new GooString(tmp->getCString(), tmp->getLength() - 4);
+          htmlFileName = new GooString(tmp->c_str(), tmp->getLength() - 4);
         }
       }
     }
@@ -291,12 +303,12 @@ int main(int argc, char *argv[]) {
     }
     delete tmp;
   } else if (fileName->cmp("fd://0") == 0) {
-      error(errCommandLine, -1, "You have to provide an output filename when reading form stdin.");
+      error(errCommandLine, -1, "You have to provide an output filename when reading from stdin.");
       goto error;
   } else {
-    p = fileName->getCString() + fileName->getLength() - 4;
+    const char *p = fileName->c_str() + fileName->getLength() - 4;
     if (!strcmp(p, ".pdf") || !strcmp(p, ".PDF"))
-      htmlFileName = new GooString(fileName->getCString(),
+      htmlFileName = new GooString(fileName->c_str(),
 				 fileName->getLength() - 4);
     else
       htmlFileName = fileName->copy();
@@ -306,23 +318,22 @@ int main(int argc, char *argv[]) {
    if (scale>3.0) scale=3.0;
    if (scale<0.5) scale=0.5;
    
-   if (complexMode || singleHtml) {
-     //noframes=gFalse;
-     stout=gFalse;
+   if (complexMode) {
+     //noframes=false;
+     stout=false;
    } 
 
    if (stout) {
-     noframes=gTrue;
-     complexMode=gFalse;
-     singleHtml=gFalse;
+     noframes=true;
+     complexMode=false;
    }
 
    if (xml)
    { 
-       complexMode = gTrue;
-       singleHtml = gFalse;
-       noframes = gTrue;
-       noMerge = gTrue;
+       complexMode = true;
+       singleHtml = false;
+       noframes = true;
+       noMerge = true;
    }
 
   // get page range
@@ -354,19 +365,14 @@ int main(int argc, char *argv[]) {
   else
       rawOrder = singleHtml;
 
-#ifdef DISABLE_OUTLINE
-  doOutline = gFalse;
-#else
   doOutline = doc->getOutline()->getItems() != nullptr;
-#endif
   // write text file
-  htmlOut = new HtmlOutputDev(doc->getCatalog(), htmlFileName->getCString(), 
-	  docTitle->getCString(), 
-	  author ? author->getCString() : nullptr,
-	  keywords ? keywords->getCString() : nullptr, 
-          subject ? subject->getCString() : nullptr, 
-	  date ? date->getCString() : nullptr,
-	  extension,
+  htmlOut = new HtmlOutputDev(doc->getCatalog(), htmlFileName->c_str(), 
+	  docTitle->c_str(), 
+	  author ? author->c_str() : nullptr,
+	  keywords ? keywords->c_str() : nullptr, 
+          subject ? subject->c_str() : nullptr, 
+	  date ? date->c_str() : nullptr,
 	  rawOrder, 
 	  firstPage,
 	  doOutline);
@@ -387,13 +393,6 @@ int main(int argc, char *argv[]) {
   {
       delete date;
   }
-
-  if (htmlOut->isOk())
-  {
-    doc->displayPages(htmlOut, firstPage, lastPage, 72 * scale, 72 * scale, 0,
-		      gTrue, gFalse, gFalse);
-    htmlOut->dumpDocOutline(doc);
-  }
   
   if ((complexMode || singleHtml) && !xml && !ignore) {
 #ifdef HAVE_SPLASH
@@ -405,21 +404,34 @@ int main(int argc, char *argv[]) {
     SplashImageFileFormat format = strcmp(extension, "jpg") ?
         splashFormatPng : splashFormatJpeg;
 
-    splashOut = new SplashOutputDevNoText(splashModeRGB8, 4, gFalse, color);
+    splashOut = new SplashOutputDevNoText(splashModeRGB8, 4, false, color);
     splashOut->startDoc(doc);
 
     for (int pg = firstPage; pg <= lastPage; ++pg) {
+      InMemoryFile imf;
       doc->displayPage(splashOut, pg,
                        72 * scale, 72 * scale,
-                       0, gTrue, gFalse, gFalse);
+                       0, true, false, false);
       SplashBitmap *bitmap = splashOut->getBitmap();
 
       imgFileName = GooString::format("{0:s}{1:03d}.{2:s}", 
-          htmlFileName->getCString(), pg, extension);
-
-      bitmap->writeImgFile(format, imgFileName->getCString(),
-                           72 * scale, 72 * scale);
-
+          htmlFileName->c_str(), pg, extension);
+      auto f1 = dataUrls ? imf.open("wb") : fopen(imgFileName->c_str(), "wb");
+      if (!f1) {
+        fprintf(stderr, "Could not open %s\n", imgFileName->c_str());
+        delete imgFileName;
+        continue;
+      }
+      bitmap->writeImgFile(format, f1, 72 * scale, 72 * scale);
+      fclose(f1);
+      if (dataUrls) {
+        htmlOut->addBackgroundImage(
+          std::string((format == splashFormatJpeg) ? "data:image/jpeg;base64," : "data:image/png;base64,") +
+          gbase64Encode(imf.getBuffer())
+        );
+      } else {
+        htmlOut->addBackgroundImage(gbasename(imgFileName->c_str()));
+      }
       delete imgFileName;
     }
 
@@ -434,7 +446,14 @@ int main(int argc, char *argv[]) {
     return -1;
 #endif
   }
-  
+
+  if (htmlOut->isOk())
+  {
+    doc->displayPages(htmlOut, firstPage, lastPage, 72 * scale, 72 * scale, 0,
+		      true, false, false);
+    htmlOut->dumpDocOutline(doc);
+  }
+
   delete htmlOut;
 
   exit_status = EXIT_SUCCESS;
@@ -446,11 +465,6 @@ int main(int argc, char *argv[]) {
   if(globalParams) delete globalParams;
 
   if(htmlFileName) delete htmlFileName;
-  HtmlFont::clear();
-  
-  // check for memory leaks
-  Object::memCheck(stderr);
-  gMemReport(stderr);
 
   return exit_status;
 }
@@ -465,7 +479,7 @@ static GooString* getInfoString(Dict *infoDict, const char *key) {
   // Value HTML escaped and converted to desired encoding
   GooString *encodedString = nullptr;
   // Is rawString UCS2 (as opposed to pdfDocEncoding)
-  GBool isUnicode;
+  bool isUnicode;
 
   obj = infoDict->lookup(key);
   if (obj.isString()) {
@@ -473,10 +487,10 @@ static GooString* getInfoString(Dict *infoDict, const char *key) {
 
     // Convert rawString to unicode
     if (rawString->hasUnicodeMarker()) {
-      isUnicode = gTrue;
+      isUnicode = true;
       unicodeLength = (obj.getString()->getLength() - 2) / 2;
     } else {
-      isUnicode = gFalse;
+      isUnicode = false;
       unicodeLength = obj.getString()->getLength();
     }
     unicodeString = new Unicode[unicodeLength];
@@ -509,7 +523,7 @@ static GooString* getInfoDate(Dict *infoDict, const char *key) {
 
   obj = infoDict->lookup(key);
   if (obj.isString()) {
-    s = obj.getString()->getCString();
+    s = obj.getString()->c_str();
     // TODO do something with the timezone info
     if ( parseDateString( s, &year, &mon, &day, &hour, &min, &sec, &tz, &tz_hour, &tz_minute ) ) {
       tmStruct.tm_year = year - 1900;
