@@ -16,7 +16,7 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2006 Dominic Lachowicz <cinamod@hotmail.com>
-// Copyright (C) 2007-2008, 2010, 2011, 2017 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2008, 2010, 2011, 2017, 2018 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2009 Jan Jockusch <jan@jockusch.de>
 // Copyright (C) 2010, 2013 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2010 Kenneth Berland <ken@hero.com>
@@ -25,6 +25,11 @@
 // Copyright (C) 2013 Yury G. Kudryashov <urkud.urkud@gmail.com>
 // Copyright (C) 2013 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
 // Copyright (C) 2015 Jeremy Echols <jechols@uoregon.edu>
+// Copyright (C) 2017 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
+// Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
+// Copyright (C) 2018 Sanchit Anand <sanxchit@gmail.com>
+// Copyright (C) 2019 Dan Shea <dan.shea@logical-innovations.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -59,6 +64,7 @@
 #include <string>
 #include <sstream>
 #include <iomanip>
+#include "Win32Console.h"
 
 static void printInfoString(FILE *f, Dict *infoDict, const char *key,
 			    const char *text1, const char *text2, UnicodeMap *uMap);
@@ -73,21 +79,22 @@ static int x = 0;
 static int y = 0;
 static int w = 0;
 static int h = 0;
-static GBool bbox = gFalse;
-static GBool bboxLayout = gFalse;
-static GBool physLayout = gFalse;
+static bool bbox = false;
+static bool bboxLayout = false;
+static bool physLayout = false;
 static double fixedPitch = 0;
-static GBool rawOrder = gFalse;
-static GBool htmlMeta = gFalse;
+static bool rawOrder = false;
+static bool discardDiag = false;
+static bool htmlMeta = false;
 static char textEncName[128] = "";
 static char textEOL[16] = "";
-static GBool noPageBreaks = gFalse;
+static bool noPageBreaks = false;
 static char ownerPassword[33] = "\001";
 static char userPassword[33] = "\001";
-static GBool quiet = gFalse;
-static GBool printVersion = gFalse;
-static GBool printHelp = gFalse;
-static GBool printEnc = gFalse;
+static bool quiet = false;
+static bool printVersion = false;
+static bool printHelp = false;
+static bool printEnc = false;
 
 static const ArgDesc argDesc[] = {
   {"-f",       argInt,      &firstPage,     0,
@@ -110,6 +117,8 @@ static const ArgDesc argDesc[] = {
    "assume fixed-pitch (or tabular) text"},
   {"-raw",     argFlag,     &rawOrder,      0,
    "keep strings in content stream order"},
+  {"-nodiag",  argFlag,     &discardDiag,   0,
+   "discard diagonal text"},
   {"-htmlmeta", argFlag,   &htmlMeta,       0,
    "generate a simple HTML file, including the meta information"},
   {"-enc",     argString,   textEncName,    sizeof(textEncName),
@@ -140,7 +149,7 @@ static const ArgDesc argDesc[] = {
    "print usage information"},
   {"-?",       argFlag,     &printHelp,     0,
    "print usage information"},
-  {NULL}
+  {}
 };
 
 static std::string myStringReplace(const std::string &inString, const std::string &oldToken, const std::string &newToken) {
@@ -176,19 +185,19 @@ int main(int argc, char *argv[]) {
   FILE *f;
   UnicodeMap *uMap;
   Object info;
-  GBool ok;
-  char *p;
+  bool ok;
   int exitCode;
 
+  Win32Console win32Console(&argc, &argv);
   exitCode = 99;
 
   // parse args
   ok = parseArgs(argDesc, &argc, argv);
   if (bboxLayout) {
-    bbox = gTrue;
+    bbox = true;
   }
   if (bbox) {
-    htmlMeta = gTrue;
+    htmlMeta = true;
   }
   if (!ok || (argc < 2 && !printEnc) || argc > 3 || printVersion || printHelp) {
     fprintf(stderr, "pdftotext version %s\n", PACKAGE_VERSION);
@@ -214,7 +223,7 @@ int main(int argc, char *argv[]) {
 
   fileName = new GooString(argv[1]);
   if (fixedPitch) {
-    physLayout = gTrue;
+    physLayout = true;
   }
 
   if (textEncName[0]) {
@@ -226,7 +235,7 @@ int main(int argc, char *argv[]) {
     }
   }
   if (noPageBreaks) {
-    globalParams->setTextPageBreaks(gFalse);
+    globalParams->setTextPageBreaks(false);
   }
   if (quiet) {
     globalParams->setErrQuiet(quiet);
@@ -243,12 +252,12 @@ int main(int argc, char *argv[]) {
   if (ownerPassword[0] != '\001') {
     ownerPW = new GooString(ownerPassword);
   } else {
-    ownerPW = NULL;
+    ownerPW = nullptr;
   }
   if (userPassword[0] != '\001') {
     userPW = new GooString(userPassword);
   } else {
-    userPW = NULL;
+    userPW = nullptr;
   }
 
   if (fileName->cmp("-") == 0) {
@@ -282,12 +291,12 @@ int main(int argc, char *argv[]) {
   if (argc == 3) {
     textFileName = new GooString(argv[2]);
   } else if (fileName->cmp("fd://0") == 0) {
-     error(errCommandLine, -1, "You have to provide an output filename when reading form stdin.");
+     error(errCommandLine, -1, "You have to provide an output filename when reading from stdin.");
      goto err2;
   } else {
-    p = fileName->getCString() + fileName->getLength() - 4;
+    const char *p = fileName->c_str() + fileName->getLength() - 4;
     if (!strcmp(p, ".pdf") || !strcmp(p, ".PDF")) {
-      textFileName = new GooString(fileName->getCString(),
+      textFileName = new GooString(fileName->c_str(),
 				 fileName->getLength() - 4);
     } else {
       textFileName = fileName->copy();
@@ -314,7 +323,7 @@ int main(int argc, char *argv[]) {
     if (!textFileName->cmp("-")) {
       f = stdout;
     } else {
-      if (!(f = fopen(textFileName->getCString(), "wb"))) {
+      if (!(f = fopen(textFileName->c_str(), "wb"))) {
 	error(errIO, -1, "Couldn't open text file '{0:t}'", textFileName);
 	exitCode = 2;
 	goto err3;
@@ -358,7 +367,7 @@ int main(int argc, char *argv[]) {
 
   // write text file
   if (htmlMeta && bbox) { // htmlMeta && is superfluous but makes gcc happier
-    textOut = new TextOutputDev(NULL, physLayout, fixedPitch, rawOrder, htmlMeta);
+    textOut = new TextOutputDev(nullptr, physLayout, fixedPitch, rawOrder, htmlMeta, discardDiag);
 
     if (textOut->isOk()) {
       if (bboxLayout) {
@@ -372,17 +381,17 @@ int main(int argc, char *argv[]) {
       fclose(f);
     }
   } else {
-    textOut = new TextOutputDev(textFileName->getCString(),
-				physLayout, fixedPitch, rawOrder, htmlMeta);
+    textOut = new TextOutputDev(textFileName->c_str(),
+				physLayout, fixedPitch, rawOrder, htmlMeta, discardDiag);
     if (textOut->isOk()) {
       if ((w==0) && (h==0) && (x==0) && (y==0)) {
 	doc->displayPages(textOut, firstPage, lastPage, resolution, resolution, 0,
-			  gTrue, gFalse, gFalse);
+			  true, false, false);
       } else {
 	
 	for (int page = firstPage; page <= lastPage; ++page) {
 	  doc->displayPageSlice(textOut, page, resolution, resolution, 0,
-			      gTrue, gFalse, gFalse, 
+			      true, false, false, 
 			      x, y, w, h);
 	}
       }
@@ -400,7 +409,7 @@ int main(int argc, char *argv[]) {
     if (!textFileName->cmp("-")) {
       f = stdout;
     } else {
-      if (!(f = fopen(textFileName->getCString(), "ab"))) {
+      if (!(f = fopen(textFileName->c_str(), "ab"))) {
 	error(errIO, -1, "Couldn't open text file '{0:t}'", textFileName);
 	exitCode = 2;
 	goto err3;
@@ -427,17 +436,13 @@ int main(int argc, char *argv[]) {
   delete globalParams;
  err0:
 
-  // check for memory leaks
-  Object::memCheck(stderr);
-  gMemReport(stderr);
-
   return exitCode;
 }
 
 static void printInfoString(FILE *f, Dict *infoDict, const char *key,
 			    const char *text1, const char *text2, UnicodeMap *uMap) {
-  GooString *s1;
-  GBool isUnicode;
+  const GooString *s1;
+  bool isUnicode;
   Unicode u;
   char buf[9];
   int i, n;
@@ -448,10 +453,10 @@ static void printInfoString(FILE *f, Dict *infoDict, const char *key,
     s1 = obj.getString();
     if ((s1->getChar(0) & 0xff) == 0xfe &&
 	(s1->getChar(1) & 0xff) == 0xff) {
-      isUnicode = gTrue;
+      isUnicode = true;
       i = 2;
     } else {
-      isUnicode = gFalse;
+      isUnicode = false;
       i = 0;
     }
     while (i < obj.getString()->getLength()) {
@@ -475,7 +480,7 @@ static void printInfoString(FILE *f, Dict *infoDict, const char *key,
 static void printInfoDate(FILE *f, Dict *infoDict, const char *key, const char *fmt) {
   Object obj = infoDict->lookup(key);
   if (obj.isString()) {
-    char *s = obj.getString()->getCString();
+    const char *s = obj.getString()->c_str();
     if (s[0] == 'D' && s[1] == ':') {
       s += 2;
     }
@@ -483,7 +488,7 @@ static void printInfoDate(FILE *f, Dict *infoDict, const char *key, const char *
   }
 }
 
-void printLine(FILE *f, TextLine *line) {
+static void printLine(FILE *f, TextLine *line) {
   double xMin, yMin, xMax, yMax;
   double lineXMin = 0, lineYMin = 0, lineXMax = 0, lineYMax = 0;
   TextWord *word;
@@ -498,9 +503,11 @@ void printLine(FILE *f, TextLine *line) {
     if (lineXMax < xMax) lineXMax = xMax;
     if (lineYMax < yMax) lineYMax = yMax;
 
-    const std::string myString = myXmlTokenReplace(word->getText()->getCString());
+    GooString *wordText = word->getText();
+    const std::string myString = myXmlTokenReplace(wordText->c_str());
     wordXML << "          <word xMin=\"" << xMin << "\" yMin=\"" << yMin << "\" xMax=\"" <<
             xMax << "\" yMax=\"" << yMax << "\">" << myString << "</word>\n";
+    delete wordText;
   }
   fprintf(f, "        <line xMin=\"%f\" yMin=\"%f\" xMax=\"%f\" yMax=\"%f\">\n",
           lineXMin, lineYMin, lineXMax, lineYMax);
@@ -510,7 +517,6 @@ void printLine(FILE *f, TextLine *line) {
 
 void printDocBBox(FILE *f, PDFDoc *doc, TextOutputDev *textOut, int first, int last) {
   double xMin, yMin, xMax, yMax;
-  TextPage *textPage;
   TextFlow *flow;
   TextBlock *blk;
   TextLine *line;
@@ -518,9 +524,8 @@ void printDocBBox(FILE *f, PDFDoc *doc, TextOutputDev *textOut, int first, int l
   fprintf(f, "<doc>\n");
   for (int page = first; page <= last; ++page) {
     fprintf(f, "  <page width=\"%f\" height=\"%f\">\n",doc->getPageMediaWidth(page), doc->getPageMediaHeight(page));
-    doc->displayPage(textOut, page, resolution, resolution, 0, gTrue, gFalse, gFalse);
-    textPage = textOut->takeText();
-    for (flow = textPage->getFlows(); flow; flow = flow->getNext()) {
+    doc->displayPage(textOut, page, resolution, resolution, 0, true, false, false);
+    for (flow = textOut->getFlows(); flow; flow = flow->getNext()) {
       fprintf(f, "    <flow>\n");
       for (blk = flow->getBlocks(); blk; blk = blk->getNext()) {
         blk->getBBox(&xMin, &yMin, &xMax, &yMax);
@@ -533,7 +538,6 @@ void printDocBBox(FILE *f, PDFDoc *doc, TextOutputDev *textOut, int first, int l
       fprintf(f, "    </flow>\n");
     }
     fprintf(f, "  </page>\n");
-    textPage->decRefCnt();
   }
   fprintf(f, "</doc>\n");
 }
@@ -542,9 +546,9 @@ void printWordBBox(FILE *f, PDFDoc *doc, TextOutputDev *textOut, int first, int
   fprintf(f, "<doc>\n");
   for (int page = first; page <= last; ++page) {
     fprintf(f, "  <page width=\"%f\" height=\"%f\">\n",doc->getPageMediaWidth(page), doc->getPageMediaHeight(page));
-    doc->displayPage(textOut, page, resolution, resolution, 0, gTrue, gFalse, gFalse);
+    doc->displayPage(textOut, page, resolution, resolution, 0, true, false, false);
     TextWordList *wordlist = textOut->makeWordList();
-    const int word_length = wordlist != NULL ? wordlist->getLength() : 0;
+    const int word_length = wordlist != nullptr ? wordlist->getLength() : 0;
     TextWord *word;
     double xMinA, yMinA, xMaxA, yMaxA;
     if (word_length == 0)
@@ -553,7 +557,7 @@ void printWordBBox(FILE *f, PDFDoc *doc, TextOutputDev *textOut, int first, int
     for (int i = 0; i < word_length; ++i) {
       word = wordlist->get(i);
       word->getBBox(&xMinA, &yMinA, &xMaxA, &yMaxA);
-      const std::string myString = myXmlTokenReplace(word->getText()->getCString());
+      const std::string myString = myXmlTokenReplace(word->getText()->c_str());
       fprintf(f,"    <word xMin=\"%f\" yMin=\"%f\" xMax=\"%f\" yMax=\"%f\">%s</word>\n", xMinA, yMinA, xMaxA, yMaxA, myString.c_str());
     }
     fprintf(f, "  </page>\n");
