@@ -17,12 +17,9 @@
 #include "xfa/fwl/theme/cfwl_utils.h"
 
 CFWL_Barcode::CFWL_Barcode(const CFWL_App* app)
-    : CFWL_Edit(app, pdfium::MakeUnique<CFWL_WidgetProperties>(), nullptr),
-      m_dwStatus(0),
-      m_type(BC_UNKNOWN),
-      m_dwAttributeMask(FWL_BCDATTRIBUTE_NONE) {}
+    : CFWL_Edit(app, pdfium::MakeUnique<CFWL_WidgetProperties>(), nullptr) {}
 
-CFWL_Barcode::~CFWL_Barcode() {}
+CFWL_Barcode::~CFWL_Barcode() = default;
 
 FWL_Type CFWL_Barcode::GetClassID() const {
   return FWL_Type::Barcode;
@@ -44,7 +41,7 @@ void CFWL_Barcode::DrawWidget(CXFA_Graphics* pGraphics,
     return;
   if ((m_pProperties->m_dwStates & FWL_WGTSTATE_Focused) == 0) {
     GenerateBarcodeImageCache();
-    if (!m_pBarcodeEngine || (m_dwStatus & XFA_BCS_EncodeSuccess) == 0)
+    if (!m_pBarcodeEngine || m_eStatus != Status::kEncodeSuccess)
       return;
 
     CFX_Matrix mt;
@@ -64,14 +61,19 @@ void CFWL_Barcode::SetType(BC_TYPE type) {
 
   m_pBarcodeEngine.reset();
   m_type = type;
-  m_dwStatus = XFA_BCS_NeedUpdate;
+  m_eStatus = Status::kNeedUpdate;
 }
 
-void CFWL_Barcode::SetText(const WideString& wsText,
-                           CFDE_TextEditEngine::RecordOperation op) {
+void CFWL_Barcode::SetText(const WideString& wsText) {
   m_pBarcodeEngine.reset();
-  m_dwStatus = XFA_BCS_NeedUpdate;
-  CFWL_Edit::SetText(wsText, op);
+  m_eStatus = Status::kNeedUpdate;
+  CFWL_Edit::SetText(wsText);
+}
+
+void CFWL_Barcode::SetTextSkipNotify(const WideString& wsText) {
+  m_pBarcodeEngine.reset();
+  m_eStatus = Status::kNeedUpdate;
+  CFWL_Edit::SetTextSkipNotify(wsText);
 }
 
 bool CFWL_Barcode::IsProtectedType() const {
@@ -79,17 +81,14 @@ bool CFWL_Barcode::IsProtectedType() const {
     return true;
 
   BC_TYPE tEngineType = m_pBarcodeEngine->GetType();
-  if (tEngineType == BC_QR_CODE || tEngineType == BC_PDF417 ||
-      tEngineType == BC_DATAMATRIX) {
-    return true;
-  }
-  return false;
+  return tEngineType == BC_QR_CODE || tEngineType == BC_PDF417 ||
+         tEngineType == BC_DATAMATRIX;
 }
 
 void CFWL_Barcode::OnProcessEvent(CFWL_Event* pEvent) {
   if (pEvent->GetType() == CFWL_Event::Type::TextWillChange) {
     m_pBarcodeEngine.reset();
-    m_dwStatus = XFA_BCS_NeedUpdate;
+    m_eStatus = Status::kNeedUpdate;
   }
   CFWL_Edit::OnProcessEvent(pEvent);
 }
@@ -151,10 +150,10 @@ void CFWL_Barcode::SetErrorCorrectionLevel(int32_t ecLevel) {
 }
 
 void CFWL_Barcode::GenerateBarcodeImageCache() {
-  if ((m_dwStatus & XFA_BCS_NeedUpdate) == 0)
+  if (m_eStatus != Status::kNeedUpdate)
     return;
 
-  m_dwStatus = 0;
+  m_eStatus = Status::kNormal;
   CreateBarcodeEngine();
   if (!m_pBarcodeEngine)
     return;
@@ -163,12 +162,12 @@ void CFWL_Barcode::GenerateBarcodeImageCache() {
   if (pTheme) {
     CFWL_ThemePart part;
     part.m_pWidget = this;
-    if (RetainPtr<CFGAS_GEFont> pFont = pTheme->GetFont(&part)) {
+    if (RetainPtr<CFGAS_GEFont> pFont = pTheme->GetFont(part)) {
       if (CFX_Font* pCXFont = pFont->GetDevFont())
         m_pBarcodeEngine->SetFont(pCXFont);
     }
-    m_pBarcodeEngine->SetFontSize(pTheme->GetFontSize(&part));
-    m_pBarcodeEngine->SetFontColor(pTheme->GetTextColor(&part));
+    m_pBarcodeEngine->SetFontSize(pTheme->GetFontSize(part));
+    m_pBarcodeEngine->SetFontColor(pTheme->GetTextColor(part));
   } else {
     m_pBarcodeEngine->SetFontSize(FWLTHEME_CAPACITY_FontSize);
   }
@@ -198,9 +197,9 @@ void CFWL_Barcode::GenerateBarcodeImageCache() {
   if (m_dwAttributeMask & FWL_BCDATTRIBUTE_ECLEVEL)
     m_pBarcodeEngine->SetErrorCorrectionLevel(m_nECLevel);
 
-  m_dwStatus = m_pBarcodeEngine->Encode(GetText().AsStringView())
-                   ? XFA_BCS_EncodeSuccess
-                   : 0;
+  m_eStatus = m_pBarcodeEngine->Encode(GetText().AsStringView())
+                  ? Status::kEncodeSuccess
+                  : Status::kNormal;
 }
 
 void CFWL_Barcode::CreateBarcodeEngine() {
