@@ -15,15 +15,15 @@ enum class FWL_KeyCommand { KeyDown, KeyUp, Char };
 
 class CFWL_MessageKey final : public CFWL_Message {
  public:
-  CFWL_MessageKey(CFWL_Widget* pSrcTarget, CFWL_Widget* pDstTarget);
+  CFWL_MessageKey(CFWL_Widget* pDstTarget,
+                  FWL_KeyCommand cmd,
+                  uint32_t flags,
+                  uint32_t keycode);
   ~CFWL_MessageKey() override;
 
-  // CFWL_Message
-  std::unique_ptr<CFWL_Message> Clone() override;
-
-  uint32_t m_dwKeyCode;
-  uint32_t m_dwFlags;
   FWL_KeyCommand m_dwCmd;
+  uint32_t m_dwFlags;
+  uint32_t m_dwKeyCode;
 };
 
 #endif  // XFA_FWL_CFWL_MESSAGEKEY_H_
