@@ -27,16 +27,16 @@ enum class FWL_MouseCommand {
 
 class CFWL_MessageMouse final : public CFWL_Message {
  public:
-  CFWL_MessageMouse(CFWL_Widget* pSrcTarget, CFWL_Widget* pDstTarget);
-  CFWL_MessageMouse(const CFWL_MessageMouse& other);
+  CFWL_MessageMouse(CFWL_Widget* pDstTarget, FWL_MouseCommand cmd);
+  CFWL_MessageMouse(CFWL_Widget* pDstTarget,
+                    FWL_MouseCommand cmd,
+                    uint32_t flags,
+                    CFX_PointF pos);
   ~CFWL_MessageMouse() override;
 
-  // CFWL_Message
-  std::unique_ptr<CFWL_Message> Clone() override;
-
-  CFX_PointF m_pos;
-  uint32_t m_dwFlags;
   FWL_MouseCommand m_dwCmd;
+  uint32_t m_dwFlags = 0;
+  CFX_PointF m_pos;
 };
 
 #endif  // XFA_FWL_CFWL_MESSAGEMOUSE_H_
