@@ -7,6 +7,7 @@
 #include "xfa/fwl/theme/cfwl_checkboxtp.h"
 
 #include "core/fxge/cfx_pathdata.h"
+#include "core/fxge/render_defines.h"
 #include "third_party/base/ptr_util.h"
 #include "xfa/fde/cfde_textout.h"
 #include "xfa/fwl/cfwl_checkbox.h"
@@ -18,7 +19,7 @@
 
 namespace {
 
-const int kSignPath = 100;
+constexpr int kSignPath = 100;
 
 CFX_PointF ScaleBezierPoint(const CFX_PointF& point) {
   CFX_PointF scaled_point(point);
@@ -29,9 +30,7 @@ CFX_PointF ScaleBezierPoint(const CFX_PointF& point) {
 
 }  // namespace
 
-CFWL_CheckBoxTP::CFWL_CheckBoxTP() : m_pThemeData(new CKBThemeData) {
-  SetThemeData();
-}
+CFWL_CheckBoxTP::CFWL_CheckBoxTP() = default;
 
 CFWL_CheckBoxTP::~CFWL_CheckBoxTP() {
   if (m_pCheckPath)
@@ -171,44 +170,6 @@ void CFWL_CheckBoxTP::DrawSignStar(CXFA_Graphics* pGraphics,
   pGraphics->RestoreGraphState();
 }
 
-void CFWL_CheckBoxTP::SetThemeData() {
-  uint32_t* pData = (uint32_t*)&m_pThemeData->clrBoxBk;
-
-  *pData++ = 0;
-  *pData++ = 0;
-  *pData++ = ArgbEncode(255, 220, 220, 215),
-  *pData++ = ArgbEncode(255, 255, 255, 255),
-  *pData++ = ArgbEncode(255, 255, 240, 207),
-  *pData++ = ArgbEncode(255, 248, 179, 48),
-  *pData++ = ArgbEncode(255, 176, 176, 167),
-  *pData++ = ArgbEncode(255, 241, 239, 239),
-  *pData++ = ArgbEncode(255, 255, 255, 255),
-  *pData++ = ArgbEncode(255, 255, 255, 255),
-  *pData++ = ArgbEncode(255, 220, 220, 215),
-  *pData++ = ArgbEncode(255, 255, 255, 255),
-  *pData++ = ArgbEncode(255, 255, 240, 207),
-  *pData++ = ArgbEncode(255, 248, 179, 48),
-  *pData++ = ArgbEncode(255, 176, 176, 167),
-  *pData++ = ArgbEncode(255, 241, 239, 239),
-  *pData++ = ArgbEncode(255, 255, 255, 255),
-  *pData++ = ArgbEncode(255, 255, 255, 255),
-  *pData++ = ArgbEncode(255, 220, 220, 215),
-  *pData++ = ArgbEncode(255, 255, 255, 255),
-  *pData++ = ArgbEncode(255, 255, 240, 207),
-  *pData++ = ArgbEncode(255, 248, 179, 48),
-  *pData++ = ArgbEncode(255, 176, 176, 167),
-  *pData++ = ArgbEncode(255, 241, 239, 239),
-  *pData++ = ArgbEncode(255, 255, 255, 255),
-  *pData++ = ArgbEncode(255, 255, 255, 255);
-  m_pThemeData->clrSignBorderNormal = ArgbEncode(255, 28, 81, 128);
-  m_pThemeData->clrSignBorderDisable = ArgbEncode(255, 202, 200, 187);
-  m_pThemeData->clrSignCheck = ArgbEncode(255, 28, 81, 128);
-  m_pThemeData->clrSignNeutral = ArgbEncode(255, 28, 134, 26);
-  m_pThemeData->clrSignNeutralNormal = ArgbEncode(255, 114, 192, 113);
-  m_pThemeData->clrSignNeutralHover = ArgbEncode(255, 33, 161, 33);
-  m_pThemeData->clrSignNeutralPressed = ArgbEncode(255, 28, 134, 26);
-}
-
 void CFWL_CheckBoxTP::InitCheckPath(float fCheckLen) {
   if (!m_pCheckPath) {
     m_pCheckPath = pdfium::MakeUnique<CXFA_GEPath>();
