@@ -56,7 +56,7 @@ bool CXFA_FFListBox::LoadWidget() {
   m_pNormalWidget->LockUpdate();
 
   for (const auto& label : m_pNode->GetChoiceListItems(false))
-    pListBox->AddString(label.AsStringView());
+    pListBox->AddString(label);
 
   uint32_t dwExtendedStyle = FWL_STYLEEXT_LTB_ShowScrollBarFocus;
   if (m_pNode->IsChoiceListMultiSelect())
@@ -75,8 +75,7 @@ bool CXFA_FFListBox::OnKillFocus(CXFA_FFWidget* pNewFocus) {
   if (!ProcessCommittedData())
     UpdateFWLData();
 
-  CXFA_FFField::OnKillFocus(pNewFocus);
-  return true;
+  return CXFA_FFField::OnKillFocus(pNewFocus);
 }
 
 bool CXFA_FFListBox::CommitData() {
@@ -165,9 +164,8 @@ void CXFA_FFListBox::SetItemState(int32_t nIndex, bool bSelected) {
   InvalidateRect();
 }
 
-void CXFA_FFListBox::InsertItem(WideStringView wsLabel, int32_t nIndex) {
-  WideString wsTemp(wsLabel);
-  ToListBox(m_pNormalWidget.get())->AddString(wsTemp.AsStringView());
+void CXFA_FFListBox::InsertItem(const WideString& wsLabel, int32_t nIndex) {
+  ToListBox(m_pNormalWidget.get())->AddString(wsLabel);
   m_pNormalWidget->Update();
   InvalidateRect();
 }
