@@ -6,28 +6,28 @@
 
 #include "xfa/fxfa/cxfa_ffrectangle.h"
 
-CXFA_FFRectangle::CXFA_FFRectangle(CXFA_WidgetAcc* pDataAcc)
-    : CXFA_FFDraw(pDataAcc) {}
+#include "xfa/fxfa/parser/cxfa_rectangle.h"
+#include "xfa/fxfa/parser/cxfa_value.h"
+
+CXFA_FFRectangle::CXFA_FFRectangle(CXFA_Node* pNode) : CXFA_FFWidget(pNode) {}
 
 CXFA_FFRectangle::~CXFA_FFRectangle() {}
 
 void CXFA_FFRectangle::RenderWidget(CXFA_Graphics* pGS,
                                     const CFX_Matrix& matrix,
-                                    uint32_t dwStatus) {
-  if (!IsMatchVisibleStatus(dwStatus))
+                                    HighlightOption highlight) {
+  if (!HasVisibleStatus())
     return;
 
-  CXFA_Value value = m_pDataAcc->GetFormValue();
+  CXFA_Value* value = m_pNode->GetFormValueIfExists();
   if (!value)
     return;
 
-  CXFA_Rectangle rtObj = value.GetRectangle();
   CFX_RectF rect = GetRectWithoutRotate();
-  if (CXFA_Margin mgWidget = m_pDataAcc->GetMargin())
-    XFA_RectWidthoutMargin(rect, mgWidget);
+  CXFA_Margin* margin = m_pNode->GetMarginIfExists();
+  XFA_RectWithoutMargin(&rect, margin);
 
   CFX_Matrix mtRotate = GetRotateMatrix();
   mtRotate.Concat(matrix);
-
-  DrawBorder(pGS, rtObj, rect, mtRotate);
+  DrawBorder(pGS, value->GetRectangleIfExists(), rect, mtRotate);
 }
