@@ -226,12 +226,12 @@ bool CXFA_FFWidgetHandler::HasEvent(CXFA_Node* pNode,
               .empty();
 }
 
-int32_t CXFA_FFWidgetHandler::ProcessEvent(CXFA_Node* pNode,
-                                           CXFA_EventParam* pParam) {
+XFA_EventError CXFA_FFWidgetHandler::ProcessEvent(CXFA_Node* pNode,
+                                                  CXFA_EventParam* pParam) {
   if (!pParam || pParam->m_eType == XFA_EVENT_Unknown)
-    return XFA_EVENTERROR_NotExist;
+    return XFA_EventError::kNotExist;
   if (!pNode || pNode->GetElementType() == XFA_Element::Draw)
-    return XFA_EVENTERROR_NotExist;
+    return XFA_EventError::kNotExist;
 
   switch (pParam->m_eType) {
     case XFA_EVENT_Calculate:
@@ -241,13 +241,13 @@ int32_t CXFA_FFWidgetHandler::ProcessEvent(CXFA_Node* pNode,
               m_pDocView->GetDoc())) {
         return pNode->ProcessValidate(m_pDocView.Get(), 0);
       }
-      return XFA_EVENTERROR_Disabled;
+      return XFA_EventError::kDisabled;
     case XFA_EVENT_InitCalculate: {
       CXFA_Calculate* calc = pNode->GetCalculateIfExists();
       if (!calc)
-        return XFA_EVENTERROR_NotExist;
+        return XFA_EventError::kNotExist;
       if (pNode->IsUserInteractive())
-        return XFA_EVENTERROR_Disabled;
+        return XFA_EventError::kDisabled;
       return pNode->ExecuteScript(m_pDocView.Get(), calc->GetScriptIfExists(),
                                   pParam);
     }
@@ -519,7 +519,7 @@ CXFA_Node* CXFA_FFWidgetHandler::CreateFormItem(XFA_Element eElement,
 
   CXFA_Node* pNewFormItem = pTemplateParent->CloneTemplateToForm(false);
   if (pParent)
-    pParent->InsertChild(pNewFormItem, pBefore);
+    pParent->InsertChildAndNotify(pNewFormItem, pBefore);
   return pNewFormItem;
 }
 
@@ -535,7 +535,7 @@ CXFA_Node* CXFA_FFWidgetHandler::CreateCopyNode(XFA_Element eElement,
                          pBefore ? pBefore->GetTemplateNodeIfExists() : nullptr)
           ->Clone(false);
   if (pParent)
-    pParent->InsertChild(pNewNode, pBefore);
+    pParent->InsertChildAndNotify(pNewNode, pBefore);
   return pNewNode;
 }
 
@@ -546,7 +546,7 @@ CXFA_Node* CXFA_FFWidgetHandler::CreateTemplateNode(XFA_Element eElement,
   CXFA_Node* pNewTemplateNode =
       pXFADoc->CreateNode(XFA_PacketType::Template, eElement);
   if (pParent)
-    pParent->InsertChild(pNewTemplateNode, pBefore);
+    pParent->InsertChildAndNotify(pNewTemplateNode, pBefore);
   return pNewTemplateNode;
 }
 
