@@ -17,22 +17,22 @@ void CXFA_FWLAdapterWidgetMgr::RepaintWidget(CFWL_Widget* pWidget) {
   if (!pWidget)
     return;
 
-  CXFA_FFWidget* pFFWidget = pWidget->GetLayoutItem();
+  auto* pFFWidget = static_cast<CXFA_FFWidget*>(pWidget->GetFFWidget());
   if (!pFFWidget)
     return;
 
-  pFFWidget->AddInvalidateRect();
+  pFFWidget->InvalidateRect();
 }
 
 bool CXFA_FWLAdapterWidgetMgr::GetPopupPos(CFWL_Widget* pWidget,
                                            float fMinHeight,
                                            float fMaxHeight,
                                            const CFX_RectF& rtAnchor,
-                                           CFX_RectF& rtPopup) {
-  CXFA_FFWidget* pFFWidget = pWidget->GetLayoutItem();
+                                           CFX_RectF* pPopupRect) {
+  auto* pFFWidget = static_cast<CXFA_FFWidget*>(pWidget->GetFFWidget());
   CFX_RectF rtRotateAnchor =
       pFFWidget->GetRotateMatrix().TransformRect(rtAnchor);
   pFFWidget->GetDoc()->GetDocEnvironment()->GetPopupPos(
-      pFFWidget, fMinHeight, fMaxHeight, rtRotateAnchor, rtPopup);
+      pFFWidget, fMinHeight, fMaxHeight, rtRotateAnchor, pPopupRect);
   return true;
 }
