@@ -8,30 +8,26 @@
 
 #include "xfa/fxfa/cxfa_ffpageview.h"
 #include "xfa/fxfa/cxfa_ffwidget.h"
-#include "xfa/fxgraphics/cxfa_graphics.h"
 
 CXFA_RenderContext::CXFA_RenderContext(CXFA_FFPageView* pPageView,
                                        const CFX_RectF& clipRect,
                                        const CFX_Matrix& matrix)
-    : m_pWidget(nullptr), m_matrix(matrix), m_rtClipRect(clipRect) {
-  matrix.GetInverse().TransformRect(m_rtClipRect);
+    : m_pWidgetIterator(pPageView->CreateWidgetIterator(
+          XFA_TRAVERSEWAY_Form,
+          XFA_WidgetStatus_Visible | XFA_WidgetStatus_Viewable)),
+      m_pWidget(m_pWidgetIterator->MoveToNext()),
+      m_matrix(matrix),
+      m_rtClipRect(clipRect) {}
 
-  m_pWidgetIterator = pPageView->CreateWidgetIterator(
-      XFA_TRAVERSEWAY_Form,
-      XFA_WidgetStatus_Visible | XFA_WidgetStatus_Viewable);
-  m_pWidget = m_pWidgetIterator->MoveToNext();
-}
-
-CXFA_RenderContext::~CXFA_RenderContext() {}
+CXFA_RenderContext::~CXFA_RenderContext() = default;
 
 void CXFA_RenderContext::DoRender(CXFA_Graphics* gs) {
   while (m_pWidget) {
-    CFX_RectF rtWidgetBox = m_pWidget->GetBBox(XFA_WidgetStatus_Visible,
-                                               CXFA_FFWidget::kDoNotDrawFocus);
-    rtWidgetBox.width += 1;
-    rtWidgetBox.height += 1;
+    CFX_RectF rtWidgetBox = m_pWidget->GetBBox(CXFA_FFWidget::kDoNotDrawFocus);
+    ++rtWidgetBox.width;
+    ++rtWidgetBox.height;
     if (rtWidgetBox.IntersectWith(m_rtClipRect))
-      m_pWidget->RenderWidget(gs, m_matrix, XFA_WidgetStatus_Highlight);
+      m_pWidget->RenderWidget(gs, m_matrix, CXFA_FFWidget::kHighlight);
 
     m_pWidget = m_pWidgetIterator->MoveToNext();
   }
