@@ -6,49 +6,47 @@
 
 #include "xfa/fxfa/parser/cxfa_arraynodelist.h"
 
+#include <utility>
 #include <vector>
 
-#include "third_party/base/stl_util.h"
-
 CXFA_ArrayNodeList::CXFA_ArrayNodeList(CXFA_Document* pDocument)
-    : CXFA_NodeList(pDocument) {}
+    : CXFA_TreeList(pDocument) {}
 
 CXFA_ArrayNodeList::~CXFA_ArrayNodeList() {}
 
-void CXFA_ArrayNodeList::SetArrayNodeList(
-    const std::vector<CXFA_Node*>& srcArray) {
+void CXFA_ArrayNodeList::SetArrayNodeList(std::vector<CXFA_Node*> srcArray) {
   if (!srcArray.empty())
-    m_array = srcArray;
+    m_array = std::move(srcArray);
 }
 
-int32_t CXFA_ArrayNodeList::GetLength() {
-  return pdfium::CollectionSize<int32_t>(m_array);
+size_t CXFA_ArrayNodeList::GetLength() {
+  return m_array.size();
 }
 
-bool CXFA_ArrayNodeList::Append(CXFA_Node* pNode) {
+void CXFA_ArrayNodeList::Append(CXFA_Node* pNode) {
   m_array.push_back(pNode);
-  return true;
 }
 
 bool CXFA_ArrayNodeList::Insert(CXFA_Node* pNewNode, CXFA_Node* pBeforeNode) {
   if (!pBeforeNode) {
     m_array.push_back(pNewNode);
-  } else {
-    auto it = std::find(m_array.begin(), m_array.end(), pBeforeNode);
-    if (it != m_array.end())
-      m_array.insert(it, pNewNode);
+    return true;
   }
+
+  auto it = std::find(m_array.begin(), m_array.end(), pBeforeNode);
+  if (it == m_array.end())
+    return false;
+
+  m_array.insert(it, pNewNode);
   return true;
 }
 
-bool CXFA_ArrayNodeList::Remove(CXFA_Node* pNode) {
+void CXFA_ArrayNodeList::Remove(CXFA_Node* pNode) {
   auto it = std::find(m_array.begin(), m_array.end(), pNode);
   if (it != m_array.end())
     m_array.erase(it);
-  return true;
 }
 
-CXFA_Node* CXFA_ArrayNodeList::Item(int32_t iIndex) {
-  int32_t iSize = pdfium::CollectionSize<int32_t>(m_array);
-  return (iIndex >= 0 && iIndex < iSize) ? m_array[iIndex] : nullptr;
+CXFA_Node* CXFA_ArrayNodeList::Item(size_t index) {
+  return index < m_array.size() ? m_array[index] : nullptr;
 }
