@@ -6,16 +6,18 @@
 
 #include "xfa/fxfa/parser/cxfa_daynames.h"
 
+#include "fxjs/xfa/cjx_node.h"
+#include "third_party/base/ptr_util.h"
+
 namespace {
 
 const CXFA_Node::PropertyData kDayNamesPropertyData[] = {
     {XFA_Element::Day, 7, 0},
-    {XFA_Element::Unknown, 0, 0}};
+};
+
 const CXFA_Node::AttributeData kDayNamesAttributeData[] = {
     {XFA_Attribute::Abbr, XFA_AttributeType::Boolean, (void*)0},
-    {XFA_Attribute::Unknown, XFA_AttributeType::Integer, nullptr}};
-
-constexpr wchar_t kDayNamesName[] = L"dayNames";
+};
 
 }  // namespace
 
@@ -27,6 +29,6 @@ CXFA_DayNames::CXFA_DayNames(CXFA_Document* doc, XFA_PacketType packet)
                 XFA_Element::DayNames,
                 kDayNamesPropertyData,
                 kDayNamesAttributeData,
-                kDayNamesName) {}
+                pdfium::MakeUnique<CJX_Node>(this)) {}
 
-CXFA_DayNames::~CXFA_DayNames() {}
+CXFA_DayNames::~CXFA_DayNames() = default;
