@@ -6,16 +6,17 @@
 
 #include "xfa/fxfa/parser/cxfa_defaulttypeface.h"
 
+#include "fxjs/xfa/cjx_node.h"
+#include "third_party/base/ptr_util.h"
+
 namespace {
 
 const CXFA_Node::AttributeData kDefaultTypefaceAttributeData[] = {
     {XFA_Attribute::Desc, XFA_AttributeType::CData, nullptr},
     {XFA_Attribute::WritingScript, XFA_AttributeType::Enum,
-     (void*)XFA_AttributeEnum::Asterisk},
+     (void*)XFA_AttributeValue::Asterisk},
     {XFA_Attribute::Lock, XFA_AttributeType::Integer, (void*)0},
-    {XFA_Attribute::Unknown, XFA_AttributeType::Integer, nullptr}};
-
-constexpr wchar_t kDefaultTypefaceName[] = L"defaultTypeface";
+};
 
 }  // namespace
 
@@ -26,8 +27,8 @@ CXFA_DefaultTypeface::CXFA_DefaultTypeface(CXFA_Document* doc,
                 XFA_XDPPACKET_Config,
                 XFA_ObjectType::NodeV,
                 XFA_Element::DefaultTypeface,
-                nullptr,
+                {},
                 kDefaultTypefaceAttributeData,
-                kDefaultTypefaceName) {}
+                pdfium::MakeUnique<CJX_Node>(this)) {}
 
-CXFA_DefaultTypeface::~CXFA_DefaultTypeface() {}
+CXFA_DefaultTypeface::~CXFA_DefaultTypeface() = default;
