@@ -18,7 +18,6 @@
 #include "third_party/base/stl_util.h"
 #include "xfa/fxfa/cxfa_ffdoc.h"
 #include "xfa/fxfa/cxfa_ffnotify.h"
-#include "xfa/fxfa/layout/cxfa_layoutprocessor.h"
 #include "xfa/fxfa/parser/cscript_datawindow.h"
 #include "xfa/fxfa/parser/cscript_eventpseudomodel.h"
 #include "xfa/fxfa/parser/cscript_hostpseudomodel.h"
@@ -49,8 +48,7 @@
 
 namespace {
 
-constexpr const wchar_t kTemplateNS[] =
-    L"http://www.xfa.org/schema/xfa-template/";
+const wchar_t kTemplateNS[] = L"http://www.xfa.org/schema/xfa-template/";
 
 struct RecurseRecord {
   CXFA_Node* pTemplateChild;
@@ -116,7 +114,7 @@ void FormValueNode_SetChildContent(CXFA_Node* pValueNode,
           }
         }
         pContentRawDataNode = pChildNode->CreateSamePacketNode(element);
-        pChildNode->InsertChild(pContentRawDataNode, nullptr);
+        pChildNode->InsertChildAndNotify(pContentRawDataNode, nullptr);
       }
       pContentRawDataNode->JSObject()->SetCData(XFA_Attribute::Value, wsContent,
                                                 false, false);
@@ -157,7 +155,7 @@ void MergeNodeRecurse(CXFA_Node* pDestNodeParent, CXFA_Node* pProtoNode) {
   }
   CXFA_Node* pNewNode = pProtoNode->Clone(true);
   pNewNode->SetTemplateNode(pProtoNode);
-  pDestNodeParent->InsertChild(pNewNode, nullptr);
+  pDestNodeParent->InsertChildAndNotify(pNewNode, nullptr);
 }
 
 void MergeNode(CXFA_Node* pDestNode, CXFA_Node* pProtoNode) {
@@ -209,12 +207,12 @@ CXFA_Node* CloneOrMergeInstanceManager(CXFA_Document* pDocument,
         break;
 
       CXFA_Node* pNextNode = pNode->GetNextSibling();
-      pFormParent->RemoveChild(pNode, true);
+      pFormParent->RemoveChildAndNotify(pNode, true);
       subforms->push_back(pNode);
       pNode = pNextNode;
     }
-    pFormParent->RemoveChild(pExistingNode, true);
-    pFormParent->InsertChild(pExistingNode, nullptr);
+    pFormParent->RemoveChildAndNotify(pExistingNode, true);
+    pFormParent->InsertChildAndNotify(pExistingNode, nullptr);
     pExistingNode->ClearFlag(XFA_NodeFlag_UnusedNode);
     pExistingNode->SetTemplateNode(pTemplateNode);
     return pExistingNode;
@@ -226,7 +224,7 @@ CXFA_Node* CloneOrMergeInstanceManager(CXFA_Document* pDocument,
       L"_" + pTemplateNode->JSObject()->GetCData(XFA_Attribute::Name);
   pNewNode->JSObject()->SetCData(XFA_Attribute::Name, wsInstMgrNodeName, false,
                                  false);
-  pFormParent->InsertChild(pNewNode, nullptr);
+  pFormParent->InsertChildAndNotify(pNewNode, nullptr);
   pNewNode->SetTemplateNode(pTemplateNode);
   return pNewNode;
 }
@@ -534,7 +532,7 @@ void CreateDataBinding(CXFA_Node* pFormNode,
               pValue->JSObject()->SetCData(XFA_Attribute::Name, L"value", false,
                                            false);
               pValue->CreateXMLMappingNode();
-              pDataNode->InsertChild(pValue, nullptr);
+              pDataNode->InsertChildAndNotify(pValue, nullptr);
               pValue->JSObject()->SetCData(XFA_Attribute::Value, text, false,
                                            false);
             }
@@ -673,9 +671,8 @@ void CreateDataBinding(CXFA_Node* pFormNode,
     }
     case XFA_FFWidgetType::kChoiceList:
       if (pFormNode->IsChoiceListMultiSelect()) {
-        std::vector<CXFA_Node*> items = pDataNode->GetNodeList(
-            XFA_NODEFILTER_Children | XFA_NODEFILTER_Properties,
-            XFA_Element::Unknown);
+        std::vector<CXFA_Node*> items = pDataNode->GetNodeListWithFilter(
+            XFA_NODEFILTER_Children | XFA_NODEFILTER_Properties);
         if (!items.empty()) {
           bool single = items.size() == 1;
           wsNormalizeValue.clear();
@@ -689,9 +686,8 @@ void CreateDataBinding(CXFA_Node* pFormNode,
           }
           CXFA_ExData* exData =
               defValue ? defValue->GetExDataIfExists() : nullptr;
-          ASSERT(exData);
-
-          exData->SetContentType(single ? L"text/plain" : L"text/xml");
+          if (exData)
+            exData->SetContentType(single ? L"text/plain" : L"text/xml");
         }
         FormValueNode_SetChildContent(defValue, wsNormalizeValue,
                                       XFA_Element::ExData);
@@ -739,7 +735,7 @@ CXFA_Node* MaybeCreateDataNode(CXFA_Document* pDocument,
         pDocument->CreateNode(XFA_PacketType::Datasets, eNodeType);
     pDataNode->JSObject()->SetCData(XFA_Attribute::Name, wsName, false, false);
     pDataNode->CreateXMLMappingNode();
-    pDataParent->InsertChild(pDataNode, nullptr);
+    pDataParent->InsertChildAndNotify(pDataNode, nullptr);
     pDataNode->SetFlag(XFA_NodeFlag_Initialized);
     return pDataNode;
   }
@@ -776,7 +772,7 @@ CXFA_Node* MaybeCreateDataNode(CXFA_Document* pDocument,
       pDataNode->JSObject()->SetEnum(XFA_Attribute::Contains,
                                      XFA_AttributeValue::MetaData, false);
     }
-    pDataParent->InsertChild(pDataNode, nullptr);
+    pDataParent->InsertChildAndNotify(pDataNode, nullptr);
     pDataNode->SetDataDescriptionNode(pDDNode);
     pDataNode->SetFlag(XFA_NodeFlag_Initialized);
     return pDataNode;
@@ -1156,8 +1152,8 @@ void UpdateBindingRelations(CXFA_Document* pDocument,
             CXFA_Node* pDataParent = pDataNode->GetParent();
             if (pDataParent != pDataScope) {
               ASSERT(pDataParent);
-              pDataParent->RemoveChild(pDataNode, true);
-              pDataScope->InsertChild(pDataNode, nullptr);
+              pDataParent->RemoveChildAndNotify(pDataNode, true);
+              pDataScope->InsertChildAndNotify(pDataNode, nullptr);
             }
           }
         }
@@ -1277,21 +1273,17 @@ void UpdateDataRelation(CXFA_Node* pDataNode, CXFA_Node* pDataDescriptionNode) {
 
 }  // namespace
 
-CXFA_Document::CXFA_Document(CXFA_FFNotify* notify)
+CXFA_Document::CXFA_Document(CXFA_FFNotify* notify,
+                             std::unique_ptr<LayoutProcessorIface> pLayout)
     : CXFA_NodeOwner(),
       notify_(notify),
-      m_pRootNode(nullptr),
-      m_eCurVersionMode(XFA_VERSION_DEFAULT),
-      m_dwDocFlags(0) {}
+      m_pLayoutProcessor(std::move(pLayout)) {
+  if (m_pLayoutProcessor)
+    m_pLayoutProcessor->SetDocument(this);
+}
 
 CXFA_Document::~CXFA_Document() = default;
 
-CXFA_LayoutProcessor* CXFA_Document::GetLayoutProcessor() {
-  if (!m_pLayoutProcessor)
-    m_pLayoutProcessor = pdfium::MakeUnique<CXFA_LayoutProcessor>(this);
-  return m_pLayoutProcessor.get();
-}
-
 void CXFA_Document::ClearLayoutData() {
   m_pLayoutProcessor.reset();
   m_pScriptContext.reset();
@@ -1384,16 +1376,9 @@ CXFA_Node* CXFA_Document::CreateNode(XFA_PacketType packet,
   return AddOwnedNode(CXFA_Node::Create(this, eElement, packet));
 }
 
-void CXFA_Document::SetFlag(uint32_t dwFlag, bool bOn) {
-  if (bOn)
-    m_dwDocFlags |= dwFlag;
-  else
-    m_dwDocFlags &= ~dwFlag;
-}
-
 bool CXFA_Document::IsInteractive() {
-  if (m_dwDocFlags & XFA_DOCFLAG_HasInteractive)
-    return !!(m_dwDocFlags & XFA_DOCFLAG_Interactive);
+  if (m_Interactive.has_value())
+    return m_Interactive.value();
 
   CXFA_Node* pConfig = ToNode(GetXFAObject(XFA_HASHCODE_Config));
   if (!pConfig)
@@ -1410,16 +1395,13 @@ bool CXFA_Document::IsInteractive() {
 
   CXFA_Interactive* pFormFiller =
       pPDF->GetChild<CXFA_Interactive>(0, XFA_Element::Interactive, false);
-  if (pFormFiller) {
-    m_dwDocFlags |= XFA_DOCFLAG_HasInteractive;
+  if (!pFormFiller)
+    return false;
 
-    WideString wsInteractive = pFormFiller->JSObject()->GetContent(false);
-    if (wsInteractive.EqualsASCII("1")) {
-      m_dwDocFlags |= XFA_DOCFLAG_Interactive;
-      return true;
-    }
-  }
-  return false;
+  WideString wsInteractive = pFormFiller->JSObject()->GetContent(false);
+  bool bInteractive = wsInteractive.EqualsASCII("1");
+  m_Interactive = bInteractive;
+  return bInteractive;
 }
 
 CXFA_LocaleMgr* CXFA_Document::GetLocaleMgr() {
@@ -1641,9 +1623,8 @@ void CXFA_Document::DoDataMerge() {
     pDatasetsRoot->JSObject()->SetCData(XFA_Attribute::Name, L"datasets", false,
                                         false);
 
-    m_pRootNode->GetXMLMappingNode()->AppendChild(pDatasetsXMLNode);
-    m_pRootNode->InsertChild(pDatasetsRoot, nullptr);
-
+    m_pRootNode->GetXMLMappingNode()->AppendLastChild(pDatasetsXMLNode);
+    m_pRootNode->InsertChildAndNotify(pDatasetsRoot, nullptr);
     pDatasetsRoot->SetXMLMappingNode(pDatasetsXMLNode);
   }
 
@@ -1685,7 +1666,7 @@ void CXFA_Document::DoDataMerge() {
         notify_->GetHDOC()->GetXMLDocument()->CreateNode<CFX_XMLElement>(
             L"xfa:data");
     pDataRoot->SetXMLMappingNode(elem);
-    pDatasetsRoot->InsertChild(pDataRoot, nullptr);
+    pDatasetsRoot->InsertChildAndNotify(pDataRoot, nullptr);
   }
 
   CXFA_DataGroup* pDataTopLevel =
@@ -1716,7 +1697,7 @@ void CXFA_Document::DoDataMerge() {
         CreateNode(XFA_PacketType::Form, XFA_Element::Form));
     ASSERT(pFormRoot);
     pFormRoot->JSObject()->SetCData(XFA_Attribute::Name, L"form", false, false);
-    m_pRootNode->InsertChild(pFormRoot, nullptr);
+    m_pRootNode->InsertChildAndNotify(pFormRoot, nullptr);
   } else {
     CXFA_NodeIteratorTemplate<CXFA_Node, CXFA_TraverseStrategy_XFANode>
         sIterator(pFormRoot);
@@ -1745,7 +1726,7 @@ void CXFA_Document::DoDataMerge() {
     pDataTopLevel->SetXMLMappingNode(elem);
 
     CXFA_Node* pBeforeNode = pDataRoot->GetFirstChild();
-    pDataRoot->InsertChild(pDataTopLevel, pBeforeNode);
+    pDataRoot->InsertChildAndNotify(pDataTopLevel, pBeforeNode);
   }
 
   ASSERT(pDataTopLevel);
@@ -1771,7 +1752,7 @@ void CXFA_Document::DoDataMerge() {
     CXFA_PageSet* pNextPageSetNode =
         pPageSetNode->GetNextSameClassSibling<CXFA_PageSet>(
             XFA_Element::PageSet);
-    pSubformSetNode->RemoveChild(pPageSetNode, true);
+    pSubformSetNode->RemoveChildAndNotify(pPageSetNode, true);
     pPageSetNode = pNextPageSetNode;
   }
 
@@ -1786,7 +1767,7 @@ void CXFA_Document::DoDataMerge() {
       if (pNode->IsContainerNode() ||
           pNode->GetElementType() == XFA_Element::InstanceManager) {
         CXFA_Node* pNext = sIterator.SkipChildrenAndMoveToNext();
-        pNode->GetParent()->RemoveChild(pNode, true);
+        pNode->GetParent()->RemoveChildAndNotify(pNode, true);
         pNode = pNext;
       } else {
         pNode->ClearFlag(XFA_NodeFlag_UnusedNode);
@@ -1804,7 +1785,7 @@ void CXFA_Document::DoDataRemerge(bool bDoDataMerge) {
   CXFA_Node* pFormRoot = ToNode(GetXFAObject(XFA_HASHCODE_Form));
   if (pFormRoot) {
     while (CXFA_Node* pNode = pFormRoot->GetFirstChild())
-      pFormRoot->RemoveChild(pNode, true);
+      pFormRoot->RemoveChildAndNotify(pNode, true);
 
     pFormRoot->SetBindingNode(nullptr);
   }
@@ -1813,7 +1794,7 @@ void CXFA_Document::DoDataRemerge(bool bDoDataMerge) {
   if (bDoDataMerge)
     DoDataMerge();
 
-  GetLayoutProcessor()->SetForceReLayout(true);
+  GetLayoutProcessor()->SetForceRelayout(true);
 }
 
 CXFA_Node* CXFA_Document::GetGlobalBinding(uint32_t dwNameHash) {
@@ -1825,3 +1806,24 @@ void CXFA_Document::RegisterGlobalBinding(uint32_t dwNameHash,
                                           CXFA_Node* pDataNode) {
   m_rgGlobalBinding[dwNameHash] = pDataNode;
 }
+
+void CXFA_Document::SetPendingNodesUnusedAndUnbound() {
+  for (CXFA_Node* pPageNode : m_pPendingPageSet) {
+    CXFA_NodeIterator sIterator(pPageNode);
+    for (CXFA_Node* pNode = sIterator.GetCurrent(); pNode;
+         pNode = sIterator.MoveToNext()) {
+      if (pNode->IsContainerNode()) {
+        CXFA_Node* pBindNode = pNode->GetBindData();
+        if (pBindNode) {
+          pBindNode->RemoveBindItem(pNode);
+          pNode->SetBindingNode(nullptr);
+        }
+      }
+      pNode->SetFlag(XFA_NodeFlag_UnusedNode);
+    }
+  }
+}
+
+CXFA_Document::LayoutProcessorIface::LayoutProcessorIface() = default;
+
+CXFA_Document::LayoutProcessorIface::~LayoutProcessorIface() = default;
