@@ -12,6 +12,7 @@
 #include <vector>
 
 #include "core/fxcrt/unowned_ptr.h"
+#include "third_party/base/optional.h"
 #include "xfa/fxfa/fxfa.h"
 #include "xfa/fxfa/parser/cxfa_localemgr.h"
 #include "xfa/fxfa/parser/cxfa_nodeowner.h"
@@ -34,13 +35,6 @@ enum XFA_VERSION {
   XFA_VERSION_MAX = 400,
 };
 
-enum XFA_DocFlag {
-  XFA_DOCFLAG_StrictScoping = 0x0001,
-  XFA_DOCFLAG_HasInteractive = 0x0002,
-  XFA_DOCFLAG_Interactive = 0x0004,
-  XFA_DOCFLAG_Scripting = 0x0008
-};
-
 class CFXJSE_Engine;
 class CJS_Runtime;
 class CScript_DataWindow;
@@ -50,13 +44,26 @@ class CScript_LayoutPseudoModel;
 class CScript_LogPseudoModel;
 class CScript_SignaturePseudoModel;
 class CXFA_FFNotify;
-class CXFA_LayoutProcessor;
 class CXFA_Node;
 class CXFA_Object;
 
 class CXFA_Document final : public CXFA_NodeOwner {
  public:
-  explicit CXFA_Document(CXFA_FFNotify* notify);
+  class LayoutProcessorIface {
+   public:
+    LayoutProcessorIface();
+    virtual ~LayoutProcessorIface();
+    virtual void SetForceRelayout(bool enable) = 0;
+
+    void SetDocument(CXFA_Document* pDocument) { m_pDocument = pDocument; }
+    CXFA_Document* GetDocument() const { return m_pDocument.Get(); }
+
+   private:
+    UnownedPtr<CXFA_Document> m_pDocument;
+  };
+
+  CXFA_Document(CXFA_FFNotify* notify,
+                std::unique_ptr<LayoutProcessorIface> pLayout);
   ~CXFA_Document() override;
 
   bool HasScriptContext() const { return !!m_pScriptContext; }
@@ -75,16 +82,18 @@ class CXFA_Document final : public CXFA_NodeOwner {
   CXFA_Node* GetNotBindNode(
       const std::vector<UnownedPtr<CXFA_Object>>& arrayNodes) const;
 
-  // Creates if not present, never returns NULL.
-  CXFA_LayoutProcessor* GetLayoutProcessor();
+  LayoutProcessorIface* GetLayoutProcessor() const {
+    return m_pLayoutProcessor.get();
+  }
 
   CXFA_Node* GetRoot() const { return m_pRootNode; }
   void SetRoot(CXFA_Node* pNewRoot) { m_pRootNode = pNewRoot; }
 
-  bool HasFlag(uint32_t dwFlag) const {
-    return (m_dwDocFlags & dwFlag) == dwFlag;
-  }
-  void SetFlag(uint32_t dwFlag, bool bOn);
+  bool is_strict_scoping() const { return m_bStrictScoping; }
+  void set_is_strict_scoping() { m_bStrictScoping = true; }
+
+  bool is_scripting() const { return m_bScripting; }
+  void set_is_scripting() { m_bScripting = true; }
 
   bool IsInteractive();
   XFA_VERSION GetCurVersionMode() { return m_eCurVersionMode; }
@@ -108,15 +117,16 @@ class CXFA_Document final : public CXFA_NodeOwner {
 
   CXFA_Node* GetGlobalBinding(uint32_t dwNameHash);
   void RegisterGlobalBinding(uint32_t dwNameHash, CXFA_Node* pDataNode);
+  void SetPendingNodesUnusedAndUnbound();
 
   std::vector<CXFA_Node*> m_pPendingPageSet;
 
  private:
   UnownedPtr<CXFA_FFNotify> const notify_;
-  CXFA_Node* m_pRootNode;
+  CXFA_Node* m_pRootNode = nullptr;
   std::map<uint32_t, CXFA_Node*> m_rgGlobalBinding;
   std::unique_ptr<CFXJSE_Engine> m_pScriptContext;
-  std::unique_ptr<CXFA_LayoutProcessor> m_pLayoutProcessor;
+  std::unique_ptr<LayoutProcessorIface> m_pLayoutProcessor;
   std::unique_ptr<CXFA_LocaleMgr> m_pLocaleMgr;
   std::unique_ptr<CScript_DataWindow> m_pScriptDataWindow;
   std::unique_ptr<CScript_EventPseudoModel> m_pScriptEvent;
@@ -124,8 +134,10 @@ class CXFA_Document final : public CXFA_NodeOwner {
   std::unique_ptr<CScript_LogPseudoModel> m_pScriptLog;
   std::unique_ptr<CScript_LayoutPseudoModel> m_pScriptLayout;
   std::unique_ptr<CScript_SignaturePseudoModel> m_pScriptSignature;
-  XFA_VERSION m_eCurVersionMode;
-  uint32_t m_dwDocFlags;
+  XFA_VERSION m_eCurVersionMode = XFA_VERSION_DEFAULT;
+  Optional<bool> m_Interactive;
+  bool m_bStrictScoping = false;
+  bool m_bScripting = false;
 };
 
 #endif  // XFA_FXFA_PARSER_CXFA_DOCUMENT_H_
