@@ -247,8 +247,7 @@ bool CXFA_DocumentParser::Parse(
 }
 
 CFX_XMLNode* CXFA_DocumentParser::ParseXMLData(const ByteString& wsXML) {
-  auto pStream =
-      pdfium::MakeRetain<CFX_ReadOnlyMemoryStream>(wsXML.AsRawSpan());
+  auto pStream = pdfium::MakeRetain<CFX_ReadOnlyMemoryStream>(wsXML.raw_span());
   xml_doc_ = LoadXML(pStream);
   if (!xml_doc_)
     return nullptr;
@@ -294,7 +293,7 @@ void CXFA_DocumentParser::ConstructXFANode(CXFA_Node* pXFANode,
             pXFAChild->JSObject()->SetCData(XFA_Attribute::Value, wsChildValue,
                                             false, false);
 
-          pXFANode->InsertChild(pXFAChild, nullptr);
+          pXFANode->InsertChildAndNotify(pXFAChild, nullptr);
           pXFAChild->SetXMLMappingNode(pXMLChild);
           pXFAChild->SetFlag(XFA_NodeFlag_Initialized);
           break;
@@ -387,7 +386,7 @@ CXFA_Node* CXFA_DocumentParser::ParseAsXDPPacket_XDP(
     pXMLConfigDOMRoot = pChildItem;
     pXFAConfigDOMRoot = ParseAsXDPPacket_Config(pXMLConfigDOMRoot);
     if (pXFAConfigDOMRoot)
-      pXFARootNode->InsertChild(pXFAConfigDOMRoot, nullptr);
+      pXFARootNode->InsertChildAndNotify(pXFAConfigDOMRoot, nullptr);
   }
 
   CFX_XMLNode* pXMLDatasetsDOMRoot = nullptr;
@@ -430,7 +429,7 @@ CXFA_Node* CXFA_DocumentParser::ParseAsXDPPacket_XDP(
       CXFA_Node* pPacketNode = ParseAsXDPPacket_Template(pElement);
       if (pPacketNode) {
         pXMLTemplateDOMRoot = pElement;
-        pXFARootNode->InsertChild(pPacketNode, nullptr);
+        pXFARootNode->InsertChildAndNotify(pPacketNode, nullptr);
       }
     } else {
       CXFA_Node* pPacketNode = ParseAsXDPPacket(pElement, ePacket);
@@ -441,7 +440,7 @@ CXFA_Node* CXFA_DocumentParser::ParseAsXDPPacket_XDP(
                 FX_HashCode_GetW(packet_info.value().name, false))) {
           return nullptr;
         }
-        pXFARootNode->InsertChild(pPacketNode, nullptr);
+        pXFARootNode->InsertChildAndNotify(pPacketNode, nullptr);
       }
     }
   }
@@ -454,13 +453,13 @@ CXFA_Node* CXFA_DocumentParser::ParseAsXDPPacket_XDP(
     CXFA_Node* pPacketNode =
         ParseAsXDPPacket(pXMLDatasetsDOMRoot, XFA_PacketType::Datasets);
     if (pPacketNode)
-      pXFARootNode->InsertChild(pPacketNode, nullptr);
+      pXFARootNode->InsertChildAndNotify(pPacketNode, nullptr);
   }
   if (pXMLFormDOMRoot) {
     CXFA_Node* pPacketNode =
         ParseAsXDPPacket(pXMLFormDOMRoot, XFA_PacketType::Form);
     if (pPacketNode)
-      pXFARootNode->InsertChild(pPacketNode, nullptr);
+      pXFARootNode->InsertChildAndNotify(pPacketNode, nullptr);
   }
 
   pXFARootNode->SetXMLMappingNode(pXMLDocumentNode);
@@ -569,16 +568,14 @@ CXFA_Node* CXFA_DocumentParser::ParseAsXDPPacket_Data(
     pDataXMLNode = pXMLDocumentNode;
   } else {
     auto* pDataElement = xml_doc_->CreateNode<CFX_XMLElement>(L"xfa:data");
-    CFX_XMLNode* pParentXMLNode = pXMLDocumentNode->GetParent();
-    if (pParentXMLNode)
-      pParentXMLNode->RemoveChildNode(pXMLDocumentNode);
+    pXMLDocumentNode->RemoveSelfIfParented();
 
     CFX_XMLElement* pElement = ToXMLElement(pXMLDocumentNode);
     pElement->RemoveAttribute(L"xmlns:xfa");
 
     // The node was either removed from the parent above, or already has no
     // parent so we can take ownership.
-    pDataElement->AppendChild(pXMLDocumentNode);
+    pDataElement->AppendLastChild(pXMLDocumentNode);
     pDataXMLNode = pDataElement;
   }
   if (!pDataXMLNode)
@@ -658,10 +655,10 @@ CXFA_Node* CXFA_DocumentParser::NormalLoader(CXFA_Node* pXFANode,
                                              CFX_XMLNode* pXMLDoc,
                                              XFA_PacketType ePacketID,
                                              bool bUseAttribute) {
-  constexpr const unsigned long kMaxExecuteRecursion = 1000;
+  constexpr size_t kMaxExecuteRecursion = 1000;
   if (m_ExecuteRecursionDepth > kMaxExecuteRecursion)
     return nullptr;
-  AutoRestorer<unsigned long> restorer(&m_ExecuteRecursionDepth);
+  AutoRestorer<size_t> restorer(&m_ExecuteRecursionDepth);
   ++m_ExecuteRecursionDepth;
 
   bool bOneOfPropertyFound = false;
@@ -710,7 +707,7 @@ CXFA_Node* CXFA_DocumentParser::NormalLoader(CXFA_Node* pXFANode,
           pXFAChild->JSObject()->SetAttribute(attr.value().attribute,
                                               it.second.AsStringView(), false);
         }
-        pXFANode->InsertChild(pXFAChild, nullptr);
+        pXFANode->InsertChildAndNotify(pXFAChild, nullptr);
         if (eType == XFA_Element::Validate || eType == XFA_Element::Locale) {
           if (ePacketID == XFA_PacketType::Config)
             ParseContentNode(pXFAChild, pXMLElement, ePacketID);
@@ -790,7 +787,7 @@ void CXFA_DocumentParser::ParseContentNode(CXFA_Node* pXFANode,
       ASSERT(pContentRawDataNode);
       pContentRawDataNode->JSObject()->SetCData(XFA_Attribute::Value, wsValue,
                                                 false, false);
-      pXFANode->InsertChild(pContentRawDataNode, nullptr);
+      pXFANode->InsertChildAndNotify(pContentRawDataNode, nullptr);
     } else {
       pXFANode->JSObject()->SetCData(XFA_Attribute::Value, wsValue, false,
                                      false);
@@ -886,7 +883,7 @@ void CXFA_DocumentParser::ParseDataGroup(CXFA_Node* pXFANode,
                                              false, false);
           pXFAMetaData->JSObject()->SetEnum(
               XFA_Attribute::Contains, XFA_AttributeValue::MetaData, false);
-          pXFAChild->InsertChild(pXFAMetaData, nullptr);
+          pXFAChild->InsertChildAndNotify(pXFAMetaData, nullptr);
           pXFAMetaData->SetXMLMappingNode(pXMLElement);
           pXFAMetaData->SetFlag(XFA_NodeFlag_Initialized);
         }
@@ -894,7 +891,7 @@ void CXFA_DocumentParser::ParseDataGroup(CXFA_Node* pXFANode,
         if (!bNeedValue)
           pXMLElement->RemoveAttribute(L"xsi:nil");
 
-        pXFANode->InsertChild(pXFAChild, nullptr);
+        pXFANode->InsertChildAndNotify(pXFAChild, nullptr);
         if (eNodeType == XFA_Element::DataGroup)
           ParseDataGroup(pXFAChild, pXMLElement, ePacketID);
         else if (bNeedValue)
@@ -918,7 +915,7 @@ void CXFA_DocumentParser::ParseDataGroup(CXFA_Node* pXFANode,
 
         pXFAChild->JSObject()->SetCData(XFA_Attribute::Value, wsText, false,
                                         false);
-        pXFANode->InsertChild(pXFAChild, nullptr);
+        pXFANode->InsertChildAndNotify(pXFAChild, nullptr);
         pXFAChild->SetXMLMappingNode(pXMLText);
         pXFAChild->SetFlag(XFA_NodeFlag_Initialized);
         continue;
@@ -970,7 +967,7 @@ void CXFA_DocumentParser::ParseDataValue(CXFA_Node* pXFANode,
                                         false, false);
         pXFAChild->JSObject()->SetCData(XFA_Attribute::Value, wsCurValue, false,
                                         false);
-        pXFANode->InsertChild(pXFAChild, nullptr);
+        pXFANode->InsertChildAndNotify(pXFAChild, nullptr);
         pXFAChild->SetXMLMappingNode(pXMLCurValueNode);
         pXFAChild->SetFlag(XFA_NodeFlag_Initialized);
         wsValueTextBuf << wsCurValue;
@@ -987,7 +984,7 @@ void CXFA_DocumentParser::ParseDataValue(CXFA_Node* pXFANode,
     pXFAChild->JSObject()->SetCData(XFA_Attribute::Name, wsNodeStr, false,
                                     false);
     ParseDataValue(pXFAChild, pXMLChild, ePacketID);
-    pXFANode->InsertChild(pXFAChild, nullptr);
+    pXFANode->InsertChildAndNotify(pXFAChild, nullptr);
     pXFAChild->SetXMLMappingNode(pXMLChild);
     pXFAChild->SetFlag(XFA_NodeFlag_Initialized);
     WideString wsCurValue =
@@ -1008,7 +1005,7 @@ void CXFA_DocumentParser::ParseDataValue(CXFA_Node* pXFANode,
                                         false, false);
         pXFAChild->JSObject()->SetCData(XFA_Attribute::Value, wsCurValue, false,
                                         false);
-        pXFANode->InsertChild(pXFAChild, nullptr);
+        pXFANode->InsertChildAndNotify(pXFAChild, nullptr);
         pXFAChild->SetXMLMappingNode(pXMLCurValueNode);
         pXFAChild->SetFlag(XFA_NodeFlag_Initialized);
       }
@@ -1031,14 +1028,14 @@ void CXFA_DocumentParser::ParseInstruction(CXFA_Node* pXFANode,
         (pXFANode->GetDocument()->RecognizeXFAVersionNumber(target_data[0]) !=
          XFA_VERSION_UNKNOWN) &&
         target_data[1].EqualsASCII("v2.7-scripting:1")) {
-      pXFANode->GetDocument()->SetFlag(XFA_DOCFLAG_Scripting, true);
+      pXFANode->GetDocument()->set_is_scripting();
     }
     return;
   }
   if (pXMLInstruction->IsAcrobat()) {
     if (target_data.size() > 1 && target_data[0].EqualsASCII("JavaScript") &&
         target_data[1].EqualsASCII("strictScoping")) {
-      pXFANode->GetDocument()->SetFlag(XFA_DOCFLAG_StrictScoping, true);
+      pXFANode->GetDocument()->set_is_strict_scoping();
     }
   }
 }
