@@ -6,7 +6,7 @@
 
 #include "xfa/fxfa/parser/cxfa_wsdladdress.h"
 
-#include "fxjs/xfa/cjx_wsdladdress.h"
+#include "fxjs/xfa/cjx_textnode.h"
 #include "third_party/base/ptr_util.h"
 
 namespace {
@@ -16,9 +16,7 @@ const CXFA_Node::AttributeData kWsdlAddressAttributeData[] = {
     {XFA_Attribute::Name, XFA_AttributeType::CData, nullptr},
     {XFA_Attribute::Use, XFA_AttributeType::CData, nullptr},
     {XFA_Attribute::Usehref, XFA_AttributeType::CData, nullptr},
-    {XFA_Attribute::Unknown, XFA_AttributeType::Integer, nullptr}};
-
-constexpr wchar_t kWsdlAddressName[] = L"wsdlAddress";
+};
 
 }  // namespace
 
@@ -28,9 +26,8 @@ CXFA_WsdlAddress::CXFA_WsdlAddress(CXFA_Document* doc, XFA_PacketType packet)
                 XFA_XDPPACKET_ConnectionSet,
                 XFA_ObjectType::TextNode,
                 XFA_Element::WsdlAddress,
-                nullptr,
+                {},
                 kWsdlAddressAttributeData,
-                kWsdlAddressName,
-                pdfium::MakeUnique<CJX_WsdlAddress>(this)) {}
+                pdfium::MakeUnique<CJX_TextNode>(this)) {}
 
-CXFA_WsdlAddress::~CXFA_WsdlAddress() {}
+CXFA_WsdlAddress::~CXFA_WsdlAddress() = default;
